/*
* QuData Chat Builder Plugin.
*
* @copyright Copyright (C) 2021, qudata.com
*/
window.editorConfig = JSON.parse(`{
    "version":    "1.0.61",
    "botManager": "https://services.qudata.com/bot/manager/api/",
    "analytics":  "https://services.qudata.com/bot/analytics/api/v1/"
  }`);

class QuBotCard {
    constructor(elem, posFree, posGrid) {
        this.posFree = posFree, this.posGrid = posGrid, this.elem = elem, this.folder = null;
    }
    type() {
        throw new Error("You have to implement the method type!");
    }
    center() {
        var cardRect = this.elem.getBoundingClientRect();
        return {
            x: (cardRect.left + cardRect.right) / 2,
            y: (cardRect.top + cardRect.bottom) / 2
        };
    }
    centerLocal() {
        var cardRect = this.elem.getBoundingClientRect();
        return {
            x: (cardRect.right - cardRect.left) / 2,
            y: (cardRect.bottom - cardRect.top) / 2
        };
    }
    rect() {
        return this.elem.getBoundingClientRect();
    }
    rectLocal(scale = 1) {
        var cardRect = this.elem.getBoundingClientRect(), parentRect = this.elem.parentElement.getBoundingClientRect();
        return {
            left: (cardRect.left - parentRect.left) / scale,
            top: (cardRect.top - parentRect.top) / scale,
            width: cardRect.width / scale,
            height: cardRect.height / scale
        };
    }
    pos() {
        var cardRect = this.elem.getBoundingClientRect();
        return {
            x: cardRect.left,
            y: cardRect.top
        };
    }
    posLocal() {
        var cardRect = this.elem.getBoundingClientRect(), parentRect = this.elem.parentElement.getBoundingClientRect();
        return {
            x: cardRect.left - parentRect.left,
            y: cardRect.top - parentRect.top
        };
    }
    visible() {
        return !this.folder || !!this.folder.open && this.folder.visible;
    }
    visibleFolder() {
        let folders = [], card = this;
        for (;card.folder; ) folders.push(card.folder), card = card.folder;
        for (card = this; folders.length; ) {
            let newCard = folders.pop();
            if (!newCard.visible()) return card;
            card = newCard;
        }
        return card;
    }
    setFolder(folderCard) {
        this.folder = folderCard;
    }
    toString() {
        return this.folder ? this.folder.toString() + "_" + this.kind : this.kind;
    }
}

class QuBotCardAdd extends QuBotCard {
    constructor(elem, posFree, posGrid) {
        super(elem, posFree, posGrid), this.kind = "add";
    }
    type() {
        return "add";
    }
    toString() {
        return this.folder ? this.folder.toString() + "_" + this.kind : this.kind;
    }
}

class QuBotCardState extends QuBotCard {
    constructor(state, elem, posFree, posGrid) {
        super(elem, posFree, posGrid), this.state = state, this.kind = "state";
    }
    type() {
        return "state";
    }
    toString() {
        return this.folder ? this.folder.toString() + "_" + this.kind + "_" + this.state : this.kind + "_" + this.state;
    }
}

class QuBotCardFolder extends QuBotCard {
    constructor(name, elem, posFree, posGrid, openFree, openGrid) {
        super(elem, posFree, posGrid), this.name = name, this.kind = "folder", this.open = !1, 
        this.folder = null, this.openFree = openFree, this.openGrid = openGrid;
    }
    type() {
        return "folder";
    }
    toString() {
        return this.folder ? this.folder.toString() + "_" + this.kind + "_" + this.name : this.kind + "_" + this.name;
    }
}

class QuBotCards {
    constructor() {
        this.cards = {
            state: [],
            add: [],
            folder: []
        };
    }
    pushStateCard(state, cardElement, posFree = null, posGrid = null) {
        state = new QuBotCardState(state, cardElement, posFree, posGrid);
        return this.cards.state.push(state), state;
    }
    pushAddCard(cardElement, posFree = null, posGrid = null) {
        cardElement = new QuBotCardAdd(cardElement, posFree, posGrid);
        return this.cards.add.push(cardElement), cardElement;
    }
    pushFolderCard(name, cardElement, posFree = null, posGrid = null, openFree = null, openGrid = null) {
        name = new QuBotCardFolder(name, cardElement, posFree, posGrid, openFree, openGrid);
        return this.cards.folder.push(name), name;
    }
    popCard(card) {
        this.cards[card.kind] = this.cards[card.kind].filter(function(value) {
            return value != card;
        });
    }
    getStateCards() {
        return this.cards.state;
    }
    getFolderCards() {
        return this.cards.folder;
    }
    getContentCards() {
        return this.cards.state.concat(this.cards.folder);
    }
    getAllCardsWithRow(r) {
        return this.getAllCards().filter(function(card) {
            return card.posGrid.r == r;
        });
    }
    getAddCards() {
        return this.cards.add;
    }
    getAllCards() {
        let cards = [];
        for (var kind in this.cards) cards = cards.concat(this.cards[kind]);
        return cards;
    }
    getCardsFromFolder(folderCard) {
        let allCards = this.getAllCards();
        return allCards.filter(function(card) {
            return card.folder == folderCard;
        });
    }
    getContentCardsFromFolder(folderCard) {
        let contentCards = this.getContentCards();
        return contentCards.filter(function(card) {
            return card.folder == folderCard;
        });
    }
    findCardByElement(elem) {
        let cards = this.getAllCards();
        return cards.find(function(card) {
            return card.elem == elem;
        });
    }
    findCardByState(state) {
        return this.cards.state.find(function(card) {
            return card.state == state;
        });
    }
    findFolderByName(name) {
        return this.cards.folder.find(function(card) {
            return card.name == name;
        });
    }
    findCardByColRow(c, r) {
        return this.getAllCards().find(function(card) {
            return card.posGrid && card.posGrid.c == c && card.posGrid.r == r;
        });
    }
    rectLocal(cards, scale) {
        let minX = null, minY = null, maxX = null, maxY = null;
        return cards.forEach(card => {
            card = card.rectLocal(scale);
            minX = null == minX || card.left < minX ? card.left : minX, minY = null == minY || card.top < minY ? card.top : minY, 
            maxX = null == maxX || card.left + card.width > maxX ? card.left + card.width : maxX, 
            maxY = null == maxY || card.top + card.height > maxY ? card.top + card.height : maxY;
        }), {
            left: minX,
            top: minY,
            width: maxX - minX,
            height: maxY - minY
        };
    }
    clear() {
        this.cards = {
            state: [],
            add: [],
            folder: []
        };
    }
    nearest(card) {
        var cards = this.getAllCards();
        return this.nearestInList(card, cards);
    }
    nearestInList(card, cards) {
        var cardOtherPos, cardPos = card.posLocal();
        let dist = -1, nearest = null;
        for (let i = 0; i < cards.length; i++) {
            let cardOther = cards[i];
            cardOther != card && cardOther.visible() && (cardOtherPos = cardOther.posLocal(), 
            cardOtherPos = Math.pow(cardPos.x - cardOtherPos.x, 2) + Math.pow(cardPos.y - cardOtherPos.y, 2), 
            (dist < 0 || cardOtherPos < dist) && (dist = cardOtherPos, nearest = cardOther));
        }
        return nearest;
    }
}

class QuBotEditorSVG {
    constructor() {
        this.svg = null, this.svgColor = "blue", this.svgSpline = !1, this.scale = 1, this.posX = 0, 
        this.posY = 0;
    }
    create() {
        this.svg = document.getElementById("qubot-svg-drawing"), this.posX = 0, this.posY = 0, 
        this.scale = 1;
    }
    clear() {
        if (this.svg) for (;1 < this.svg.childElementCount; ) this.svg.lastChild.remove();
    }
    drawPathArrow(path, color, alpha = 1) {
        let fig = document.createElementNS("http://www.w3.org/2000/svg", "path");
        fig.setAttribute("d", path), fig.style.stroke = color, fig.style.strokeWidth = "2px", 
        fig.style.strokeOpacity = alpha, fig.style.fill = "transparent", fig.style.markerEnd = "url(#qubot-arrow-" + color + ")", 
        fig.style.markerStart = "url(#qubot-circle-" + color + ")", fig.classList.add("qubot-svg-shadow"), 
        this.svg.appendChild(fig);
    }
    drawPathPoint(path, color, alpha = 0) {
        let fig = document.createElementNS("http://www.w3.org/2000/svg", "path");
        fig.setAttribute("d", path), fig.style.stroke = color, fig.style.strokeWidth = "2px", 
        fig.style.strokeOpacity = "0", fig.style.fill = "transparent", fig.style.markerEnd = "none", 
        fig.style.markerStart = "url(#qubot-big-circle-" + color + ")", fig.classList.add("qubot-svg-shadow"), 
        this.svg.appendChild(fig);
    }
    drawPath(path, color) {
        let fig = document.createElementNS("http://www.w3.org/2000/svg", "path");
        fig.setAttribute("d", path), fig.style.stroke = color, fig.style.strokeWidth = "2px", 
        fig.style.fill = "transparent", this.svg.appendChild(fig);
    }
    drawSpline(cx1, cy1, w1, h1, cx2, cy2, w2, h2, scale = 0, color = "gray", alpha = 1) {
        let curve = .5, px1 = cx1, py1, sx1, sy1, px2, py2 = cy2, sx2, sy2;
        px2 = Math.abs(cx1 - cx2) < w1 / 8 ? (px1 = cx1 + w1, cx2 + w2 + 2) : cx1 + w1 < cx2 - w2 ? (px1 = cx1 + w1, 
        cx2 - w2 - 2) : cx2 + w2 < cx1 - w1 ? (px1 = cx1 - w1, cx2 + w2 + 2) : cx2 + w2 < cx1 + w1 ? (px1 = cx1 + w1, 
        cx2 + w2 + 2) : cx1 - w1 < cx2 - w2 ? (px1 = cx1 - w1, cx2 - w2 - 2) : (px1 = cx1 - w1, 
        cx2 + w2 + 2), py1 = cy1, curve *= Math.abs(px1 - px2), curve < 20 && (curve = 20), 
        sx1 = px1 + Math.sign(px1 - cx1) * curve, sy1 = py1, sy2 = py2, sx2 = px2 + Math.sign(px2 - cx2) * curve;
        cy2 = "M" + Math.round(px1) + "," + Math.round(py1) + " C " + Math.round(sx1) + "," + Math.round(sy1) + " " + Math.round(sx2) + "," + Math.round(sy2) + " " + Math.round(px2) + "," + Math.round(py2);
        this.drawPathArrow(cy2, color, alpha);
    }
    drawArrow(cx1, cy1, w1, h1, cx2, cy2, w2, h2, scale = 0, color = "blue", alpha = 1) {
        let px2, py2 = cy2;
        px2 = cx1 + w1 < cx2 - w2 ? cx2 - w2 + 10 : cx2 + w2 - 10, this.drawPathArrow("M " + cx1 + " " + (cy1 + h1) + " L " + px2 + " " + py2, color, alpha);
    }
    drawPoint(cx1, cy1, w1, h1, scale = 0, color = "blue", alpha = 1) {
        let px2, py2 = cy1;
        px2 = cx1 + w1 < cx1 - w1 ? cx1 - w1 + 10 : cx1 + w1 - 10, this.drawPathPoint("M " + cx1 + " " + (cy1 + h1) + " L " + px2 + " " + py2, color, alpha);
    }
    drawLines(points, color = "green") {
        let path = "";
        for (let i = 0; i < points.length; i++) path += 0 == i ? "M " : " L ", path += points[i].x + " " + points[i].y;
        this.drawPath(path, color);
    }
}

class QuBotEditorList extends QuBotEditorBase {
    constructor(cards, lists) {
        super(), this.cardOpacity = "0.9", this.canvas = new QuBotEditorSVG(), this.space = null, 
        this.cards = cards, this.lists = lists, this.active = !1, this.activeFolder = null;
    }
    init() {}
    show() {
        this.active = !0;
    }
    hide() {
        this.active = !1;
    }
    placeCard(card) {
        this.active && ("add" == card.kind ? card.caption.onmousedown = this.mousedownCardAdd.bind(this) : "folder" == card.kind ? card.caption.onmousedown = this.mousedownCardFolder.bind(this) : card.caption.onmousedown = this.mousedownCardState.bind(this));
    }
    placeCardInFolder(card, folderCard, beforeCard = 0, afterCard) {
        card.setFolder(folderCard);
    }
    onResizeCardEvent(card) {}
    findCardByState(state) {
        return this.cards.findCardByState(state);
    }
    findCardByElement(obj) {
        return this.cards.findCardByElement(obj);
    }
    clear() {
        this.canvas.clear(), this.cards.clear();
    }
    create() {
        this.space = document.getElementById("qubot-state-space"), this.canvas.create();
    }
    createCardState(state, kind, save = null) {
        null == state && (state = qubotEditor.newStateName(), this.createEmptyState(state, kind));
        let cardElem = this.add(this.space, "qubot-state"), content = (cardElem.id = "qubot-state-" + state, 
        cardElem.contentEditable = !1, cardElem.classList.add("qubot-disable-select"), this.add(cardElem, "content")), caption = (content.setAttribute("onclick", "qubotEditor.listEditor.onclickCard(this.parentNode)"), 
        this.add(cardElem, "caption"));
        caption.innerHTML = state, caption.classList.add("qubot-disable-select"), this.add(cardElem, "label-start"), 
        this.openCardState(state, cardElem), this.addBtnDel(cardElem, "qubotEditor.listEditor.onclickBtnDelCard(this)", -3, -3);
        kind = this.addBtnAdd(cardElem, "qubotEditor.listEditor.onclickBtnAddCard(this)", !1, !1, -5, -3);
        cardElem.setAttribute("onmouseover", "qubotEditor.listEditor.mouseoverCard(this)"), 
        cardElem.setAttribute("onmouseenter", "qubotEditor.listEditor.mouseenterCardState(this)");
        let card;
        return save ? (card = this.cards.pushStateCard(state, cardElem, {
            x: save.x,
            y: save.y
        }, {
            c: save.c,
            r: save.r
        }), card.setFolder(this.cards.findFolderByName(save.folder))) : card = this.cards.pushStateCard(state, cardElem), 
        card.btnAdd = kind, card.caption = caption, card;
    }
    createCardAdd() {
        let cardElem = this.add(this.space, "qubot-add-state"), caption = (cardElem.id = "qubot-add-state", 
        cardElem.setAttribute("onclick", "qubotEditor.listEditor.onclickCardAdd(this);"), 
        this.add(cardElem, "caption"));
        caption.innerHTML = "Drag me or click", caption.classList.add("qubot-disable-select");
        var content = this.add(cardElem, "content"), content = (this.add(content, "qubot-icon-add-state"), 
        cardElem.setAttribute("onmouseover", "qubotEditor.listEditor.mouseoverCard(this)"), 
        this.cards.pushAddCard(cardElem, null, null));
        return content;
    }
    createCardFolder(name = null, save = null) {
        var folders = this.cards.getFolderCards().length;
        if (name || (name = "New folder", 0 < folders && (name += "_" + (folders + 1)), 
        null != (name = prompt("Please enter new folder name:", name)) && "" != name)) {
            let cardElem = this.add(this.space, "qubot-folder"), caption = (cardElem.id = "qubot-folder", 
            cardElem.classList.add("qubot-disable-select"), this.add(cardElem, "caption")), content = (caption.innerHTML = name, 
            caption.classList.add("qubot-disable-select"), this.add(cardElem, "content"));
            folders = this.add(content, "qubot-icon-folder-closed");
            this.addBtnDel(cardElem, "qubotEditor.listEditor.onclickBtnDelCloseCardFolder(this)"), 
            cardElem.setAttribute("onmouseover", "qubotEditor.listEditor.mouseoverCard(this)"), 
            content.setAttribute("onclick", "qubotEditor.listEditor.onclickCardFolder(this.parentElement);");
            let card;
            card = save ? this.cards.pushFolderCard(name, cardElem, {
                x: save.x,
                y: save.y
            }, {
                c: save.c,
                r: save.r
            }, save.openFree, save.openGrid) : this.cards.pushFolderCard(name, cardElem), card.caption = caption, 
            card.icon = folders;
            save = qubotEditor.bot.state, name = this.cards.findCardByState(save);
            return name && this.lists.placeCardAfter(card, name), this.activeFolder = card, 
            card;
        }
    }
    copyCardFolder() {
        var bot, folderDeps;
        this.activeFolder ? (bot = qubotEditor.bot, this.saveCards(), folderDeps = {
            states: {},
            cards: {
                states: {},
                folders: {}
            }
        }, this.collectDeps(bot, this.activeFolder, folderDeps), this.lists.clipboard.copyFolder = {
            name: this.activeFolder.name,
            deps: folderDeps
        }) : qubotEditor.help.alert("NoActiveFolder");
    }
    deepCopy(obj) {
        return JSON.parse(JSON.stringify(obj));
    }
    collectDeps(bot, card, save) {
        if ("folder" == card.kind) {
            save.cards.folders[card.name] = this.deepCopy(bot.cards.folders[card.name]);
            let folderCards = this.cards.getContentCardsFromFolder(this.activeFolder);
            folderCards.forEach(card => {
                this.collectDeps(bot, card, save);
            });
        } else "state" == card.kind && (save.states[card.state] = this.deepCopy(bot.states[card.state]), 
        save.cards.states[card.state] = this.deepCopy(bot.cards.states[card.state]));
    }
    pasteCardFolder() {
        var folderRenames, state;
        this.lists.clipboard.copyFolder && (this.pasteBotPart(this.lists.clipboard.copyFolder.deps, {}, folderRenames = {}), 
        (folderRenames = this.cards.findFolderByName(folderRenames[this.lists.clipboard.copyFolder.name])) && (state = qubotEditor.bot.state, 
        state = this.cards.findCardByState(state), this.lists.placeCardAfter(folderRenames, state)));
    }
    pasteBotPart(part, stateRenames, folderRenames) {
        part = this.deepCopy(part), this.saveCards();
        let bot = qubotEditor.bot;
        for (var folderName in this.collectRenames(stateRenames, part.states, bot.states), 
        this.collectRenames(folderRenames, part.cards.folders, bot.cards.folders), part.cards.folders) {
            let folderSave = part.cards.folders[folderName];
            folderSave.openFree = folderSave.openGrid = !1, this.createCardFolder(folderRenames[folderName], folderSave);
        }
        for (var stateName in part.states) {
            stateName = part.states[stateName];
            let states = qubotEditor.stateEditor.items.getStates(stateName);
            states.forEach(ss => {
                ss.state in stateRenames && ("state" in ss.action ? ss.action.state = stateRenames[ss.state] : "goto" in ss.action ? ss.action.goto = stateRenames[ss.state] : "run" in ss.action && (ss.action.run = stateRenames[ss.state]));
            });
        }
        for (let stateName in part.states) {
            var stateSave = part.states[stateName];
            bot.states[stateRenames[stateName]] = stateSave;
            let stateCardSave = part.cards.states[stateName];
            if (stateCardSave.folder) {
                stateCardSave.folder = folderRenames[stateCardSave.folder];
                let card = this.createCardState(stateRenames[stateName], null, stateCardSave);
                card.visible() || this.hideCard(card);
            } else this.createCardState(stateRenames[stateName], null, stateCardSave);
        }
    }
    renameState(oldState, newState) {
        let card = this.cards.findCardByState(oldState);
        card && (card.state = newState, card.elem.id = "qubot-state-" + newState, card.elem.getElementsByClassName("caption")[0].innerHTML = newState), 
        this.save();
    }
    save() {
        this.saveCards(), qubotEditor.myBots.save();
    }
    saveCards() {
        let stateCardsSave = {}, folderCardsSave = {}, stateCards = this.cards.getContentCards(), bot = (stateCards.forEach(card => {
            let cardSave = {
                c: 0,
                r: 0,
                x: 0,
                y: 0
            };
            card.posGrid && (cardSave.c = card.posGrid.c, cardSave.r = card.posGrid.r), card.posFree && (cardSave.x = card.posFree.x, 
            cardSave.y = card.posFree.y), card.folder && (cardSave.folder = card.folder.name), 
            "state" == card.kind ? stateCardsSave[card.state] = cardSave : "folder" == card.kind && (folderCardsSave[card.name] = cardSave, 
            "grid" == this.kindView ? folderCardsSave[card.name].openGrid = card.open : folderCardsSave[card.name].openFree = card.open);
        }), qubotEditor.bot);
        bot.view = this.kindView, bot.cards.states = stateCardsSave, bot.cards.folders = folderCardsSave;
    }
    collectRenames(renames, fromObj, toObj) {
        for (var field in fromObj) if (field in toObj) {
            var oldName = field;
            let newName, i = 1;
            for (;newName = oldName + "_" + i, i++, newName in toObj; );
            renames[oldName] = newName;
        } else renames[field] = field;
    }
    renameCardFolder() {
        var oldName;
        this.activeFolder ? (oldName = this.activeFolder.name, null != (oldName = prompt("Please enter new folder name:", oldName)) && "" != oldName && (this.activeFolder.name = oldName, 
        this.activeFolder.caption.innerHTML = oldName, this.save())) : qubotEditor.help.alert("NoActiveFolder");
    }
    openCardCurState() {
        this.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state));
    }
    openCardState(state, card) {
        this.LANGUAGE = qubotEditor.stateEditor.LANGUAGE;
        let body = card.getElementsByClassName("content")[0], messages = body.getElementsByClassName("message");
        if (void 0 !== messages) for (;0 < messages.length; ) messages[0].remove();
        let inputs = body.getElementsByClassName("item-input");
        if (void 0 !== inputs) for (;0 < inputs.length; ) inputs[0].remove();
        let message = null, items = qubotEditor.bot.states[state], kindInput = null, isInput = null;
        if (items.forEach(item => {
            if ("text" === item.type) {
                message = message || this.add(body, "message");
                let txt = this.add(message, "text");
                txt.classList.add("qubot-disable-select"), txt.innerHTML = this.getText(item.text).replace(/\n/g, "<br>"), 
                void 0 !== item.color && (txt.style.color = item.color), item.center && (txt.style.textAlign = "center");
            } else if ("row" === item.type) {
                message = message || this.add(body, "message");
                let row = this.add(message, "item-row");
                item.items.forEach(btn => {
                    let button = this.add(row, "button");
                    var text = this.getText(btn.button).replace(/\n/g, "<br>");
                    "check" === btn.kind ? button.innerHTML = "☐ " + text : "radio" === btn.kind ? button.innerHTML = "○ " + text : button.innerHTML = text, 
                    "left" === btn.align ? button.style.textAlign = "left" : "right" === btn.align && (button.style.textAlign = "right"), 
                    button.classList.add("qubot-disable-select");
                });
            } else if ("images" === item.type) {
                message = message || this.add(body, "message");
                let images = this.add(message, "item-images");
                item.items.forEach(image => {
                    let img = this.add(images, "image", "img");
                    img.classList.add("qubot-disable-select"), img.setAttribute("onload", "qubotEditor.listEditor.showArrows(qubotEditor.bot.state)"), 
                    img.src = image.image, image.width && (img.style.width = image.width + "%"), 0 === image.center && (img.style.marginLeft = img.style.marginRight = "0px");
                });
            } else if ("map" === item.type) {
                message = message || this.add(body, "message");
                let map = this.add(message, "map");
                map.classList.add("qubot-disable-select");
                var src = item.url.match(/src\s*=\s*"(.+?)"/);
                src && src.length && (map.innerHTML = `<iframe ${src[0]} width="100%"  style="border:0; pointer-events: none;" loading="lazy"></iframe>`);
            } else if ("video" === item.type) {
                message = message || this.add(body, "message");
                let video = this.add(message, "video");
                video.classList.add("qubot-disable-select"), video.innerHTML = "🎞️";
            } else if ("audio" === item.type) {
                message = message || this.add(body, "message");
                let audio = this.add(message, "audio");
                audio.classList.add("qubot-disable-select"), audio.innerHTML = "🎧";
            } else "message" === item.type ? message = this.add(body, "message") : "input" == item.type && (kindInput = item.kind, 
            isInput = item.actions && 0 < item.actions.length);
        }), isInput) {
            let inp = this.add(body, "item-input");
            "text" === kindInput ? inp.innerHTML = "✍🏽" : "intent" === kindInput ? (inp.innerHTML = "🙊", 
            inp.style.backgroundColor = "#dfd") : "int" === kindInput || "float" === kindInput ? inp.innerHTML = "🔢" : "email" === kindInput ? inp.innerHTML = "📧" : "phone" === kindInput ? inp.innerHTML = "📞" : "MMDDYYYY" === kindInput || "DDMMYYYY" === kindInput ? inp.innerHTML = "📆" : "time" === kindInput && (inp.innerHTML = "⌚");
        }
        state == qubotEditor.bot.start && card.classList.add("qubot-start-state");
    }
    setStartState(state) {
        qubotEditor.bot.start = state, qubotEditor.myBots.bot && (qubotEditor.myBots.bot.start = state);
        let cards = this.space.getElementsByClassName("qubot-state");
        for (let i = 0; i < cards.length; i++) {
            cards[i].classList.remove("qubot-start-state");
            let state = cards[i].id.substring(12);
            state === qubotEditor.bot.start && cards[i].classList.add("qubot-start-state");
        }
        qubotEditor.save();
    }
    addLinkClasses(state) {
        let obj = document.getElementById("qubot-state-" + state);
        if (obj) {
            this.removeAddedClasses(), obj.classList.add("qubot-state-active");
            let states = qubotEditor.stateEditor.items.getStates(qubotEditor.bot.states[state]);
            states.forEach(s => {
                if (s.state && s.state.length && s.state in qubotEditor.bot.states) {
                    var stateCard = this.cards.findCardByState(s.state);
                    if (stateCard) {
                        let link = stateCard.elem;
                        void 0 !== link && state !== s.state && link.classList.add("qubot-state-link");
                    }
                }
            });
        }
    }
    removeAddedClasses() {
        var list = this.space.getElementsByClassName("qubot-state");
        Array.from(list).forEach(card => {
            card.classList.remove("qubot-state-link"), card.classList.remove("qubot-state-link-main"), 
            card.classList.remove("qubot-state-active");
        });
    }
    setCurState(state) {
        state && (qubotEditor.bot.state = state), this.removeAddedClasses(), this.addLinkClasses(qubotEditor.bot.state), 
        this.showArrows();
    }
    drawArrows(state1, states, color, alpha = 1) {
        let card1 = this.cards.findCardByState(state1);
        if (card1) {
            if (!card1.visible()) {
                var folderCard = card1.visibleFolder();
                if (!folderCard) return;
                card1 = folderCard;
            }
            states = states || qubotEditor.stateEditor.items.getStates(qubotEditor.bot.states[state1]);
            let rect0 = this.canvas.svg.getBoundingClientRect(), splines = {}, arrows = {}, points = {};
            for (var spline in states.forEach(state2 => {
                if (state2 && state2.state && state2.state.length && state2.state in qubotEditor.bot.states) {
                    let card2 = this.cards.findCardByState(state2.state);
                    if (card2) {
                        let rect1;
                        if ("state" == card1.kind) if (0 <= state2.id) rect1 = card1.elem.getElementsByClassName("button")[state2.id].getBoundingClientRect(); else {
                            if (rect1 = card1.elem.getBoundingClientRect(), -1 === state2.id) {
                                let itemInput = card1.elem.getElementsByClassName("item-input");
                                rect1 = (0 < itemInput.length ? itemInput[0] : card1.elem).getBoundingClientRect();
                            }
                            -2 === state2.id && (rect1 = card1.elem.getBoundingClientRect());
                        } else rect1 = card1.elem.getBoundingClientRect();
                        (card2.visible() || (card2 = card2.visibleFolder(), card2)) && card1 != card2 && (state2 = card2.elem.getBoundingClientRect(), 
                        "free" == this.kindView ? (args = [ rect1.left - rect0.left + rect1.width / 2, rect1.top - rect0.top + rect1.height / 2, rect1.width / 2, rect1.height / 2, state2.left - rect0.left + state2.width / 2, state2.top - rect0.top + 10, state2.width / 2, 10, 1, color, alpha ], 
                        splines[args.join()] = args) : "green" == color ? function(args) {
                            points[args.join()] = args;
                        }([ rect1.left - rect0.left + rect1.width / 2, rect1.top - rect0.top + rect1.height / 2, rect1.width / 2, rect1.height / 2, 1, color, alpha ]) : function(args) {
                            arrows[args.join()] = args;
                        }([ rect1.left - rect0.left + rect1.width / 2, rect1.top - rect0.top + rect1.height / 2, rect1.width / 2, rect1.height / 2, state2.left - rect0.left + state2.width / 2, state2.top - rect0.top + 10, state2.width / 2, 10, 1, color, alpha ]));
                    }
                }
                var args;
            }), splines) this.canvas.drawSpline.apply(this.canvas, splines[spline]);
            for (var arrow in arrows) this.canvas.drawArrow.apply(this.canvas, arrows[arrow]);
            for (var point in points) this.canvas.drawPoint.apply(this.canvas, points[point]);
        }
    }
    copyStep() {
        let obj = this.space.getElementsByClassName("qubot-state-active")[0];
        var state = obj.id.substring(12);
        this.lists.clipboard.copyStep = JSON.parse(JSON.stringify(qubotEditor.bot.states[state]));
    }
    pasteStep() {
        this.lists.clipboard.copyStep && (this.onAddCard(null, !0, "copy"), this.lists.clipboard.copyStep = JSON.parse(JSON.stringify(this.lists.clipboard.copyStep)));
    }
    onclickCard(obj) {
        qubotEditor.dontSave = !0;
        obj = obj.id.substring(12);
        qubotEditor.stateEditor.actions.changingState ? (qubotEditor.stateEditor.actions.changeState(obj, qubotEditor.stateEditor.actions.changingState), 
        this.removeAddedClasses(), this.addLinkClasses(qubotEditor.bot.state), this.showArrows(), 
        qubotEditor.stateEditor.actions.show()) : (qubotEditor.bot.state = obj, qubotEditor.stateEditor.items.set(qubotEditor.bot.states[obj]), 
        this.setCurState(obj), qubotEditor.stateEditor.set(qubotEditor.bot.states[obj])), 
        qubotEditor.dontSave = !1, qubotEditor.stateEditor.actions.wasOnClickState = !1;
    }
    onclickCardAdd(obj) {
        this.onAddCard(obj, !1);
    }
    onclickCardFolder(obj) {
        obj = this.cards.findCardByElement(obj);
        this.activeFolder = obj;
    }
    cardFolderOpen(folderCard) {
        folderCard.open = !0, folderCard.icon.className = "qubot-icon-folder-opened";
    }
    cardFolderClose(folderCard) {
        folderCard.open = !1, folderCard.icon.className = "qubot-icon-folder-closed";
    }
    onclickBtnAddCard(obj) {
        this.onAddCard(obj, !0);
    }
    onAddCard(obj, after = !0, kind = null) {
        {
            if (null !== qubotEditor.myBots.botNumber) return obj ? after && (obj = obj.parentNode) : obj = this.space.getElementsByClassName("qubot-state-active")[0], 
            qubotEditor.bot.posGrid || (qubotEditor.bot.posGrid = {}), qubotEditor.bot.posFree || (qubotEditor.bot.posFree = {}), 
            kind = this.createCardState(null, kind), (obj = this.cards.findCardByElement(obj)).folder ? after ? this.lists.placeCardInFolder(kind, obj.folder, null, obj) : this.lists.placeCardInFolder(kind, obj.folder, obj, null) : after ? this.lists.placeCardAfter(kind, obj) : this.lists.placeCardBefore(kind, obj), 
            this.save(), qubotEditor.setCurStateEditor(kind.state), kind;
            qubotEditor.help.alert("YouCannotEdit");
        }
    }
    async onclickBtnDelCard(obj) {
        null !== qubotEditor.myBots.botNumber ? 1 !== obj.parentNode.parentNode.getElementsByClassName("qubot-state").length ? await qubotEditor.help.confirm("DelState") && (this.init(), 
        obj = this.cards.findCardByElement(obj.parentNode), this.lists.removeCard(obj)) : qubotEditor.help.alert("OneState") : qubotEditor.help.alert("YouCannotEdit");
    }
    async onclickBtnDelCloseCardFolder(obj) {
        obj = this.cards.findCardByElement(obj.parentNode);
        obj.open ? this.cardFolderClose(obj) : null !== qubotEditor.myBots.botNumber ? await qubotEditor.help.confirm("DelFolder") && (this.init(), 
        this.lists.removeCard(obj)) : qubotEditor.help.alert("YouCannotEdit");
    }
    onclickStartStateBtn(obj) {
        var cards = this.space.getElementsByClassName("qubot-state");
        Array.from(cards).forEach(card => {
            card.getElementsByClassName("checkbox")[0].checked = !1;
        }), obj.checked = !0, qubotEditor.bot.start = obj.parentNode.id.substring(12);
    }
    keypress(event) {
        qubotEditor.stateEditor.startEditText || event.shiftKey && (43 == (event = event.keyCode) ? this.zoomIn(1.01) : 95 != event && 45 != event || this.zoomOut(.99));
    }
    viewRect() {
        var parentRect = this.space.parentElement.getBoundingClientRect();
        return {
            x: parentRect.x + this.space.offsetLeft,
            y: parentRect.y + this.space.offsetTop,
            width: parentRect.width - this.space.offsetLeft,
            height: parentRect.height - this.space.offsetTop
        };
    }
    viewCenter() {
        var viewRect = this.viewRect();
        return {
            x: viewRect.width / 2,
            y: viewRect.height / 2
        };
    }
    scale() {
        return this.canvas.scale;
    }
    zoomIn(scale, pos) {
        pos = pos || ("free" == this.kindView ? this.viewCenter() : {
            x: 0,
            y: 0
        }), this.canvas.scale *= scale, 100 < this.canvas.scale && (this.canvas.scale = 100, 
        scale = 1);
        var dX = (pos.x + this.space.parentElement.scrollLeft) * (scale - 1), pos = (pos.y + this.space.parentElement.scrollTop) * (scale - 1);
        this.space.style.transform = `translate( -50%, -50%) scale(${this.canvas.scale}) translate( 50%, 50%)`, 
        this.space.parentElement.scrollLeft += dX, this.space.parentElement.scrollTop += pos, 
        this.showArrows();
    }
    zoomOut(scale, pos) {
        pos = pos || ("free" == this.kindView ? this.viewCenter() : {
            x: 0,
            y: 0
        }), this.canvas.scale *= scale, this.canvas.scale < .01 && (this.canvas.scale = .01, 
        scale = 1);
        var dX = (pos.x + this.space.parentElement.scrollLeft) * (scale - 1), pos = (pos.y + this.space.parentElement.scrollTop) * (scale - 1);
        this.space.style.transform = `translate( -50%, -50%) scale(${this.canvas.scale}) translate( 50%, 50%)`, 
        this.space.parentElement.scrollLeft += dX, this.space.parentElement.scrollTop += pos, 
        this.showArrows();
    }
    mouseoverCard(e) {}
    mousedownCardFolderResize(e) {}
    mouseenterCardState(e) {}
    createEmptyState(state, kind) {
        kind ? "copy" == kind && this.lists.clipboard.copyStep ? qubotEditor.bot.states[state] = this.lists.clipboard.copyStep : "yesno" == kind ? qubotEditor.bot.states[state] = [ {
            type: "text",
            text: {
                en: "Text"
            }
        }, {
            type: "row",
            items: [ {
                type: "button",
                button: {
                    en: "✔ Yes",
                    de: "✔ Ja",
                    fr: "✔ Oui",
                    es: "✔ Sí",
                    jp: "✔ はい",
                    ru: "✔ Да",
                    ua: "✔ Так"
                },
                actions: []
            }, {
                type: "button",
                button: {
                    en: "❌ No",
                    de: "❌ Nein",
                    fr: "❌ Non",
                    es: "❌ No",
                    jp: "❌ いいえ",
                    ru: "❌ Нет",
                    ua: "✔ Ні"
                },
                actions: []
            } ]
        }, {
            type: "input",
            kind: "text",
            actions: []
        } ] : "lang" == kind && (qubotEditor.bot.states[state] = [ {
            type: "actions",
            actions: [ {
                state: state
            } ]
        }, {
            type: "text",
            text: {
                en: "What language do you prefer to communicate in?",
                ru: "На каком языке Вы хотите общаться?",
                ua: "Якою мовою Ви бажаєте спілкуватись?",
                de: "In welcher Sprache möchten Sie kommunizieren?",
                fr: "Dans quelle langue souhaitez-vous communiquer ?",
                es: "¿En qué idioma quieres comunicarte?"
            }
        }, {
            type: "row",
            items: [ {
                type: "button",
                button: {
                    en: "English",
                    ru: "English",
                    ua: "English",
                    de: "English",
                    fr: "English",
                    es: "English"
                },
                actions: [ {
                    slot: "LANGUAGE",
                    value: "en"
                } ]
            }, {
                type: "button",
                button: {
                    en: "Русский",
                    ru: "Русский",
                    ua: "Русский",
                    de: "Русский",
                    fr: "Русский",
                    es: "Русский"
                },
                actions: [ {
                    slot: "LANGUAGE",
                    value: "ru"
                } ]
            } ]
        }, {
            type: "row",
            items: [ {
                type: "button",
                button: {
                    en: "Española",
                    ru: "Española",
                    ua: "Española",
                    de: "Española",
                    fr: "Española",
                    es: "Española"
                },
                actions: [ {
                    slot: "LANGUAGE",
                    value: "es"
                } ]
            }, {
                type: "button",
                button: {
                    en: "Український",
                    ru: "Український",
                    ua: "Український",
                    de: "Український",
                    fr: "Український",
                    es: "Український"
                },
                actions: [ {
                    slot: "LANGUAGE",
                    value: "ua"
                } ]
            } ]
        }, {
            type: "row",
            items: [ {
                type: "button",
                button: {
                    en: "Deutsch",
                    ru: "Deutsch",
                    ua: "Deutsch",
                    de: "Deutsch",
                    fr: "Deutsch",
                    es: "Deutsch"
                },
                actions: [ {
                    slot: "LANGUAGE",
                    value: "de"
                } ]
            }, {
                type: "button",
                button: {
                    en: "Français",
                    ru: "Français",
                    ua: "Français",
                    de: "Français",
                    fr: "Français",
                    es: "Français"
                },
                actions: [ {
                    slot: "LANGUAGE",
                    value: "fr"
                } ]
            } ]
        }, {
            type: "input",
            kind: "text",
            actions: []
        } ]) : qubotEditor.bot.states[state] = [ {
            type: "text",
            text: {
                en: "Text"
            }
        }, {
            type: "row",
            items: [ {
                type: "button",
                button: {
                    en: "Button"
                },
                actions: []
            } ]
        }, {
            type: "input",
            kind: "text",
            actions: []
        } ];
    }
    createZoom() {
        let zoom = document.getElementById("qubot-zoom");
        for (;zoom.children.length; ) zoom.firstChild.remove();
        let zoomIn = this.add(zoom, "qubot-zoom-in"), zoomOut = (zoomIn.id = "qubot-zoom-in", 
        zoomIn.innerHTML = "Shift+", this.add(zoom, "qubot-zoom-out"));
        zoomOut.id = "qubot-zoom-out", zoomOut.innerHTML = "Shift-", zoom.style = "display:flex; flex-direction:row; margin-right:10px;", 
        zoomIn.setAttribute("onclick", "qubotEditor.listEditor.zoomIn(1.1)"), zoomOut.setAttribute("onclick", "qubotEditor.listEditor.zoomOut(0.9)");
    }
}

class QuBotEditorListFree extends QuBotEditorList {
    constructor(cards, lists) {
        super(cards, lists), this.kindView = "free";
    }
    init() {}
    create() {
        super.create();
    }
    show() {
        super.show(), this.space.style.transform = `translate( -50%, -50%) scale(${this.canvas.scale}) translate( 50%, 50%)`, 
        this.kindView = "free", this.canvas.svg.style.zIndex = "1", this.space.style.cursor = "grab", 
        this.space.onmousedown = this.mousedownSpace.bind(this), this.initFolders(), this.align(), 
        this.showFree(), this.openFolders(), this.canvas.svgColor = "gray", this.canvas.svgSpline = !0, 
        this.canvas.svg.classList.remove("qubot-svg-shadow"), this.space.parentElement.scrollLeft = 0, 
        this.space.parentElement.scrollTop = 0, navigator.maxTouchPoints || navigator.msMaxTouchPoints || (this.space.parentElement.onwheel = this.onwheel.bind(this)), 
        this.showArrows(), this.save();
    }
    initFolders() {
        let folderCards = this.cards.getFolderCards();
        folderCards.forEach(folderCard => {
            folderCard.open = !1;
        });
    }
    openFolders() {
        let folderCards = this.cards.getFolderCards();
        folderCards.forEach(folderCard => {
            folderCard.open = !!folderCard.openFree, folderCard.open && this.cardFolderOpen(folderCard, !1);
        });
    }
    align() {
        let minX = 0, minY = 0, cards = this.cards.getContentCards(), lastPos = {
            x: 0,
            y: 0
        };
        cards.forEach(card => {
            card.visible() && (card.posFree ? (card.posFree.x < minX && (minX = card.posFree.x), 
            card.posFree.y < minY && (minY = card.posFree.y), lastPos.x = card.posFree.x, lastPos.y = card.posFree.y) : (card.posFree = {
                x: lastPos.x + 100,
                y: lastPos.y + 100
            }, lastPos.x += 100, lastPos.y += 100));
        }), cards.forEach(card => {
            card.posFree && (card.posFree.x -= minX, card.posFree.y -= minY);
        });
    }
    showFree() {
        this.active && (this.hideCards(), this.showTree(null, {
            x: 0,
            y: 0
        }, {
            x: 0,
            y: 0
        }), this.showArrows());
    }
    showTree(card, lastPos, offset) {
        if (card && this.showCard(card, lastPos, offset), !card || "folder" == card.kind && card.open) {
            let children = this.cards.getContentCardsFromFolder(card);
            children.forEach(card => {
                this.showTree(card, lastPos, offset);
            });
        }
    }
    showCard(card, lastPos, offset) {
        let p = card.posFree;
        p ? (lastPos.x = p.x, lastPos.y = p.y) : (p = card.posFree = {
            x: lastPos.x + 100,
            y: lastPos.y + 100
        }, lastPos.x += 100, lastPos.y += 100), this.placeCard(card, p.x + offset.x, p.y + offset.y), 
        this.updateCard(card);
    }
    hideTree(parent) {
        if (!parent || "folder" == parent.kind && parent.open) {
            let children = this.cards.getContentCardsFromFolder(null);
            children.forEach(card => {
                this.hideTree(children);
            });
        }
        parent && this.hideCard(parent);
    }
    leftTopPos(cards) {
        let left = null, top = null;
        return cards.forEach(card => {
            card.visible() && card.posFree && ((null == left || card.posFree.x < left) && (left = card.posFree.x), 
            (null == top || card.posFree.y < top) && (top = card.posFree.y));
        }), {
            left: left,
            top: top
        };
    }
    hideCards() {
        var cards = this.cards.getAllCards();
        for (let i = 0; i < cards.length; i++) {
            var card = cards[i];
            this.hideCard(card);
        }
    }
    hideCard(card) {
        let cardElem = card.elem;
        cardElem.style.display = "none";
    }
    updateCard(card) {
        var cardCenter;
        "state" == card.kind && (cardCenter = card.centerLocal().y, card.btnAdd.style.top = cardCenter - 14 + "px", 
        card.btnAdd.style.bottom = null);
    }
    placeCardAfter(card, anotherCard) {
        var anotherCardElem;
        !this.active && card.posFree || (anotherCardElem = anotherCard.elem, this.active ? this.placeCard(card, anotherCardElem.offsetLeft + 100, anotherCardElem.offsetTop + 100, anotherCardElem) : anotherCard.posFree && this.placeCard(card, anotherCard.posFree.x + 100, anotherCard.posFree.y + 100, anotherCardElem), 
        this.save());
    }
    placeCardBefore(card, anotherCard) {
        var anotherCardElem;
        !this.active && card.posFree || (anotherCardElem = anotherCard.elem, this.active ? this.placeCard(card, anotherCardElem.offsetLeft - 100, anotherCardElem.offsetTop - 100, anotherCardElem) : anotherCard.posFree && this.placeCard(card, anotherCard.posFree.x - 100, anotherCard.posFree.y - 100, anotherCardElem), 
        this.save());
    }
    placeCardInFolder(card, folderCard, beforeCard = null, afterCard = null) {
        super.placeCardInFolder(card, folderCard, beforeCard, afterCard), beforeCard && (this.active ? this.placeCard(card, beforeCard.elem.offsetLeft - 100, beforeCard.elem.offsetTop - 100, beforeCard.elem) : beforeCard.posFree && this.placeCard(card, beforeCard.posFree.x - 100, beforeCard.posFree.y - 100, beforeCard.elem)), 
        afterCard && (this.active ? this.placeCard(card, afterCard.elem.offsetLeft + 100, afterCard.elem.offsetTop + 100, afterCard.elem) : afterCard.posFree && this.placeCard(card, afterCard.posFree.x + 100, afterCard.posFree.y + 100, afterCard.elem)), 
        this.updateFolderCard(folderCard);
    }
    placeCard(card, x, y, after = null) {
        if (super.placeCard(card), this.active) {
            let cardElem = card.elem;
            after ? this.space.insertBefore(card.elem, after.elem) : this.space.appendChild(cardElem), 
            this.moveCard(card, x, y), cardElem.style.position = "absolute", cardElem.style.opacity = this.cardOpacity, 
            cardElem.style.display = "flex";
        }
        card.posFree = {
            x: x,
            y: y
        };
    }
    moveCard(card, x, y) {
        this.active && (card.elem.style.left = x + "px", card.elem.style.top = y + "px"), 
        card.posFree = {
            x: x,
            y: y
        };
    }
    removeCard(card) {
        this.showFree();
    }
    hide() {
        super.hide();
        let folderCards = this.cards.getFolderCards();
        folderCards.forEach(folderCard => {
            folderCard.icon.style.display = "flex", folderCard.elem.style.transition = "", folderCard.elem.style.width = "", 
            folderCard.elem.style.height = "";
        });
        var cards = this.cards.getContentCards();
        for (let i = 0; i < cards.length; i++) {
            let cardElem = cards[i].elem;
            cardElem.style.left = null, cardElem.style.top = null, cardElem.style.position = null;
        }
        this.space.parentElement.onwheel = null;
    }
    showArrows() {
        this.canvas.clear(), this.showAllArrows();
    }
    clear() {
        if (this.space) for (;this.space.firstChild; ) this.space.removeChild(this.space.firstChild);
        super.clear();
    }
    showAllArrows() {
        let in_states = {};
        for (var state in qubotEditor.bot.states) {
            var states = qubotEditor.stateEditor.items.getStates(qubotEditor.bot.states[state]);
            for (let i = 0; i < states.length; i++) states[i].state === qubotEditor.bot.state && (in_states[state] = 1);
        }
        for (let state in in_states) {
            let states = qubotEditor.stateEditor.items.getStates(qubotEditor.bot.states[state]);
            for (let i = 0; i < states.length; i++) states[i].state !== qubotEditor.bot.state && (states[i] = null);
            this.drawArrows(state, states, "green", .3);
        }
        for (let state in qubotEditor.bot.states) state === qubotEditor.bot.state ? this.drawArrows(state, null, "blue", .5) : state in in_states || this.drawArrows(state, null, "gray", .1);
    }
    mousedownCardState(e) {
        if (!qubotEditor.stateEditor.actions.changingState) {
            e.stopPropagation(), this.cardGrab = !0, this.mousePosX = e.clientX, this.mousePosY = e.clientY;
            e = e.path || e.composedPath && e.composedPath();
            this.cardMove = this.cards.findCardByElement(e[1]);
            let card = this.cardMove.elem;
            this.moveCard(this.cardMove, card.offsetLeft, card.offsetTop), card.style.position = "absolute", 
            card.style.zIndex = 100, document.onmouseup = this.mouseupCardState.bind(this), 
            document.onmousemove = this.mousemoveCardState.bind(this), card.onmouseleave = this.mousemoveCardState.bind(this), 
            this.showArrows();
        }
    }
    mousemoveCardState(e) {
        e.stopPropagation();
        var dX = (this.mousePosX - e.clientX) / this.canvas.scale, dY = (this.mousePosY - e.clientY) / this.canvas.scale, card = this.cardMove.elem;
        0 <= card.offsetLeft - dX && (this.mousePosX = e.clientX, this.moveCard(this.cardMove, card.offsetLeft - dX, card.offsetTop)), 
        0 <= card.offsetTop - dY && (this.mousePosY = e.clientY, this.moveCard(this.cardMove, card.offsetLeft, card.offsetTop - dY)), 
        this.showArrows();
    }
    mouseupCardState(e) {
        e.stopPropagation(), this.cardMove.posFree = {
            x: this.cardMove.elem.offsetLeft,
            y: this.cardMove.elem.offsetTop
        }, document.onmouseup = null, document.onmousemove = null;
        let folderCards = this.cards.getFolderCards(), insertedIntoFolder = !1;
        folderCards.forEach(folderCard => {
            var folderRect = folderCard.rect(), cardRect = this.cardMove.rect();
            this.intersect(cardRect, folderRect) && (this.placeCardInFolder(this.cardMove, folderCard), 
            insertedIntoFolder = !0, folderCard.open || this.hideCard(this.cardMove));
        }), this.cardMove.folder && !insertedIntoFolder && (e = this.cardMove.folder, this.cardMove.setFolder(null), 
        this.updateFolderCard(e));
        let card = this.cardMove.elem;
        card.style.zIndex = null, card.onmouseleave = null, this.cardMove = null, this.cardGrab = !1, 
        this.save();
    }
    intersect(rect1, rect2) {
        return !(rect2.left > rect1.right || rect2.right < rect1.left || rect2.top > rect1.bottom || rect2.bottom < rect1.top);
    }
    mousedownCardFolder(e) {
        if (!qubotEditor.stateEditor.actions.changingState) {
            e.stopPropagation(), this.mousePosX = e.clientX, this.mousePosY = e.clientY;
            e = e.path || e.composedPath && e.composedPath();
            let card = this.cards.findCardByElement(e[1]);
            this.activeFolder = card, card.style.position = "absolute", card.style.zIndex = 100;
        }
    }
    mousedownSpace(e) {
        e.preventDefault(), e = e || window.event, this.cardGrab || (this.mousePosX = e.clientX, 
        this.mousePosY = e.clientY, document.onmouseup = this.mouseupSpace.bind(this), document.onmousemove = this.mousemoveSpace.bind(this));
    }
    mousemoveSpace(e) {
        e.preventDefault(), e = e || window.event;
        var dX = this.mousePosX - e.clientX, dY = this.mousePosY - e.clientY;
        this.space.parentElement.scrollLeft += dX, this.space.parentElement.scrollTop += dY, 
        this.mousePosX = e.clientX, this.mousePosY = e.clientY, this.showArrows();
    }
    mouseupSpace(e) {
        e.preventDefault(), document.onmouseup = null, document.onmousemove = null, this.save();
    }
    onwheel(e) {
        var rect;
        e.wheelDelta && Math.abs(e.wheelDelta) < 120 || (e.preventDefault(), rect = this.viewRect(), 
        rect = {
            x: e.clientX - rect.x,
            y: e.clientY - rect.y
        }, 1 < (e = 1 - e.deltaY / 1e3) ? this.zoomIn(e, rect) : this.zoomOut(e, rect));
    }
    onclickCardFolder(obj) {
        super.onclickCardFolder(obj);
        obj = this.cards.findCardByElement(obj);
        obj.open || (this.cardFolderOpen(obj), this.save());
    }
    cardFolderOpen(folderCard, shift = !0) {
        super.cardFolderOpen(folderCard), folderCard.icon.style.display = "none", this.updateFolderCard(folderCard, !1, shift), 
        this.showArrows();
    }
    cardFolderClose(folderCard) {
        super.cardFolderClose(folderCard);
        var scale = this.scale(), oldFolderCardRect = folderCard.rectLocal(scale), scale = (folderCard.icon.style.display = "flex", 
        folderCard.elem.style.transition = "", folderCard.elem.style.width = "", folderCard.elem.style.height = "", 
        folderCard.rectLocal(scale));
        let children = this.cards.getContentCardsFromFolder(folderCard);
        children.forEach(card => {
            this.hideTree(card);
        });
        var otherCards = this.cards.getContentCards().filter(card => card.visible() && card.folder != folderCard);
        this.resizeSubArea(oldFolderCardRect, scale, otherCards, !1), folderCard.width = scale.width, 
        folderCard.height = scale.height, this.showArrows();
    }
    updateFolderCard(folderCard, anim = !0, shiftOther = !0) {
        if (this.active && folderCard.open) {
            var scale = this.scale(), folderCaptionRect = folderCard.caption.getBoundingClientRect();
            let children = this.cards.getContentCardsFromFolder(folderCard);
            var childrenLeftTop = this.leftTopPos(children);
            const folderBorders_left = 10, folderBorders_top = folderCaptionRect.height / scale + 10, folderBorders_right = 10, folderBorders_bottom = 10;
            let childrenOffset = {
                x: folderCard.posFree.x - childrenLeftTop.left + folderBorders_left,
                y: folderCard.posFree.y - childrenLeftTop.top + folderBorders_top
            };
            children.forEach(card => {
                var lastPos = {
                    x: folderCard.posFree.x,
                    y: folderCard.posFree.y
                };
                this.showTree(card, lastPos, childrenOffset);
            });
            folderCaptionRect = folderCard.rectLocal(scale);
            let newFolderCardRect = null;
            0 < children.length ? (newFolderCardRect = this.cards.rectLocal(children, scale), 
            newFolderCardRect.width += folderBorders_left + folderBorders_right, newFolderCardRect.height += folderBorders_top + folderBorders_bottom) : newFolderCardRect = {
                left: folderCaptionRect.left,
                right: folderCaptionRect.right,
                width: 2 * folderCaptionRect.width,
                height: 2 * folderCaptionRect.height
            }, folderCard.elem.style.transition = anim ? "width 1s, height 1s" : "", folderCard.elem.style.width = newFolderCardRect.width + "px", 
            folderCard.elem.style.height = newFolderCardRect.height + "px", newFolderCardRect.width += 2, 
            newFolderCardRect.height += 2;
            childrenLeftTop = this.cards.getContentCards().filter(card => card.visible() && card.folder != folderCard);
            folderCard.width = newFolderCardRect.width, folderCard.height = newFolderCardRect.height, 
            shiftOther && this.resizeSubArea(folderCaptionRect, newFolderCardRect, childrenLeftTop, !0), 
            this.showArrows();
        }
    }
    resizeSubArea(fromRect, toRect, cards, expand) {
        let scale = this.scale(), areaShift = {
            x: toRect.width - fromRect.width,
            y: toRect.height - fromRect.height
        };
        cards.forEach(card => {
            var cardRect = card.rectLocal(scale);
            let offset = {
                x: 0,
                y: 0
            };
            cardRect.left + cardRect.width > fromRect.left && cardRect.top > fromRect.top + fromRect.height && (offset.y += areaShift.y), 
            cardRect.top + cardRect.height > fromRect.top && cardRect.left > fromRect.left + fromRect.width && (offset.x += areaShift.x), 
            expand && offset.y && offset.x && (offset.x = 0), !expand && offset.x && offset.y && (offset.y = 0), 
            (offset.x || offset.y) && this.showCard(card, {
                x: 0,
                y: 0
            }, offset);
        });
    }
    mousedownCardFolder(e) {
        qubotEditor.stateEditor.actions.changingState || (e.stopPropagation(), this.mousePosX = e.clientX, 
        this.mousePosY = e.clientY, e = e.path || e.composedPath && e.composedPath(), e = this.cards.findCardByElement(e[1]), 
        this.activeFolder = e, this.dragCardFolder(e));
    }
    mousemoveCardFolder(e) {
        if (e.stopPropagation(), this.cardMove) {
            let dX = (this.mousePosX - e.clientX) / this.canvas.scale, dY = (this.mousePosY - e.clientY) / this.canvas.scale;
            var card = this.cardMove.elem;
            let children = this.cards.getContentCardsFromFolder(this.cardMove);
            0 <= card.offsetLeft - dX && (this.mousePosX = e.clientX, this.moveCard(this.cardMove, card.offsetLeft - dX, card.offsetTop), 
            this.cardMove.open && children.forEach(card => {
                this.moveCard(card, card.elem.offsetLeft - dX, card.elem.offsetTop);
            })), 0 <= card.offsetTop - dY && (this.mousePosY = e.clientY, this.moveCard(this.cardMove, card.offsetLeft, card.offsetTop - dY), 
            this.cardMove.open && children.forEach(card => {
                this.moveCard(card, card.elem.offsetLeft, card.elem.offsetTop - dY);
            })), this.showArrows(qubotEditor.bot.state);
        }
    }
    mouseupCardFolder(e) {
        e.stopPropagation(), this.cardGrab && this.dropCardFolder();
    }
    dragCardFolder(card) {
        1 != this.cards.getContentCards().length && (this.cardMove = card, this.cardGrab = !0, 
        document.onmouseup = this.mouseupCardFolder.bind(this), document.onmousemove = this.mousemoveCardFolder.bind(this), 
        this.cardMove.elem.onmouseleave = this.mousemoveCardFolder.bind(this));
    }
    dropCardFolder() {
        this.cardMove.posFree = {
            x: this.cardMove.elem.offsetLeft,
            y: this.cardMove.elem.offsetTop
        }, document.onmouseup = null, document.onmousemove = null;
        let card = this.cardMove.elem;
        card.style.zIndex = null, card.onmouseleave = null, this.cardMove = null, this.cardGrab = !1, 
        this.save();
    }
}

class QuBotEditorListGrid extends QuBotEditorList {
    constructor(cards, lists) {
        super(cards, lists), this.kindView = "grid";
    }
    init() {}
    create() {
        super.create(), this.columns = 5, this.offset = 5, this.grid || (this.grid = document.createElement("div"), 
        this.grid.style.display = "grid", this.grid.style.gridTemplateColumns = "repeat(" + this.columns + ", 1fr)", 
        this.grid.style.columnGap = this.offset + "px", this.grid.style.rowGap = this.offset + "px", 
        this.grid.style.padding = this.offset + "px", this.grid.style.gridAutoRows = "minmax(60px, auto)", 
        this.grid.classList.add("qubot-disable-select")), this.rowMenu || (this.rowMenu = this.createRowMenu());
    }
    show() {
        super.show(), this.space.style.transform = `translate( -50%, -50%) scale(${this.canvas.scale}) translate( 50%, 50%)`, 
        this.canvas.svg.style.zIndex = "10", this.space.style.cursor = "default", this.space.onmousedown = null, 
        this.space.style.minWidth = "800px", this.space.style.minHeight = "none", this.space.appendChild(this.grid), 
        this.space.onmousedown = null, this.canvas.svg.style.zIndex = "10", this.canvas.svgColor = "blue", 
        this.canvas.svgSpline = !1, this.canvas.clear(), this.rowMenu.r = 0, this.space.parentElement.scrollLeft = 0, 
        this.space.parentElement.scrollTop = 0, this.initFolders(), this.showGrid(), this.save();
    }
    initFolders() {
        let folderCards = this.cards.getFolderCards();
        folderCards.forEach(folderCard => {
            folderCard.open = !!folderCard.openGrid;
        });
    }
    placeAddCard(cards, c, r, folderCard) {
        let addCard = 0 < cards.length ? cards.pop() : this.createCardAdd();
        addCard.setFolder(folderCard), this.placeCardToPos(addCard, c, r);
    }
    showGrid() {
        var addCards;
        this.active && (this.hideCards(), addCards = [ ...this.cards.getAddCards() ], this.showFolderRows(0, null, addCards), 
        this.showRowMenu(), this.showArrows());
    }
    showRowMenu() {
        var rowsPos = this.gridRowsPos(), rowsPos = (rowsPos[this.rowMenu.r] + rowsPos[this.rowMenu.r + 1]) / 2;
        this.rowMenu.elem.style.top = rowsPos - this.rowMenu.elem.getBoundingClientRect().height / 2 + "px", 
        this.rowMenu.elem.style.display = "flex";
    }
    showFolderRows(row, folderCard, addCards) {
        var unsorted = this.cards.getContentCardsFromFolder(folderCard);
        if (0 == unsorted.length) return row - 1;
        var cards = this.sortByGrid(unsorted);
        let prev = null, breakCards = {};
        for (let i = 0; i < cards.length; i++) {
            var card = cards[i];
            prev && prev.posGrid && card.posGrid && prev.posGrid.r != card.posGrid.r && (breakCards[prev] = !0), 
            prev = card;
        }
        var startCol = folderCard ? folderCard.posGrid.c : 0;
        let c = startCol, r = row;
        prev = null;
        var stateColumns = this.columns - 1;
        let nextFolder = null;
        for (let i = 0; i < cards.length; i++) {
            let card = cards[i], newLine = !1;
            prev && breakCards[prev] && (newLine = !0), !card.posGrid && c + 1 >= stateColumns && (newLine = !0), 
            newLine && nextFolder && (r = this.showFolderRows(r + 1, nextFolder, addCards), 
            nextFolder = null), c = newLine ? startCol : c, r = newLine ? r + 1 : r, this.placeCardToPos(card, c, r), 
            "folder" == (prev = card).kind && card.open && (nextFolder = card), c += 1;
        }
        return nextFolder ? r = this.showFolderRows(r + 1, nextFolder, addCards) : c, r;
    }
    showCard(card) {
        let cardElem = card.elem;
        cardElem.style.left = null, cardElem.style.top = null, cardElem.style.position = null, 
        cardElem.style.zIndex = null, cardElem.style.display = "flex", cardElem.style.gridColumn = card.posGrid.c + 1, 
        cardElem.style.gridRow = card.posGrid.r + 1, this.grid.appendChild(cardElem);
    }
    hideCard(card) {
        let cardElem = card.elem;
        cardElem.style.gridColumn = null, cardElem.style.gridRow = null, cardElem.style.display = "none", 
        "add" == card.kind && (card.folder = null), "folder" == card.kind && (card.icon.className = "qubot-icon-folder-closed");
    }
    createRowMenu() {
        let menu = {
            r: 0
        };
        var parent = document.getElementById("qubot-states-list");
        let menuElem = this.add(parent, "qubot-grid-row-menu"), drag = (menuElem.style.position = "absolute", 
        menuElem.style.top = "20px", menuElem.style.left = "0px", this.add(menuElem, "qubot-icon-drag-row")), add = this.add(menuElem, "qubot-btn-plus-row");
        return menuElem.classList.add("qubot-disable-select"), this.add(add, "qubot-icon-plus-row"), 
        drag.onmousedown = this.mousedownRowMenuDrag.bind(this), add.onmousedown = this.mousedownRowMenuAdd.bind(this), 
        menu.elem = menuElem, menu;
    }
    zoomIn(scale, pos) {
        super.zoomIn(scale, pos), this.showRowMenu();
    }
    zoomOut(scale, pos) {
        super.zoomOut(scale, pos), this.showRowMenu();
    }
    placeCardAfter(card, anotherCard) {
        if (this.active || !card.posGrid) {
            anotherCard = anotherCard.posGrid;
            let pos;
            anotherCard ? (pos = {
                c: anotherCard.c + 1,
                r: anotherCard.r
            }, !this.active && pos.c > this.columns && (pos.c = 0, pos.r += 1), this.insertCard(card, pos.c, pos.r)) : this.appendCard(card), 
            this.save();
        }
    }
    placeCardBefore(card, anotherCard) {
        !this.active && card.posGrid || ((anotherCard = anotherCard.posGrid) ? this.insertCard(card, anotherCard.c, anotherCard.r) : this.appendCard(card), 
        this.save());
    }
    placeCardInFolder(card, folderCard, beforeCard = null, afterCard = null) {
        super.placeCardInFolder(card, folderCard, beforeCard), beforeCard ? this.insertCard(card, beforeCard.posGrid.c, beforeCard.posGrid.r) : afterCard ? this.insertCard(card, afterCard.posGrid.c + 1, afterCard.posGrid.r) : (beforeCard = this.cards.getContentCardsFromFolder(folderCard), 
        this.appendCard(card, beforeCard));
    }
    placeCard(card) {
        this.placeCardToPos(card, card.posGrid.c, card.posGrid.r);
    }
    placeCardToPos(card, c, r) {
        super.placeCard(card), card.posGrid = {
            c: c,
            r: r
        }, this.active && (card.visible() ? this.showCard(card) : this.hideCard(card));
    }
    placeCardOnSpace(card, x, y) {
        let cardElem = card.elem;
        cardElem.style.gridColumn = null, cardElem.style.gridRow = null, cardElem.style.left = x + "px", 
        cardElem.style.top = y + "px", cardElem.style.position = "absolute", cardElem.style.zIndex = 1e3, 
        this.space.appendChild(cardElem);
    }
    removeCard(card) {
        this.extractCard(card), this.showGrid();
    }
    extractCard(card) {
        var cardElem = card.elem, posGrid = (this.active && (cardElem = {
            x: cardElem.offsetLeft,
            y: cardElem.offsetTop
        }, this.placeCardOnSpace(card, cardElem.x, cardElem.y)), card.posGrid);
        if (posGrid) {
            var cards = this.cards.getAllCards();
            for (let i = 0; i < cards.length; i++) {
                var cardOther = cards[i];
                cardOther == card || !cardOther.posGrid || cardOther.posGrid.r != posGrid.r || cardOther.posGrid.c < posGrid.c || this.placeCardToPos(cardOther, cardOther.posGrid.c - 1, cardOther.posGrid.r);
            }
        }
    }
    gridPos(card) {
        var folderPosGrid;
        if (card.folder) return folderPosGrid = this.gridPos(card.folder), {
            c: card.posGrid.c,
            r: folderPosGrid.r + card.posGrid.r
        };
    }
    sortByGrid(cards) {
        return cards.sort((a, b) => null == a.posGrid || null == b.posGrid ? -1 : a.posGrid.r == b.posGrid.r ? a.posGrid.c > b.posGrid.c ? 1 : -1 : a.posGrid.r > b.posGrid.r ? 1 : -1);
    }
    extractCardsRow(cards, r) {
        let pos = [];
        for (let i = 0; i < cards.length; i++) {
            var cardElem = cards[i].elem;
            pos[i] = {
                x: cardElem.offsetLeft,
                y: cardElem.offsetTop
            };
        }
        for (let i = 0; i < cards.length; i++) {
            let card = cards[i];
            this.placeCardOnSpace(card, pos[i].x, pos[i].y);
        }
        var allCards = this.cards.getAllCards();
        for (let i = 0; i < allCards.length; i++) {
            var cardOther = allCards[i];
            cardOther.posGrid.r <= r || this.placeCardToPos(cardOther, cardOther.posGrid.c, cardOther.posGrid.r - 1);
        }
    }
    insertCard(card, c, r) {
        var cards = this.cards.getContentCards();
        for (let i = 0; i < cards.length; i++) {
            var cardOther = cards[i];
            cardOther == card || cardOther.posGrid.r != r || cardOther.posGrid.c < c || this.placeCardToPos(cardOther, cardOther.posGrid.c + 1, cardOther.posGrid.r);
        }
        this.placeCardToPos(card, c, r), this.showGrid();
    }
    appendCard(card, cards = null) {
        cards = (cards = cards || this.cards.getContentCards()).filter(function(cardv) {
            return cardv != card;
        });
        cards = this.lastCard(cards);
        cards ? this.placeCardToPos(card, cards.posGrid.c + 1, cards.posGrid.r) : this.placeCardToPos(card, 0, 0), 
        this.showGrid();
    }
    firstCard(cards) {
        return 0 == cards.length ? null : this.sortByGrid(cards)[0];
    }
    lastCard(cards) {
        if (0 == cards.length) return null;
        cards = this.sortByGrid(cards);
        return cards[cards.length - 1];
    }
    insertCardsRow(cards, r) {
        var cardsAll = this.cards.getAllCards();
        for (let i = 0; i < cardsAll.length; i++) {
            let cardOther = cardsAll[i];
            cards.find(card => card == cardOther) || cardOther.posGrid.r < r || this.placeCardToPos(cardOther, cardOther.posGrid.c, cardOther.posGrid.r + 1);
        }
        for (let i = 0; i < cards.length; i++) {
            var card = cards[i];
            this.placeCardToPos(card, card.posGrid.c, r);
        }
    }
    hide() {
        super.hide(), this.hideCards(), this.grid.parent && this.space.removeChild(this.grid), 
        this.rowMenu.elem.style.display = "none";
    }
    hideCards() {
        var cards = this.cards.getAllCards();
        for (let i = 0; i < cards.length; i++) {
            var card = cards[i];
            this.hideCard(card);
        }
    }
    clear() {
        if (this.space) for (;this.space.firstChild; ) this.space.removeChild(this.space.firstChild);
        if (this.grid) for (;this.grid.firstChild; ) this.grid.removeChild(this.grid.firstChild);
        super.clear();
    }
    updateCard(card) {
        var cardCenter, rowsPos;
        "state" == card.kind && (card == this.cardMove ? (cardCenter = card.centerLocal().y, 
        card.btnAdd.style.top = cardCenter - 14 + "px") : (cardCenter = card.posGrid.r, 
        rowsPos = ((rowsPos = this.gridRowsPos())[cardCenter + 1] - rowsPos[cardCenter]) / 2, 
        card.btnAdd.style.top = rowsPos - 14 + "px"), card.btnAdd.style.bottom = null);
    }
    onResizeCardEvent(card) {
        this.showArrows();
    }
    showArrows() {
        this.canvas.clear(), this.drawFolderBorder(), this.drawArrows(qubotEditor.bot.state, null, "blue", .5);
        let in_states = {};
        for (var state in qubotEditor.bot.states) {
            var states = qubotEditor.stateEditor.items.getStates(qubotEditor.bot.states[state]);
            for (let i = 0; i < states.length; i++) states[i].state === qubotEditor.bot.state && (in_states[state] = 1);
        }
        for (let state in in_states) {
            let states = qubotEditor.stateEditor.items.getStates(qubotEditor.bot.states[state]);
            for (let i = 0; i < states.length; i++) states[i].state !== qubotEditor.bot.state && (states[i] = null);
            this.drawArrows(state, states, "green", .3);
        }
    }
    drawFolderBorderNew(folder) {
        let children = this.cards.getCardsFromFolder(folder);
        if (children.forEach(card => {
            "folder" == card.kind && card.open && this.drawFolderBorder(card);
        }), folder && folder.open) {
            children = children.filter(card => card != this.cardMove), this.firstCard(children), 
            this.lastCard(children);
            this.gridColumnsPos(), this.gridRowsPos();
        }
    }
    drawFolderBorder(folder) {
        let children = this.cards.getCardsFromFolder(folder);
        if (children.forEach(card => {
            "folder" == card.kind && card.open && this.drawFolderBorder(card);
        }), folder && folder.open) {
            let edges = {}, addEdge = function(c, r) {
                var hash = c + "_" + r;
                return edges[hash] ? edges[hash].n += 1 : edges[hash] = {
                    c: c,
                    r: r,
                    n: 1,
                    was: !1
                }, edges[hash];
            }, getEdge = function(c, r, l) {
                c = c + "_" + r;
                if (edges[c] && !edges[c].was) return edges[c].l = l, edges[c];
            };
            let firstEdge = null;
            for (var hash in children.push(folder), children.forEach(card => {
                var pos, edge;
                card != this.cardMove && (pos = card.posGrid, edge = addEdge(pos.c + .5, pos.r), 
                addEdge(pos.c + 1, pos.r + .5), addEdge(pos.c + .5, pos.r + 1), addEdge(pos.c, pos.r + .5), 
                card == folder && (firstEdge = edge));
            }), edges) {
                let edge = edges[hash];
                1 < edge.n && (edge.was = !0);
            }
            let edgesPath = [], edge = firstEdge;
            for (;edge; ) edge.was = !0, edgesPath.push(edge), edge = (c = edge.c, r = edge.r, 
            c % 1 ? getEdge(c + .5, r - .5, 1) || getEdge(c + 1, r, 0) || getEdge(c + .5, r + .5, 0) || getEdge(c - .5, r + .5, 0) || getEdge(c - 1, r, 1) || getEdge(c - .5, r - .5, 1) : getEdge(c + .5, r + .5, 0) || getEdge(c, r + 1, 0) || getEdge(c - .5, r + .5, 1) || getEdge(c - .5, r - .5, 1) || getEdge(c, r - 1, 1) || getEdge(c + .5, r - .5, 0));
            let pointsPath = [], colsPos = this.gridColumnsPos(), rowsPos = this.gridRowsPos();
            function getPoint(c, r) {
                return {
                    x: colsPos[c],
                    y: rowsPos[r]
                };
            }
            firstEdge = edgesPath[0];
            let point = getPoint(firstEdge.c - .5, firstEdge.r);
            pointsPath.push(point);
            for (let i = 0; i < edgesPath.length; i++) {
                let edge = edgesPath[i];
                point = edge.l ? edge.c % 1 ? getPoint(edge.c - .5, edge.r) : getPoint(edge.c, edge.r - .5) : edge.c % 1 ? getPoint(edge.c + .5, edge.r) : getPoint(edge.c, edge.r + .5), 
                pointsPath.push(point);
            }
            var svgRect = this.canvas.svg.getBoundingClientRect(), gridRect = this.grid.getBoundingClientRect();
            let offsetY = gridRect.top - svgRect.top, offsetX = gridRect.left - svgRect.left;
            pointsPath.forEach(point => {
                point.x += offsetX, point.y += offsetY;
            });
            this.canvas.drawLines(pointsPath, "#CC8000");
        }
        var c, r;
    }
    gridColumnsPos() {
        let cards = this.cards.getAllCards(), widths = [], pos = (cards.forEach(card => {
            var w;
            card != this.cardMove && (w = card.elem.getBoundingClientRect().width, (!widths[card.posGrid.c] || widths[card.posGrid.c] < w) && (widths[card.posGrid.c] = w));
        }), this.offset / 2), columns = [];
        columns.push(pos);
        for (let i = 0; i < widths.length; i++) pos += widths[i] + this.offset, columns.push(pos);
        return columns;
    }
    gridColumnsPosOld() {
        let cards = this.cards.getAllCards(), widths = [], pos = (cards.forEach(card => {
            var w;
            card != this.cardMove && (w = card.elem.getBoundingClientRect().width, (!widths[card.posGrid.c] || widths[card.posGrid.c] < w) && (widths[card.posGrid.c] = w));
        }), this.offset * this.scale() / 2), columns = [];
        columns.push(pos);
        for (let i = 0; i < widths.length; i++) pos += widths[i] + this.offset * this.scale(), 
        columns.push(pos);
        return columns;
    }
    gridRowsPos() {
        let cards = this.cards.getAllCards(), heights = [], pos = (cards.forEach(card => {
            var h;
            card != this.cardMove && (h = card.elem.getBoundingClientRect().height, (!heights[card.posGrid.r] || heights[card.posGrid.r] < h) && (heights[card.posGrid.r] = h));
        }), this.offset * this.scale() / 2), rows = [];
        rows.push(pos);
        for (let i = 0; i < heights.length; i++) pos += heights[i] + this.offset * this.scale(), 
        rows.push(pos);
        return rows;
    }
    mousedownCardState(e) {
        qubotEditor.stateEditor.actions.changingState || (e.stopPropagation(), this.mousePosX = e.clientX, 
        this.mousePosY = e.clientY, e = e.path || e.composedPath && e.composedPath(), e = this.cards.findCardByElement(e[1]), 
        this.dragCardState(e));
    }
    mousemoveCardState(e) {
        if (e.stopPropagation(), this.cardMove) {
            var dX = (this.mousePosX - e.clientX) / this.canvas.scale, dY = (this.mousePosY - e.clientY) / this.canvas.scale;
            let card = this.cardMove.elem;
            0 <= card.offsetLeft - dX && (this.mousePosX = e.clientX, card.style.left = card.offsetLeft - dX + "px"), 
            0 <= card.offsetTop - dY && (this.mousePosY = e.clientY, card.style.top = card.offsetTop - dY + "px"), 
            this.showArrows(qubotEditor.bot.state);
        }
    }
    mouseupCardState(e) {
        e.stopPropagation(), this.cardGrab && this.dropCardState();
    }
    dragCardState(card) {
        1 != this.cards.getContentCards().length && (this.cardMove = card, this.cardGrab = !0, 
        this.extractCard(card), this.updateCard(card), this.showArrows(), document.onmouseup = this.mouseupCardState.bind(this), 
        document.onmousemove = this.mousemoveCardState.bind(this), card.elem.onmouseleave = this.mousemoveCardState.bind(this));
    }
    dropCardState() {
        let cardMove = this.cardMove, nearestCard = this.nearestRC(cardMove);
        var moveRect, nearestRect;
        nearestCard ? (moveRect = this.cardMove.rectLocal(), nearestRect = nearestCard.rectLocal(), 
        moveRect.top >= nearestRect.top + nearestRect.height ? (this.cardMove.setFolder(null), 
        this.insertCard(this.cardMove, 0, nearestCard.posGrid.r + 1)) : moveRect.left >= nearestRect.left + nearestRect.width ? (this.cardMove.setFolder(nearestCard.folder), 
        this.lists.placeCardAfter(this.cardMove, nearestCard)) : "folder" == nearestCard.kind ? this.lists.placeCardInFolder(this.cardMove, nearestCard) : nearestCard.folder ? this.lists.placeCardInFolder(this.cardMove, nearestCard.folder, nearestCard) : (this.cardMove.setFolder(null), 
        this.lists.placeCardBefore(this.cardMove, nearestCard))) : this.insertCard(this.cardMove, 0, 0), 
        this.showGrid(), document.onmouseup = null, document.onmousemove = null, cardMove.elem.onmouseleave = null, 
        this.cardMove = null, this.updateCard(cardMove), this.save(), this.showArrows(), 
        this.cardGrab = !1;
    }
    mousedownCardAdd(e) {
        qubotEditor.stateEditor.actions.changingState || (e.stopPropagation(), this.mousePosY = e.clientY, 
        1 != this.cards.getAddCards().length && (e = e.path || e.composedPath && e.composedPath(), 
        e = this.cards.findCardByElement(e[1]), this.dragCardAdd(e)));
    }
    dragCardAdd(card) {
        var r = card.posGrid.r, cards = this.cards.getAllCardsWithRow(r);
        1 != cards.length && (this.cardAddGrab = !0, this.cardsMove = cards, this.extractCardsRow(cards, r), 
        document.onmouseup = this.mouseupCardAdd.bind(this), document.onmousemove = this.mousemoveCardAdd.bind(this), 
        card.elem.onmouseleave = this.mousemoveCardAdd.bind(this));
    }
    mousemoveCardAdd(e) {
        if (e.stopPropagation(), this.cardsMove && 0 != this.cardsMove.length) {
            var dY = (this.mousePosY - e.clientY) / this.canvas.scale;
            if (0 < this.cardsMove[this.cardsMove.length - 1].elem.offsetTop - dY) {
                this.mousePosY = e.clientY;
                for (let j = 0; j < this.cardsMove.length; j++) {
                    let c = this.cardsMove[j].elem;
                    c.style.top = c.offsetTop - dY + "px";
                }
            }
            this.showArrows(qubotEditor.bot.state);
        }
    }
    dropCardAdd() {
        var cards = this.cards.getAllCards().filter(function(card) {
            return !(card in this.cardsMove);
        }.bind(this)), cards = this.cards.nearestInList(this.cardsMove[0], cards);
        this.insertCardsRow(this.cardsMove, cards.posGrid.r), document.onmouseup = null, 
        document.onmousemove = null, card.elem.onmouseleave = null, this.cardsMove = null, 
        this.save(), this.showArrows(), this.cardAddGrab = !1;
    }
    mouseupCardAdd(e) {
        e.stopPropagation(), this.cardAddGrab && this.dropCardAdd();
    }
    mousedownCardFolder(e) {
        qubotEditor.stateEditor.actions.changingState || (e.stopPropagation(), this.mousePosX = e.clientX, 
        this.mousePosY = e.clientY, e = e.path || e.composedPath && e.composedPath(), e = this.cards.findCardByElement(e[1]), 
        this.activeFolder = e, this.cardFolderClose(e), this.showGrid(), this.dragCardFolder(e));
    }
    mousemoveCardFolder(e) {
        if (e.stopPropagation(), this.cardMove) {
            var dX = (this.mousePosX - e.clientX) / this.canvas.scale, dY = (this.mousePosY - e.clientY) / this.canvas.scale;
            let card = this.cardMove.elem;
            0 <= card.offsetLeft - dX && (this.mousePosX = e.clientX, card.style.left = card.offsetLeft - dX + "px"), 
            0 <= card.offsetTop - dY && (this.mousePosY = e.clientY, card.style.top = card.offsetTop - dY + "px"), 
            this.showArrows(qubotEditor.bot.state);
        }
    }
    mouseupCardFolder(e) {
        e.stopPropagation(), this.cardGrab && this.dropCardFolder();
    }
    dragCardFolder(card) {
        1 != this.cards.getContentCards().length && (this.cardMove = card, this.cardGrab = !0, 
        this.extractCard(card), document.onmouseup = this.mouseupCardFolder.bind(this), 
        document.onmousemove = this.mousemoveCardFolder.bind(this), this.cardMove.elem.onmouseleave = this.mousemoveCardFolder.bind(this));
    }
    dropCardFolder() {
        let nearestCard = this.nearestRC(this.cardMove);
        var moveRect, nearestRect;
        nearestCard ? (moveRect = this.cardMove.rectLocal(), nearestRect = nearestCard.rectLocal(), 
        moveRect.top >= nearestRect.top + nearestRect.height ? (this.cardMove.setFolder(null), 
        this.insertCard(this.cardMove, 0, nearestCard.posGrid.r + 1)) : moveRect.left >= nearestRect.left + nearestRect.width ? this.lists.placeCardAfter(this.cardMove, nearestCard) : this.lists.placeCardBefore(this.cardMove, nearestCard)) : (this.cardMove.setFolder(null), 
        this.placeCardToPos(this.cardMove, 0, 0)), this.showGrid(), document.onmouseup = null, 
        document.onmousemove = null, this.cardMove.elem.onmouseleave = null, this.cardMove = null, 
        this.save(), this.showArrows(), this.cardGrab = !1;
    }
    onclickCardFolder(obj) {
        super.onclickCardFolder(obj);
        obj = this.cards.findCardByElement(obj);
        obj.open ? this.cardFolderClose(obj) : this.cardFolderOpen(obj), this.showGrid(), 
        this.save();
    }
    cardFolderOpen(folderCard) {
        let folderCards = this.cards.getCardsFromFolder(folderCard);
        if (0 != folderCards.length) {
            let r = folderCard.posGrid.r, folderRowCards = (folderCards = this.cards.getFolderCards(), 
            folderCards.filter(card => card.posGrid && card.posGrid.r == r && card != folderCard));
            folderRowCards.forEach(card => this.cardFolderClose(card)), super.cardFolderOpen(folderCard);
        }
    }
    nearestRC(card) {
        var cardPos = card.posLocal(card);
        let nearestRow = this.nearestRowPos(cardPos.x, cardPos.y), rowCards;
        for (;rowCards = this.cards.getAllCardsWithRow(nearestRow).filter(cardOther => cardOther != card && cardOther.visible()), 
        --nearestRow, 0 == rowCards.length && 0 <= nearestRow; );
        return this.cards.nearestInList(card, rowCards);
    }
    nearestRowCenter(x, y) {
        var rowsPos = this.gridRowsPos();
        let r = 0, dist = -1, rowCenter = 0;
        for (let i = 0; i < rowsPos.length; i++) {
            if (i < rowsPos.length - 1) rowCenter = (rowsPos[i] + rowsPos[i + 1]) / 2; else {
                if (!(0 < i)) break;
                rowCenter = rowsPos[i] + (rowsPos[i] - rowsPos[i - 1]) / 2;
            }
            var curDist = Math.abs(rowCenter - y);
            (dist < 0 || curDist < dist) && (dist = curDist, r = i);
        }
        return r;
    }
    nearestRowPos(x, y) {
        var rowsPos = this.gridRowsPos();
        let r = 0, dist = -1;
        for (let i = 0; i < rowsPos.length; i++) {
            var curDist = Math.abs(rowsPos[i] - y);
            (dist < 0 || curDist < dist) && (dist = curDist, r = i);
        }
        return r;
    }
    mousedownRowMenuDrag(e) {
        var r = this.nearestRowCenter(0, this.rowMenu.elem.offsetTop);
        let cards = this.cards.getAllCardsWithRow(r);
        cards = cards.filter(card => card.visible()), 0 != cards.length && (e.stopPropagation(), 
        cards.forEach(card => {
            "folder" == card.kind && this.cardFolderClose(card);
        }), this.showGrid(), this.mousePosY = e.clientY, this.rowMenuDragGrab = !0, this.cardsMove = cards, 
        this.extractCardsRow(cards, r), document.onmouseup = this.mouseupRowMenuDrag.bind(this), 
        document.onmousemove = this.mousemoveRowMenuDrag.bind(this), this.rowMenu.elem.onmouseleave = this.mousemoveRowMenuDrag.bind(this));
    }
    mousemoveRowMenuDrag(e) {
        e.stopPropagation();
        var dY = this.mousePosY - e.clientY;
        if (0 < this.cardsMove[this.cardsMove.length - 1].elem.offsetTop - dY) {
            this.mousePosY = e.clientY;
            for (let j = 0; j < this.cardsMove.length; j++) {
                let c = this.cardsMove[j].elem;
                c.style.top = c.offsetTop - dY / this.canvas.scale + "px";
            }
            this.rowMenu.elem.style.top = this.rowMenu.elem.offsetTop - dY + "px";
        }
        this.showArrows(qubotEditor.bot.state);
    }
    mouseupRowMenuDrag(e) {
        e.stopPropagation(), this.cardsMove && this.dropRowMenuDrag(), document.onmouseup = null, 
        document.onmousemove = null, document.onmouseover = null, this.rowMenu.elem.onmouseleave = null, 
        this.cardsMove = null, this.showRowMenu(), this.showGrid(), this.cardsMove = !1, 
        this.save();
    }
    dropRowMenuDrag() {
        var r = this.nearestRowCenter(0, this.rowMenu.elem.offsetTop);
        this.rowMenu.r = r, this.insertCardsRow(this.cardsMove, r);
    }
    mousedownRowMenuAdd(e) {
        var card, nearest;
        null !== qubotEditor.myBots.botNumber ? (card = this.createCardState(null, null), 
        (nearest = this.cards.findCardByColRow(0, this.rowMenu.r)) && nearest.folder ? this.placeCardInFolder(card, nearest.folder, null, nearest) : this.placeCardToPos(card, 0, this.rowMenu.r + 1), 
        this.insertCardsRow([ card ], this.rowMenu.r + 1), this.save(), this.showGrid(), 
        qubotEditor.setCurStateEditor(card.state)) : qubotEditor.help.alert("YouCannotEdit");
    }
    mouseoverCard(obj) {
        this.cardsMove || (obj = this.cards.findCardByElement(obj)) && (this.rowMenu.r = obj.posGrid.r, 
        this.showRowMenu());
    }
    mouseenterCardState(obj) {
        obj = this.cards.findCardByElement(obj);
        obj && this.updateCard(obj);
    }
}

class QuBotEditorLists extends QuBotEditorBase {
    constructor() {
        super(), this.cards = new QuBotCards(), this.list = null, this.listGrid = new QuBotEditorListGrid(this.cards, this), 
        this.listFree = new QuBotEditorListFree(this.cards, this), this.lists = [ this.listGrid, this.listFree ], 
        this.kindView = "grid", this.clipboard = {
            copyStep: null,
            copyFolder: null
        };
    }
    select(kindView) {
        var newList = kindView && "grid" != kindView ? this.listFree : this.listGrid;
        return this.list && this.list != newList && this.list.hide(), this.list = newList, 
        this.kindView = kindView, this.list;
    }
    create() {
        document.getElementById("qubot-states-list");
        document.addEventListener("keypress", this.keypress.bind(this)), this.listGrid.create(), 
        this.listFree.create();
        var folder, bot = qubotEditor.bot, folderCards = qubotEditor.bot.cards.folders;
        for (folder in folderCards) {
            var folderSave = folderCards[folder];
            this.list.createCardFolder(folder, folderSave);
        }
        var state, stateSave, stateCards = qubotEditor.bot.cards.states;
        for (state in bot.states) state in stateCards ? (stateSave = stateCards[state], 
        this.list.createCardState(state, null, stateSave)) : this.list.createCardState(state, null);
    }
    keypress(event) {
        this.list.keypress(event);
    }
    placeCardAfter(card, anotherCard) {
        for (const list of this.lists) list.placeCardAfter(card, anotherCard);
    }
    placeCardBefore(card, anotherCard) {
        for (const list of this.lists) list.placeCardBefore(card, anotherCard);
    }
    placeCardInFolder(card, folderCard, beforeCard = null, afterCard = null) {
        for (const list of this.lists) list.placeCardInFolder(card, folderCard, beforeCard, afterCard);
    }
    removeCard(card) {
        if ("folder" == card.kind) {
            let childs = this.cards.getContentCardsFromFolder(card);
            childs.forEach(child => {
                this.removeCard(child);
            });
        }
        this.cards.popCard(card);
        for (const list of this.lists) list.removeCard(card);
        "state" == card.kind && this.removeState(card.state), card.elem.remove(), this.list.save(), 
        this.list.show();
    }
    removeState(state) {
        if (delete qubotEditor.bot.states[state], qubotEditor.bot.start === state || !qubotEditor.bot.start.length) for (var s in qubotEditor.bot.states) {
            qubotEditor.bot.start = s, qubotEditor.bot.state = s;
            break;
        }
        if (qubotEditor.bot.state === state) for (let s in qubotEditor.bot.states) {
            qubotEditor.bot.state = s;
            break;
        }
        qubotEditor.renameState(state, ""), qubotEditor.setCurStateEditor(qubotEditor.bot.state), 
        this.list.save();
    }
}

class QuBotEditorMyBots extends QuBotEditorBase {
    constructor() {
        super(), this.bots = [], this.bot = null, this.botNumber = null;
    }
    createMyBotsMenu() {
        let menu = qubotEditor.menuMyBots;
        for (;menu.childElementCount; ) menu.lastChild.remove();
        let a;
        for (let i = 0; i < this.bots.length; i++) a = this.add(qubotEditor.menuMyBots, "item"), 
        a.innerHTML = this.bots[i].name, i === this.botNumber && a.classList.add("checked"), 
        a.setAttribute("onclick", `qubotEditor.myBots.setBot(${i})`);
        var lst;
        if (qubotEditor.menuTemplates && (lst = qubotEditor.menuTemplates.getElementsByClassName("item"), 
        Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        })), qubotEditor.menuExamples) {
            let lst = qubotEditor.menuExamples.getElementsByClassName("item");
            Array.from(lst).forEach(a => {
                a.classList.remove("checked");
            });
        }
    }
    emptyBot(name, id) {
        return {
            id: id,
            name: name,
            start: "MAIN",
            states: {
                MAIN: [ {
                    type: "text",
                    text: {
                        en: "Text"
                    }
                }, {
                    type: "row",
                    items: [ {
                        type: "button",
                        button: {
                            en: "Button"
                        },
                        actions: []
                    } ]
                }, {
                    type: "input",
                    kind: "text",
                    actions: []
                } ]
            },
            slots: {
                LANGUAGE: "en",
                INPUT: ""
            },
            cards: {
                states: {
                    MAIN: {
                        r: 0,
                        c: 0,
                        x: 0,
                        y: 0
                    }
                },
                folders: {}
            }
        };
    }
    newBot(name, load = !0) {
        var bid;
        (name || null != (name = prompt("Please enter your bot name:", "Bot " + (this.bots.length + 1))) && "" != name) && (bid = this.bidGenerator(), 
        this.bots.push({
            id: bid,
            name: name
        }), this.save(), this.bot = this.emptyBot(name, bid), this.save(), this.botNumber = this.bots.length - 1, 
        load && qubotEditor.loadBot(this.bot), this.createMyBotsMenu());
    }
    async deleteBot() {
        var botID;
        null !== this.bot ? await qubotEditor.help.confirm("DeleteBot") && (botID = this.bot.id, 
        this.bots.splice(this.botNumber, 1), this.bot = null, qubotData.deleteBot(botID), 
        0 === this.bots.length ? (this.botNumber = 0, this.newBot("Bot 1")) : this.setBot(this.botNumber - 1), 
        this.save(), this.createMyBotsMenu(), qubotEditor.resetEditorAndAllCards()) : qubotEditor.help.alert("NoActiveBot");
    }
    renameBot() {
        var name;
        null !== this.bot ? (name = prompt("Please enter your new name for  bot", this.bots[this.botNumber].name)) && (this.bots[this.botNumber].name = name, 
        qubotEditor.botName.innerHTML = name, this.bot.name = name, qubotManager.renameBotEvent(this.bot.id, name), 
        this.save(), this.createMyBotsMenu()) : qubotEditor.help.alert("NoBotsForRename");
    }
    saveCurrentBot() {
        null !== this.bot && (this.bot.slots = qubotEditor.bot.slots, this.bot.states = qubotEditor.bot.states, 
        this.bot.start = qubotEditor.bot.start, this.bot.cards = qubotEditor.bot.cards, 
        this.bot.view = qubotEditor.bot.view, this.save());
    }
    setBot(i) {
        null !== this.bot && i === this.botNumber || (this.saveCurrentBot(), this.botNumber = i < 0 ? 0 : i, 
        this.bot = null, this.load(this.bots[this.botNumber].id, this.setBot1.bind(this)));
    }
    setBot1() {
        qubotEditor.loadBot(this.bot), this.createMyBotsMenu(), this.saveCurrentBot();
        var lst = qubotEditor.menuTemplates.getElementsByClassName("item");
        Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        }), lst = qubotEditor.menuExamples.getElementsByClassName("item"), Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        });
    }
    cloneBot() {
        var bid, name = prompt("You clone the current bot by creating a new one. Enter a name for it:", "Bot " + (this.bots.length + 1));
        name && (this.saveCurrentBot(), bid = this.bidGenerator(), this.bot = {}, this.bot.id = bid, 
        this.bot.name = name, this.bot.view = qubotEditor.bot.view, this.bot.states = JSON.parse(JSON.stringify(qubotEditor.bot.states)), 
        this.bot.slots = JSON.parse(JSON.stringify(qubotEditor.bot.slots)), this.bot.start = JSON.parse(JSON.stringify(qubotEditor.bot.start)), 
        this.bot.cards = JSON.parse(JSON.stringify(qubotEditor.bot.cards)), this.bots.push({
            id: bid,
            name: name
        }), this.botNumber = this.bots.length - 1, bid = qubotEditor.menuTemplates.getElementsByClassName("item"), 
        Array.from(bid).forEach(a => {
            a.classList.remove("checked");
        }), this.save(), qubotEditor.loadBot(this.bot), this.createMyBotsMenu());
    }
    async clear() {
        await qubotEditor.help.confirm("DeleteAllBots") && await qubotEditor.help.confirm("DeleteAllBots2") && (this.bots = [], 
        this.bot = null, this.save(), this.createMyBotsMenu(), this.botNumber = null);
    }
    save() {
        if (!(this.botNumber < 0) && (qubotData.save("qubotEditorMyBots", JSON.stringify({
            bots: this.bots,
            botNumber: this.botNumber
        })), null !== this.bot)) {
            ("" + this.bot.id).length < 5 && (this.bot.id = this.bidGenerator(), this.bots[this.botNumber].id = this.bot.id);
            var state, slot, slots = {};
            for (state in this.bot.states) state && state.length && this.bot.states[state] && qubotEditor.stateEditor.items.getSlots(this.bot.states[state], slots);
            for (slot in slots) !slot.length || slot in this.bot.slots || (this.bot.slots[slot] = "");
            for (let slot in this.bot.slots) this.bot.slots[slot] = null, "LANGUAGE" === slot && (this.bot.slots[slot] = "en");
            this.bot.version = window.editorConfig.version, qubotEditor.bot.version = window.editorConfig.version, 
            this.bot.view = qubotEditor.bot.view, qubotData.saveBot(this.bot.id, this.bot.name, JSON.stringify(this.bot));
        }
    }
    loadBots(callback) {
        this.callback = callback, qubotData.load("qubotEditorMyBots", function(data) {
            if (data) {
                if (!("bots" in data)) return this.bots = [], this.bot = null, void this.callback();
                this.bots = data.bots, "botNumber" in data && (null !== (data = data.botNumber) && 0 <= data && data < this.bots.length && (this.botNumber = data)), 
                this.callback();
            } else this.callback();
        }.bind(this));
    }
    load(id, callback) {
        this.loadID = id, this.callback = callback, qubotData.loadBot(this.loadID, function(data) {
            data ? this.bot = data : (this.bot = this.emptyBot("My first bot", this.loadID), 
            this.save()), this.callback();
        }.bind(this));
    }
    bidGenerator(a = null) {
        return a ? (a ^ 16 * Math.random() >> a / 4).toString(16) : ("10000000100040008000" + 1e11).replace(/[018]/g, this.bidGenerator);
    }
}

class QuBotEditorExamples {
    constructor() {
        this.bots = {
            Media: {
                id: 0,
                name: "Media",
                start: "MAIN",
                version: "1.0.0",
                states: {
                    MAIN: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "video",
                        url: "https://qudata.com/qubot/qudata.mp4",
                        stop: 1,
                        control: 0,
                        play: 1,
                        loop: 0
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Next "
                            },
                            actions: [ {
                                state: "STEP_2"
                            } ]
                        } ]
                    } ],
                    STEP_2: [ {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/owl_a.png",
                            width: "75"
                        } ]
                    }, {
                        type: "audio",
                        url: "https://qudata.com/qubot/QuOwl.mp3",
                        stop: 1,
                        play: 1,
                        control: 0,
                        loop: 0
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Back"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 80,
                            y: 14
                        },
                        STEP_2: {
                            c: 1,
                            r: 0,
                            x: 368,
                            y: 148
                        }
                    },
                    folders: {}
                }
            },
            Messages: {
                id: 0,
                name: "Messages",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "note",
                        note: "Below are the properties of the first message. \nThey only need to be set if you want to add the typing effect and/or change the message background color."
                    }, {
                        type: "message",
                        typing: "2000",
                        back: "#ffff00"
                    }, {
                        type: "text",
                        text: {
                            en: "Text 1"
                        },
                        color: "#ff1900",
                        center: 1
                    }, {
                        type: "note",
                        note: 'The following text, "Text 2", will be displayed in a new message with a typing effect before it.'
                    }, {
                        type: "message",
                        typing: "2000"
                    }, {
                        type: "text",
                        text: {
                            en: "Text 2"
                        }
                    }, {
                        type: "note",
                        note: 'The "Next" button opens STEP_2. \nIn this step, there is a "goto" action that instantly opens STEP_3.'
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Next"
                            },
                            actions: [ {
                                state: "STEP_2"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "STEP_2"
                        } ]
                    } ],
                    STEP_2: [ {
                        type: "text",
                        text: {
                            en: "Text 3"
                        }
                    }, {
                        type: "note",
                        note: 'Click the rocket to see the "goto" action, taking you instantly to STEP_3. The typing effect precedes the step.'
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "STEP_3"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_3: [ {
                        type: "message",
                        typing: "2000"
                    }, {
                        type: "text",
                        text: {
                            en: 'Text 4\n<b class="green">Clicking on the Clear button will clear the chat history.</b>'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Clear"
                            },
                            actions: [ {
                                state: "STEP_4"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_4: [ {
                        type: "note",
                        note: 'Click the rocket and set the value -1 in the "clear" field to clear the chat history. All previous messages in the current session will be deleted, and then the text of the current step will be displayed.\n'
                    }, {
                        type: "actions",
                        actions: [ {
                            clear: "-1"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: '<b class="green">Chat cleared</b>'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Next"
                            },
                            actions: [ {
                                state: "STEP_5"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: ""
                        } ]
                    } ],
                    STEP_5: [ {
                        type: "text",
                        text: {
                            en: "Text A"
                        }
                    }, {
                        type: "message"
                    }, {
                        type: "text",
                        text: {
                            en: "Text B"
                        }
                    }, {
                        type: "message"
                    }, {
                        type: "text",
                        text: {
                            en: 'Text C\n<b class="green">After clicking "Next", two previous messages will be deleted.</b>'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Next"
                            },
                            actions: [ {
                                state: "STEP_6"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_6: [ {
                        type: "note",
                        note: 'Click the rocket to set the number of previous messages to be cleared in the "clear" field.'
                    }, {
                        type: "actions",
                        actions: [ {
                            clear: "2"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: '<b class="green">2 previous messages were deleted.</b>'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Home"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 54,
                            y: 0
                        },
                        STEP_2: {
                            c: 0,
                            r: 1,
                            x: 57,
                            y: 112
                        },
                        STEP_3: {
                            c: 1,
                            r: 1,
                            x: 321,
                            y: 112
                        },
                        STEP_4: {
                            c: 0,
                            r: 2,
                            x: 51,
                            y: 213
                        },
                        STEP_5: {
                            c: 0,
                            r: 3,
                            x: 118,
                            y: 321
                        },
                        STEP_6: {
                            c: 1,
                            r: 3,
                            x: 419,
                            y: 335
                        }
                    },
                    folders: {}
                }
            },
            Scripts: {
                id: 0,
                name: "Scripts",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "text",
                        text: {
                            en: "Scripts are needed to perform complex slot operations or to display dynamic visual content."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Show the script text and images"
                            },
                            actions: [ {
                                state: "TEXTS_IMAGES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: ""
                        } ]
                    } ],
                    TEXTS_IMAGES: [ {
                        type: "note",
                        note: "The scripts are displayed in the actions, as you click the rocket.\nSee the comments to the script code for explanation."
                    }, {
                        type: "actions",
                        actions: [ {
                            script: '# Set the current language to Spanish\nset(LANGUAGE, "es")  \n\n# This text gets a string, not an object.\n# It will be displayed in English.\n# Tag <br> - newline\ntext("Hello<br> World")\n\n# The text in the active language \n# will be used.\ntext(\n{\n    en: "Hello World",\n    es: "Hola"\n})\n\n# Set back to English\nset(LANGUAGE, "en")\n\n# A line of images starts:\nimages()\n# Images are displayed in it:\nimage("https://qudata.com/qubot/im/owl_a.png")\nimage("https://qudata.com/qubot/im/owl_b.png")'
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: 'Show the script "WHILE_IF"'
                            },
                            actions: [ {
                                state: "WHILE_IF"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: ""
                        } ]
                    } ],
                    WHILE_IF: [ {
                        type: "actions",
                        actions: [ {
                            script: '# The while loop repeats as long as \n# the condition is true\nset(INDEX, 1)\nwhile($INDEX <= 5,\n    text("INDEX = $INDEX")\n    set(INDEX, $INDEX + 1)\n)\n\n# In the "if" action, the first argument\n# is a boolean condition.\n# If it is true, then the script from \n# the second argument is executed.\n# Otherwise, the script from the third \n# argument is executed.\nif($INDEX == 1,\n    text("INDEX == 1")\n,\n    text("INDEX != 1")\n    text("There can be many actions")\n)'
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Next"
                            },
                            actions: [ {
                                state: "MORE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    MORE: [ {
                        type: "text",
                        text: {
                            en: "More script examples can be found in the Examples (Lists and Checkboxes) ."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Main"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    INDEX: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 25,
                            y: 9
                        },
                        TEXTS_IMAGES: {
                            c: 1,
                            r: 0,
                            x: 350,
                            y: 14
                        },
                        WHILE_IF: {
                            c: 2,
                            r: 0,
                            x: 349,
                            y: 202
                        },
                        MORE: {
                            c: 3,
                            r: 0,
                            x: 96,
                            y: 200
                        }
                    },
                    folders: {}
                }
            },
            List: {
                id: 0,
                name: "Lists",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "note",
                        note: "This example demonstrates adding objects to a list. Click on the actions (🚀) for further comments. Comments are also found in the buttons and cards of the next steps."
                    }, {
                        type: "actions",
                        actions: [ {
                            note: 'There is no step action in product buttons. Therefore, there will be a transition to this step. Another step is specified under "Order" the button.'
                        }, {
                            state: "AMOUNT"
                        }, {
                            note: "The name of the current step is stored in the SAVE_STEP slot. Thus, the bot will be able to return here from the ORDER step. So, the ORDER step can be accessed in two ways: from this step and from ADD_ITEM_TO_ORDER."
                        }, {
                            slot: "SAVE_STEP",
                            value: '"MAIN"'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "You can choose the following products:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🥛 Milk"
                            },
                            actions: [ {
                                note: "The name of the product is stored in the ITEM slot, and its price in the PRICE slot."
                            }, {
                                slot: "ITEM",
                                value: '"Milk"'
                            }, {
                                slot: "PRICE",
                                value: "10"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🍷 Wine"
                            },
                            actions: [ {
                                slot: "ITEM",
                                value: '"Wine"'
                            }, {
                                slot: "PRICE",
                                value: "100"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🧀 Cheese"
                            },
                            actions: [ {
                                slot: "ITEM",
                                value: '"Cheese"'
                            }, {
                                slot: "PRICE",
                                value: "50"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🍬 Sweets"
                            },
                            actions: [ {
                                slot: "ITEM",
                                value: '"Candy"'
                            }, {
                                slot: "PRICE",
                                value: "2"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🛒 Order"
                            },
                            actions: [ {
                                state: "ORDER"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    AMOUNT: [ {
                        type: "actions",
                        actions: [ {
                            state: "ADD_ITEM_TO_ORDER"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "$ITEM is a great choice!\nHow many units do you want to order?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "1"
                            },
                            actions: [ {
                                slot: "AMOUNT",
                                value: "1"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "2"
                            },
                            actions: [ {
                                slot: "AMOUNT",
                                value: "2"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "3"
                            },
                            actions: [ {
                                slot: "AMOUNT",
                                value: "3"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "I want to enter a different amount"
                            },
                            actions: [ {
                                state: "ANY_AMOUNT"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ORDER: [ {
                        type: "actions",
                        actions: [ {
                            script: "len(ORDER, LEN)"
                        }, {
                            if: "$LEN > 0",
                            actions: [ {
                                script: 'foreach(ORDER,\n   text("{{$ITEM_INDEX+1}}. $I  $A * $P = {{$A * $P}}")\n)\ntotal(ORDER, TOTAL, $A*$P)\ntext("Total: <b>$TOTAL</b>")'
                            } ]
                        }, {
                            if: "$LEN == 0",
                            actions: [ {
                                script: 'text("You haven\'t ordered anything yet.")'
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "$SAVE_STEP",
                                true: "ADD_ITEM_TO_ORDER"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🛠️ Remove"
                            },
                            actions: [ {
                                state: "REMOVE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙊 Clear all"
                            },
                            actions: [ {
                                state: "ORDER"
                            }, {
                                script: "clear(ORDER)"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ADD_ITEM_TO_ORDER: [ {
                        type: "actions",
                        actions: [ {
                            note: 'In the script, the "add" action adds an object consisting of three key-value pairs (I, A, P) to the ORDER list. The object is surrounded by curly braces. Values are taken from slots with dollar signs preceding their names.\n\nThe second argument of the "add" action lists the keys which values have to be aggregated (increased) when added. Below is the quantity of prooducts (key "A"). Aggregation occurs if the keys from the third argument list match. And the "I" key value stores the name of the product.'
                        }, {
                            script: "add(ORDER, [A], [I],  \n   {I: $ITEM, A: $AMOUNT, P: $PRICE} \n)"
                        }, {
                            slot: "SAVE_STEP",
                            value: '"ADD_ITEM_TO_ORDER"'
                        } ],
                        script: "aaa"
                    }, {
                        type: "text",
                        text: {
                            en: "I added $AMOUNT units of $ITEM to the order."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🧾 Let's add more?"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🛒 Order"
                            },
                            actions: [ {
                                state: "ORDER"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    REMOVE: [ {
                        type: "note",
                        note: "Click the edit field to check the important comments in the Item properties."
                    }, {
                        type: "text",
                        text: {
                            en: "How many units would you like to remove from the order?"
                        }
                    }, {
                        type: "input",
                        kind: "int",
                        actions: [ {
                            note: 'A user is expected to enter an integer, which will be stored in the INPUT slot.\n\nIn the "if" condition, we check if he number is positive, in which case the product is removed from the list. \n\nThe ORDER list items are numbered from 0. Therefore, one is subtracted from $INPUT.'
                        }, {
                            script: "len(ORDER, COUNT)"
                        }, {
                            if: "$INPUT \nand $INPUT > 0 \nand $INPUT <= $COUNT",
                            actions: [ {
                                script: "remove(ORDER, $INPUT-1)"
                            }, {
                                state: "ORDER"
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "WRONG_ITEM_INDEX"
                            } ]
                        } ]
                    } ],
                    ANY_AMOUNT: [ {
                        type: "text",
                        text: {
                            en: "How many units would you like to order?"
                        }
                    }, {
                        type: "input",
                        kind: "int",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                state: "ADD_ITEM_TO_ORDER"
                            }, {
                                slot: "AMOUNT",
                                value: "$INPUT"
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "WRONG_AMOUNT"
                            } ]
                        } ]
                    } ],
                    WRONG_AMOUNT: [ {
                        type: "text",
                        text: {
                            en: "The amount has to be an integer!"
                        },
                        color: "#ff0000"
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "ANY_AMOUNT"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    WRONG_ITEM_INDEX: [ {
                        type: "text",
                        text: {
                            en: "The number of the removed units can span from 1 to {{$COUNT+1}}."
                        },
                        color: "#ff0000"
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "REMOVE"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    ITEM: "",
                    AMOUNT: "",
                    SAVE_STATE: "",
                    ORDER: "",
                    PRICE: "",
                    SAVE_STEP: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 0,
                            y: 0
                        },
                        AMOUNT: {
                            c: 1,
                            r: 0,
                            x: 256,
                            y: 0
                        },
                        ORDER: {
                            c: 0,
                            r: 1,
                            x: 269,
                            y: 275
                        },
                        ADD_ITEM_TO_ORDER: {
                            c: 2,
                            r: 0,
                            x: 534,
                            y: 0
                        },
                        REMOVE: {
                            c: 1,
                            r: 1,
                            x: 557,
                            y: 275
                        },
                        ANY_AMOUNT: {
                            c: 0,
                            r: 2,
                            x: 256,
                            y: 146
                        },
                        WRONG_AMOUNT: {
                            c: 1,
                            r: 2,
                            x: 542,
                            y: 142
                        },
                        WRONG_ITEM_INDEX: {
                            c: 2,
                            r: 1,
                            x: 804,
                            y: 278
                        }
                    },
                    folders: {}
                }
            },
            Checkboxes: {
                id: 0,
                name: "Checkboxes",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "actions",
                        actions: [ {
                            clear: "-1"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "<b>Multiple choice:</b>"
                        },
                        color: "#006400",
                        center: 0
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Option1"
                            },
                            actions: [],
                            kind: "check",
                            checked: !0
                        }, {
                            type: "button",
                            button: {
                                en: "Option2"
                            },
                            actions: [],
                            kind: "check",
                            checked: !0
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Option3"
                            },
                            actions: [],
                            kind: "check",
                            checked: !1
                        }, {
                            type: "button",
                            button: {
                                en: "Option4"
                            },
                            actions: [],
                            kind: "check",
                            checked: !0
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "<b>Single choice:</b>"
                        },
                        color: "#006400",
                        center: 0
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "R1"
                            },
                            actions: [ {
                                state: ""
                            } ],
                            kind: "radio",
                            checked: !1
                        }, {
                            type: "button",
                            button: {
                                en: "R2"
                            },
                            actions: [ {
                                state: ""
                            } ],
                            kind: "radio",
                            checked: !0
                        }, {
                            type: "button",
                            button: {
                                en: "R3"
                            },
                            actions: [],
                            kind: "radio",
                            checked: !1
                        } ]
                    }, {
                        type: "note",
                        note: "Click the 🚀 to see the script, which obtains the values of the selected buttons. Actions check_get, radio_get are to be called after the buttons appear."
                    }, {
                        type: "actions",
                        actions: [ {
                            script: "get_check(CHECKS)\nsum(CHECKS, COUNT)\nget_radio(RADIOS)"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "<b>Script results:</b>\nCHECKS = $CHECKS\nChecked first: $CHECKS[0], checked last: $CHECKS[-1], total: $COUNT\n\nRADIOS = $RADIOS"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Next"
                            },
                            actions: [ {
                                state: "STEP_2"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: ""
                        } ]
                    } ],
                    STEP_2: [ {
                        type: "actions",
                        actions: [ {
                            clear: "-1"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Op1"
                            },
                            actions: [ {
                                state: ""
                            } ],
                            kind: "check",
                            checked: !1
                        }, {
                            type: "button",
                            button: {
                                en: "Op2"
                            },
                            actions: [ {
                                state: ""
                            } ],
                            kind: "check",
                            checked: !1
                        }, {
                            type: "button",
                            button: {
                                en: "Op3"
                            },
                            actions: [ {
                                state: ""
                            } ],
                            kind: "check",
                            checked: !1
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Back"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "note",
                        note: "When you first enter the step, the values of the checkboxes are set to [0,1,0]."
                    }, {
                        type: "actions",
                        actions: [ {
                            script: "if(not $WAS,\n    set(CHECK, [0,1,0])\n    set_check(CHECK)\n    set(WAS, 1)    \n)"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: ""
                        } ]
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    CHECKS: "",
                    COUNT: "",
                    CHECKS1: "",
                    CHEKS: "",
                    RADIOS: "",
                    CHECK: "",
                    WAS: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 160,
                            y: 3
                        },
                        STEP_2: {
                            c: 1,
                            r: 0,
                            x: 50,
                            y: 295
                        }
                    },
                    folders: {}
                }
            },
            Flipping: {
                id: 0,
                name: "Flipping",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "note",
                        note: "It's an example of flipping through images. Click on the actions (rocket) below."
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Save the full path to image in the URL slot."
                        }, {
                            slot: "URL",
                            value: '"https://qudata.com/qubot/im/ex/flipp/"'
                        }, {
                            note: 'The START slot is a list (square brackets). Its elements are objects (curly braces).\n\nEach object consists of the "im" key (file name) and the "nm" key (the name of the actress).'
                        }, {
                            slot: "STARS",
                            value: '[\n{im: "Jolie.jpg",    nm: "Angelina Jolie"},\n{im: "Jovovich.jpg", nm: "Milla Jovovich"},\n{im: "Kidman.jpg",   nm: "Nicole Kidman"},\n{im: "Stone.jpg",    nm: "Sharon Stone"},\n]'
                        }, {
                            note: 'In script action "len" takes the length of the list from the first argument and writes it to the slot from the second argument. \n\nThus, the LEN slot will be equal to 4.'
                        }, {
                            script: "len(STARS, LEN)"
                        }, {
                            note: "The INDEX slot will contain the number of the current image (starting from zero)."
                        }, {
                            slot: "INDEX",
                            value: "0"
                        }, {
                            note: "When all is set, we will jump instantly to step STEP_2."
                        }, {
                            goto: "STEP_2"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_2: [ {
                        type: "note",
                        note: "In the actions below, the chat history is cleared and the current image is displayed."
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "If the value equals -1, the whole chat history will be cleared. If the number is positive, the defined number of previous messages will be deleted."
                        }, {
                            clear: "-1"
                        }, {
                            note: 'In the script, the "image" action outputs an image to the current message. The action argument is the address of the image. It is formed from the $URL prefix and the image name saved in the STARS list.\n\nThe $STARS[$INDEX] entry gives an item in the STARS list numbered $INDEX. Then, after a dot, you specify the name of the key which value you want to extract from the item.'
                        }, {
                            script: "image($URL + $STARS[$INDEX].im)"
                        } ]
                    }, {
                        type: "note",
                        note: 'The text below contains the name of the actress. It is obtained from the item of the STARS list with the number $INDEX, using the "nm" key (see the explanations above in the rocket).'
                    }, {
                        type: "text",
                        text: {
                            en: "$STARS[$INDEX].nm"
                        },
                        center: 1
                    }, {
                        type: "note",
                        note: "Flip through the images with these two buttons. Click them to check the respective actions."
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "◁"
                            },
                            actions: [ {
                                note: "Pressing the previous button decreases the image number by one. The value of the slot is obtained by adding the dollar symbol to its name."
                            }, {
                                slot: "INDEX",
                                value: "$INDEX - 1"
                            }, {
                                note: "If the image number becomes less than zero, then it will be equal to the length of the list minus one (the elements are numbered starting from zero)."
                            }, {
                                if: "$INDEX < 0",
                                actions: [ {
                                    slot: "INDEX",
                                    value: "$LEN - 1"
                                } ]
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "▷"
                            },
                            actions: [ {
                                note: "Pressing the next button increases the image number by one. The value of the slot is obtained by adding the dollar symbol to its name."
                            }, {
                                slot: "INDEX",
                                value: "$INDEX + 1"
                            }, {
                                note: "If the number of the image becomes greater than or equal to the length of the list, then it will be set to zero (the elements are numbered from zero)."
                            }, {
                                if: "$INDEX >= $LEN",
                                actions: [ {
                                    slot: "INDEX",
                                    value: "0"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Select"
                            },
                            actions: [ {
                                state: "STEP_3"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_3: [ {
                        type: "text",
                        text: {
                            en: "Your choice is <b>$STARS[$INDEX].nm</b>"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Back"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: ""
                        } ]
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    STARS: "",
                    INDEX: "",
                    LEN: "",
                    URL: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 32,
                            y: 110
                        },
                        STEP_2: {
                            c: 1,
                            r: 0,
                            x: 288,
                            y: 0
                        },
                        STEP_3: {
                            c: 2,
                            r: 0,
                            x: 254,
                            y: 231
                        }
                    },
                    folders: {}
                }
            },
            Save: {
                id: 0,
                name: "Save data",
                start: "NAME",
                states: {
                    AGE: [ {
                        type: "text",
                        text: {
                            en: "How old are you?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Back"
                            },
                            actions: [ {
                                state: "NAME"
                            } ]
                        } ]
                    }, {
                        type: "note",
                        note: "Click on the input line below."
                    }, {
                        type: "input",
                        actions: [ {
                            note: "The input line (INPUT slot) is stored in the AGE slot."
                        }, {
                            slot: "AGE",
                            value: "$INPUT"
                        }, {
                            note: "The save data action is called. The Word Press repository can be viewed by their plugin.\n\nThe stored data is an object, which are key-values in curly brackets. Below the keys are kind, name, and age with the following values:\nkind - arbitrary string\nname - obtains the value of the NAME slot\nage - obtains the value of the AGE slot\n\nTo obtain the slot value, put the dollar sign before its name."
                        }, {
                            action: "save",
                            storage: "wp",
                            key: "person",
                            value: '{\n    kind: "person",\n    name: $NAME,\n    age:  $AGE\n}'
                        }, {
                            note: "After entering the text, pass to the NAME step."
                        }, {
                            state: "NAME"
                        } ],
                        kind: "text"
                    } ],
                    NAME: [ {
                        type: "note",
                        note: "Here is an example of saving the user-entered data.\n\nAfter entering the name and age, the save action is called. Check the input properties of the AGE step."
                    }, {
                        type: "text",
                        text: {
                            en: "What is your name?"
                        }
                    }, {
                        type: "note",
                        note: "Click on the input line below."
                    }, {
                        type: "input",
                        actions: [ {
                            note: "The input line (INPUT slot) is stored in the NAME slot."
                        }, {
                            slot: "NAME",
                            value: "$INPUT"
                        }, {
                            note: "After entering the text, pass to the AGE step."
                        }, {
                            state: "AGE"
                        } ],
                        kind: "text"
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    NAME: "",
                    AGE: ""
                },
                cards: {
                    states: {
                        AGE: {
                            c: 1,
                            r: 0,
                            x: 344,
                            y: 117
                        },
                        NAME: {
                            c: 0,
                            r: 0,
                            x: 98,
                            y: 17
                        }
                    },
                    folders: {}
                }
            },
            NLU: {
                id: 0,
                name: "NLU",
                start: "NLU_INTENTS",
                version: "1.0.0",
                states: {
                    NLU_INTENTS: [ {
                        type: "note",
                        note: "Below (monkey) are the affirmative intentions of users. They are made in two languages (English and Russian). The language switch is in the upper right corner of the step editor."
                    }, {
                        type: "intents",
                        actions: [ {
                            intent: "YES",
                            words: {
                                en: "yes; of course; with pleasure; i think yes; okay; \nuh-huh; duh; dah; always; y; ya;  yup; indeed; \nthat sounds good; correct;  alright; good; ok; k; kk; okidoki; okey dokey; sure; for sure; eya; aye; aye aye; jeah; fine;  yeah; yea; yep; cool; you got it;\nabsolutely; totally; certainly; sign me in; you betcha; \nyou bet; alrighty; very well; gladly; definitely; \nagree; agreed; i agree; by all means; \nsounds good; undoubtedly; i’d be delighted; \naffirmative; positive; no doubt; no doubts; \nwithout a doubt; without doubts; sure thing; \nno worries; obviously; a-okey; a-ok; oki-doki; \noki-dokie; okay-dokay; oki doki; oki dokie; \nokidokie; o' course; yeahh; yup; right; \nrighty right; all right; deal; okey; okey-doke; \nyah; doubtless; beyond doubt; surely; \ndoubtlessly; i’m sure; i am sure; confident; \ni’m confident; i am confident; it’s correct; \nit is correct; this is correct; i’m certain;\ni am certain; true; it’s true; it is true; why not; exactly;",
                                ru: "да; пожалуй; конечно; давайте; давай;\nуговорила; уговорил; охотно; ok; ок; ага; \nугу; да-да; разумеется; обязательно; \nестественно; безусловно; пожалуй да; \nда пожалуйста; да конечно; ну давайте; \nну давай; давай конечно; ладно; ладно уж;  \nо да; с радостью; с удовольствием; а как же; \nа то; а то как же; верно; именно; вот именно; \nхорошо; согласен; согласна; соглашусь; \nпожалуй соглашусь; еще бы; несомненно; \nопределенно; точно; плюс; подтверждаю; \nсамо собой разумеется; ну разумеется; \nтак точно; ответ положительный; наверное да; \nскорее всего; я за; всенепременно; таки да;\nбез сомнения; без сомнений; одобряю; за; \nне против; я не против; почему бы и нет; \nпочему бы и да; наверно да; правильно;\nэто правильно; соглашаюсь; я соглашаюсь;"
                            }
                        }, {
                            intent: "NO",
                            words: {
                                en: "no; n; nope; not yet; not today; next time; nay; \nnuh; nuh-uh; pass; i’ll pass; nae; can’t; i can’t;\nnot now; never; ne'er; i don't want; i don't think so;\ndon't like that; no way; nah; sorry but no; \nnot ever; certainly not; by no means; not likely; \nno thanks; i think not; unfortunately not; \nnot this time; not for me; it’s not my thing; \ni will pass; i’m not interested; not interested; \ncan not; cannot; I’d rather not; can’t make it; \ni can’t make it; forget it; hell no; negative; \nminus; no siree; reject; deny; refuse; i reject; \ni deny; i refuse; no indeed; not really; far from it;\nnot nearly; of course not; absolutely not; \nmost certainly not; under no circumstances;\nin no circumstances; not once; at no time;\nnot on your life; not in your life; not at all;\nby no manner of means; not true; it’s not true; \nit is not true; this is not true; incorrect;\nit’s incorrect; it is incorrect; this is incorrect;\nnot correct; it’s not correct; it is not correct; \nthis is not correct; wrong; it’s wrong; it is wrong;\nthis is wrong; false; improper; it’s improper; \nit is improper; this is improper; untrue;\nit’s untrue; it is untrue; this is untrue;\ninaccurate; it’s inaccurate; it is inaccurate; \nthis is inaccurate; imprecise; it’s imprecise; \nit is imprecise; this is imprecise; inexact; \nit’s inexact; it is inexact; this is inexact;\nnot exact; not exactly;",
                                ru: "не; нет; неа; sorry; сори; сорри;\nпока нет; нет ещё; нет еще; никак нет; \nни в коем случае; никогда; ни за что;\nдаже не подумаю; не хочу; я не хочу; \nне буду; я не буду; жаль, но нет; \nне сейчас; в другой раз; не сегодня; \nувы нет; вряд ли; в ряд ли; врядли; \nни за какие коврижки; вовсе нет;\nи в помине нет; не нужно; нетушки; \nответ отрицательный; нефига; ни фига;\nнифига; и не подумаю; на за что на свете; \nкак бы не так; как бы ни так; какбы ни так; \nкакбы не так; не нужно; мне это не нужно; \nя не согласен; не согласен; согласия не даю; \nне согласна; я не согласна; не соглашусь;\nне соглашаюсь; я не соглашаюсь;\nпротив; я против; конечно нет; нет конечно;\nнаверно нет; наверное нет; нет наверно; \nнет наверное; да нет наверное; да нет наверно;\nне верно; неверно; это неверно; это не верно;\nнеправильно; не правильно; это неправильно; \nэто не правильно; не подтверждаю; не одобряю;"
                            }
                        }, {
                            intent: "GREET",
                            words: {
                                en: "hi; hello; great; hey; hello there; hi there; moin;\nhey there; let's go; hey dude; good day; hey man;\ngood afternoon; good morning; goodmorning; \ngood evening; goodevening; morning; afternoon; \nevening; ahoy; greetings; howdy; welcome; \nheya; hi-ya; hiya; hey bro; yo; hey you; \nhey girl; sup; nice to meet you; whazzup;\nlong time no see; it’s been a while; \ngreat to meet you; good eve; good morn;\ngood to meet you; good to see you; \ngreat to see you; good to meet ya; \ngood to see ya; great to see ya; \ngreat to meet ya; mornin’; good mornin’;\nevenin’; good evenin’; sup bro; salut;",
                                ru: "здорово; здравствуй; здравствуйте; доброе;\nпривет; приветик; приветики; приветствую; \nсалют; хай; хаюшки; хэлло; хэло; але; алё; \nалло; ало; hello; hi; доброе утро; добрый вечер; \nдобрый день; доброго времени суток; я слушаю;\nмоё почтение; позвольте поприветствовать вас;\nприветствую вас; сердечно приветствую вас;\nрада вас видеть; рада тебя видеть; рад вас видеть; \nрад тебя видеть; рад видеть; рада видеть;\nдень добрый; драсте; драсти; дратути; здарова; \nдарова; алле; аллё; слушаю; приветули; \nдоброй ночи; дратуте; дд;"
                            }
                        }, {
                            intent: "GOODBYE",
                            words: {
                                en: "bye; bye bye; bye-bye; goodbye; see you later;\nhave a nice day; see you around; adieu; adios; \ngood night; cu; good by; see ya; cee you later; \nfarewell; see ya later; I’m off; take it easy; \ntill next time; until next time; keep in touch; \nta ta; tatty bye; later; lateres; take care; \nbe good; talk to you later; i need to go; \nneed to go; gotta go; hasta la vista; \ncatch you later; till later; can’t stay; chop chop; \ni’m out; i’ve got to get going; i’m out of here; \nsee you soon; i must be going; got to go; g2g; \nttyl; talk to ya later; text to you later; \ntext you later; text to ya later; text ya later; \ntoo-da-loo; have to go; i have to get going;",
                                ru: "пока; покеда; прощайте; чао; пока пока;\nбывай здоров; бывайте здоровы; \nвсего вам доброго; всего доброго; \nвсего тебе доброго; всего хорошего;\nдо встречи; до завтра; до свидания;\nдо скорой встречи; позвольте попрощаться;\nразрешите откланяться; разрешите попрощаться; \nдо скорого свидания; чао бамбино; свидимся; \nувидимся; спокойной ночи; споки; мне пора;\nпора бежать; нужно отлучиться; нужно отойти;\nпоговорим позже; спишемся; спишемся позже;\nспишемся потом; нужно идти; бывай; бывайте;\nнадо идти; надо отойти; поговорим потом;\nпоговорим не сейчас; поговорим несейчас; \nспишемся не сейчас; спишемся несейчас;\nотойду; отошел; отошла; всех благ;"
                            }
                        } ]
                    }, {
                        type: "intents",
                        actions: [ {
                            intent: "MOOD_HAPPY",
                            words: {
                                en: "perfect; great; amazing; feeling like a king; i enjoy;\nwonderful; i am feeling very good; splendid; joy;\ni am great; i am amazing; i am going to save the world;\nsuper stoked; extremely good; so so perfect; so good;\nso perfect; thumbs up; the best; over the moon; ecstatic;\non cloud nine; on top of the world; happy; i’m happy; glad; \ni’m glad; delighted; i’m delighted; pleased; i’m pleased;\njump for joy; thrilled; i’m thrilled; i’m weeping for joy;\nexcellent; well done; awesome; pleasure; my pleasure;\npleasure is all mine; stunning; i’m doing great;\nI am doing great; magnificent; spectacular; remarkable;\nyay; hooray; hurrah; woo-hoo; wahoo; whee; yahoo; \nyee-haw; yippee; tip-top; top-notch; neat; superb; \nsuper-duper; never been better; i’ve never been better; \ni have never been better; never felt better; \ni’ve never felt better; i have never felt better;\nfantastic; fabulous; feel fine; feeling fine; i’m fine;\ni am fine; i feel good; feeling good; marvelous;\nnot bad; i am happy; happy as the day is long; \nhappy as a lark; happy as a clam;",
                                ru: "идеально; отлично; замечательно; нормально; \nхорошо; прекрасно; чудесно; великолепно; \nизумительно; волшебно; сказочно; потрясно; \nкруто; здорово;  опупительно; класcно; \nкласно; офигительно; все хорошо;\nя чувствую себя очень хорошо; очень хорошо; \nв норме; не плохо; шикарно; лучше не бывает; \nв прекрасном настроении; клёво;\nв прекрасном расположении духа; агонь;\nабалденно; обалденно; оболденно; клево;"
                            }
                        }, {
                            intent: "MOOD_UNHAPPY",
                            words: {
                                en: "my day was horrible; i am sad; i don't feel very well;\ni am disappointed; super sad; i'm so sad; sad;\nvery sad; unhappy; not good; not very good; \nextremly sad; so saad; so sad; extremely sad;\ni feel sad; i feel bad; bad; unwell; i feel blue; \ni feel so down; i’m out of sorts; gloomy; so bad;\nrainy days; upset; i am upset; i’m upset; \ndepressed; i’m depressed; fed up; sick and tired; \ni’m fed up; i am fed up; i’m sick and tired; miserable;\ni am sick and tired; i am miserable;\ni’m miserable; bitter; under the weather; \ni’m under the weather; i am under the weather;\nnot in the mood; in no mood; out of humor;\ni’m unhappy; i am unhappy; not great; not fine;\nannoying; i am sorrowful; i’m sorrowful; sorrowful;\nmournful; i’m mournful; i am mournful; tragic;\nmy day was terrible; terrible; horrible; awful;\nmy day was awful; unpleasant; dreadful; \ni am feeling dreadful; i'm feeling dreadful; \nmy day was dreadful; pathetic; i'm pathetic; \ni am pathetic; lousy; i am feeling lousy; \ni'm feeling lousy; broken-hearted; heartbroken; \ni'm heartbroken; i am heartbroken; moody; \ni am moody; i'm moody; i'm feeling blue; \ni am feeling blue;",
                                ru: "плохо; отвратительно; хреново; хреновастенько; \nфигово; ужасно; ужастно; депрессия; хандра; \nне хорошо; не очень хорошо; мне плохо;\nя не очень хорошо себя чувствую; грустно; \nмой день был ужасен; мне грустно; я расстроена;\nя расстроен; супер грустно; мне так грустно; \nя очень грустный; я несчастный; хуже не бывает;\nмне крайне печально; так грустно; хуже некуда;\nя в депрессии; у меня депресняк; жизнь боль;\nочень грустно; пиши пропало; все пропало; все плохо;\nя так несчастен; я несчастен; отвратительно;\nнехорошо; не хорошо; все тлен; печально; \nпечальненько; плак; плак плак; печалька; \nя плохо себя чувствую; я грущу; я грустная;\nя несчастная; я несчастна; несчастлив; несчастлива;\nнесчастливый; дурно; худо; адски; уныло;\nтак себе; нет настроения; не в настроении;\nне в духе; отстойно; отстой; я растроен; я растроена;\nдепресия; в депресии; депрессуха; депресуха;\nдипрессия; в дипрессии; дипресняк; дипресуха;\nдипрессуха; грусть; беда; печаль; погано;"
                            }
                        }, {
                            intent: "THANKS",
                            words: {
                                ru: "и вам спасибо; спасибо вам; спасибо тебе;\nспасибо большое; спасибо огромное; благодарю;\nблагодарю вас; благодарю тебя;\nмоя благодарность вам; спасибки; вы очень любезны;\nспасибо; благодарю; спасибки; спасибочки;\nспс; благодарочка; благодарствую; \nпокорно благодарю; спасибище; мерси;\nсенкью; сенкс; большое спасибо; \nогромное спасибо; признателен;\nпризнательна; искренне признателен;\nискренне признательна; от души спасибо;\nот души благодарю; ты меня выручил;\nочень выручил; выручил; глубоко признателен;\nглубоко признательна; я твой должник;\nя твоя должница; так приятно;\nпасиба; пасибо; премного благодарен;\nблагодарна; сенки; сэнкью; сэнкс; сэнки;\nс меня должок; буду должен; буду должна;\nчтобы я без тебя делал; ты незаменим;\nчто бы я без тебя делала;",
                                en: "thanks; thank you; with pleasure; ty; \nthank you very much; tnx; thankee;\nmy appreciation; thank you so much; \ngrateful; i’m grateful; i am grateful;\ni am so grateful; you have my gratitude;\ni sincerely appreciate; i appreciate;\nmy gratitude; many thanks; thankful; \ni’m thankful; i am thankful; and thank you; \nthanks a lot; thank you a lot; tnx a lot;\nmy gratitude to you; you are very kind;\ni am grateful for your help; thanks a bunch;\nmuch obliged; thank you for your support;\nit’s very kind of you; i owe you one;\ni owe you big time; you’re too kind;\ni can’t thank you enough; very grateful;\nthanks a ton; how thoughtful of you;\nyou are the best; you made my day;  you have my thanks;"
                            }
                        } ]
                    }, {
                        type: "note",
                        note: "The following intent contains some questions from the user. This and the previous item can be copied to your project (gray menu on the left)."
                    }, {
                        type: "intents",
                        actions: [ {
                            intent: "ARE_YOU_BOT",
                            words: {
                                en: "are you a bot?; are you a human?;\nam I talking to a bot?;\nam I talking to a human?;\nwhat are you?; who are you?;\nyou are a bot; you’re bot; \naren’t you a bot?; you’re not a human;\nyou’re not a person; are you alive?;\nyou are not alive; you’re not alive; \ni’m not talking to a human, right?;\nare you a machine?; are you a robot?;\nyou machine?; you’re machine?; \nyou’re machine; you’re a machine;\nyou are a machine; you are machine;\nare you robot?; you robot?; \nyou’re robot?; you’re robot; you’re a robot;\nyou are a robot; you are robot;\nare you AI?; you are AI; AI?;\nam i texting with a bot?; are you an android?;\nam I talking to an android?; \nyou are an android; are you a droid?;\nam I talking to a droid?; you are a droid?;\nam i texting with a machine?;\nam i texting with an android?;\nam i texting with a droid?; \nam i texting with a robot?;",
                                ru: "вы - бот?; ты - бот?;вы бот?; ты бот?; \nты бот; ты же бот; вы бот; вы же бот;\nты не человек?; вы не человек?\nвы человек?; ты человек?; \nя разговариваю с ботом?;\nя разговариваю с человеком?;\nты кто?; по-моему ты не человек;\nразве ты человек?; ты точно бот;\nразве ты не человек?;\nмне кажется ты не человек;\nмне кажется ты бот; ты точно не человек;\nты живой?; ты не живой; ты машина?; \nты машина; а ты машина?; а ты не машина?;\nвы машина?; вы не машина?;"
                            }
                        }, {
                            intent: "WHAT_IS_YOUR_NAME",
                            words: {
                                ru: "как тебя зовут?; а как тебя зовут?;\nа как вас зовут?; как вас зовут?;\nтвое имя; могу ли я узнать твоё имя?;\nно у тебя имя есть?; твое имя?\nкак мне тебя называть?; а фамилия у тебя есть?;\nкакое у тебя имя?; а имя у тебя какое?;\nимя?; как я могу обращаться?;\nкак я могу к вам обращаться?;\nкак я могу к тебе обращаться?;\nя могу обращаться по имени?; \nкак мне к тебе обращаться?; как я могу тебя называть?;\nкак мне к вам обращаться?; как мне вас называть?;\nкак я могу вас называть?;\nскажи свое имя; напиши свое имя;\nпредставься; представься; представьтесь;\nпредставься пожалуйста;  представьтесь пожалуйста;\nкак звать тебя?; как звать-то тебя?;\nкак звать то тебя?; как называть тебя?;\nкак тебя называть?; как зовут?;\nкак к тебе обращаться?; как звать?;\nимя у тебя есть?; имя есть?; \nзвать тебя как?; звать-то тебя как?; \nзвать то тебя как?; называть тебя как?;\nтебя как звать?; тебя как звать-то?; \nтебя как звать то?;\nкак звать вас?; как звать-то вас?;\nкак звать то вас?; как называть вас?;\nкак вас называть?; имя у вас есть?; \nкак к вам обращаться?; \nзвать вас как?; звать-то вас как?; \nзвать то вас как?; называть вас как?;\nвас как звать?; вас как звать-то?;\nвас как звать то?; имя фамилия есть?;",
                                en: "what's your name?; what is your name?;\nyour name; name?; may i know your name?;\nbut do you have a name?; how can I call you?;\nwhat can i call you?; do you have a surname?;\ni want to know your name;\nintroduce; introduce yourself;  i want your name;\ncan you tell me your name?; tell me your name;\nwhat was your name?; didn’t catch your name;\ncatch your name; got a name?; you got a name?;\nhow do you call yourself?; state your name; \nwhat do they call you?; identify; identify yourself;\nwrite your name; say your name; how do I call you?; \nwhat do I call you?;  i wanna know your name;\nhow do they call you?; spell your name; text your name;"
                            }
                        }, {
                            intent: "DO_YOU_HAVE_MIND",
                            words: {
                                ru: "ты умеешь думать?;\nу тебя есть сознание?; компьютер умеет думать?;\nбот умеет думать?; ты понимаешь?; ты меня понимаешь?;\nты понимаешь что я тебе говорю?;\nесть ли мозг у компьютера?;\nты умный?; ты разумен?; \nу тебя есть разум?; у тебя есть мысли?;\nу тебя есть свои мысли?;\nу тебя есть собственные мысли?;\nу тебя есть мозг?; у тебя есть мозги?;",
                                en: "can you think; are you able to think?;\ndo you have consciousness?;\nbot can think?; does bot think;\ndo bots think; what do you think?;\ncomputer can think?; you understand?;\ncan you understand me?;\ndo you understand me?;\ndo you understand what I'm telling you;\ndoes a computer have a brain?;\ndo you have a brain?;\nyou are clever?; are you clever;\ndo you have a mind?;\ndo you have your own mind?;\ndo you have thoughts; \ndo you have your own thoughts?;\nare you intelligent?; aren’t you intelligent;\ncan a bot think?; do you have an opinion?;\ndo you have your own opinion?;\ncan a bot have a mind; are you smart?;\ndo you have intelligence?; are you wise?;\ndo you have self-awareness?;\nare you aware of yourself; are you witty?;"
                            }
                        }, {
                            intent: "DO_YOU_HAVE_FEELINGS",
                            words: {
                                ru: "у тебя есть чувства?;\nкомпьютер умеет чувствовать?;\nты умеешь любить?; ты умеешь чувствовать?;\nты можешь чувствовать?; ты способен чувствовать?\nты способен любить?; ты чувствуешь боль?;\nу тебя есть чувства?; что ты чувствуешь?;\nчто ты можешь чувствовать?; ты умеешь злиться?;\nты способен злиться?; ты устал?; ты можешь устать?;",
                                en: "do you have feelings?; what are your feelings?\ncan the computer feel?; can you love?; can you feel;\nare you able to love?; do you feel pain;\ndo you feel anything?; what are your feelings?;\nwhat do you feel; can you love?; do you love;\ncan you get angry; do you get angry?;\ncan you get upset?; do you get upset?;\ncan you get tired; are you tired;\ndo you get tired?; can you get annoyed?; \ndo you get annoyed; are you happy?; can you be happy?;\nhave you ever been happy;\ndoes anything makes you happy?; \ndoes anything makes you sad;\ncan you hate; do you hate?; can a bot have feelings?\ncan i hurt your feelings; are you afraid of anything?;\ncan you be scared; can you be jealous?;\nare you jealous of anything; do you envy other bots?;\ndo you feel joy; are you joyful?;"
                            }
                        } ]
                    }, {
                        type: "intents",
                        actions: [ {
                            intent: "I_WANT",
                            words: {
                                ru: "хочу; я хочу; мне нужно; \nмне очень нужно;  мне очень хочется; \nя мечтаю; желаю; я желаю; мне хочется;\nне могу дождаться; мое желание; хочу не могу;\nхотелось бы; давно мечтаю о;",
                                en: "want; i want; i need; i want it very much; i’d like; \ni would like; i’d love; i would love; craving; \ni’m craving; I feel like; i am in the mood for; \ni’m in the mood for; i’m dying for; desire; \nwish; i wish; more than anything; my wish;\nmy desire; wants; want so much; want it so much;\ni fancy; i hope for; i was hoping; i set my heart on;\ni have an appetite for; i crave; demand; i demand; \ni hanker; hankering; covet; i covet; wish for; \nlong for; yearn for; dream of; aspire to; i wanna;\nhanker for; hanker after; hunger after; wanna;\nhunger for; thirst for; ache for; burn for; pant for;"
                            }
                        } ]
                    }, {
                        type: "note",
                        note: "Click on actions (rocket) for further comments."
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "For the convenience of testing, we will immediately set the current language.",
                            goto: "PRE"
                        }, {
                            slot: "LANGUAGE",
                            value: '"en"'
                        }, {
                            note: "This slot is useful for creating variability in the DONT_UNDERSTAND step."
                        }, {
                            slot: "WAS_UNKNOWN",
                            value: "0"
                        }, {
                            note: "Instant jump to the TEST step where you can test your intentions."
                        }, {
                            goto: ""
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TEST: [ {
                        type: "text",
                        text: {
                            en: 'Intent recognition test. \nEnter any text:\n\nIntent: <b class="green">$INTENT</b>\nconfidence: $CONFIDENCE',
                            ru: 'Распознавания намерения.\nВведите любой текст:\n\nНамерениe: <b class="green">$INTENT</b>\nУверенность: $CONFIDENCE'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Change language",
                                ru: "🌐 Сменить язык"
                            },
                            actions: [ {
                                state: "CHANGE_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙊 Talk with bot",
                                ru: "🙊 Поговорить с ботом"
                            },
                            actions: [ {
                                state: "FIRST_GREET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "TEST"
                        } ]
                    } ],
                    CHANGE_LANGUAGE: [ {
                        type: "actions",
                        actions: [ {
                            state: "TEST"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "What language do you prefer to communicate in?",
                            ru: "На каком языке Вы хотите общаться?",
                            ua: "Якою мовою Ви бажаєте спілкуватись?",
                            de: "In welcher Sprache möchten Sie kommunizieren?",
                            fr: "Dans quelle langue souhaitez-vous communiquer ?",
                            es: "¿En qué idioma quieres comunicarte?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English",
                                ua: "English",
                                de: "English",
                                fr: "English",
                                es: "English"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "en"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Русский",
                                ru: "Русский",
                                ua: "Русский",
                                de: "Русский",
                                fr: "Русский",
                                es: "Русский"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "ru"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CHITCHAT: [ {
                        type: "note",
                        note: "This is the key step in which user intent is analyzed. Click on the actions (rocket)."
                    }, {
                        type: "actions",
                        actions: [ {
                            note: 'If the level of confidence in recognizing the intention is low, go to the step "DONT_UNDERSTAND"'
                        }, {
                            if: "$CONFIDENCE < 30",
                            actions: [ {
                                goto: "DONT_UNDERSTAND"
                            } ]
                        }, {
                            note: "The bot remembers the user's last intentions. The branch statement selects the next replica. Entering text in transition steps returns to that step again."
                        }, {
                            switch: "$INTENT",
                            actions: [ {
                                case: '"GREET"',
                                actions: [ {
                                    goto: "GREET"
                                } ]
                            }, {
                                case: '"GOODBYE"',
                                actions: [ {
                                    goto: "GOODBYE"
                                } ]
                            }, {
                                case: '"MOOD_UNHAPPY"',
                                actions: [ {
                                    goto: "MOOD_UNHAPPY"
                                } ]
                            }, {
                                case: '"MOOD_HAPPY"',
                                actions: [ {
                                    goto: "MOOD_HAPPY"
                                } ]
                            }, {
                                default: "",
                                actions: [ {
                                    goto: "CHITCHAT_2"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    GOODBYE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Bye Bye",
                            ru: "Пока-пока"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    GREET_HAPPY: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Wonderful! I'm happy for you.",
                            ru: "Замечательно! Я рад за Вас."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "I_AM_CLEVER"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    GREET_UNHAPPY: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "How sad... What happened?",
                            ru: "Как печально... А что случилось?"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "GREET_UNHAPPY_CLEAR"
                        } ]
                    } ],
                    DONT_UNDERSTAND: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "WAS_UNKNOWN",
                            value: "$WAS_UNKNOWN + 1"
                        }, {
                            switch: "$WAS_UNKNOWN",
                            actions: [ {
                                case: "2",
                                actions: [ {
                                    goto: "DONT_UNKNOWN_2"
                                } ]
                            }, {
                                case: "3",
                                actions: [ {
                                    goto: "DONT_UNKNOWN_3"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "I do not understand",
                            ru: "Я Вас не понимаю"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    GREET_UNHAPPY_CLEAR: [ {
                        type: "message",
                        typing: "2000"
                    }, {
                        type: "text",
                        text: {
                            en: "It's clear. But don't worry, everything will work out!",
                            ru: "Понятно. Но не переживайте, всё наладится!"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "I_AM_CLEVER"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    AFTER_GREET: [ {
                        type: "message",
                        typing: "2000"
                    }, {
                        type: "text",
                        text: {
                            en: "Nice to meet you, $NAME. How is your mood?",
                            ru: "Рад знакомству, $NAME. Как Ваше настроение?"
                        }
                    }, {
                        type: "note",
                        note: "Click on the properties of the input field"
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            note: "First of all, the input property kind: intent is set. This launches the intent recognition algorithm. The result is placed in the INTENT slot."
                        }, {
                            note: 'If the level of confidence in recognizing the intention is low, go to the step "DONT_UNDERSTAND"'
                        }, {
                            if: "$CONFIDENCE < 30",
                            actions: [ {
                                goto: "DONT_UNDERSTAND"
                            } ]
                        }, {
                            note: "Then transitions are set in the switch statement for the expected two intents. Otherwise, it goes to the CHITCHAT step."
                        }, {
                            switch: "$INTENT",
                            actions: [ {
                                case: '"MOOD_HAPPY"',
                                actions: [ {
                                    state: "GREET_HAPPY"
                                } ]
                            }, {
                                case: '"MOOD_UNHAPPY"',
                                actions: [ {
                                    state: "GREET_UNHAPPY"
                                } ]
                            }, {
                                default: "",
                                actions: [ {
                                    state: "CHITCHAT"
                                } ]
                            } ]
                        } ]
                    } ],
                    FIRST_GREET: [ {
                        type: "note",
                        note: "This step begins the hello dialogue chain. The user may not respond as expected. In this case, there will be a transition to the CHITCHAT step in which all intentions known to the bot are analyzed."
                    }, {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Hi, my name is Qu. What is your name?",
                            ru: "Привет, меня зовут Кью. А как зовут Вас?"
                        }
                    }, {
                        type: "note",
                        note: "Click on the properties of the input field"
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            slot: "NAME",
                            value: "$INPUT"
                        }, {
                            state: "AFTER_GREET"
                        } ]
                    } ],
                    GREET: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We already kind of said hello ...",
                            ru: "Мы уже вроде поздоровались..."
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    I_AM_CLEVER: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Do you agree that I am very smart?",
                            ru: "Вы согласны, что я очень умный?"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            if: "$CONFIDENCE < 30",
                            actions: [ {
                                goto: "DONT_UNDERSTAND"
                            } ]
                        }, {
                            switch: "$INTENT",
                            actions: [ {
                                case: '"YES"',
                                actions: [ {
                                    state: "CLEVER_YES"
                                } ]
                            }, {
                                case: '"NO"',
                                actions: [ {
                                    state: "CLEVER_NO"
                                } ]
                            }, {
                                default: "",
                                actions: [ {
                                    state: "CHITCHAT"
                                } ]
                            } ]
                        } ]
                    } ],
                    CLEVER_YES: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "And I completely agree with you!",
                            ru: "И я с Вами совершенно согласен!"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    CLEVER_NO: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Well nothing. Soon you will understand how wrong you are...",
                            ru: "Ну, ничего. Скоро Вы поймёте как Вы неправы..."
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    MOOD_UNHAPPY: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Yes, it's sad...",
                            ru: "Да, это печально..."
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT",
                            true: "MOOD_HAPPY"
                        } ]
                    } ],
                    MOOD_HAPPY: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "I'm happy for you",
                            ru: "Я рад за Вас"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    ARE_YOU_BOT: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Yes, I'm an artificial intelligence. Hope it doesn't scare you?",
                            ru: "Да я искусственный разум. Надеюсь, это Вас не пугает?"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            switch: "$INTENT",
                            actions: [ {
                                case: '"YES"',
                                actions: [ {
                                    state: "ARE_YOU_BOT_YES"
                                } ]
                            }, {
                                case: '"NO"',
                                actions: [ {
                                    state: "ARE_YOU_BOT_YES"
                                } ]
                            }, {
                                default: "",
                                actions: [ {
                                    state: "CHITCHAT"
                                } ]
                            } ]
                        } ]
                    } ],
                    WHAT_IS_YOUR_NAME: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "I already introduced myself. My name is Qu.",
                            ru: "Я уже представлялся. Меня зовут Кью."
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    DO_YOU_HAVE_MIND: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "What do you think? Of course yes!",
                            ru: "Я Вы как думаете? Конечно - да!"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT",
                            true: "DO_YOU_HAVE_FEELINGS"
                        } ]
                    } ],
                    DO_YOU_HAVE_FEELINGS: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "I am very worried when they do not understand me. And it's even worse when I don't understand. After all, I'm a very smart bot, right?",
                            ru: "Я очень переживаю, когда меня не понимают. А ещё хуже когда не понимаю я. Ведь я очень умный бот не так ли?"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            if: "$CONFIDENCE < 30",
                            actions: [ {
                                goto: "DONT_UNDERSTAND"
                            } ]
                        }, {
                            switch: "$INTENT",
                            actions: [ {
                                case: '"YES"',
                                actions: [ {
                                    state: "CLEVER_YES"
                                } ]
                            }, {
                                case: '"NO"',
                                actions: [ {
                                    state: "CLEVER_NO"
                                } ]
                            }, {
                                default: "",
                                actions: [ {
                                    state: "CHITCHAT"
                                } ]
                            } ]
                        } ]
                    } ],
                    I_WANT: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Satisfying your desires is my main goal!",
                            ru: "Удовлетворить Ваши желания моя основная цель!"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT",
                            true: "DONT_UNDERSTAND"
                        } ]
                    } ],
                    CHITCHAT_2: [ {
                        type: "note",
                        note: "For convenience, we have split CHITCHAT into two steps. Click on аctions:"
                    }, {
                        type: "actions",
                        actions: [ {
                            switch: "$INTENT",
                            actions: [ {
                                case: '"ARE_YOU_BOT"',
                                actions: [ {
                                    goto: "ARE_YOU_BOT",
                                    true: "WHAT_IS_YOUR_NAME"
                                } ]
                            }, {
                                case: '"WHAT_IS_YOUR_NAME"',
                                actions: [ {
                                    goto: "WHAT_IS_YOUR_NAME",
                                    true: "DO_YOU_HAVE_MIND"
                                } ]
                            }, {
                                case: '"DO_YOU_HAVE_MIND"',
                                actions: [ {
                                    goto: "DO_YOU_HAVE_MIND",
                                    true: "DO_YOU_HAVE_FEELINGS"
                                } ]
                            }, {
                                case: '"DO_YOU_HAVE_FEELINGS"',
                                actions: [ {
                                    goto: "DO_YOU_HAVE_FEELINGS"
                                } ]
                            }, {
                                case: '"THANKS"',
                                actions: [ {
                                    goto: ""
                                } ]
                            }, {
                                case: '"I_WANT"',
                                actions: [ {
                                    goto: "I_WANT"
                                } ]
                            }, {
                                default: "",
                                actions: [ {
                                    goto: "DONT_UNDERSTAND"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    DONT_UNKNOWN_2: [ {
                        type: "text",
                        text: {
                            en: "Sadly, I don't seem to catch you again.",
                            ru: "Печально, но кажется я снова Вас не понимаю."
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    DONT_UNKNOWN_3: [ {
                        type: "text",
                        text: {
                            en: "Again the difficulty of understanding. Forgive me, I'm just an example.",
                            ru: "Опять сложности с пониманием. Вы меня простите, я же только пример. "
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "We will repeat again the three steps DONT_UNDERSTAND"
                        }, {
                            slot: "WAS_UNKNOWN",
                            value: "0"
                        } ]
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    ARE_YOU_BOT_YES: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "It is right. I am your assistant. Why be afraid of me?",
                            ru: "Это правильно. Я Ваш помощник. Чего меня бояться?"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    ARE_YOU_BOT_NO: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Fear not, I am good and harmless.",
                            ru: "Не бойтесь, я хороший и безобидный."
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ],
                    PRE: [ {
                        type: "text",
                        text: {
                            en: "First you can test intents. Remember that the bot only understands the intent classes it knows:\n<b>YES, NO, GREET, GOODBYE, MOOD_HAPPY, MOOD_UNHAPPY, THANKS, ARE_YOU_BOT, WHAT_IS_YOUR_NAME, DO_YOU_HAVE_MIND, DO_YOU_HAVE_FEELINGS, I_WANT</b>",
                            ru: "Сначала Вы можете тестировать намерения. Помните, что бот понимает только известные ему классы намерений:\n<b>YES, NO, GREET, GOODBYE, MOOD_HAPPY, MOOD_UNHAPPY, THANKS, ARE_YOU_BOT, WHAT_IS_YOUR_NAME, DO_YOU_HAVE_MIND, DO_YOU_HAVE_FEELINGS, I_WANT</b>"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "TEST"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    THANKS: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "You are welcome",
                            ru: "Пожалуйста"
                        }
                    }, {
                        type: "input",
                        kind: "intent",
                        actions: [ {
                            state: "CHITCHAT"
                        } ]
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    NAME: "",
                    WAS_UNKNOWN: "",
                    INTENT: "",
                    CONFIDENCE: ""
                },
                cards: {
                    states: {
                        NLU_INTENTS: {
                            c: 0,
                            r: 0,
                            x: 6,
                            y: 23
                        },
                        TEST: {
                            c: 2,
                            r: 0,
                            x: 489,
                            y: 5
                        },
                        CHANGE_LANGUAGE: {
                            c: 3,
                            r: 0,
                            x: 749,
                            y: 18
                        },
                        CHITCHAT: {
                            c: 0,
                            r: 5,
                            x: 326,
                            y: 575
                        },
                        GOODBYE: {
                            c: 0,
                            r: 6,
                            x: 274,
                            y: 772
                        },
                        GREET_HAPPY: {
                            c: 0,
                            r: 3,
                            x: 23,
                            y: 267
                        },
                        GREET_UNHAPPY: {
                            c: 0,
                            r: 2,
                            x: 610,
                            y: 273
                        },
                        DONT_UNDERSTAND: {
                            c: 0,
                            r: 8,
                            x: 35,
                            y: 1206
                        },
                        GREET_UNHAPPY_CLEAR: {
                            c: 1,
                            r: 2,
                            x: 869,
                            y: 276
                        },
                        AFTER_GREET: {
                            c: 1,
                            r: 1,
                            x: 341,
                            y: 221
                        },
                        FIRST_GREET: {
                            c: 0,
                            r: 1,
                            x: 329,
                            y: 139
                        },
                        GREET: {
                            c: 3,
                            r: 6,
                            x: 17,
                            y: 769
                        },
                        I_AM_CLEVER: {
                            c: 0,
                            r: 4,
                            x: 333,
                            y: 388
                        },
                        CLEVER_YES: {
                            c: 2,
                            r: 4,
                            x: 42,
                            y: 436
                        },
                        CLEVER_NO: {
                            c: 1,
                            r: 4,
                            x: 674,
                            y: 437
                        },
                        MOOD_UNHAPPY: {
                            c: 1,
                            r: 6,
                            x: 733,
                            y: 768
                        },
                        MOOD_HAPPY: {
                            c: 2,
                            r: 6,
                            x: 504,
                            y: 768
                        },
                        ARE_YOU_BOT: {
                            c: 0,
                            r: 9,
                            x: 385,
                            y: 880
                        },
                        WHAT_IS_YOUR_NAME: {
                            c: 0,
                            r: 7,
                            x: 108,
                            y: 877
                        },
                        DO_YOU_HAVE_MIND: {
                            c: 1,
                            r: 7,
                            x: 22,
                            y: 1037
                        },
                        DO_YOU_HAVE_FEELINGS: {
                            c: 2,
                            r: 7,
                            x: 827,
                            y: 1005
                        },
                        I_WANT: {
                            c: 3,
                            r: 7,
                            x: 657,
                            y: 890
                        },
                        CHITCHAT_2: {
                            c: 1,
                            r: 5,
                            x: 331,
                            y: 671
                        },
                        DONT_UNKNOWN_2: {
                            c: 1,
                            r: 8,
                            x: 363,
                            y: 1208
                        },
                        DONT_UNKNOWN_3: {
                            c: 2,
                            r: 8,
                            x: 615,
                            y: 1198
                        },
                        ARE_YOU_BOT_YES: {
                            c: 1,
                            r: 9,
                            x: 245,
                            y: 976
                        },
                        ARE_YOU_BOT_NO: {
                            c: 2,
                            r: 9,
                            x: 488,
                            y: 983
                        },
                        PRE: {
                            c: 1,
                            r: 0,
                            x: 234,
                            y: 3
                        },
                        THANKS: {
                            c: 4,
                            r: 7,
                            x: 875,
                            y: 902
                        }
                    },
                    folders: {}
                }
            }
        };
    }
}

class QuBotEditorTemplates {
    constructor() {
        this.bots = {
            CUSTOMER: {
                id: 0,
                name: "Customer",
                start: "BEGIN",
                version: "1.0.0",
                states: {
                    CUSTOMER_WRONG_NAME: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Very strange name $INPUT. Could you introduce yourself differently:",
                            ru: "Очень странное имя $INPUT. Не могли бы Вы представиться по другому:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            if: "len($INPUT) > 1",
                            actions: [ {
                                state: "CUSTOMER_GREET"
                            }, {
                                slot: "CUSTOMER_NAME",
                                value: "$INPUT"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Our mail: info@qudata.com, and yours, $CUSTOMER_NAME?",
                            ru: "Наша почта: info@qudata.com, а Ваша, $CUSTOMER_NAME?"
                        }
                    }, {
                        type: "input",
                        kind: "email",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                slot: "CUSTOMER_EMAIL",
                                value: "$INPUT"
                            }, {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL_WRONG: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We will not be able to write to you at this address. Will you try again?",
                            ru: "Мы не сможем написать Вам по такому адресу. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, please enter your phone number:",
                            ru: "$CUSTOMER_NAME, введите пожалуйста Ваш телефон:"
                        }
                    }, {
                        type: "input",
                        kind: "phone",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                slot: "CUSTOMER_PHONE",
                                value: "$INPUT"
                            }, {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_PHONE_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_PHONE_WRONG: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We can't call that phone. Will you try again?",
                            ru: "Мы не сможем позвонить по такому телефону. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE_MAYBE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, maybe you will leave your phone? We can send messages in the messenger.",
                            ru: "$CUSTOMER_NAME, может быть Вы оставите Ваш телефон? Мы можем присылать сообщения в месенджере."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_EMAIL_MAYBE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, would you mind leaving your email? Sometimes we will send in very useful information.",
                            ru: "$CUSTOMER_NAME, Вы не против оставить Вашу электронную почту? Иногда мы будем присылать очень полезную информацию."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_MESSAGE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Write something about yourself or leave any of your suggestions. Your feedback is very important to us.",
                            ru: "Напишите что-нибудь о себе или оставьте любые Ваши предложения. Ваше мнение очень важно для нас."
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_END"
                        }, {
                            slot: "CUSTOMER_MESSAGE",
                            value: "$INPUT"
                        } ]
                    } ],
                    CUSTOMER_END: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "customer",
                            value: '{\n"kind":    "customer",\n"name":    $CUSTOMER_NAME,\n"phone":   $CUSTOMER_PHONE,\n"mail":    $CUSTOMER_EMAIL,\n"message": $CUSTOMER_MESSAGE\n}'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Thank you very much $CUSTOMER_NAME 😊, we will contact you shortly.",
                            ru: "Спасибо большое $CUSTOMER_NAME 😊, мы свяжемся с Вами в ближайшее время."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "END"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHECK: [ {
                        type: "text",
                        text: {
                            en: "Is that correct?",
                            ru: "Всё верно?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_MESSAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🤖 Change name",
                                ru: "🤖 Изменить имя"
                            },
                            actions: [ {
                                state: "CUSTOMER_NAME"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "💻Change mail",
                                ru: "💻Изменить почту"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Change phone",
                                ru: "📞 Изменить телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BEFORE_CHECK: [ {
                        type: "actions",
                        actions: [ {
                            if: "$CUSTOMER_PHONE and $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "PHONE_AND_EMAIL"
                            } ]
                        }, {
                            if: "$CUSTOMER_PHONE",
                            actions: [ {
                                goto: "ONLY_PHONE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "ONLY_EMAIL"
                            } ]
                        }, {
                            goto: "NOTHING"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PHONE_AND_EMAIL: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by mail <b> $CUSTOMER_EMAIL </b> and phone <b> $CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b> и телефону <b>$CUSTOMER_PHONE</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_PHONE: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact phone <b>$CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по телефону <b>$CUSTOMER_PHONE</b>. "
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b>$CUSTOMER_NAME </b> and you prefer to contact by by mail <b>$CUSTOMER_EMAIL</b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NOTHING: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, you have left no phone or mail.",
                            ru: "$CUSTOMER_NAME, Вы не оставили, ни телефона, ни почты."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_NAME: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Please introduce yourself 🙏:",
                            ru: "Пожалуйста, введите своё имя 🙏:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_GREET"
                            }, {
                                slot: "CUSTOMER_NAME",
                                value: "$INPUT"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_START: [ {
                        type: "actions",
                        actions: [ {
                            if: "not $CUSTOMER_NAME",
                            actions: [ {
                                goto: "CUSTOMER_NAME"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL and not $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_CHANNEL"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL and $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_EMAIL_MAYBE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL and not $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_PHONE_MAYBE"
                            } ]
                        }, {
                            goto: "BEFORE_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BEGIN: [ {
                        type: "text",
                        text: {
                            en: "Welcome to our website!",
                            ru: "Приветствую Вас на нашем сайте!"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Select your language",
                                ru: "🌐 Выберите Ваш язык"
                            },
                            actions: [ {
                                state: "CHANGE_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📧 Start",
                                ru: "📧 Начать"
                            },
                            actions: [ {
                                state: "CUSTOMER_START"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    END: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Thanks a lot",
                            ru: "Спасибо большое"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Back to top",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "BEGIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CHANGE_LANGUAGE: [ {
                        type: "actions",
                        actions: [ {
                            state: "BEGIN"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "What language do you prefer to communicate in?",
                            ru: "На каком языке Вы хотите общаться?",
                            ua: "Якою мовою Ви бажаєте спілкуватись?",
                            de: "In welcher Sprache möchten Sie kommunizieren?",
                            fr: "Dans quelle langue souhaitez-vous communiquer ?",
                            es: "¿En qué idioma quieres comunicarte?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English",
                                ua: "English",
                                de: "English",
                                fr: "English",
                                es: "English"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "en"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Русский",
                                ru: "Русский",
                                ua: "Русский",
                                de: "Русский",
                                fr: "Русский",
                                es: "Русский"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "ru"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHANNEL: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We may want to contact you. Is it more convenient for you to communicate by phone or email?",
                            ru: "Возможно, мы захотим связаться с Вами. Вам удобнее общаться по телефону или по электронной почте?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Phone",
                                ru: "📞 Телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "💻 Email",
                                ru: "💻 Почта"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_GREET: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Nice to meet you $CUSTOMER_NAME!",
                            ru: "Рад знакомству $CUSTOMER_NAME!"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_START"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    CUSTOMER_NAME: "",
                    CUSTOMER_PHONE: "",
                    CUSTOMER_MESSAGE: "",
                    CUSTOMER_EMAIL: ""
                },
                cards: {
                    states: {
                        CUSTOMER_WRONG_NAME: {
                            c: 1,
                            r: 3,
                            x: 628,
                            y: 164,
                            folder: "Customer"
                        },
                        CUSTOMER_CHANNEL: {
                            c: 3,
                            r: 3,
                            x: 418,
                            y: 259,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL: {
                            c: 0,
                            r: 4,
                            x: 703,
                            y: 389,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL_WRONG: {
                            c: 1,
                            r: 4,
                            x: 718,
                            y: 531,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE: {
                            c: 0,
                            r: 5,
                            x: 210,
                            y: 388,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE_WRONG: {
                            c: 1,
                            r: 5,
                            x: 189,
                            y: 535,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE_MAYBE: {
                            c: 2,
                            r: 4,
                            x: 432,
                            y: 533,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL_MAYBE: {
                            c: 2,
                            r: 5,
                            x: 428,
                            y: 382,
                            folder: "Customer"
                        },
                        CUSTOMER_MESSAGE: {
                            c: 1,
                            r: 8,
                            x: 300,
                            y: -100,
                            folder: "Customer"
                        },
                        CUSTOMER_END: {
                            c: 2,
                            r: 8,
                            x: 500,
                            y: -100,
                            folder: "Customer"
                        },
                        CUSTOMER_CHECK: {
                            c: 0,
                            r: 8,
                            x: 300,
                            y: -100,
                            folder: "Customer"
                        },
                        BEFORE_CHECK: {
                            c: 0,
                            r: 6,
                            x: 184,
                            y: 667,
                            folder: "Customer"
                        },
                        PHONE_AND_EMAIL: {
                            c: 0,
                            r: 7,
                            x: 380,
                            y: 670,
                            folder: "Customer"
                        },
                        ONLY_PHONE: {
                            c: 1,
                            r: 7,
                            x: 571,
                            y: 670,
                            folder: "Customer"
                        },
                        ONLY_EMAIL: {
                            c: 2,
                            r: 7,
                            x: 764,
                            y: 671,
                            folder: "Customer"
                        },
                        NOTHING: {
                            c: 3,
                            r: 7,
                            x: 575,
                            y: 762,
                            folder: "Customer"
                        },
                        CUSTOMER_NAME: {
                            c: 0,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "Customer"
                        },
                        CUSTOMER_START: {
                            c: 0,
                            r: 2,
                            x: 600,
                            y: 0,
                            folder: "Customer"
                        },
                        BEGIN: {
                            c: 0,
                            r: 0,
                            x: 280,
                            y: 147
                        },
                        END: {
                            c: 0,
                            r: 9,
                            x: 599,
                            y: 265
                        },
                        CHANGE_LANGUAGE: {
                            c: 1,
                            r: 0,
                            x: 25,
                            y: 30
                        },
                        CUSTOMER_GREET: {
                            c: 2,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "Customer"
                        }
                    },
                    folders: {
                        Customer: {
                            c: 0,
                            r: 1,
                            x: 30,
                            y: 301
                        }
                    }
                }
            },
            HR: {
                id: 0,
                name: "HR-manager",
                start: "HR_GREED",
                version: "1.0.0",
                states: {
                    HR_GREED: [ {
                        type: "input",
                        actions: [],
                        kind: "text"
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "TEST_ML",
                            value: "10000"
                        }, {
                            slot: "TEST_SALES",
                            value: "20000"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Hello! I'm a virtual HR Manager with <b>QuData</b>. ",
                            ru: "С Вами на связи виртуальный HR-менеджер компании <b>Qudata</b>."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/owl_a.png",
                            width: 50
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "We have an open position of a ML Specialist and a Sales Manager.",
                            ru: "Мы ищем специалиста по ML и менеджера по продажам."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "I can tell you about our company, and do a preliminary interview.",
                            ru: "Я могу познакомить Вас с нашей компанией, а также провести первый этап собеседования."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "ℹ Company",
                                ru: "ℹ О компании"
                            },
                            actions: [ {
                                state: "COMPANY_INFO1"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🙇 Interview",
                                ru: "🙇 Интервью"
                            },
                            actions: [ {
                                state: "JOB_MENU"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Change language",
                                ru: "🌐 Сменить язык"
                            },
                            actions: [ {
                                state: "CHANGE_LANGUAGE"
                            } ]
                        } ]
                    } ],
                    CHANGE_LANGUAGE: [ {
                        type: "actions",
                        actions: [ {
                            state: "HR_GREED"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Select your language:",
                            ru: "Выберите язык общения:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "'en'"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Русский",
                                ru: "Русский"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "'ru'"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    COMPANY_INFO1: [ {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/hr/hr_01.jpg"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "<b>QuData</b> - IT company in the field of machine learning and artificial intelligence.",
                            ru: "<b>QuData</b> - IT компания, в области машинного обучения и искусственного интеллекта."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "We are a team of professionals aming to change the game with cutting-edge technologies.",
                            ru: "У нас работают высококлассные специалисты, использующие самые передовые технологии."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "ℹ More info",
                                ru: "ℹ Ещё"
                            },
                            actions: [ {
                                state: "COMPANY_INFO2"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "↻ Back",
                                ru: "↻ Назад"
                            },
                            actions: [ {
                                state: "HR_GREED"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙇 I'd like to apply",
                                ru: "🙇 Хочу к вам"
                            },
                            actions: [ {
                                state: "JOB_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    COMPANY_INFO2: [ {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/hr/hr_02.jpg"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Our greatest value is our team. We appreciate professionalism and creativity.",
                            ru: "Самая большая ценность - это наш коллектив. Мы ценим профессионализм и креативность."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "ℹ More",
                                ru: "ℹ Ещё"
                            },
                            actions: [ {
                                state: "COMPANY_INFO3"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙇 I'd like to apply",
                                ru: "🙇 Хочу к вам"
                            },
                            actions: [ {
                                state: "JOB_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    COMPANY_INFO3: [ {
                        type: "text",
                        text: {
                            en: "We have Python, C ++ and JavaScript developers, machine learning specialists, linguists and marketers.",
                            ru: "У нас работают Python, C++, JavaScript программисты, специалисты по машинному обучению, лингвисты и маркетологи."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "So, are you ready for an interview?",
                            ru: "Ну что, готовы пройти собеседование прямо сейчас?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "👌 Yes!",
                                ru: "👌 Да, конечно!"
                            },
                            actions: [ {
                                state: "JOB_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🙊 No",
                                ru: "🙊 Нет"
                            },
                            actions: [ {
                                state: "HR_GREED"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    JOB_MENU: [ {
                        type: "text",
                        text: {
                            en: "Select a vacancy",
                            ru: "Выберите вакансию, которая Вас интересует"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "ML Engineer",
                                ru: "специалист по ML"
                            },
                            actions: [ {
                                state: "JOB_MENU_ML"
                            }, {
                                slot: "TEST",
                                value: "1"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Sales Manager",
                                ru: "менеджер по продажам"
                            },
                            actions: [ {
                                state: "JOB_MENU_SALES"
                            }, {
                                slot: "TEST",
                                value: "2"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "HR_GREED"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    JOB_MENU_ML: [ {
                        type: "text",
                        text: {
                            en: "Skills and expertise:\n\n➣ Knowledge of tensor calculus\n➣ Knowledge of probability theory\n➣ Strong knowledge of Python\n➣ Profound knowledge of neural networks\n➣ Working with TensorFlow and PyTorch\n➣ Numpy, scikit-learn, pandas, scipy and matplotlib libraries",
                            ru: "Основные требования:\n\n➣ Знание тензорного анализа\n➣ Знание теории вероятности\n➣ Уверенное владение Python\n➣ Глубокие знания о нейронных сетях\n➣ Работа  с TensorFlow и PyTorch\n➣ Библиотеки numpy, scikit-learn, pandas, scipy, matplotlib"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Interview",
                                ru: "Собеседование"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            }, {
                                if: "$CUSTOMER_NAME and ($CUSTOMER_PHONE or $CUSTOMER_EMAIL)",
                                actions: [ {
                                    state: "EXPIRIANCE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Vacancies",
                                ru: "Вакансии"
                            },
                            actions: [ {
                                state: "JOB_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    JOB_MENU_SALES: [ {
                        type: "text",
                        text: {
                            en: "Job requirements:\n\n➣ Communication skills\n➣ Purposefulness\n➣ Ability to listen\n➣ Knowledge of the sales theory",
                            ru: "Основные требования:\n\n➣ Коммуникабельность\n➣ Целеустремленность\n➣ Умение слушать\n➣ Знание теории продаж"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Interview",
                                ru: "Собеседование"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            }, {
                                if: "$CUSTOMER_NAME and ($CUSTOMER_PHONE or $CUSTOMER_EMAIL)",
                                actions: [ {
                                    state: "EXPIRIANCE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Vacansies",
                                ru: "Вакансии"
                            },
                            actions: [ {
                                state: "JOB_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    EXPIRIANCE: [ {
                        type: "actions",
                        actions: [ {
                            state: "START_TEST"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Your work experience:",
                            ru: "Ваш опыт работы:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "3+ years",
                                ru: "более 3-х лет"
                            },
                            actions: [ {
                                slot: "EXPERIENCE",
                                value: "3"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "1-3 years",
                                ru: "1-3 года"
                            },
                            actions: [ {
                                slot: "EXPERIENCE",
                                value: "1"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "less than a year",
                                ru: "менее года"
                            },
                            actions: [ {
                                slot: "EXPERIENCE",
                                value: "0"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    START_TEST: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, let's take a test. You have half a minute per question. Ready?",
                            ru: "$CUSTOMER_NAME, приступим к тесту. На каждый вопрос у Вас будет пол минуты. Готовы?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⏳ Yes!",
                                ru: "⏳ Да!"
                            },
                            actions: [ {
                                state: "TEST_ML1"
                            }, {
                                if: "$TEST == 2",
                                actions: [ {
                                    state: "TEST_SALES1"
                                } ]
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "HR_GREED"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    TEST_ML1: [ {
                        type: "actions",
                        actions: [ {
                            state: "TEST_ML2"
                        } ]
                    }, {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_ML2"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[1/4] Overfitted model may occur if the model...",
                            ru: "[1/4] Переобучение модели может возникать, когда:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "is very good",
                                ru: "Модель очень хорошая"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "has too many parameters",
                                ru: "В модели много параметров"
                            },
                            actions: [ {
                                slot: "TEST_ML",
                                value: "$TEST_ML + 1000"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "takes too long to train",
                                ru: "Модель очень долго учится"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    TEST_ML2: [ {
                        type: "actions",
                        actions: [ {
                            state: "TEST_ML3"
                        } ]
                    }, {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_ML3"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[2/4] The output of a convolution of a (2,3) and a (3,4) matrix:",
                            ru: "[2/4] Матричная свёртка двух матриц с формами (2,3) и (3,4) даёт матрицу с формой:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "(3,4)",
                                ru: "(3,4)"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "(2,3,4)",
                                ru: "(2,3,4)"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "(2,4)",
                                ru: "(2,4)"
                            },
                            actions: [ {
                                slot: "TEST_ML",
                                value: "$TEST_ML + 100"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    TEST_ML3: [ {
                        type: "actions",
                        actions: [ {
                            state: "TEST_ML4"
                        } ]
                    }, {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_ML4"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[3/4] Recurrent neural networks are used to recognize patterns in:",
                            ru: "[3/4] Рекуррентные нейронные сети используются для поиска закономерностей в:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "ordinal data",
                                ru: "упорядоченных данных"
                            },
                            actions: [ {
                                slot: "TEST_ML",
                                value: "$TEST_ML + 10"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "images",
                                ru: "изображениях"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "a regression problem",
                                ru: "в задаче регрессии"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    TEST_ML4: [ {
                        type: "actions",
                        actions: [ {
                            state: "TEST_ML_END"
                        } ]
                    }, {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_ML_END"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[4/4] The Transformer architecture is most commonly used instead of:",
                            ru: "[4/4] Архитектура Transformer наиболее типично используется вместо:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "recurrent networks",
                                ru: "рекуррентных сетей"
                            },
                            actions: [ {
                                slot: "TEST_ML",
                                value: "$TEST_ML + 1"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "convolutional networks",
                                ru: "свёрточных сетей"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "fully connected networks",
                                ru: "полносвязных сетей"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    TEST_ML_END: [ {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "ml",
                            value: '{\nname:  $CUSTOMER_NAME,\nphone: $CUSTOMER_PHONE,\nemail: $CUSTOMER_EMAIL,\njob:   "ML",\ntest:  $TEST_ML,\nexperience: $EXPERIENCE,\n}'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Well done $CUSTOMER_NAME!<br>Your ML test binary is $TEST_ML.\nOur biological HR will contact you shortly.",
                            ru: "Молодец $CUSTOMER_NAME!<br>Ваш бинарный код ML-теста равен $TEST_ML.\nНаш биологический HR скоро свяжется с Вами."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "HR_GREED"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    TEST_SALES1: [ {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_SALES2"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[1/4] A sales manager should respond to objection starting with:\n\n     • 1. You get it wrong, and let me explain why\n     • 2. I agree with you\n     • 3. I see your point\n\nEnter the answer number:            \n",
                            ru: "[1/4] Работу с возражением менеджер по продажам должен начинать так:\n\n    • 1. Вы несколько неправы, и я сейчас расскажу вам, в чем именно\n    • 2. Согласен с Вами\n    • 3. Я понимаю Вашу позицию\n\nВведите номер ответа:\n"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "TEST_SALES2"
                        }, {
                            if: "$INPUT == 3",
                            actions: [ {
                                slot: "TEST_SALES",
                                value: "$TEST_SALES + 1000"
                            } ]
                        } ]
                    } ],
                    TEST_SALES2: [ {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_SALES3"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[2/4] The goal of the first phone call to a potential client is:\n\n    • 1. To sale\n    • 2. To leave a good impression of you and your company\n    • 3. Both of the above\n\nEnter the answer number:\n",
                            ru: "[2/4] Цель первого телефонного разговора с потенциальным клиентом:\n\n    • 1. Продажа\n    • 2. Создание благоприятного впечатления о себе и своей компании\n    • 3. Все перечисленные    \n            \nВведите номер ответа:\n"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "TEST_SALES3"
                        }, {
                            if: "$INPUT == 3",
                            actions: [ {
                                slot: "TEST_SALES",
                                value: "$TEST_SALES + 100"
                            } ]
                        } ]
                    } ],
                    TEST_SALES3: [ {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_SALES4"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "[3/4] You called, and a secretary answered the phone. What does it mean for you?\n\n    • 1. Secretaries only interfere with your purpose, so there is no point talking to them.\n    • 2. You may collect more information about a potential client.\n    • 3. You can make an acquaintance with a girl.\n\nEnter the answer number:",
                            ru: "[3/4] Вы позвонили в компанию, и трубку взял секретарь. Что это означает для Вас?\n\n    • 1. На секретаря лучше не попадать: они только мешают.\n    • 2. Можно собрать дополнительную информацию о потенциальном клиенте.\n    • 3. Можно познакомиться с девушкой.\n\nВведите номер ответа:                            \n"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "TEST_SALES4"
                        }, {
                            if: "$INPUT == 2",
                            actions: [ {
                                slot: "TEST_SALES",
                                value: "$TEST_SALES + 10"
                            } ]
                        } ]
                    } ],
                    TEST_SALES4: [ {
                        type: "text",
                        text: {
                            en: "[4/4] What information means most in the market analysis?\n\n    • 1. Customers, your product, competitors.\n    • 2. Market segmentation, SWOT-analysis, ABC analysis.\n    • 3. Oil prices, political situation, seasonality.\n\nEnter the answer number:",
                            ru: "[4/4] Что наиболее важно менеджеру по продажам при анализе ситуации на рынке?\n\n    • 1. Клиенты, свой продукт, конкуренты.\n    • 2. Сегментация рынка, SWOT-анализ, ABC анализ.\n    • 3. Цены на нефть, политическая ситуация, сезонность.\n\nВведите номер ответа:                            "
                        }
                    }, {
                        type: "timer",
                        time: "30000",
                        actions: [ {
                            state: "TEST_SALES_END"
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "TEST_SALES_END"
                        }, {
                            if: "$INPUT == 1",
                            actions: [ {
                                slot: "TEST_SALES",
                                value: "$TEST_SALES + 1"
                            } ]
                        } ]
                    } ],
                    TEST_SALES_END: [ {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "sales",
                            value: '{\nname:  $CUSTOMER_NAME,\nphone: $CUSTOMER_PHONE,\nemail: $CUSTOMER_EMAIL,\njob:   "SALES",\ntest:  $TEST_SALES,\nexperience: $EXPERIENCE,\n}'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Well done $CUSTOMER_NAME !\n\nYour ML test binary is <b>$TEST_SALES</b>.\nOur biological HR will contact you shortly.\n",
                            ru: "Молодец $CUSTOMER_NAME!\n \nВаш бинарный код теста равен <b>$TEST_SALES</b>.\nНаш биологический HR скоро свяжется с Вами.\n"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "HR_GREED"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    CUSTOMER_WRONG_NAME: [ {
                        type: "text",
                        text: {
                            en: "Very strange name $CUSTOMER_NAME. Could you introduce yourself differently:",
                            ru: "Очень странное имя $CUSTOMER_NAME. Не могли бы Ва представиться по другому:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_GREET"
                        }, {
                            slot: "CUSTOMER_NAME",
                            value: "$INPUT"
                        }, {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_GREET: [ {
                        type: "text",
                        text: {
                            en: "Nice to meet you $CUSTOMER_NAME!\nWe may want to contact you. Is it more convenient for you to communicate by phone or email?",
                            ru: "Рад знакомству $CUSTOMER_NAME!\nВозможно, мы захотим связаться с Вами. Вам удобнее общаться по телефону или по электронной почте?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Phone",
                                ru: "📞 Телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "💻 Email",
                                ru: "💻 Почта"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_WELLCOME: [ {
                        type: "text",
                        text: {
                            en: "My name is Qu. Please introduce yourself as well  🙏: ",
                            ru: "Меня зовут Кью. Представьтесь пожалуйста и Вы 🙏:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_GREET"
                        }, {
                            slot: "CUSTOMER_NAME",
                            value: "$INPUT"
                        }, {
                            if: "$CUSTOMER_PHONE or $CUSTOMER_EMAIL",
                            actions: [ {
                                state: "BEFORE_CHECK"
                            } ]
                        }, {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "Our mail: info@qudata.com, and yours, $CUSTOMER_NAME?",
                            ru: "Наша почта: info@qudata.com, а Ваша, $CUSTOMER_NAME?"
                        }
                    }, {
                        type: "input",
                        kind: "email",
                        actions: [ {
                            slot: "CUSTOMER_EMAIL",
                            value: "$INPUT"
                        }, {
                            state: "BEFORE_CHECK"
                        }, {
                            if: "not $CUSTOMER_PHONE",
                            actions: [ {
                                state: "CUSTOMER_PHONE_MAYBE"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL_WRONG: [ {
                        type: "text",
                        text: {
                            en: "We will not be able to write to you at this address. Will you try again?",
                            ru: "Мы не сможем написать Вам по такому адресу. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, please enter your phone number:",
                            ru: "$CUSTOMER_NAME, введите пожалуйста Ваш телефон:"
                        }
                    }, {
                        type: "input",
                        kind: "phone",
                        actions: [ {
                            slot: "CUSTOMER_PHONE",
                            value: "$INPUT"
                        }, {
                            slot: "CUSTOMER_PHONE",
                            value: "$INPUT",
                            state: "BEFORE_CHECK"
                        }, {
                            if: "not $CUSTOMER_EMAIL",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_MAYBE"
                            } ]
                        }, {
                            if: "not $CUSTOMER_PHONE",
                            actions: [ {
                                state: "CUSTOMER_PHONE_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_PHONE_WRONG: [ {
                        type: "text",
                        text: {
                            en: "We can't call that phone. Will you try again?",
                            ru: "Мы не сможем позвонить по такому телефону. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE_MAYBE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, maybe you will leave your phone? We can send messages in the messenger.",
                            ru: "$CUSTOMER_NAME, может быть Вы оставите Ваш телефон? Мы можем присылать сообщения в месенджере."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_EMAIL_MAYBE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, would you mind leaving your email? Sometimes we will send in very useful information.",
                            ru: "$CUSTOMER_NAME, Вы не против оставить Вашу электронную почту? Иногда мы будем присылать очень полезную информацию."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHECK: [ {
                        type: "text",
                        text: {
                            en: "Is that correct?",
                            ru: "Всё верно?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "EXPIRIANCE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🤖 Change name",
                                ru: "🤖 Изменить имя"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "💻Change mail",
                                ru: "💻Изменить почту"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Change phone",
                                ru: "📞 Изменить телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BEFORE_CHECK: [ {
                        type: "actions",
                        actions: [ {
                            if: "$CUSTOMER_PHONE and $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "PHONE_AND_EMAIL"
                            } ]
                        }, {
                            if: "$CUSTOMER_PHONE",
                            actions: [ {
                                goto: "ONLY_PHONE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "ONLY_EMAIL"
                            } ]
                        }, {
                            if: "not $CUSTOMER_PHONE and  not $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "NOTHING"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PHONE_AND_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by mail <b> $CUSTOMER_EMAIL </b> and phone <b> $CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b> и телефону <b>$CUSTOMER_PHONE</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_PHONE: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact phone <b>$CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по телефону <b>$CUSTOMER_PHONE</b>. "
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by by mail <b> $CUSTOMER_EMAIL </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NOTHING: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, you have left no phone or mail.",
                            ru: "$CUSTOMER_NAME, Вы не оставили, ни телефона, ни почты."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    NAME: "",
                    TEST_ML: "",
                    TEST_SALES: "",
                    TEST: "",
                    PHONE: "",
                    EMAIL: "",
                    EXPERIENCE: "",
                    CUSTOMER_NAME: "",
                    CUSTOMER_EMAIL: "",
                    CUSTOMER_PHONE: "",
                    CUSTOMER_MESSAGE: ""
                },
                cards: {
                    states: {
                        HR_GREED: {
                            c: 0,
                            r: 0,
                            x: 751,
                            y: 7
                        },
                        CHANGE_LANGUAGE: {
                            c: 1,
                            r: 0,
                            x: 1001,
                            y: 4
                        },
                        COMPANY_INFO1: {
                            c: 2,
                            r: 0,
                            x: 474,
                            y: 5
                        },
                        COMPANY_INFO2: {
                            c: 3,
                            r: 0,
                            x: 246,
                            y: 0
                        },
                        COMPANY_INFO3: {
                            c: 4,
                            r: 0,
                            x: 0,
                            y: 1
                        },
                        JOB_MENU: {
                            c: 0,
                            r: 1,
                            x: 408,
                            y: 312
                        },
                        JOB_MENU_ML: {
                            c: 1,
                            r: 1,
                            x: 101,
                            y: 297
                        },
                        JOB_MENU_SALES: {
                            c: 2,
                            r: 1,
                            x: 787,
                            y: 350
                        },
                        EXPIRIANCE: {
                            c: 0,
                            r: 8,
                            x: 692,
                            y: 646
                        },
                        START_TEST: {
                            c: 1,
                            r: 8,
                            x: 300,
                            y: 454
                        },
                        TEST_ML1: {
                            c: 1,
                            r: 5,
                            x: 311,
                            y: 951,
                            folder: "TEST_ML"
                        },
                        TEST_ML2: {
                            c: 2,
                            r: 5,
                            x: 545,
                            y: 946,
                            folder: "TEST_ML"
                        },
                        TEST_ML3: {
                            c: 3,
                            r: 5,
                            x: 787,
                            y: 942,
                            folder: "TEST_ML"
                        },
                        TEST_ML4: {
                            c: 4,
                            r: 5,
                            x: 1026,
                            y: 935,
                            folder: "TEST_ML"
                        },
                        TEST_ML_END: {
                            c: 5,
                            r: 5,
                            x: 1262,
                            y: 966,
                            folder: "TEST_ML"
                        },
                        TEST_SALES1: {
                            c: 1,
                            r: 5,
                            x: 313,
                            y: 1099,
                            folder: "TEST_SALES"
                        },
                        TEST_SALES2: {
                            c: 2,
                            r: 5,
                            x: 548,
                            y: 1095,
                            folder: "TEST_SALES"
                        },
                        TEST_SALES3: {
                            c: 3,
                            r: 5,
                            x: 796,
                            y: 1083,
                            folder: "TEST_SALES"
                        },
                        TEST_SALES4: {
                            c: 4,
                            r: 5,
                            x: 1032,
                            y: 1076,
                            folder: "TEST_SALES"
                        },
                        TEST_SALES_END: {
                            c: 5,
                            r: 5,
                            x: 1267,
                            y: 1094,
                            folder: "TEST_SALES"
                        },
                        CUSTOMER_WRONG_NAME: {
                            c: 2,
                            r: 3,
                            x: 463,
                            y: 16,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_GREET: {
                            c: 1,
                            r: 3,
                            x: 253,
                            y: 111,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_WELLCOME: {
                            c: 0,
                            r: 3,
                            x: 22,
                            y: 12,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_EMAIL: {
                            c: 0,
                            r: 5,
                            x: 538,
                            y: 241,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_EMAIL_WRONG: {
                            c: 2,
                            r: 5,
                            x: 553,
                            y: 383,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_PHONE: {
                            c: 0,
                            r: 4,
                            x: 45,
                            y: 240,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_PHONE_WRONG: {
                            c: 2,
                            r: 4,
                            x: 24,
                            y: 387,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_PHONE_MAYBE: {
                            c: 1,
                            r: 5,
                            x: 267,
                            y: 385,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_EMAIL_MAYBE: {
                            c: 1,
                            r: 4,
                            x: 263,
                            y: 234,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_CHECK: {
                            c: 0,
                            r: 7,
                            x: 932,
                            y: 539,
                            folder: "CUSTOMER"
                        },
                        BEFORE_CHECK: {
                            c: 0,
                            r: 6,
                            x: 19,
                            y: 519,
                            folder: "CUSTOMER"
                        },
                        PHONE_AND_EMAIL: {
                            c: 4,
                            r: 6,
                            x: 215,
                            y: 522,
                            folder: "CUSTOMER"
                        },
                        ONLY_PHONE: {
                            c: 3,
                            r: 6,
                            x: 406,
                            y: 522,
                            folder: "CUSTOMER"
                        },
                        ONLY_EMAIL: {
                            c: 2,
                            r: 6,
                            x: 599,
                            y: 523,
                            folder: "CUSTOMER"
                        },
                        NOTHING: {
                            c: 1,
                            r: 6,
                            x: 410,
                            y: 614,
                            folder: "CUSTOMER"
                        }
                    },
                    folders: {
                        TEST_SALES: {
                            c: 0,
                            r: 10,
                            x: 100,
                            y: 654
                        },
                        TEST_ML: {
                            c: 0,
                            r: 9,
                            x: 100,
                            y: 554
                        },
                        CUSTOMER: {
                            c: 0,
                            r: 2,
                            x: 396,
                            y: 658
                        }
                    }
                }
            },
            FAQ: {
                id: 0,
                name: "FAQ",
                start: "MAIN",
                version: "1.0.0",
                view: "grid",
                states: {
                    FAQ_WHY: [ {
                        type: "text",
                        text: {
                            ru: "1. Клиенты должны видеть, что бизнес с ними находится на связи 24 часа в сутки.",
                            en: "1. The customers have to see your business is accessible 24/7."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "2. Чат боты расширяют каналы продаж в месенджерах и социальных сетях.",
                            en: "2. Bots let you expand your sales channels to messengers and social networks."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "3. Чат боты снижают издержки, беря на себя многие рутинные действия.",
                            en: "3. Chatbots reduce costs by performing routine work."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "4. Клиенты ощущают личное отношение, что повышает их доверие к компании.",
                            en: "4. The customers appreciate personal approach, which establishes credibility."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: '5. Люди любят общаться не с текстом на сайте, а с "живым" собеседником.',
                            en: "5. People prefer maintaining dialog, instead simply browsing a website for information."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Понятно",
                                en: "⇧ OK"
                            },
                            actions: [ {
                                state: "FAQ_MAIN"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "⇩ Непонятно",
                                en: "⇩ Learn more"
                            },
                            actions: [ {
                                state: "FAQ_HOW"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    FAQ_KINDS: [ {
                        type: "text",
                        text: {
                            ru: "По способу коммуникации с человеком боты бывают с кнопочной навигацией и понимающие естественный язык.",
                            en: "According to the communicational method, tbots fall into those with button navigation and those understanding natural language."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "По алгоритмам поведения боты могут быть основаны на правилах или быть самообучающимися.",
                            en: "According to behavioral algorithms, chatbots are divided into determined and self-learning."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Понятно",
                                en: "⇧ OK"
                            },
                            actions: [ {
                                state: "FAQ_MAIN"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "⇩ Непонятно",
                                en: "⇩ Learn more"
                            },
                            actions: [ {
                                state: "FAQ_HOW"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    FAQ_HOW: [ {
                        type: "text",
                        text: {
                            ru: "Нужно связаться со специалистами нашей компании.",
                            en: "Contact our team and describe what you need."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "👫 Contact us",
                                ru: "👫 Связаться с нами"
                            },
                            actions: [ {
                                state: "CUSTOMER_START"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Назад",
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "FAQ_MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    FAQ_COST_NLU: [ {
                        type: "actions",
                        actions: [ {
                            state: "FAQ_COST_LOGIC"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            ru: "Сколько примерно намерений человека должен понимать чат-бот?",
                            en: "Roughly, how many intents should a chatbot understand?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "10",
                                en: "10"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE + 5000"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "50",
                                en: "50"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE + 20000"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "100..",
                                en: "100"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE + 50000"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "🦉 Что такое намерение?",
                                en: "🦉 What is intent?"
                            },
                            actions: [ {
                                state: "FAQ_COST_INTENT"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    FAQ_COST_INTENT: [ {
                        type: "text",
                        text: {
                            ru: "<b>Намерение</b> (intent) - это один из классов, к которому необходимо отнести введенный человеком текст.",
                            en: "The <b>intent</b> is a class used to attribute user texts."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "Например, намерение <b>GREET</b> должно срабатывать, если человек пишет '<i>преветик</i>', а намерение <b>WANT_CHEAPER</b> после ввода '<i>мне бы что-то подешевше...</i>'",
                            en: "For example, <b>GREET</b> is the intent to be fired when a user types '<i>Hello</i>', while <b>WANT_CHEAPER</b> should be triggered after '<i>I'd need something less expensive..."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "Для каждого намерения необходимо подготовить примеры, обучить нейронную сеть и в дальнейшем переобучать её на реальных диалогах.",
                            en: "Sample texts provided for each intent allows neural network to learn, and further train using real dialogs."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Понятно",
                                en: "⇧ OK"
                            },
                            actions: [ {
                                state: "FAQ_COST_NLU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "⇩ Непонятно",
                                en: "⇩ Learn more"
                            },
                            actions: [ {
                                state: "FAQ_HOW"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    FAQ_COST_LOGIC: [ {
                        type: "text",
                        text: {
                            ru: "Логика бота будет основана на правилах (детерминирована) или на алгоритмах искусственного интеллекта?",
                            en: "Should the bot logics be based on rules (determined) or AI algorithms?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "На правилах",
                                en: "Rules"
                            },
                            actions: [ {
                                state: "FAQ_COST_END"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "На алгоритмах ИИ",
                                en: "AI algorithms"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE * 10"
                            }, {
                                state: "FAQ_COST_END"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "🦉 Непонятно",
                                en: "⇩ Learn more"
                            },
                            actions: [ {
                                state: "FAQ_LOGIC"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    FAQ_COST_END: [ {
                        type: "text",
                        text: {
                            ru: "Ориентировочная стоимость бота будет $BOT_PRICE USD.",
                            en: "The approximate chatbot cost is $BOT_PRICE USD."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Понятно",
                                en: "⇧ OK"
                            },
                            actions: [ {
                                state: "FAQ_MAIN"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "⇩ Непонятно",
                                en: "⇩ Learn more"
                            },
                            actions: [ {
                                state: "FAQ_HOW"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    FAQ_COST: [ {
                        type: "actions",
                        actions: [ {
                            slot: "BOT_PRICE",
                            value: "0"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            ru: "Не простой вопрос 🙊...",
                            en: "That depends 🙊..."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "Прежде всего, давайте попробуем прояснить несколько моментов.",
                            en: "First let's clear up a couple of points."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "Как человек будет взаимодействовать с ботом?",
                            en: "How is a user supposed to communicate with the bot?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "При помощи кнопок",
                                en: "Using buttons"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE + 1000"
                            }, {
                                state: "FAQ_COST_LOGIC"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Вводя текст",
                                en: "Typing answers"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE + 2000"
                            }, {
                                state: "FAQ_COST_NLU"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Оба способа",
                                en: "Both"
                            },
                            actions: [ {
                                slot: "BOT_PRICE",
                                value: "$BOT_PRICE + 3000"
                            }, {
                                state: "FAQ_COST_NLU"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "🦉Непонятно ",
                                en: "⇩ Learn more"
                            },
                            actions: [ {
                                state: "FAQ_INPUT"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Назад",
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "FAQ_MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    FAQ_MAIN: [ {
                        type: "text",
                        text: {
                            ru: "Что Вас интересует:",
                            en: "Select a question:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Для чего нужен чат-бот?",
                                en: "What are the advantages of a chatbot?"
                            },
                            actions: [ {
                                state: "FAQ_WHY"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Какие бывают чат-боты?",
                                en: "What types of bots exist?"
                            },
                            actions: [ {
                                state: "FAQ_KINDS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Сколько стоит чат-бот?",
                                en: "How much does it cost to create a bot?"
                            },
                            actions: [ {
                                state: "FAQ_COST"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Как сделать чат-бот?",
                                en: "How can I get a customized chatbot?"
                            },
                            actions: [ {
                                state: "FAQ_HOW"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "❌ Не это",
                                en: "❌ Other one"
                            },
                            actions: [ {
                                state: "CUSTOMER_START"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                ru: "🏠 В начало",
                                en: "🏠 Home"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    FAQ_LOGIC: [ {
                        type: "text",
                        text: {
                            ru: "Мне непонятно:",
                            en: "Going further:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: " Что такое правила?",
                                en: "What are rules?"
                            },
                            actions: [ {
                                state: "FAQ_LOGIC_RULES"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "Что такое алгритмы ИИ?",
                                en: "What are AI algorithms?"
                            },
                            actions: [ {
                                state: "FAQ_LOGIC_AI"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Назад",
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "FAQ_COST_LOGIC"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FAQ_LOGIC_RULES: [ {
                        type: "text",
                        text: {
                            ru: "Бот содержит жёсткий алгоритм реакции на действия пользователя. Его ответы могут варьироваться, но выводимая информация полностью контролируется разработчиком.",
                            en: "A bot follows a strict response algorithm. While the replies may vary, the provided information is fully controlled by a developer."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "➕ поведения бота контролируемо, невозможны странные или неполиткорректные ответы.",
                            en: "➕ Full control over chatbot behavior excludes irrelevant or inappropriate replies."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: '➖ не могут отклонятся от заданной программы и иногда выглядят не достаточно "интеллектуально".',
                            en: '➖ Unable to deviate from the script, bots may seem not that "intelligent".'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Назад",
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "FAQ_LOGIC"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FAQ_LOGIC_AI: [ {
                        type: "text",
                        text: {
                            ru: 'Бот использует нейронные сети и другие методы искусственного интеллекта. Это "чёрный ящик", поведение которого настраивается при помощи машинного обучения.',
                            en: "A bot uses neural networks and other AI-based methods. It's behavior is set using machine learning."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: '➕ Могут выглядеть "почти как люди". Адекватно реагировать в неожиданных ситуациях и подстраиваться под поведение пользователей.',
                            en: '➕ Bots simulate "human behavior". They can respond adequately to non-standard situations and adapt to user behavior.'
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "➖ Поведение может быть непредсказуемым и требуют длительной настройки с анализом поведения реальных пользователей.",
                            en: "➖ Bot behavior may be unpredictable and require time-consuming configuration and analysis of actual human responses."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Назад",
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "FAQ_LOGIC"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FAQ_INPUT: [ {
                        type: "text",
                        text: {
                            ru: "При кнопочной навигации диалог реализуется в результате нажатия на кнопки с текстом:",
                            en: "In case of button navigation, a user maintains the dialog by clicking buttons:"
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "➕ люди не любят писать и им проще нажимать на кнопки.",
                            en: "➕ Users rather prefer clicking an option than typing."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "➖ навигация иногда оказывается довольно длинной.",
                            en: "➖ Navigation may prove strange and time-consuming."
                        }
                    }, {
                        type: "text",
                        text: {
                            ru: "Боты, понимающие ввод текста на естестественном языке дороже, но в них удобнее навигация, т.к. можно сразу перейти к нужной теме. Кроме этого к такому боту больше доверия, как к живому собеседнику.",
                            en: "Chatbots that can process natural language are more expensive, however they have better navigation, as they can get straight to the point. They also enjoy higher credibility, as a real interlocutor."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                ru: "⇧ Назад",
                                en: "⇧ Back"
                            },
                            actions: [ {
                                state: "FAQ_COST"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_WRONG_NAME: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Very strange name $INPUT. Could you introduce yourself differently:",
                            ru: "Очень странное имя $INPUT. Не могли бы Вы представиться по другому:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            if: "len($INPUT) > 1",
                            actions: [ {
                                state: "CUSTOMER_GREET"
                            }, {
                                slot: "CUSTOMER_NAME",
                                value: "$INPUT"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Our mail: info@qudata.com, and yours, $CUSTOMER_NAME?",
                            ru: "Наша почта: info@qudata.com, а Ваша, $CUSTOMER_NAME?"
                        }
                    }, {
                        type: "input",
                        kind: "email",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                slot: "CUSTOMER_EMAIL",
                                value: "$INPUT"
                            }, {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL_WRONG: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We will not be able to write to you at this address. Will you try again?",
                            ru: "Мы не сможем написать Вам по такому адресу. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, please enter your phone number:",
                            ru: "$CUSTOMER_NAME, введите пожалуйста Ваш телефон:"
                        }
                    }, {
                        type: "input",
                        kind: "phone",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                slot: "CUSTOMER_PHONE",
                                value: "$INPUT"
                            }, {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_PHONE_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_PHONE_WRONG: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We can't call that phone. Will you try again?",
                            ru: "Мы не сможем позвонить по такому телефону. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE_MAYBE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, maybe you will leave your phone? We can send messages in the messenger.",
                            ru: "$CUSTOMER_NAME, может быть Вы оставите Ваш телефон? Мы можем присылать сообщения в месенджере."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_EMAIL_MAYBE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, would you mind leaving your email? Sometimes we will send in very useful information.",
                            ru: "$CUSTOMER_NAME, Вы не против оставить Вашу электронную почту? Иногда мы будем присылать очень полезную информацию."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_MESSAGE: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Write something about yourself or leave any of your suggestions. Your feedback is very important to us.",
                            ru: "Напишите что-нибудь о себе или оставьте любые Ваши предложения. Ваше мнение очень важно для нас."
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_END"
                        }, {
                            slot: "CUSTOMER_MESSAGE",
                            value: "$INPUT"
                        } ]
                    } ],
                    CUSTOMER_END: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "customer",
                            value: '{\n"kind":    "customer",\n"name":    $CUSTOMER_NAME,\n"phone":   $CUSTOMER_PHONE,\n"mail":    $CUSTOMER_EMAIL,\n"message": $CUSTOMER_MESSAGE\n}'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Thank you very much $CUSTOMER_NAME 😊, we will contact you shortly.",
                            ru: "Спасибо большое $CUSTOMER_NAME 😊, мы свяжемся с Вами в ближайшее время."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "MAIN"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHECK: [ {
                        type: "text",
                        text: {
                            en: "Is that correct?",
                            ru: "Всё верно?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_MESSAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🤖 Change name",
                                ru: "🤖 Изменить имя"
                            },
                            actions: [ {
                                state: "CUSTOMER_NAME"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "💻Change mail",
                                ru: "💻Изменить почту"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Change phone",
                                ru: "📞 Изменить телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BEFORE_CHECK: [ {
                        type: "actions",
                        actions: [ {
                            if: "$CUSTOMER_PHONE and $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "PHONE_AND_EMAIL"
                            } ]
                        }, {
                            if: "$CUSTOMER_PHONE",
                            actions: [ {
                                goto: "ONLY_PHONE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "ONLY_EMAIL"
                            } ]
                        }, {
                            goto: "NOTHING"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PHONE_AND_EMAIL: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by mail <b> $CUSTOMER_EMAIL </b> and phone <b> $CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b> и телефону <b>$CUSTOMER_PHONE</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_PHONE: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact phone <b>$CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по телефону <b>$CUSTOMER_PHONE</b>. "
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b>$CUSTOMER_NAME </b> and you prefer to contact by by mail <b>$CUSTOMER_EMAIL</b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NOTHING: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, you have left no phone or mail.",
                            ru: "$CUSTOMER_NAME, Вы не оставили, ни телефона, ни почты."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_NAME: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Please introduce yourself 🙏:",
                            ru: "Пожалуйста, введите своё имя 🙏:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_GREET"
                            }, {
                                slot: "CUSTOMER_NAME",
                                value: "$INPUT"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_START: [ {
                        type: "actions",
                        actions: [ {
                            if: "not $CUSTOMER_NAME",
                            actions: [ {
                                goto: "CUSTOMER_NAME"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL and not $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_CHANNEL"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL and $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_EMAIL_MAYBE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL and not $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_PHONE_MAYBE"
                            } ]
                        }, {
                            goto: "BEFORE_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHANNEL: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "We may want to contact you. Is it more convenient for you to communicate by phone or email?",
                            ru: "Возможно, мы захотим связаться с Вами. Вам удобнее общаться по телефону или по электронной почте?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Phone",
                                ru: "📞 Телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "💻 Email",
                                ru: "💻 Почта"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_GREET: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Nice to meet you $CUSTOMER_NAME!",
                            ru: "Рад знакомству $CUSTOMER_NAME!"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_START"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    MAIN: [ {
                        type: "text",
                        text: {
                            en: "Welcome to <b>QuData</b> website!",
                            ru: "Приветствуем Вас на сайте компании  <b>QuData</b>!"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "👫 Contact us",
                                ru: "👫 Связаться с нами"
                            },
                            actions: [ {
                                state: "CUSTOMER_START"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🦉 Ask a question",
                                ru: "🦉 Задать вопрос"
                            },
                            actions: [ {
                                state: "FAQ_MAIN"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Change language",
                                ru: "🌐 Сменить язык"
                            },
                            actions: [ {
                                state: "LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    LANGUAGE: [ {
                        type: "actions",
                        actions: [ {
                            state: "MAIN"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "What language do you prefer to communicate in?",
                            ru: "На каком языке Вы хотите общаться?",
                            ua: "Якою мовою Ви бажаєте спілкуватись?",
                            de: "In welcher Sprache möchten Sie kommunizieren?",
                            fr: "Dans quelle langue souhaitez-vous communiquer ?",
                            es: "¿En qué idioma quieres comunicarte?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English",
                                ua: "English",
                                de: "English",
                                fr: "English",
                                es: "English"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "en"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Русский",
                                ru: "Русский",
                                ua: "Русский",
                                de: "Русский",
                                fr: "Русский",
                                es: "Русский"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "ru"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    BOT_PRICE: "",
                    CUSTOMER_NAME: "",
                    CUSTOMER_EMAIL: "",
                    CUSTOMER_PHONE: "",
                    CUSTOMER_MESSAGE: ""
                },
                cards: {
                    states: {
                        FAQ_WHY: {
                            c: 1,
                            r: 1,
                            x: 2,
                            y: 174
                        },
                        FAQ_KINDS: {
                            c: 2,
                            r: 1,
                            x: 228,
                            y: 185
                        },
                        FAQ_HOW: {
                            c: 3,
                            r: 1,
                            x: 633,
                            y: 24
                        },
                        FAQ_COST_NLU: {
                            c: 2,
                            r: 2,
                            x: 233,
                            y: 398
                        },
                        FAQ_COST_INTENT: {
                            c: 3,
                            r: 2,
                            x: 227,
                            y: 547
                        },
                        FAQ_COST_LOGIC: {
                            c: 0,
                            r: 3,
                            x: 516,
                            y: 379
                        },
                        FAQ_COST_END: {
                            c: 0,
                            r: 4,
                            x: 2,
                            y: 420
                        },
                        FAQ_COST: {
                            c: 0,
                            r: 2,
                            x: 518,
                            y: 170
                        },
                        FAQ_MAIN: {
                            c: 0,
                            r: 1,
                            x: 263,
                            y: 0
                        },
                        FAQ_LOGIC: {
                            c: 1,
                            r: 3,
                            x: 494,
                            y: 550
                        },
                        FAQ_LOGIC_RULES: {
                            c: 2,
                            r: 3,
                            x: 773,
                            y: 490
                        },
                        FAQ_LOGIC_AI: {
                            c: 3,
                            r: 3,
                            x: 784,
                            y: 691
                        },
                        FAQ_INPUT: {
                            c: 1,
                            r: 2,
                            x: 782,
                            y: 270
                        },
                        CUSTOMER_WRONG_NAME: {
                            c: 2,
                            r: 6,
                            x: 640,
                            y: 1242,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL: {
                            c: 1,
                            r: 7,
                            x: 715,
                            y: 1467,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL_WRONG: {
                            c: 2,
                            r: 7,
                            x: 730,
                            y: 1609,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE: {
                            c: 1,
                            r: 8,
                            x: 222,
                            y: 1466,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE_WRONG: {
                            c: 2,
                            r: 8,
                            x: 201,
                            y: 1613,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE_MAYBE: {
                            c: 3,
                            r: 7,
                            x: 444,
                            y: 1611,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL_MAYBE: {
                            c: 3,
                            r: 8,
                            x: 440,
                            y: 1460,
                            folder: "Customer"
                        },
                        CUSTOMER_MESSAGE: {
                            c: 2,
                            r: 11,
                            x: 312,
                            y: 978,
                            folder: "Customer"
                        },
                        CUSTOMER_END: {
                            c: 3,
                            r: 11,
                            x: 512,
                            y: 978,
                            folder: "Customer"
                        },
                        CUSTOMER_CHECK: {
                            c: 1,
                            r: 11,
                            x: 312,
                            y: 978,
                            folder: "Customer"
                        },
                        BEFORE_CHECK: {
                            c: 1,
                            r: 9,
                            x: 196,
                            y: 1745,
                            folder: "Customer"
                        },
                        PHONE_AND_EMAIL: {
                            c: 1,
                            r: 10,
                            x: 392,
                            y: 1748,
                            folder: "Customer"
                        },
                        ONLY_PHONE: {
                            c: 2,
                            r: 10,
                            x: 583,
                            y: 1748,
                            folder: "Customer"
                        },
                        ONLY_EMAIL: {
                            c: 3,
                            r: 10,
                            x: 776,
                            y: 1749,
                            folder: "Customer"
                        },
                        NOTHING: {
                            c: 4,
                            r: 10,
                            x: 587,
                            y: 1840,
                            folder: "Customer"
                        },
                        CUSTOMER_NAME: {
                            c: 1,
                            r: 6,
                            x: 12,
                            y: 1078,
                            folder: "Customer"
                        },
                        CUSTOMER_START: {
                            c: 2,
                            r: 5,
                            x: 612,
                            y: 1078,
                            folder: "Customer"
                        },
                        CUSTOMER_CHANNEL: {
                            c: 4,
                            r: 6,
                            x: 430,
                            y: 1337,
                            folder: "Customer"
                        },
                        CUSTOMER_GREET: {
                            c: 3,
                            r: 6,
                            x: 12,
                            y: 1078,
                            folder: "Customer"
                        },
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 363,
                            y: 100
                        },
                        LANGUAGE: {
                            c: 1,
                            r: 0,
                            x: 463,
                            y: 200
                        }
                    },
                    folders: {
                        Customer: {
                            c: 0,
                            r: 5,
                            x: 4,
                            y: 798,
                            openGrid: !1
                        }
                    }
                }
            },
            Scheduling: {
                id: 0,
                name: "Scheduling",
                start: "MAIN",
                version: "1.0.0",
                states: {
                    MAIN: [ {
                        type: "text",
                        text: {
                            en: "My name is QuBot, and I will help you to schedule a meeting with Mrs. Jane Smith. You will just need to give me some necessary information, and I will automatically add the planned meeting to Mrs. Smith's calendar. Are you ready to start?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Yes!"
                            },
                            actions: [ {
                                state: "SELECT_MEET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    ENTER_TIME2: [ {
                        type: "text",
                        text: {
                            en: "At what time would you like to meet Mrs. Smith? Please use 12-hour clock format and Eastern Time (UTC - 05:00/ - 04:00)"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_TIME",
                            value: '"CONFIRM2"'
                        }, {
                            slot: "SS_FROM_ENTER_TIME",
                            value: '"ENTER_TIME2"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_TIME"
                        }, {
                            slot: "time",
                            value: "$INPUT"
                        } ],
                        kind: "time"
                    } ],
                    ENTER_DATE2: [ {
                        type: "text",
                        text: {
                            en: "Can you please write the date when you would like to meet Mrs. Smith. Use this format: mm/dd/yyyy"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_DATE",
                            value: '"ENTER_TIME2"'
                        }, {
                            slot: "SS_FROM_ENTER_DATE",
                            value: '"ENTER_DATE2"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            slot: "date",
                            value: "$INPUT",
                            state: "CHECK_DATE"
                        } ],
                        kind: "DDMMYYYY"
                    } ],
                    ENTER_PHONE2: [ {
                        type: "text",
                        text: {
                            en: "Can you please leave me your phone number, so Mrs. Smith can contact you before the meeting . Please use this format: +1234567890"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_PHONE",
                            value: '"ENTER_DATE2"'
                        }, {
                            slot: "SS_FROM_ENTER_PHONE",
                            value: '"ENTER_PHONE2"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_PHONE"
                        }, {
                            slot: "phone",
                            value: "$INPUT"
                        } ],
                        kind: "phone"
                    } ],
                    SELECT_MEET: [ {
                        type: "text",
                        text: {
                            en: "Would you like to meet Mrs. Smith in person or online?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "In Person"
                            },
                            actions: [ {
                                state: "SELECT_PLACE1"
                            }, {
                                slot: "meetType",
                                value: '"In Person"'
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Online"
                            },
                            actions: [ {
                                state: "SELECT_MESSENGER"
                            }, {
                                slot: "meetType",
                                value: '"Online"'
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    CONFIRM2: [ {
                        type: "text",
                        text: {
                            en: 'Ok, so Mrs. Smith will meet you at $address on <b>$date</b> at <b>$time</b>, and your phone number is <b>$phone</b>?\nIf this is correct click the "Yes" button, and I will add the information to Mrs. Smith\'s calendar.'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Yes"
                            },
                            actions: [ {
                                state: "FINISH"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Reschedule"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    CONFIRM1: [ {
                        type: "text",
                        text: {
                            en: 'Ok, so Mrs. Smith will meet you at her office on <b>$date</b> at <b>$time</b> , and your phone number is <b>$phone</b>?\nIf this is correct click the "<b>Yes</b>" button, and I will add the information to Mrs. Smith\'s calendar.'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Yes"
                            },
                            actions: [ {
                                state: "FINISH"
                            }, {
                                action: "save",
                                storage: "wp",
                                key: "",
                                value: ""
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Reschedule"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    SELECT_MESSENGER: [ {
                        type: "actions",
                        actions: [ {
                            clear: "-1"
                        }, {
                            slot: "messengers",
                            value: '[\n\t{name: "Telegram"},\n\t{name: "Skype"},\n\t{name: "Viber"},\n\t{name: "WhatsApp"},\n\t{name: "Facebook"},\n]'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "On which communication platform would you like to talk to Mrs. Smith? Choose one of the buttons below."
                        },
                        center: 0
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Telegram"
                            },
                            actions: [],
                            kind: "check",
                            checked: !1
                        }, {
                            type: "button",
                            button: {
                                en: "Skype"
                            },
                            actions: [ {
                                slot: "messanger",
                                value: '"Skype"'
                            } ],
                            kind: "check",
                            checked: !1
                        }, {
                            type: "button",
                            button: {
                                en: "Viber"
                            },
                            actions: [ {
                                slot: "messanger",
                                value: '"Viber"'
                            } ],
                            kind: "check",
                            checked: !0
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "WhatsApp"
                            },
                            actions: [ {
                                slot: "messanger",
                                value: '"WhatsApp"'
                            } ],
                            kind: "check",
                            checked: !0
                        }, {
                            type: "button",
                            button: {
                                en: "Facebook"
                            },
                            actions: [ {
                                slot: "messanger",
                                value: '"Facebook Messenger"'
                            } ],
                            kind: "check",
                            checked: !0
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Accept"
                            },
                            actions: [ {
                                state: "ENTER_CONTACT3"
                            }, {
                                script: "get_check(CHECKS)"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    ENTER_DATE3: [ {
                        type: "text",
                        text: {
                            en: "On what date would you like to meet Mrs. Smith ?"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_DATE",
                            value: '"ENTER_TIME3"'
                        }, {
                            slot: "SS_FROM_ENTER_DATE",
                            value: '"ENTER_DATE3"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_DATE"
                        }, {
                            slot: "date",
                            value: "$INPUT"
                        } ],
                        kind: "DDMMYYYY"
                    } ],
                    ENTER_TIME3: [ {
                        type: "text",
                        text: {
                            en: "At what time would you like to meet Mrs. Smith? Please use 12-hour clock format and Easter Time (UTC - 05:00/ - 04:00)."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_TIME",
                            value: '"CONFIRM3"'
                        }, {
                            slot: "SS_FROM_ENTER_TIME",
                            value: '"ENTER_TIME3"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_TIME"
                        }, {
                            slot: "time",
                            value: "$INPUT"
                        } ],
                        kind: "time"
                    } ],
                    CONFIRM3: [ {
                        type: "text",
                        text: {
                            en: 'Ok, so Mrs. Smith will meet you on $rezStr on <b>$date</b> at <b>$time</b>, and your contact info is <b>$contactinfo</b>? \nIf this is correct click the "<b>Yes</b>" button, and I will add the information to Mrs. Smith\'s calendar.'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Yes"
                            },
                            actions: [ {
                                state: "FINISH"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Reschedule"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    FINISH: [ {
                        type: "text",
                        text: {
                            en: "Well done, your meeting with Mrs. Smith is scheduled. She is looking forward to seeing you. Thank you and goodbye."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "",
                            value: '{\n"Meet Type": \t\t\t$meetType,\n"Place Type": \t\t$placeType,\n"Phone": \t\t\t\t\t$phone,\n"Date": \t\t\t\t\t$date,\n"Time":\t\t\t\t\t\t$time,\n"Address":\t\t\t\t$address,\n"Messanger":\t\t\t$messanger,\n"Contact Info":\t\t$contactinfo\n}'
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Start over"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    ENTER_CONTACT3: [ {
                        type: "actions",
                        actions: [ {
                            script: 'set(i, 0)\nset(firstEl, true)\nset(rezStr, "")\n\nforeach(CHECKS, \n  if($CHECKS[$i] == 1,\n    if($firstEl,\n      set(rezStr, $rezStr + $messengers[$i].name)\n      set(firstEl, false),\n      set(rezStr, $rezStr + " or " + $messengers[$i].name)\n    )\n  )\n  set(i, $i+1)\n)'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Give me your some contact info by messenger(s) $rezStr, so Mrs. Smith can contact you before the meeting."
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "ENTER_DATE3"
                        }, {
                            slot: "contactinfo",
                            value: "$INPUT"
                        } ],
                        kind: "text"
                    } ],
                    SELECT_PLACE1: [ {
                        type: "text",
                        text: {
                            en: "Would you like to meet Mrs. Smith at her office, or at some other place?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Office"
                            },
                            actions: [ {
                                state: "ADDRESS1"
                            }, {
                                slot: "placeType",
                                value: '"At her office"'
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Other place"
                            },
                            actions: [ {
                                state: "ENTER_ADDRESS2"
                            }, {
                                slot: "placeType",
                                value: '"At other place"'
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    ADDRESS1: [ {
                        type: "text",
                        text: {
                            en: "Mrs. Smith's office is locted at the 2nd floor of our company building at 43 6th Avenue, NYC, NY, USA. Take some time and copy or write down this address."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Got it!"
                            },
                            actions: [ {
                                state: "ENTER_PHONE1"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    ENTER_PHONE1: [ {
                        type: "text",
                        text: {
                            en: "Can you please leave me your phone number, so Mrs. Smith can contact you before the meeting . Please use this format: +1234567890"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_PHONE",
                            value: '"ENTER_DATE1"'
                        }, {
                            slot: "SS_FROM_ENTER_PHONE",
                            value: '"ENTER_PHONE1"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_PHONE",
                            true: "ENTER_PHONE2"
                        }, {
                            slot: "phone",
                            value: "$INPUT"
                        } ],
                        kind: "phone"
                    } ],
                    ENTER_DATE1: [ {
                        type: "text",
                        text: {
                            en: "Can you please write the date when you would like to meet Mrs. Smith. Use this format: mm/dd/yyyy"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_DATE",
                            value: '"ENTER_TIME1"'
                        }, {
                            slot: "SS_FROM_ENTER_DATE",
                            value: '"ENTER_DATE1"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            slot: "date",
                            value: "$INPUT"
                        }, {
                            state: "CHECK_DATE"
                        } ],
                        kind: "DDMMYYYY"
                    } ],
                    ENTER_TIME1: [ {
                        type: "text",
                        text: {
                            en: "At what time would you like to meet Mrs. Smith? Please use 12-hour clock format and Eastern Time (UTC - 05:00/ - 04:00)"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            slot: "SS_AFTER_ENTER_TIME",
                            value: '"CONFIRM1"'
                        }, {
                            slot: "SS_FROM_ENTER_TIME",
                            value: '"ENTER_TIME1"'
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_TIME"
                        }, {
                            slot: "time",
                            value: "$INPUT"
                        } ],
                        kind: "time"
                    } ],
                    ENTER_ADDRESS2: [ {
                        type: "text",
                        text: {
                            en: "Can you please give me the exact address where you would like to meet Mrs. Smith?"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_ADDRESS2"
                        }, {
                            slot: "address",
                            value: "$INPUT"
                        } ],
                        kind: "text"
                    } ],
                    CHECK_ADDRESS2: [ {
                        type: "text",
                        text: {
                            en: "So, you would like to meet Mrs. Smith at $address, is that right?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Confirm"
                            },
                            actions: [ {
                                state: "ENTER_PHONE2"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "RE-ENTER"
                            },
                            actions: [ {
                                state: "ENTER_ADDRESS2"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            slot: "address",
                            value: "$INPUT"
                        } ],
                        kind: "text"
                    } ],
                    ENTER_PHONE_AGAIN: [ {
                        type: "text",
                        text: {
                            en: "RE-ENTER the phone number"
                        }
                    }, {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "$SS_FROM_ENTER_PHONE",
                            true: "ENTER_PHONE_AGAIN"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CHECK_PHONE: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "actions",
                        actions: [ {
                            if: "not $phone",
                            actions: [ {
                                goto: "ENTER_PHONE_AGAIN",
                                true: "ENTER_PHONE_AGAIN"
                            } ]
                        }, {
                            goto: "$SS_AFTER_ENTER_PHONE",
                            true: "CHECK_PHONE"
                        } ]
                    } ],
                    CHECK_DATE: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "actions",
                        actions: [ {
                            if: "not $date",
                            actions: [ {
                                goto: "ENTER_DATE_AGAIN"
                            } ]
                        }, {
                            goto: "$SS_AFTER_ENTER_DATE"
                        } ]
                    } ],
                    ENTER_DATE_AGAIN: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "text",
                        text: {
                            en: "RE-ENTER the date.\nUse this format: mm/dd/yyyy"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "$SS_FROM_ENTER_DATE"
                        } ]
                    } ],
                    CHECK_TIME: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "actions",
                        actions: [ {
                            if: "not $time",
                            actions: [ {
                                goto: "ENTER_TIME_AGAIN"
                            } ]
                        }, {
                            goto: "$SS_AFTER_ENTER_TIME"
                        } ]
                    } ],
                    ENTER_TIME_AGAIN: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "text",
                        text: {
                            en: "RE-ENTER the time.\nPlease use 12-hour clock format and Eastern Time (UTC - 05:00/ - 04:00)"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "$SS_FROM_ENTER_TIME",
                            true: "ENTER_TIME2"
                        } ]
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    date: "",
                    phone: "",
                    time: "",
                    address: "",
                    contactinfo: "",
                    SS_FROM_ENTER_PHONE: "",
                    messengers: "",
                    SS_AFTER_ENTER_PHONE: "",
                    SS_AFTER_ENTER_DATE: "",
                    SS_FROM_ENTER_DATE: "",
                    SS_AFTER_ENTER_TIME: "",
                    SS_FROM_ENTER_TIME: "",
                    meetType: "",
                    messanger: "",
                    CHECKS: "",
                    i: "",
                    firstEl: "",
                    rezStr: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 36,
                            y: 61
                        },
                        ENTER_TIME2: {
                            c: 8,
                            r: 2,
                            x: 596,
                            y: 813
                        },
                        ENTER_DATE2: {
                            c: 7,
                            r: 2,
                            x: 594,
                            y: 684
                        },
                        ENTER_PHONE2: {
                            c: 6,
                            r: 2,
                            x: 596,
                            y: 572
                        },
                        SELECT_MEET: {
                            c: 1,
                            r: 0,
                            x: 336,
                            y: 159
                        },
                        CONFIRM2: {
                            c: 9,
                            r: 2,
                            x: 607,
                            y: 945
                        },
                        CONFIRM1: {
                            c: 7,
                            r: 1,
                            x: 282,
                            y: 932
                        },
                        SELECT_MESSENGER: {
                            c: 0,
                            r: 3,
                            x: 970,
                            y: 0
                        },
                        ENTER_DATE3: {
                            c: 2,
                            r: 3,
                            x: 973,
                            y: 271
                        },
                        ENTER_TIME3: {
                            c: 3,
                            r: 3,
                            x: 975,
                            y: 353
                        },
                        CONFIRM3: {
                            c: 4,
                            r: 3,
                            x: 974,
                            y: 454
                        },
                        FINISH: {
                            c: 0,
                            r: 4,
                            x: 439,
                            y: 1174
                        },
                        ENTER_CONTACT3: {
                            c: 1,
                            r: 3,
                            x: 970,
                            y: 177
                        },
                        SELECT_PLACE1: {
                            c: 0,
                            r: 1,
                            x: 374,
                            y: 286
                        },
                        ADDRESS1: {
                            c: 2,
                            r: 1,
                            x: 280,
                            y: 460
                        },
                        ENTER_PHONE1: {
                            c: 3,
                            r: 1,
                            x: 281,
                            y: 591
                        },
                        ENTER_DATE1: {
                            c: 5,
                            r: 1,
                            x: 279,
                            y: 702
                        },
                        ENTER_TIME1: {
                            c: 6,
                            r: 1,
                            x: 282,
                            y: 800
                        },
                        ENTER_ADDRESS2: {
                            c: 1,
                            r: 1,
                            x: 592,
                            y: 361
                        },
                        CHECK_ADDRESS2: {
                            c: 5,
                            r: 2,
                            x: 594,
                            y: 460
                        },
                        ENTER_PHONE_AGAIN: {
                            c: 4,
                            r: 2,
                            x: 10,
                            y: 629
                        },
                        CHECK_PHONE: {
                            c: 4,
                            r: 1,
                            x: 11,
                            y: 544
                        },
                        CHECK_DATE: {
                            c: 0,
                            r: 2,
                            x: 13,
                            y: 727
                        },
                        ENTER_DATE_AGAIN: {
                            c: 1,
                            r: 2,
                            x: 13,
                            y: 809
                        },
                        CHECK_TIME: {
                            c: 2,
                            r: 2,
                            x: 15,
                            y: 901
                        },
                        ENTER_TIME_AGAIN: {
                            c: 3,
                            r: 2,
                            x: 13,
                            y: 985
                        }
                    },
                    folders: {}
                }
            },
            Booking: {
                id: 0,
                name: "Booking",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "text",
                        text: {
                            en: "Welcome! I am a QuBot that will help you book a hotel according to your needs."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Yes, need some help"
                            },
                            actions: [ {
                                state: "ASK1"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    PHRASE1: [ {
                        type: "text",
                        text: {
                            en: "No issues. That's why we are here."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Good!"
                            },
                            actions: [ {
                                state: "ENTER_DAYS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    ENTER_DAYS: [ {
                        type: "text",
                        text: {
                            en: "How many days are you planning?"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "ENTER_DATE"
                        }, {
                            slot: "days",
                            value: "$INPUT"
                        } ],
                        kind: "int"
                    } ],
                    ENTER_COUNT_PERSON: [ {
                        type: "text",
                        text: {
                            en: "How many people will be accompanying you?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Just me"
                            },
                            actions: [ {
                                state: "ENTER_NAME"
                            }, {
                                slot: "countPerson",
                                value: '"Just me"'
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "There is 2 of us"
                            },
                            actions: [ {
                                state: "ENTER_NAME"
                            }, {
                                slot: "countPerson",
                                value: '"There is 2 of us"'
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "There is more than 2"
                            },
                            actions: [ {
                                state: "ENTER_NAME"
                            }, {
                                slot: "countPerson",
                                value: '"There is more than 2"'
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "I am talking my whole family"
                            },
                            actions: [ {
                                state: "ENTER_NAME"
                            }, {
                                slot: "countPerson",
                                value: '"I am talking my whole family"'
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    ENTER_NAME: [ {
                        type: "text",
                        text: {
                            en: "Sorry. I should have asked before. What's your name?"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "ENTER_KIDS"
                        }, {
                            slot: "name",
                            value: "$INPUT"
                        } ],
                        kind: "text"
                    } ],
                    ASK1: [ {
                        type: "text",
                        text: {
                            en: "Have you decided where you want to go?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Tourism"
                            },
                            actions: [ {
                                state: "ENTER_DAYS"
                            }, {
                                slot: "goal",
                                value: '"Tourism"'
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Guests"
                            },
                            actions: [ {
                                state: "ENTER_DAYS"
                            }, {
                                slot: "goal",
                                value: '"Guests"'
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Sport"
                            },
                            actions: [ {
                                state: "ENTER_DAYS"
                            }, {
                                slot: "goal",
                                value: '"Sport"'
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Business"
                            },
                            actions: [ {
                                state: "ENTER_DAYS"
                            }, {
                                slot: "goal",
                                value: '"Business"'
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Work"
                            },
                            actions: [ {
                                state: "ENTER_DAYS"
                            }, {
                                slot: "goal",
                                value: '"Guests"'
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Still planning"
                            },
                            actions: [ {
                                state: "PHRASE1"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    ENTER_DATE: [ {
                        type: "text",
                        text: {
                            en: "When are you planning to start?\nEnter date in format dd.mm.yyyy"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "ENTER_COUNT_PERSON"
                        }, {
                            slot: "date",
                            value: "$INPUT"
                        } ],
                        kind: "text"
                    } ],
                    ENTER_category_hotel: [ {
                        type: "text",
                        text: {
                            en: "Right\nWhich category of hotel do you prefer?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Budget"
                            },
                            actions: [ {
                                state: "ENTER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Standard"
                            },
                            actions: [ {
                                state: "ENTER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Premium"
                            },
                            actions: [ {
                                state: "ENTER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    ENTER_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "Can you share your email address with us? We respect your privacy"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "ENTER_PHONE"
                        }, {
                            slot: "email",
                            value: "$INPUT"
                        } ],
                        kind: "email"
                    } ],
                    ENTER_PHONE: [ {
                        type: "text",
                        text: {
                            en: "And your mobile number also please"
                        }
                    }, {
                        type: "input",
                        actions: [ {
                            state: "CHECK_PHONE"
                        }, {
                            slot: "phone",
                            value: "$INPUT"
                        } ],
                        kind: "phone"
                    } ],
                    ENTER_requirements: [ {
                        type: "text",
                        text: {
                            en: "Thanks a lot.\nDo you have any additional requirements for the trip?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip"
                            },
                            actions: [ {
                                state: "PHRASE2"
                            }, {
                                slot: "requirements",
                                value: '"None"'
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            state: "PHRASE2"
                        }, {
                            slot: "requirements",
                            value: "$INPUT"
                        } ],
                        kind: "text"
                    } ],
                    PHRASE2: [ {
                        type: "text",
                        text: {
                            en: "We are done asking questions\nNow you just relax and get ready for the trip of your lifetime"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Let's get a resume!"
                            },
                            actions: [ {
                                state: "RESUME"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    FINISH: [ {
                        type: "text",
                        text: {
                            en: "We will get in touch with you soon. Bye"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Thanks. Start over"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    ENTER_KIDS: [ {
                        type: "text",
                        text: {
                            en: "Okay\nAre there any kids with you?\nPlease enter a count kids"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "No, it's just me"
                            },
                            actions: [ {
                                state: "ENTER_category_hotel"
                            }, {
                                slot: "countKids",
                                value: "0"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            slot: "countKids",
                            value: "$INPUT"
                        }, {
                            state: "ENTER_category_hotel"
                        } ],
                        kind: "int"
                    } ],
                    RESUME: [ {
                        type: "text",
                        text: {
                            en: "Let's get a resume!\n\nYour name: $name\nPurpose of your trip: $goal\nNumber of days: $days\nTravel date: $date\nNumber of accompanying persons: $countPerson\nAmount of children: $countKids\nYour phone number: $phone\nYour email: $email\nAdditional requirements: $requirements"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Good"
                            },
                            actions: [ {
                                state: "FINISH"
                            }, {
                                action: "save",
                                storage: "wp",
                                key: "",
                                value: '{"name":$name,\n"goal":$goal,\n"days":$days,\n"date":$date,\n"countPerson":$countPerson,\n"countKids":$countKids,\n"phone":$phone,\n"email":$email,\n"requirements":$requirements}'
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Start over"
                            },
                            actions: [ {
                                state: "ASK1"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CHECK_PHONE: [ {
                        type: "input",
                        kind: "text",
                        actions: []
                    }, {
                        type: "actions",
                        actions: [ {
                            if: "$phone == null",
                            actions: [ {
                                goto: "ENTER_PHONE_AGAIN"
                            } ]
                        }, {
                            goto: "ENTER_requirements"
                        } ]
                    } ],
                    ENTER_PHONE_AGAIN: [ {
                        type: "text",
                        text: {
                            en: "Повторите ввод номера телефона"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "ENTER_PHONE"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    date: "",
                    name: "",
                    email: "",
                    phone: "",
                    days: "",
                    goal: "",
                    countKids: "",
                    people: "",
                    a: "",
                    "ф": "",
                    requirements: "",
                    countPerson: "",
                    asd: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 0,
                            y: 0
                        },
                        PHRASE1: {
                            c: 2,
                            r: 0,
                            x: 562,
                            y: 6
                        },
                        ENTER_DAYS: {
                            c: 3,
                            r: 0,
                            x: 886,
                            y: 16
                        },
                        ENTER_COUNT_PERSON: {
                            c: 5,
                            r: 0,
                            x: 886,
                            y: 245
                        },
                        ENTER_NAME: {
                            c: 0,
                            r: 1,
                            x: 893,
                            y: 420
                        },
                        ASK1: {
                            c: 1,
                            r: 0,
                            x: 289,
                            y: 174
                        },
                        ENTER_DATE: {
                            c: 4,
                            r: 0,
                            x: 877,
                            y: 126
                        },
                        ENTER_category_hotel: {
                            c: 2,
                            r: 1,
                            x: 614,
                            y: 450
                        },
                        ENTER_EMAIL: {
                            c: 0,
                            r: 2,
                            x: 387,
                            y: 595
                        },
                        ENTER_PHONE: {
                            c: 1,
                            r: 2,
                            x: 389,
                            y: 709
                        },
                        ENTER_requirements: {
                            c: 4,
                            r: 2,
                            x: 391,
                            y: 832
                        },
                        PHRASE2: {
                            c: 5,
                            r: 2,
                            x: 402,
                            y: 959
                        },
                        FINISH: {
                            c: 7,
                            r: 2,
                            x: 798,
                            y: 1124
                        },
                        ENTER_KIDS: {
                            c: 1,
                            r: 1,
                            x: 894,
                            y: 526
                        },
                        RESUME: {
                            c: 6,
                            r: 2,
                            x: 783,
                            y: 913
                        },
                        CHECK_PHONE: {
                            c: 2,
                            r: 2,
                            x: 134,
                            y: 672
                        },
                        ENTER_PHONE_AGAIN: {
                            c: 3,
                            r: 2,
                            x: 132,
                            y: 791
                        }
                    },
                    folders: {}
                }
            },
            AboutUsTmpl: {
                id: 0,
                name: "About us",
                start: "MAIN",
                states: {
                    MAIN: [ {
                        type: "text",
                        text: {
                            en: "This is the home page. Greet your customers and write here briefly about your company.",
                            ru: "Это главная страница. Поприветствуйте ваших клиентов и напишите здесь коротко о своей компании."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🔑 Our services",
                                ru: "🔑 Наши услуги"
                            },
                            actions: [ {
                                state: "SERVICES"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: " 💼 Our products",
                                ru: "💼 Наши продукты"
                            },
                            actions: [ {
                                state: "PRODUCTS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⭐ About Us",
                                ru: "⭐ О нас"
                            },
                            actions: [ {
                                state: "ABOUT_US"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🚩 Our advantages",
                                ru: "🚩 Наши преимущества"
                            },
                            actions: [ {
                                state: "ADVANTAGES"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Change language",
                                ru: "🌐 Сменить язык"
                            },
                            actions: [ {
                                state: "CHANGE_LANGUAGE",
                                true: "CHANGE_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CHANGE_LANGUAGE: [ {
                        type: "text",
                        text: {
                            en: "Select the language of communication:",
                            ru: "Выберите язык общения:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English"
                            },
                            actions: [ {
                                state: "MAIN"
                            }, {
                                slot: "LANGUAGE",
                                value: "'en'"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Russian",
                                ru: "Русский"
                            },
                            actions: [ {
                                state: "MAIN"
                            }, {
                                slot: "LANGUAGE",
                                value: "'ru'"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SERVICES: [ {
                        type: "text",
                        text: {
                            en: "Briefly describe what you are willing to do for your visitor by directing them to specific landing pages.",
                            ru: "Кратко расскажите о том, что вы готовы сделать для посетителя, направив его на конкретные целевые страницы."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: " Service 1",
                                ru: "Услуга 1"
                            },
                            actions: [ {
                                state: "SERVICES_1"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: " Service 2",
                                ru: "Услуга 2"
                            },
                            actions: [ {
                                state: "SERVICES_2"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: " Service 3",
                                ru: "Услуга 3"
                            },
                            actions: [ {
                                state: "SERVICES_3"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: " ✉ Leave a request",
                                ru: "✉ Оставить заявку"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME",
                                true: "SERVICES"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ABOUT_US: [ {
                        type: "text",
                        text: {
                            en: "Tell us about what you are doing and for whom. Talk about visitor problems that you solve with your products and services.",
                            ru: "Расскажите о том, что вы делаете и для кого. Говорите о проблемах посетителей, которые вы решаете с помощью товаров и услуг. "
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: " ✉ Leave a request",
                                ru: "✉ Оставить заявку"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME",
                                true: "ADVANTAGES"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📍 Contacts",
                                ru: "📍 Контакты"
                            },
                            actions: [ {
                                state: "CONTACTS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CONTACT_MAP: [ {
                        type: "map",
                        url: '<iframe src="https://www.google.com/maps/embed?pb=!1m18!1m12!1m3!1d174295.14385241855!2d7.254786928750355!3d46.95458452641973!2m3!1f0!2f0!3f0!3m2!1i1024!2i768!4f13.1!3m3!1m2!1s0x478e39c0d43a1b77%3A0xcb555ffe0457659a!2z0JHQtdGA0L0sINCo0LLQtdC50YbQsNGA0LjRjw!5e0!3m2!1sru!2sua!4v1641461629417!5m2!1sru!2sua" width="600" height="450" style="border:0;" allowfullscreen="" loading="lazy"></iframe>',
                        height: "300"
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "👉 View route",
                                ru: "👉 Посмотреть маршрут"
                            },
                            actions: [ {
                                state: "https://google.com/maps/dir/?api=1&destination=48.45614445037599,35.03230596938486"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "CONTACTS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CONTACTS: [ {
                        type: "text",
                        text: {
                            en: "📍 <b>Our address</b>:\nEnter your address here\n\n🌐 <b>Official site</b>: <a href='https://site.com/' target='_blank'> site.com </a>\n📞 <b>Our phone</b>: Enter your phone\n📬 <b>Our mail</b>: <a href='mailto:info@company.com'> info@company.com </a>\n\n• <b> YouTube channel </b>: <a href='https://www.youtube.com/channel/UCLXENpdTmOFPwZjcu7WLApA' target='_blank'> Channel company</a>\n• <b> Telegram </b>: <a href='https://t.me/company' target='_blank'> @company </a>\n• <b> WhatsApp </b>: <a href='https://api.whatsapp.com/send/?phone' target='_blank'> Go to chat </a>",
                            ru: "📍 <b>Наш адрес</b>:\nУкажите ваш адрес здесь\n\n🌐 <b>Официальный сайт</b>: <a href='https://site.com/' target='_blank'>site.com</a>\n📞 <b>Наш телефон</b>: Укажите ваш телефон\n📬 <b>Наша почта</b>: <a href='mailto:info@company.com'>info@company.com</a>\n\n• <b>YouTube канал</b>: <a href='https://www.youtube.com/channel/' target='_blank'>Channel company</a>\n• <b>Telegram</b>: <a href='https://t.me/company target='_blank'>@company</a>\n• <b>WhatsApp</b>: <a href='https://api.whatsapp.com/send/?phone' target='_blank'>Go to chat</a>"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "👉 Show us on the map",
                                ru: "👉 Показать нас на карте"
                            },
                            actions: [ {
                                state: "CONTACT_MAP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SERVICES_1: [ {
                        type: "text",
                        text: {
                            en: "Describe the service you provide here.",
                            ru: "Опишите здесь услугу, которую вы предоставляете."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "SERVICES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SERVICES_2: [ {
                        type: "text",
                        text: {
                            en: "Describe the service you provide here.",
                            ru: "Опишите здесь услугу, которую вы предоставляете."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "SERVICES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SERVICES_3: [ {
                        type: "text",
                        text: {
                            en: "Describe the service you provide here.",
                            ru: "Опишите здесь услугу, которую вы предоставляете."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "SERVICES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PRODUCTS: [ {
                        type: "text",
                        text: {
                            en: "Describe your company's products here.",
                            ru: "Опишите здесь продукты вашей компании."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Product 1",
                                ru: "Продукт 1"
                            },
                            actions: [ {
                                state: "PRODUCTS_1"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Product 2",
                                ru: "Продукт 2"
                            },
                            actions: [ {
                                state: "PRODUCTS_2"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Product 3",
                                ru: "Продукт 3"
                            },
                            actions: [ {
                                state: "PRODUCTS_3"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✉ Leave a request",
                                ru: "✉ Оставить заявку"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PRODUCTS_1: [ {
                        type: "text",
                        text: {
                            en: "Describe your company's product here.",
                            ru: "Опишите здесь продукт вашей компании."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "PRODUCTS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PRODUCTS_2: [ {
                        type: "text",
                        text: {
                            en: "Describe your company's product here.",
                            ru: "Опишите здесь продукт вашей компании."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "PRODUCTS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PRODUCTS_3: [ {
                        type: "text",
                        text: {
                            en: "Describe your company's product here.",
                            ru: "Опишите здесь продукт вашей компании."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⬅️ Return",
                                ru: "⬅️ Вернуться"
                            },
                            actions: [ {
                                state: "PRODUCTS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ADVANTAGES: [ {
                        type: "text",
                        text: {
                            en: "Describe the specific benefits of your company. The “About Us in Numbers” trick works well. Collect all possible numbers and present them to your potential clients profitably.\n\nFor example: work experience, exact specification of the assortment, duration of the warranty, number of customers, speed of coffee preparation.",
                            ru: "Опишите конкретные преимущества вашей компании. Хорошо работает прием «О нас в цифрах». Соберите все возможные цифры и выгодно презентуйте их вашим потенциальным клиентам.\n\nНапример: опыт работы, точное указание ассортимента, длительность гарантии, количество заказчиков, скорость приготовления кофе."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✉ Leave a request",
                                ru: "✉ Оставить заявку"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME",
                                true: "CONTACTS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_WRONG_NAME: [ {
                        type: "text",
                        text: {
                            en: "Very strange name $CUSTOMER_NAME. Could you introduce yourself differently:",
                            ru: "Очень странное имя $CUSTOMER_NAME. Не могли бы Ва представиться по другому:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_GREET"
                        }, {
                            slot: "CUSTOMER_NAME",
                            value: "$INPUT"
                        }, {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_GREET: [ {
                        type: "text",
                        text: {
                            en: "We are glad to meet $CUSTOMER_NAME!\nPlease indicate your preferred communication channel.",
                            ru: "Рады знакомству $CUSTOMER_NAME!\nУкажите, пожалуйста, предпочитаемый канал связи с Вами."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Phone",
                                ru: "📞 Телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "💻 Email",
                                ru: "💻 Почта"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_WELLCOME: [ {
                        type: "text",
                        text: {
                            en: "Please enter your name:",
                            ru: "Введите, пожалуйста, свое имя:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Change dialogue language",
                                ru: "🌐 Сменить язык общения"
                            },
                            actions: [ {
                                state: "CUSTOMER_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_GREET"
                        }, {
                            slot: "CUSTOMER_NAME",
                            value: "$INPUT"
                        }, {
                            if: "$CUSTOMER_PHONE or $CUSTOMER_EMAIL",
                            actions: [ {
                                state: "BEFORE_CHECK"
                            } ]
                        }, {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "Please enter your email so that our staff can contact you.",
                            ru: "Введите, пожалуйста, свою электронную почту, чтобы наши сотрудники могли с Вами связаться."
                        }
                    }, {
                        type: "input",
                        kind: "email",
                        actions: [ {
                            slot: "CUSTOMER_EMAIL",
                            value: "$INPUT"
                        }, {
                            state: "BEFORE_CHECK"
                        }, {
                            if: "not $CUSTOMER_PHONE",
                            actions: [ {
                                state: "CUSTOMER_PHONE_MAYBE"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL_WRONG: [ {
                        type: "text",
                        text: {
                            en: "We will not be able to write to you at this address. Will you try again?",
                            ru: "Мы не сможем написать Вам по такому адресу. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, please enter your phone number:",
                            ru: "$CUSTOMER_NAME, введите пожалуйста Ваш телефон:"
                        }
                    }, {
                        type: "input",
                        kind: "phone",
                        actions: [ {
                            slot: "CUSTOMER_PHONE",
                            value: "$INPUT"
                        }, {
                            slot: "CUSTOMER_PHONE",
                            value: "$INPUT",
                            state: "BEFORE_CHECK"
                        }, {
                            if: "not $CUSTOMER_EMAIL",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_MAYBE"
                            } ]
                        }, {
                            if: "not $CUSTOMER_PHONE",
                            actions: [ {
                                state: "CUSTOMER_PHONE_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_PHONE_WRONG: [ {
                        type: "text",
                        text: {
                            en: "We can't call that phone. Will you try again?",
                            ru: "Мы не сможем позвонить по такому телефону. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE_MAYBE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, maybe you will leave your phone?",
                            ru: "$CUSTOMER_NAME, может быть Вы оставите Ваш телефон? "
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_EMAIL_MAYBE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, would you like to leave your email to contact you?",
                            ru: "$CUSTOMER_NAME, хотите оставить Вашу электронную почту для связи с Вами?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_MESSAGE: [ {
                        type: "text",
                        text: {
                            en: "Leave here your application, wishes or comments.",
                            ru: "Оставьте здесь свою заявку, пожелания или замечания."
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_END"
                        }, {
                            slot: "CUSTOMER_MESSAGE",
                            value: "$INPUT"
                        } ]
                    } ],
                    CUSTOMER_END: [ {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "customer",
                            value: '{\n"kind":    "customer",\n"name":    $CUSTOMER_NAME,\n"phone":   $CUSTOMER_PHONE,\n"mail":    $CUSTOMER_EMAIL,\n"message": $CUSTOMER_MESSAGE\n}'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Thank you very much $CUSTOMER_NAME 😊, we will contact you shortly.",
                            ru: "Спасибо большое $CUSTOMER_NAME 😊, мы свяжемся с Вами в ближайшее время."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "END"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHECK: [ {
                        type: "text",
                        text: {
                            en: "Is that correct?",
                            ru: "Всё верно?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_MESSAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🤖 Change name",
                                ru: "🤖 Изменить имя"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "💻Change mail",
                                ru: "💻Изменить почту"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Change phone",
                                ru: "📞 Изменить телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_LANGUAGE: [ {
                        type: "text",
                        text: {
                            en: "What language do you prefer to communicate in?",
                            ru: "На каком языке Вы предпочитаете общение?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            }, {
                                slot: "LANGUAGE",
                                value: "en"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Русский",
                                ru: "Русский"
                            },
                            actions: [ {
                                state: "CUSTOMER_WELLCOME"
                            }, {
                                slot: "LANGUAGE",
                                value: "ru"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BEFORE_CHECK: [ {
                        type: "actions",
                        actions: [ {
                            if: "$CUSTOMER_PHONE and $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "PHONE_AND_EMAIL"
                            } ]
                        }, {
                            if: "$CUSTOMER_PHONE",
                            actions: [ {
                                goto: "ONLY_PHONE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "ONLY_EMAIL"
                            } ]
                        }, {
                            if: "not $CUSTOMER_PHONE and  not $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "NOTHING"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PHONE_AND_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by mail <b> $CUSTOMER_EMAIL </b> and phone <b> $CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b> и телефону <b>$CUSTOMER_PHONE</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_PHONE: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact phone <b>$CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по телефону <b>$CUSTOMER_PHONE</b>. "
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by by mail <b> $CUSTOMER_EMAIL </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NOTHING: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, you have left no phone or mail.",
                            ru: "$CUSTOMER_NAME, Вы не оставили, ни телефона, ни почты."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    END: [ {
                        type: "text",
                        text: {
                            en: "Your application is accepted.",
                            ru: "Ваша заявка принята."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    CUSTOMER_EMAIL: "",
                    CUSTOMER_NAME: "",
                    CUSTOMER_PHONE: "",
                    CUSTOMER_MESSAGE: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 27,
                            y: 47
                        },
                        CHANGE_LANGUAGE: {
                            c: 2,
                            r: 0,
                            x: 307,
                            y: 295
                        },
                        SERVICES: {
                            c: 3,
                            r: 0,
                            x: 313,
                            y: 49
                        },
                        ABOUT_US: {
                            c: 0,
                            r: 2,
                            x: 307,
                            y: 783
                        },
                        CONTACT_MAP: {
                            c: 2,
                            r: 2,
                            x: 803,
                            y: 781
                        },
                        CONTACTS: {
                            c: 1,
                            r: 2,
                            x: 556,
                            y: 784
                        },
                        SERVICES_1: {
                            c: 4,
                            r: 0,
                            x: 575,
                            y: 45
                        },
                        SERVICES_2: {
                            c: 5,
                            r: 0,
                            x: 576,
                            y: 153
                        },
                        SERVICES_3: {
                            c: 6,
                            r: 0,
                            x: 578,
                            y: 260
                        },
                        PRODUCTS: {
                            c: 0,
                            r: 1,
                            x: 311,
                            y: 467
                        },
                        PRODUCTS_1: {
                            c: 1,
                            r: 1,
                            x: 581,
                            y: 427
                        },
                        PRODUCTS_2: {
                            c: 2,
                            r: 1,
                            x: 584,
                            y: 540
                        },
                        PRODUCTS_3: {
                            c: 3,
                            r: 1,
                            x: 587,
                            y: 656
                        },
                        ADVANTAGES: {
                            c: 0,
                            r: 3,
                            x: 21,
                            y: 296
                        },
                        CUSTOMER_WRONG_NAME: {
                            c: 3,
                            r: 1,
                            x: 463,
                            y: 16,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_GREET: {
                            c: 2,
                            r: 1,
                            x: 253,
                            y: 111,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_WELLCOME: {
                            c: 1,
                            r: 1,
                            x: 22,
                            y: 12,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_EMAIL: {
                            c: 1,
                            r: 3,
                            x: 538,
                            y: 241,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_EMAIL_WRONG: {
                            c: 3,
                            r: 3,
                            x: 553,
                            y: 383,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_PHONE: {
                            c: 1,
                            r: 2,
                            x: 45,
                            y: 240,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_PHONE_WRONG: {
                            c: 3,
                            r: 2,
                            x: 24,
                            y: 387,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_PHONE_MAYBE: {
                            c: 2,
                            r: 3,
                            x: 267,
                            y: 385,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_EMAIL_MAYBE: {
                            c: 2,
                            r: 2,
                            x: 263,
                            y: 234,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_MESSAGE: {
                            c: 2,
                            r: 5,
                            x: 626,
                            y: 673,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_END: {
                            c: 3,
                            r: 5,
                            x: 4,
                            y: 683,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_CHECK: {
                            c: 1,
                            r: 5,
                            x: 932,
                            y: 539,
                            folder: "CUSTOMER"
                        },
                        CUSTOMER_LANGUAGE: {
                            c: 4,
                            r: 1,
                            x: 244,
                            y: 11,
                            folder: "CUSTOMER"
                        },
                        BEFORE_CHECK: {
                            c: 1,
                            r: 4,
                            x: 19,
                            y: 519,
                            folder: "CUSTOMER"
                        },
                        PHONE_AND_EMAIL: {
                            c: 5,
                            r: 4,
                            x: 215,
                            y: 522,
                            folder: "CUSTOMER"
                        },
                        ONLY_PHONE: {
                            c: 4,
                            r: 4,
                            x: 406,
                            y: 522,
                            folder: "CUSTOMER"
                        },
                        ONLY_EMAIL: {
                            c: 3,
                            r: 4,
                            x: 599,
                            y: 523,
                            folder: "CUSTOMER"
                        },
                        NOTHING: {
                            c: 2,
                            r: 4,
                            x: 410,
                            y: 614,
                            folder: "CUSTOMER"
                        },
                        END: {
                            c: 4,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CUSTOMER"
                        }
                    },
                    folders: {
                        CUSTOMER: {
                            c: 1,
                            r: 0,
                            x: 23,
                            y: 703
                        }
                    }
                }
            },
            BJ: {
                id: 0,
                name: "Blackjack",
                start: "MAIN",
                version: "1.0.50",
                view: "free",
                states: {
                    TABLE: [ {
                        type: "actions",
                        actions: [ {
                            clear: "0"
                        }, {
                            note: "If the points are more than 21, the player loses."
                        }, {
                            if: "$PLAYER_SCORE > 21",
                            actions: [ {
                                goto: "LOSE"
                            } ]
                        }, {
                            note: "If the points are equal to 21, check if the player wins."
                        }, {
                            if: "$PLAYER_SCORE == 21",
                            actions: [ {
                                goto: "CHECK_WIN"
                            } ]
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Your score: $PLAYER_SCORE",
                            ru: "Ваши очки: $PLAYER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the player's cards"
                        }, {
                            script: "foreach(PLAYER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "👌 Hit",
                                ru: "👌 Ещё"
                            },
                            actions: [ {
                                state: "TABLE"
                            }, {
                                note: "Add a card to the player's hand and increase his points"
                            }, {
                                script: "CD = randint(0, $COUNT - 1)\nPR = randint(0, 3)\n\npush(PLAYER_HAND, \n  $PREFIX[$PR] + $CARDS[$CD].img)\nset(PLAYER_SCORE,\n  $PLAYER_SCORE + $CARDS[$CD].pt)"
                            }, {
                                note: "We check if there is an Ace among the cards, if there are more than 21 points. If so, we subtract 10 points."
                            }, {
                                if: "$PLAYER_SCORE > 21 and $CARDS[$CD].pt == 11",
                                actions: [ {
                                    slot: "PLAYER_SCORE",
                                    value: "$PLAYER_SCORE - 10"
                                } ]
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "✋ Stand",
                                ru: "✋ Хватит"
                            },
                            actions: [ {
                                state: "ADD_DEALER_CARDS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    MAIN: [ {
                        type: "actions",
                        actions: [ {
                            note: "Initializing slots:\nMONEY - player's starting credits\nURL   - link to the server with images\nPREFIX - prefix for cards\nCARDS - array with card data\n- img - the name of the image on the server\n- pt  - card points"
                        }, {
                            slot: "MONEY",
                            value: "800"
                        }, {
                            slot: "URL",
                            value: "'https://qudata.com/qubot/im/ts/bj/'"
                        }, {
                            note: "Prefix for cards"
                        }, {
                            slot: "PREFIX",
                            value: "['c', 'd', 'h', 's']"
                        }, {
                            note: "Card data"
                        }, {
                            slot: "CARDS",
                            value: "[\n { 'img': '2.png', 'pt': 2 },\n { 'img': '3.png', 'pt': 3 },\n { 'img': '4.png', 'pt': 4 },\n { 'img': '5.png', 'pt': 5 },\n { 'img': '6.png', 'pt': 6 },\n { 'img': '7.png', 'pt': 7 },\n { 'img': '8.png', 'pt': 8 },\n { 'img': '9.png', 'pt': 9 },\n { 'img': '10.png', 'pt': 10 },\n { 'img': 'J.png', 'pt': 10 },\n { 'img': 'Q.png', 'pt': 10 },\n { 'img': 'K.png', 'pt': 10 },\n { 'img': 'A.png', 'pt': 11 }\n]"
                        }, {
                            note: "Obtain the number of cards"
                        }, {
                            script: "len(CARDS, COUNT)"
                        } ]
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/bj/blackjack.png"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "In <b>Blackjack</b>, or 21, you have to decide whether to <b>'hit'</b> (take another card) or <b>'stand'</b> (keep the hand you are dealt).",
                            ru: "В игре <b>'Блэкджек'</b>, или 21, вы можете выбирать между <b>'ещё'</b> (взять карту) или <b>'хватит'</b> (оставить руку)"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🚀 Run",
                                ru: "🚀 Начать"
                            },
                            actions: [ {
                                state: "SELECT_BET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            slot: "",
                            value: ""
                        } ],
                        kind: "text"
                    } ],
                    SELECT_BET: [ {
                        type: "actions",
                        actions: [ {
                            note: "Resetting the dealer's and player's points"
                        }, {
                            script: "DEALER_SCORE = 0\nPLAYER_SCORE = 0"
                        }, {
                            note: "We set a starting hand of two cards for the dealer and the player, after clearing the previous cards (if any)."
                        }, {
                            script: "clear(DEALER_HAND)\n\nINDEX = 2\n\nwhile($INDEX > 0,\n  CD = randint(0, $COUNT - 1)\n  PR = randint(0, 3)\n  push(DEALER_HAND, \n    $PREFIX[$PR] + $CARDS[$CD].img)\n  set(DEALER_SCORE, \n\t\t$DEALER_SCORE + $CARDS[$CD].pt)\n  INDEX = $INDEX - 1\n)"
                        }, {
                            script: "clear(PLAYER_HAND)\n\nINDEX = 2\n\nwhile($INDEX > 0,\n  CD = randint(0, $COUNT - 1)\n  PR = randint(0, 3)\n  push(PLAYER_HAND, \n    $PREFIX[$PR] + $CARDS[$CD].img)\n  set(PLAYER_SCORE, \n    $PLAYER_SCORE+ $CARDS[$CD].pt)\n  INDEX = $INDEX - 1\n)"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "You have <b>\\$ $MONEY</b>. Place your bet 💰:",
                            ru: "У Вас есть <b>\\$ $MONEY</b>. Сделайте ставку 💰:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "$ 10"
                            },
                            actions: [ {
                                note: "Store the bet"
                            }, {
                                slot: "BET",
                                value: "10"
                            }, {
                                state: "DEALER_TABLE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "$ 100"
                            },
                            actions: [ {
                                note: "Store the bet"
                            }, {
                                slot: "BET",
                                value: "100"
                            }, {
                                state: "DEALER_TABLE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "$ 1000"
                            },
                            actions: [ {
                                note: "Store the bet"
                            }, {
                                slot: "BET",
                                value: "1000"
                            }, {
                                state: "DEALER_TABLE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "All-in",
                                ru: "Ставлю всё"
                            },
                            actions: [ {
                                note: "Store the bet"
                            }, {
                                slot: "BET",
                                value: "$MONEY"
                            }, {
                                state: "DEALER_TABLE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [ {
                            note: "Store the bet"
                        }, {
                            if: "$INPUT",
                            actions: [ {
                                slot: "BET",
                                value: "$INPUT"
                            }, {
                                state: "DEALER_TABLE"
                            } ]
                        } ],
                        kind: "int"
                    } ],
                    LOSE: [ {
                        type: "actions",
                        actions: [ {
                            clear: "1"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Dealer's score: $DEALER_SCORE\n",
                            ru: "Очки дилера: $DEALER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the dealer's cards"
                        }, {
                            script: "images()\n\nforeach(DEALER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Your score: $PLAYER_SCORE",
                            ru: "Ваши очки: $PLAYER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the player's cards"
                        }, {
                            script: "images()\n\nforeach(PLAYER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "😢 You lose!",
                            ru: "😢 Вы проиграли!"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Deduct the bet amount from the player's credits"
                        }, {
                            slot: "MONEY",
                            value: "$MONEY - $BET"
                        }, {
                            note: "If the player goes bankrupt, the game is over"
                        }, {
                            if: "$MONEY <= 0",
                            actions: [ {
                                goto: "NO_MONEY"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙈 Play again",
                                ru: "🙈 Играть снова"
                            },
                            actions: [ {
                                state: "SELECT_BET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    WIN: [ {
                        type: "actions",
                        actions: [ {
                            clear: "1"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Dealer's score: $DEALER_SCORE",
                            ru: "Очки дилера: $DEALER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the dealer's cards"
                        }, {
                            script: "images()\n\nforeach(DEALER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Your score: $PLAYER_SCORE",
                            ru: "Ваши очки: $PLAYER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the player's cards"
                        }, {
                            script: "images()\n\nforeach(PLAYER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "🤩 You win!",
                            ru: "🤩 Вы победили!"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙈 Play again",
                                ru: "🙈 Играть снова"
                            },
                            actions: [ {
                                note: "Increase the player's credits by the amount of the bet and continue playing"
                            }, {
                                slot: "MONEY",
                                value: "$MONEY + $BET"
                            }, {
                                state: "SELECT_BET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    PASS: [ {
                        type: "actions",
                        actions: [ {
                            clear: "1"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Dealer score: $DEALER_SCORE",
                            ru: "Очки дилера: $DEALER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the dealer's cards"
                        }, {
                            script: "images()\n\nforeach(DEALER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Your score: $PLAYER_SCORE",
                            ru: "Ваши очки: $PLAYER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the player's cards"
                        }, {
                            script: "images()\n\nforeach(PLAYER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "🙂 We have a draw!",
                            ru: "🙂 Ничья!"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙈 Play again",
                                ru: "🙈 Играть снова"
                            },
                            actions: [ {
                                state: "SELECT_BET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    CHECK_WIN: [ {
                        type: "actions",
                        actions: [ {
                            note: "If the player has more points than the dealer or the dealer scored more than 21, then the player wins."
                        }, {
                            if: "$PLAYER_SCORE > $DEALER_SCORE or $DEALER_SCORE > 21",
                            actions: [ {
                                goto: "WIN"
                            } ]
                        }, {
                            note: "If the player's points are less than the dealer's, the player loses."
                        }, {
                            if: "$PLAYER_SCORE < $DEALER_SCORE",
                            actions: [ {
                                goto: "LOSE"
                            } ]
                        }, {
                            note: "If the points are equal, then it's a draw."
                        }, {
                            goto: "PASS"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ADD_DEALER_CARDS: [ {
                        type: "actions",
                        actions: [ {
                            note: "If the dealer's point score exceeds 15 or player's score, then we check the game results."
                        }, {
                            if: "$DEALER_SCORE > 15 or $DEALER_SCORE > $PLAYER_SCORE",
                            actions: [ {
                                goto: "CHECK_WIN"
                            } ]
                        }, {
                            note: "Add a card to the dealer's hand and increase the point score."
                        }, {
                            script: "CD = randint(0, $COUNT - 1)\nPR = randint(0, 3)\n\npush(DEALER_HAND, \n  $PREFIX[$PR] + $CARDS[$CD].img)\nset(DEALER_SCORE, \n  $DEALER_SCORE + $CARDS[$CD].pt)"
                        }, {
                            note: "We check if there is an Ace among the cards, if the point score exceeds 21. If so, we subtract 10 points."
                        }, {
                            if: "$DEALER_SCORE > 21 and $CARDS[$CD].pt == 11",
                            actions: [ {
                                slot: "DEALER_SCORE",
                                value: "$DEALER_SCORE - 10"
                            } ]
                        }, {
                            note: "Run the current step again"
                        }, {
                            goto: "ADD_DEALER_CARDS"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    DEALER_TABLE: [ {
                        type: "actions",
                        actions: [ {
                            note: "If money is not enough, notify the player about it."
                        }, {
                            if: "$MONEY < $BET",
                            actions: [ {
                                goto: "NO_MONEY_FOR_BET"
                            } ]
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Dealer's score: $DEALER_SCORE",
                            ru: "Очки дилера: $DEALER_SCORE"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            note: "Display the dealer's cards"
                        }, {
                            script: "foreach(DEALER_HAND,\n  image($URL + $ITEM_VALUE, 20))"
                        } ]
                    }, {
                        type: "timer",
                        time: "1000",
                        actions: [ {
                            state: "TABLE"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NO_MONEY_FOR_BET: [ {
                        type: "text",
                        text: {
                            en: "You don't  have \\$ $BET",
                            ru: "У Вас нет \\$ $BET"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🙈 Play again",
                                ru: "🙈 Играть снова"
                            },
                            actions: [ {
                                state: "SELECT_BET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    NO_MONEY: [ {
                        type: "text",
                        text: {
                            en: "You have no money 😢"
                        }
                    }, {
                        type: "timer",
                        time: "1000",
                        actions: [ {
                            state: "CUSTOMER_START"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_WRONG_NAME: [ {
                        type: "text",
                        text: {
                            en: "Very strange name $INPUT. Could you introduce yourself differently:",
                            ru: "Очень странное имя $INPUT. Не могли бы Вы представиться по другому:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            if: "len($INPUT) > 1",
                            actions: [ {
                                state: "CUSTOMER_GREET"
                            }, {
                                slot: "CUSTOMER_NAME",
                                value: "$INPUT"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "Our mail: info@qudata.com, and yours, $CUSTOMER_NAME?",
                            ru: "Наша почта: info@qudata.com, а Ваша, $CUSTOMER_NAME?"
                        }
                    }, {
                        type: "input",
                        kind: "email",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                slot: "CUSTOMER_EMAIL",
                                value: "$INPUT"
                            }, {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_EMAIL_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_EMAIL_WRONG: [ {
                        type: "text",
                        text: {
                            en: "We will not be able to write to you at this address. Will you try again?",
                            ru: "Мы не сможем написать Вам по такому адресу. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_PHONE",
                                actions: [ {
                                    state: "CUSTOMER_PHONE_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, please enter your phone number:",
                            ru: "$CUSTOMER_NAME, введите пожалуйста Ваш телефон:"
                        }
                    }, {
                        type: "input",
                        kind: "phone",
                        actions: [ {
                            if: "$INPUT",
                            actions: [ {
                                slot: "CUSTOMER_PHONE",
                                value: "$INPUT"
                            }, {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_PHONE_WRONG"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_PHONE_WRONG: [ {
                        type: "text",
                        text: {
                            en: "We can't call that phone. Will you try again?",
                            ru: "Мы не сможем позвонить по такому телефону. Попробуете ещё раз?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_PHONE_MAYBE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, maybe you will leave your phone? We can send messages in the messenger.",
                            ru: "$CUSTOMER_NAME, может быть Вы оставите Ваш телефон? Мы можем присылать сообщения в месенджере."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            }, {
                                if: "not $CUSTOMER_EMAIL",
                                actions: [ {
                                    state: "CUSTOMER_EMAIL_MAYBE"
                                } ]
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_EMAIL_MAYBE: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, would you mind leaving your email? Sometimes we will send in very useful information.",
                            ru: "$CUSTOMER_NAME, Вы не против оставить Вашу электронную почту? Иногда мы будем присылать очень полезную информацию."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "❌ No",
                                ru: "❌ Нет"
                            },
                            actions: [ {
                                state: "BEFORE_CHECK"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_MESSAGE: [ {
                        type: "text",
                        text: {
                            en: "Write something about yourself or leave any of your suggestions. Your feedback is very important to us.",
                            ru: "Напишите что-нибудь о себе или оставьте любые Ваши предложения. Ваше мнение очень важно для нас."
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            state: "CUSTOMER_END"
                        }, {
                            slot: "CUSTOMER_MESSAGE",
                            value: "$INPUT"
                        } ]
                    } ],
                    CUSTOMER_END: [ {
                        type: "actions",
                        actions: [ {
                            action: "save",
                            storage: "wp",
                            key: "customer",
                            value: '{\n"kind":    "customer",\n"name":    $CUSTOMER_NAME,\n"phone":   $CUSTOMER_PHONE,\n"mail":    $CUSTOMER_EMAIL,\n"message": $CUSTOMER_MESSAGE\n}'
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Thank you very much $CUSTOMER_NAME 😊, we will contact you shortly.",
                            ru: "Спасибо большое $CUSTOMER_NAME 😊, мы свяжемся с Вами в ближайшее время."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Get $ 1000",
                                ru: "Получить 1000 $"
                            },
                            actions: [ {
                                slot: "MONEY",
                                value: "1000"
                            }, {
                                state: "SELECT_BET"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHECK: [ {
                        type: "text",
                        text: {
                            en: "Is that correct?",
                            ru: "Всё верно?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "✔ Yes",
                                ru: "✔ Да"
                            },
                            actions: [ {
                                state: "CUSTOMER_MESSAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🤖 Change name",
                                ru: "🤖 Изменить имя"
                            },
                            actions: [ {
                                state: "CUSTOMER_NAME"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "💻Change mail",
                                ru: "💻Изменить почту"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Change phone",
                                ru: "📞 Изменить телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BEFORE_CHECK: [ {
                        type: "actions",
                        actions: [ {
                            if: "$CUSTOMER_PHONE and $CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "PHONE_AND_EMAIL"
                            } ]
                        }, {
                            if: "$CUSTOMER_PHONE",
                            actions: [ {
                                goto: "ONLY_PHONE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL",
                            actions: [ {
                                goto: "ONLY_EMAIL"
                            } ]
                        }, {
                            goto: "NOTHING"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PHONE_AND_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact by mail <b> $CUSTOMER_EMAIL </b> and phone <b> $CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b> и телефону <b>$CUSTOMER_PHONE</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_EMAIL: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b>$CUSTOMER_NAME </b> and you prefer to contact by by mail <b>$CUSTOMER_EMAIL</b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по почте <b>$CUSTOMER_EMAIL</b>."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NOTHING: [ {
                        type: "text",
                        text: {
                            en: "$CUSTOMER_NAME, you have left no phone or mail.",
                            ru: "$CUSTOMER_NAME, Вы не оставили, ни телефона, ни почты."
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_NAME: [ {
                        type: "text",
                        text: {
                            en: "Please introduce yourself 🙏:",
                            ru: "Пожалуйста, введите своё имя 🙏:"
                        }
                    }, {
                        type: "input",
                        kind: "text",
                        actions: [ {
                            if: "len($INPUT) < 2",
                            actions: [ {
                                state: "CUSTOMER_WRONG_NAME"
                            } ]
                        }, {
                            else: "",
                            actions: [ {
                                state: "CUSTOMER_GREET"
                            }, {
                                slot: "CUSTOMER_NAME",
                                value: "$INPUT"
                            } ]
                        } ]
                    } ],
                    CUSTOMER_START: [ {
                        type: "actions",
                        actions: [ {
                            if: "not $CUSTOMER_NAME",
                            actions: [ {
                                goto: "CUSTOMER_NAME"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL and not $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_CHANNEL"
                            } ]
                        }, {
                            if: "not $CUSTOMER_EMAIL and $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_EMAIL_MAYBE"
                            } ]
                        }, {
                            if: "$CUSTOMER_EMAIL and not $CUSTOMER_PHONE",
                            actions: [ {
                                goto: "CUSTOMER_PHONE_MAYBE"
                            } ]
                        }, {
                            goto: "BEFORE_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_CHANNEL: [ {
                        type: "text",
                        text: {
                            en: "We may want to contact you. Is it more convenient for you to communicate by phone or email?",
                            ru: "Возможно, мы захотим связаться с Вами. Вам удобнее общаться по телефону или по электронной почте?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "📞 Phone",
                                ru: "📞 Телефон"
                            },
                            actions: [ {
                                state: "CUSTOMER_PHONE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "💻 Email",
                                ru: "💻 Почта"
                            },
                            actions: [ {
                                state: "CUSTOMER_EMAIL"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CUSTOMER_GREET: [ {
                        type: "text",
                        text: {
                            en: "Nice to meet you $CUSTOMER_NAME!",
                            ru: "Рад знакомству $CUSTOMER_NAME!"
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_START"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ONLY_PHONE: [ {
                        type: "text",
                        text: {
                            en: "And so, your name is <b> $CUSTOMER_NAME </b> and you prefer to contact phone <b>$CUSTOMER_PHONE </b>. ",
                            ru: "И так, Вас зовут <b>$CUSTOMER_NAME</b> и Вы предпочитаете связь по телефону <b>$CUSTOMER_PHONE</b>. "
                        }
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "CUSTOMER_CHECK"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: "",
                    MONEY: "",
                    CARDS: "",
                    DEALER_SCORE: "",
                    URL: "",
                    PREFIX: "",
                    BET: "",
                    CUSTOMER_NAME: "",
                    CUSTOMER_EMAIL: "",
                    CUSTOMER_PHONE: "",
                    CUSTOMER_MESSAGE: "",
                    COUNT: ""
                },
                cards: {
                    states: {
                        TABLE: {
                            c: 3,
                            r: 0,
                            x: 1195,
                            y: 129
                        },
                        MAIN: {
                            c: 0,
                            r: 0,
                            x: 26,
                            y: 10
                        },
                        SELECT_BET: {
                            c: 1,
                            r: 0,
                            x: 426,
                            y: 190
                        },
                        LOSE: {
                            c: 1,
                            r: 1,
                            x: 82,
                            y: 336
                        },
                        WIN: {
                            c: 0,
                            r: 1,
                            x: 386,
                            y: 446
                        },
                        PASS: {
                            c: 2,
                            r: 1,
                            x: 705,
                            y: 531
                        },
                        CHECK_WIN: {
                            c: 5,
                            r: 0,
                            x: 772,
                            y: 335
                        },
                        ADD_DEALER_CARDS: {
                            c: 4,
                            r: 0,
                            x: 1273,
                            y: 441
                        },
                        DEALER_TABLE: {
                            c: 2,
                            r: 0,
                            x: 826,
                            y: 43
                        },
                        NO_MONEY_FOR_BET: {
                            c: 3,
                            r: 1,
                            x: 283,
                            y: 29
                        },
                        NO_MONEY: {
                            c: 1,
                            r: 2,
                            x: 56,
                            y: 543
                        },
                        CUSTOMER_WRONG_NAME: {
                            c: 1,
                            r: 4,
                            x: 832,
                            y: 929.0070840450559,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL: {
                            c: 0,
                            r: 5,
                            x: 1085,
                            y: 1155.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL_WRONG: {
                            c: 1,
                            r: 5,
                            x: 1904,
                            y: 1076.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE: {
                            c: 0,
                            r: 6,
                            x: 318,
                            y: 1256.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE_WRONG: {
                            c: 1,
                            r: 6,
                            x: 719,
                            y: 1129.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_PHONE_MAYBE: {
                            c: 2,
                            r: 5,
                            x: 644,
                            y: 1390.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_EMAIL_MAYBE: {
                            c: 2,
                            r: 6,
                            x: 966,
                            y: 1574.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_MESSAGE: {
                            c: 2,
                            r: 3,
                            x: 1959,
                            y: 880.0124332656075,
                            folder: "Customer"
                        },
                        CUSTOMER_END: {
                            c: 1,
                            r: 3,
                            x: 1693,
                            y: 763.0124439610204,
                            folder: "Customer"
                        },
                        CUSTOMER_CHECK: {
                            c: 3,
                            r: 4,
                            x: 1564,
                            y: 941.0124332656075,
                            folder: "Customer"
                        },
                        BEFORE_CHECK: {
                            c: 3,
                            r: 5,
                            x: 1661,
                            y: 1379.007084045056,
                            folder: "Customer"
                        },
                        PHONE_AND_EMAIL: {
                            c: 0,
                            r: 7,
                            x: 1213,
                            y: 1322.007084045056,
                            folder: "Customer"
                        },
                        ONLY_EMAIL: {
                            c: 2,
                            r: 7,
                            x: 1452,
                            y: 1586.007084045056,
                            folder: "Customer"
                        },
                        NOTHING: {
                            c: 3,
                            r: 7,
                            x: 2023,
                            y: 1362.007084045056,
                            folder: "Customer"
                        },
                        CUSTOMER_NAME: {
                            c: 0,
                            r: 4,
                            x: 460,
                            y: 847.0070840450559,
                            folder: "Customer"
                        },
                        CUSTOMER_START: {
                            c: 0,
                            r: 3,
                            x: 813,
                            y: 777.0070840450559,
                            folder: "Customer"
                        },
                        CUSTOMER_CHANNEL: {
                            c: 3,
                            r: 3,
                            x: 1214,
                            y: 818.0070840450559,
                            folder: "Customer"
                        },
                        CUSTOMER_GREET: {
                            c: 2,
                            r: 4,
                            x: 430,
                            y: 1053.007084045056,
                            folder: "Customer"
                        },
                        ONLY_PHONE: {
                            c: 1,
                            r: 7,
                            x: 1210,
                            y: 1494.007084045056,
                            folder: "Customer"
                        }
                    },
                    folders: {
                        Customer: {
                            c: 0,
                            r: 2,
                            x: 308,
                            y: 728,
                            openFree: !0
                        }
                    }
                }
            },
            HELP: {
                id: "help",
                name: "Help",
                start: "GREET",
                version: "1.0.52",
                view: "grid",
                states: {
                    MAIN: [ {
                        type: "text",
                        text: {
                            en: "Messages that appear in the bot window are called <b>steps</b>. These steps are shown as cards on the right.",
                            ru: "Сообщения, появляющиеся в окне бота, называются <b>шагами</b>. Справа эти шаги изображены в виде карточек."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "You might want to know how to:",
                            ru: "Возможно, Вы хотите узнать, как:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Create steps",
                                ru: "Создавать шаги"
                            },
                            actions: [ {
                                state: "CREATE_STEP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Edit steps",
                                ru: "Редактировать шаги"
                            },
                            actions: [ {
                                state: "OPEN_PANEL"
                            } ]
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: ""
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🦉 FAQ",
                                ru: "🦉 Частые вопросы"
                            },
                            actions: [ {
                                state: "FAQ"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🌐 Change language",
                                ru: "🌐 Сменить язык"
                            },
                            actions: [ {
                                state: "CHANGE_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    COLOR_STATE: [ {
                        type: "text",
                        text: {
                            en: "The <b>pink</b> card is the one which is currently edited.",
                            ru: "<b>Малиновый цвет</b> означает выбранный шаг, который сейчас редактируется."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "The <b>blue</b> cards mark the steps, where a user may navigate to from the current one, by clicking a button, entering the text or after the timer runs out.",
                            ru: "<b>Голубой цвет</b> имеют карточки шагов, в которые можно перейти из выбранного шага при помощи кнопок, текстового ввода или таймера."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>Arrows</b> show navigation upon the button click.",
                            ru: "<b>Стрелки</b> показывают, из каких кнопок происходят переходы."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>Green dots</b> mark the steps which may precede the selected one.",
                            ru: "<b>Зелёные точки</b> показывают шаги, из которых можно попасть в выбранный шаг."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FAQ"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 На главную"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    LINKS: [ {
                        type: "text",
                        text: {
                            en: "To set navigation to any step, click the button in the editor, and then click the <b>step</b> field in the item properties. Then select the card to navigate to upon the button click.",
                            ru: "Чтобы задать переход, кликните на кнопку в редакторе.\nВ свойствах кнопки кликните на поле шага, а затем на карточку того состояния в который нужно перейти при нажатии кнопки."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "Navigation upon text entering, timer and actions are set in the same way.",
                            ru: "Аналогично делаются переходы при вводе текста, в таймере и действиях."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_links.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "EDIT_STEP_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "LINKS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    SCRIPT: [ {
                        type: "text",
                        text: {
                            en: "Scripts are used to process lists or to display dynamic content.",
                            ru: "При помощи скриптов работают со списками или выводят динамический контент."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "Scripts consist of actions, each of which is a function:",
                            ru: "Скрипты состоят из действий, каждое из которых - это функция:"
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>action1</b> (arg1, arg2, ...)\n<b>action2</b> (arg1, arg2, ...)",
                            ru: "<b>действие1</b>(аргумент1,аргумент2,...)\n<b>действие2</b>(аргумент1,аргумент2,...)"
                        },
                        pre: 1
                    }, {
                        type: "text",
                        text: {
                            en: "In its tern, some action arguments can be scripts.",
                            ru: "Некоторые аргументы действий, в свою очередь, могут быть скриптами."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Content output",
                                ru: "Вывод контента"
                            },
                            actions: [ {
                                state: "SCRIPT_CONTENT"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Common actions",
                                ru: "Общие действия"
                            },
                            actions: [ {
                                state: "SCRIPT_GENERAL"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "List processing",
                                ru: "Работа со списками"
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: []
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_CONTENT: [ {
                        type: "text",
                        text: {
                            en: "The visible items are set in the step editor. They can also be created dynamically within scripts.",
                            ru: "Визуальные элементы задаются в редакторе шага. Кроме этого, их можно создавать динамически внутри скриптов."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>text</b>&nbsp;- text output",
                                ru: "<b>text</b>&nbsp;- вывод текста"
                            },
                            actions: [ {
                                state: "SCRIPT_TEXT"
                            } ],
                            align: "left",
                            kind: "checkbox"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>image</b>&nbsp;- display images",
                                ru: "<b>image</b>&nbsp;- вывод изображения"
                            },
                            actions: [ {
                                state: "SCRIPT_IMAGE"
                            } ],
                            align: "left",
                            kind: "checkbox"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>get_check</b>&nbsp;- add checkbox",
                                ru: "<b>get_check</b>&nbsp;- чекбоксы"
                            },
                            actions: [ {
                                state: "SCRIPT_CHECK"
                            } ],
                            kind: "checkbox",
                            checked: !1,
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_LIST: [ {
                        type: "text",
                        text: {
                            en: "Lists are an important slot type. For example, they may store user orders or other data.",
                            ru: "Списки - важный вид слотов. В них можно сохранять, например, сделанный пользователем заказ и другие данные."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>clear</b>&nbsp;- clear the list",
                                ru: "<b>clear</b>&nbsp;- очистка списка"
                            },
                            actions: [ {
                                state: "SCRIPT_CLEAR"
                            } ],
                            align: "left",
                            kind: "checkbox"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>len</b>&nbsp;- number of elements",
                                ru: "<b>len</b>&nbsp;- число элементов "
                            },
                            actions: [ {
                                state: "SCRIPT_LEN"
                            } ],
                            align: "left",
                            kind: "checkbox"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>push</b>&nbsp;- add in the end",
                                ru: "<b>push</b>&nbsp;- добавить в конец"
                            },
                            actions: [ {
                                state: "SCRIPT_PUSH"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>add</b>&nbsp;- add elements",
                                ru: "<b>add</b>&nbsp;- добавить элементы "
                            },
                            actions: [ {
                                state: "SCRIPT_ADD"
                            } ],
                            align: "left",
                            kind: "checkbox"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>sum</b>&nbsp;- sum up the values",
                                ru: "<b>sum</b>&nbsp;- сумма значений"
                            },
                            actions: [ {
                                state: "SCRIPT_SUM"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>total</b>&nbsp;- expression sum",
                                ru: "<b>total</b>&nbsp;- сумма выражения"
                            },
                            actions: [ {
                                state: "SCRIPT_TOTAL"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>get</b>&nbsp;- list element",
                                ru: "<b>get</b>&nbsp;- элемент списка"
                            },
                            actions: [ {
                                state: "SCRIPT_GET"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>foreach</b>&nbsp;- for all elements",
                                ru: "<b>foreach</b>&nbsp;- для всех элементов"
                            },
                            actions: [ {
                                state: "SCRIPT_FOREACH"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_CLEAR: [ {
                        type: "text",
                        text: {
                            en: "<b> clear </b> (LIST) clears the LIST slot. After that it will be an empty list [].",
                            ru: "<b>clear</b>(LIST) - очищает слот с именем LIST. После этого он будет пустым списком []."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>set</b>  (LIST, [1,2,3,4])\n<b>clear</b>(LIST)",
                            ru: "<b>set</b>  (LIST, [1,2,3,4])\n<b>clear</b>(LIST)"
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_ADD: [ {
                        type: "text",
                        text: {
                            en: '<b>add</b> (list, agrg, keys [, cond], item)  adds the "item" element to the LIST list, which is an object, consisting of keys and values.',
                            ru: "<b>add</b>(list, agrg, keys [, cond], item) - добавить в список list элемент item, являющийся объектом (состоит из ключей и значений). "
                        }
                    }, {
                        type: "text",
                        text: {
                            en: 'The "agrg" list contains the keys to be aggregated (incremented) when an item is added, in case the keys match the values from the "keys" list.\nIf there is no match, then the item is added to the end of the list. If there is a "cond" argument, then it may have a boolean condition containing the keys of the list elements. Adding (or aggregating) will only happen if this condition is true.',
                            ru: "В списке agrg перечислятся ключи, которые необходимо проагрегировать (увеличить) при добавлении item. В списке keys перечисляются ключи, при совпадении с которыми это необходимо делать. Если совпадения не произошло, то item добавляется в конец списка. Если есть аргумент cond, то в нём может быть логическое условие, содержащее ключи элементов списка. Добавление (или агрегирование) произойдёт только, если это условие истинно."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_TEXT: [ {
                        type: "text",
                        text: {
                            en: "<b>text </b> (string) outputs the string to a message. The string can be multilingual, in which case it is passed as an object (in curly brackets) with two-letter language codes and texts as values:",
                            ru: "<b>text</b>(string) - вывод строки в сообщение. Строка может быть мультиязычной. В этом случае она передаётся как объект (в фигурных скобках) с ключами языков (два символа) и текстами в качестве  значений:"
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b>text</b>("Hello, World!") \n<b>text</b>({\n    en: "Hello, World!",\n    es: "¡Hola, mundo!"\n})',
                            ru: '<b>text</b>("Hello, World!") \n<b>text</b>({\n    en: "Hello, World!",\n    ru: "Привет, мир!"\n})'
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_CONTENT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_IMAGE: [ {
                        type: "text",
                        text: {
                            en: '<b>image</b> (url) displays the image in a message. This action is to be preceded by the <b>images</b> action. All the following "image" items will be displayed in one string.',
                            ru: "<b>image</b>(url) - вывод изображения в сообщение. Перед этим действием, необходимо поставить действие <b>images</b>. Все идущие после него image будут выведены в одну строку."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b>set</b>  (URL, "https://qu.com/q/im/")     \n<b>images</b>()     \n<b>image</b>(\\$URL + "owl_a.png")     \n<b>image</b>(\\$URL + "owl_b.png")  ',
                            ru: '<b>set</b>  (URL, "https://qu.com/q/im/")     \n<b>images</b>()     \n<b>image</b>(\\$URL + "owl_a.png")     \n<b>image</b>(\\$URL + "owl_b.png")  '
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_CONTENT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CHANGE_LANGUAGE: [ {
                        type: "actions",
                        actions: [ {
                            state: "MAIN"
                        } ]
                    }, {
                        type: "text",
                        text: {
                            en: "Select the language to proceed.",
                            ru: "На каком языке Вы хотите общаться?",
                            ua: "Якою мовою Ви бажаєте спілкуватись?",
                            de: "In welcher Sprache möchten Sie kommunizieren?",
                            fr: "Dans quelle langue souhaitez-vous communiquer ?",
                            es: "¿En qué idioma quieres comunicarte?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "English",
                                ru: "English",
                                ua: "English",
                                de: "English",
                                fr: "English",
                                es: "English"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "en"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Russian",
                                ru: "Русский",
                                ua: "Русский",
                                de: "Русский",
                                fr: "Русский",
                                es: "Русский"
                            },
                            actions: [ {
                                slot: "LANGUAGE",
                                value: "ru"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    START_STEP: [ {
                        type: "text",
                        text: {
                            en: "The bot starts a dialog from the initial cart indicated as <b>start</b> (in the top right). It also has a bold border.",
                            ru: "Бот начинает диалог с начального шага. У такой карточки написано <b>start</b> в правом верхнем углу, и она имеет более жирную рамку."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_start_vis.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "SELECT_START"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    GREET: [ {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "text",
                        text: {
                            en: "Welcome to the QuBot editor.",
                            ru: "Добро пожаловать в редактор QuBot."
                        }
                    }, {
                        type: "message",
                        typing: "1000"
                    }, {
                        type: "actions",
                        actions: [ {
                            goto: "MAIN"
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FAQ: [ {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "How to set the default language?",
                                ru: "Как изменить начальный язык?"
                            },
                            actions: [ {
                                state: "START_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Where to get the map embed code?",
                                ru: "Где взять ссылку для карты?"
                            },
                            actions: [ {
                                state: "MAP_LINK"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Where to copy the image URL?",
                                ru: "Где взять ссылку для картинки?"
                            },
                            actions: [ {
                                state: "IMG_LINK"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "What do the card colors mean?",
                                ru: "Что означают цвета карточек ?"
                            },
                            actions: [ {
                                state: "COLOR_STATE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FOLDERS: [ {
                        type: "text",
                        text: {
                            en: '<b class="green">Folders</b> contain step cards.',
                            ru: '<b class="green">Папки</b> содержат карточки шагов. '
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "In the <b>Grid</b> view mode, you can click on a folder to open it and see all the containing cards.",
                            ru: "В режиме <b>View</b> - <b>Grid</b>, клик на папку открывает или закрывает её. В открытом состоянии видны все карточки, находящиеся внутри папки."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "EDIT_STEPS_IN_DIR"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_POSITION: [ {
                        type: "text",
                        text: {
                            en: "There are two modes of card replacement: <b>Grid</b> and <b>Free</b>. You can swich between them in <b>View</b>.",
                            ru: "Есть два режима упорядочивания карточек шагов: <b>Grid</b> и <b>Free</b>. Они переключаются в меню <b>View</b>."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_view.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "GRID_VIEW"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ITEMS: [ {
                        type: "text",
                        text: {
                            en: "Steps may have <b>visible</b> items, displayed to a user, and <b>logical</b> items, which a user can't see and are only used for the bot logic.",
                            ru: "В шаге могут использовать <b>визуальные</b> элементы, которые видны пользователю, и <b>логические</b> элементы, которые пользователь не видит и которые отвечают за логику бота."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "EDIT_STEP_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "VISUAL_ITEMS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    GOTO: [ {
                        type: "text",
                        text: {
                            en: "There are 3 types of navigation.",
                            ru: "Существует три вида переходов."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b class="green">step</b> is triggered upon an event. It may be set for a button or in the input field properties.',
                            ru: '<b class="green">step</b> - переход по событию. Он задаётся, например, в кнопке или свойствах поля ввода.'
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b class="green">goto</b> is the instant navigation which needs no event. All further properties and items below the "goto" are ignored.',
                            ru: '<b class="green">goto</b> - мгновенный переход без ожидания события. Все дальнейшие свойства и элементы ниже goto игнорируются.'
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b class="green">run</b> runs the selected step (in the current step, all items are displayed and actions are performed). Then we return to where "run" was called.',
                            ru: '<b class="green">run</b> - выполняет указанный шаг (в текущем шаге рисуются его элементы и совершаются его действия). Затем происходит возврат на то место, откуда run был вызван.'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "LINKS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    DEF_STEP: [ {
                        type: "text",
                        text: {
                            en: 'If the "step" property is not set for a button, text, or timer, then when an event occurs, the current step will run again.',
                            ru: "Если свойство step в  кнопке (тексте, таймере) не задано, то при возникновении события будет повторно показан текущий шаг."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: 'The default navigation can be defined in the "actions" (rocket) at the start of the step. It will be if the event object does not have the "step" property.',
                            ru: "Переход по умолчанию можно  определить в действиях (ракета) в начале шага. Этот переход будет использован, если в событийном объекте нет свойства step."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "LINKS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ACTIONS: [ {
                        type: "text",
                        text: {
                            en: "Actions (rocket) may contain various items: conditions (<b>if</b>), branch statements (<b>switch</b>), setting slots (variables), scripts, etc. All of them are visible when you click the plus panel in the item properties.",
                            ru: "Действия (ракета) могут содержать различные элементы: условия (<b>if</b>), операторы ветвления (<b>switch</b>), установку слотов (переменных), скрипты и т.п. Все они видны при нажатии на плюс панели свойств элемента."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "What are scripts?",
                                ru: "Что такое скрипты?"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "EDIT_STEP_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    MAP_LINK: [ {
                        type: "text",
                        text: {
                            en: 'Open <a href="https://www.google.com/maps/"  target="_blank">Google Maps</a>, and click the menu icon (3 horizontal lines) in the top left. Select "Share or embed map", switch to the "Embed a map" tab, and click "COPY HTML". Paste the link to the <b>map</b> item.',
                            ru: 'Необходимо зайти на <a href="https://www.google.com.ua/maps/"  target="_blank">Google Map</a>.  В левом верхнем углу нажать на три параллельные линии. Выбрать "Ссылка/код" и закладку "Встраивание карт". Затем нажать "КОПИРОВАТЬ HTML". Полученную ссылку вставить в элемент <b>map</b>.'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FAQ"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    IMG_LINK: [ {
                        type: "text",
                        text: {
                            en: "You may use any image URL on the web.",
                            ru: "Ссылки на картинки можно вставлять из любых ресурсов интернета."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "It's a good idea to upload images to the WordPress media library and use them for the purpose.",
                            ru: "Хорошей идей является использование изображений, загруженных в медиа галерею WordPress."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FAQ"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    START_LANGUAGE: [ {
                        type: "text",
                        text: {
                            en: "The default language can be set in the start step. Add <b>actions</b> (rocket) as the first item, and set the required language value (en, ru, ...) in the LANGUAGE slot.",
                            ru: "Начальный язык можно задать в стартовом состоянии. Для этого первым элементом ставятся действия (ракета). В них добавляется слот LANGUAGE со значением нужного языка (en, ru, ...)"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FAQ"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_CHECK: [ {
                        type: "text",
                        text: {
                            en: '<b>get_check</b> (CHECKS) places the values of buttons marked as "check" into the CHECKS slot list (see Examples -Checkboxes).',
                            ru: "<b>get_check</b>(CHECKS) - помещает значения кнопок, помеченных как чекбоксы, в слот-список CHECKS.  См. пример Checkboxes."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_CONTENT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_RADIO: [ {
                        type: "text",
                        text: {
                            en: '<b>get_radio</b> (CHECKS) places the values of buttons marked as "radio" into the CHECKS slot list (see Examples -Checkboxes).',
                            ru: "<b>get_radio</b>(CHECKS) - помещает значения кнопок, помеченных как radio,  кнопки в слот-список CHECKS.  См. пример Checkboxes."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_CONTENT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_GENERAL: [ {
                        type: "text",
                        text: {
                            en: "These actions allow you to change slots, check conditions, and perform repetitive actions.",
                            ru: "Эти действия позволяют менять слоты, проверять условия и совершать повторяющиеся действия."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>set</b> &nbsp;- slot value setting",
                                ru: "<b>set</b>&nbsp;- задание значения слота"
                            },
                            actions: [ {
                                state: "SCRIPT_SET"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>if</b> &nbsp;- conditional operator",
                                ru: "<b>if</b> &nbsp;- условный оператор"
                            },
                            actions: [ {
                                state: "SCRIPT_IF"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "<b>while</b>&nbsp;- loop",
                                ru: "<b>while</b>&nbsp;- цикл"
                            },
                            actions: [ {
                                state: "SCRIPT_WHILE"
                            } ],
                            align: "left"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_IF: [ {
                        type: "text",
                        text: {
                            en: '<b>if</b> (cond, script_true [, script_false])  evaluates the boolean expression of "cond". If it is true, then the "script_true" script is executed. Otherwise the "script_false" script is executed (if applicable).',
                            ru: "<b>if</b>(cond, script_true [, script_false]) - вычисление логического выражения cond. Если оно истинно, то выполняется скрипт script_true, иначе - скрипт script_false (последнего может не быть). "
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "These two scripts have to be separated by a comma.",
                            ru: "Между этими двумя скриптами должна быть запятая."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b>if</b>(\\$NUM1 > 1 or \\$STR1 == "Wine", \n    <b>set</b> (NUM2, 5)\n    <b>text</b>("\\$NUM2")\n    ,\n    <b>text</b>("false")\n)',
                            ru: '<b>if</b>(\\$NUM1 > 1 or \\$STR1 == "Wine", \n    <b>set</b> (NUM2, 5)\n    <b>text</b>("\\$NUM2")\n    ,\n    <b>text</b>("false")\n)'
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_GENERAL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_SET: [ {
                        type: "text",
                        text: {
                            en: '<b>set</b> (slot, value) sets the slot named "slot" to "value". If the slot with this name is missing, then it is added to the slot list.',
                            ru: "<b>set</b>(slot, value) - установка слота с именем slot в значение value. Если слота с таким именем не было, то он добавляется в список слотов."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "There is no need to add <b>\\$</b> before the slot name in the first argument. However the second argument has to have it, in case it is an expression aimed to obtain the value of a certain slot.",
                            ru: "Символ \\$ перед именем слота в первом аргументе ставить не нужно. Во втором аргументе он должен быть, если это выражение, в котором нужно получить значение некоторого слота."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b>set</b>(NUM1, 3.14)\n<b>set</b>(STR1, "Wine")\n<b>set</b>(NUM2, 2 * \\$NUM1 )\n<b>set</b>(STR2, \\$STR1 + " and cheese" )\n<b>set</b>(LIST, [1, 2, "Wine"] )\n<b>set</b>(OBJ,  { n: 1, s: "Wine"  } )',
                            ru: '<b>set</b>(NUM1, 3.14)\n<b>set</b>(STR1, "Wine")\n<b>set</b>(NUM2, 2 * \\$NUM1 )\n<b>set</b>(STR2, \\$STR1 + " and cheese" )\n<b>set</b>(LIST, [1, 2, "Wine"] )\n<b>set</b>(OBJ,  { n: 1, s: "Wine"  } )'
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_GENERAL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_WHILE: [ {
                        type: "text",
                        text: {
                            en: '<b>while</b> (cond, script) keeps running the "script" script while the boolean expression "cond" is true.',
                            ru: "<b>while</b>(cond, script) - повторное выполнение скрипта script, пока истинно логическое выражение cond. "
                        }
                    }, {
                        type: "text",
                        text: {
                            en: 'The script has to be subject to changes, which in its turn changes the truth of "cond". To be on the safe side, the number of circles is limited to 1000.',
                            ru: "Естественно, в скрипте должны происходить изменения, меняющие истинность cond. Для защиты, число повторов ограничено 1000."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: 'set(INDEX, 1)\nwhile(\\$INDEX <= 10, \n    text("\\$INDEX")\n    set (INDEX, \\$INDEX+1)\n)',
                            ru: '<b>set</b>(INDEX, 1)\n<b>while</b>(\\$INDEX <= 10, \n    <b>text</b>("\\$INDEX")\n    <b>set</b> (INDEX, \\$INDEX+1)\n)'
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_GENERAL"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_SUM: [ {
                        type: "text",
                        text: {
                            en: '<b> sum </b> (list, result) sums up the numeric elements of the LIST, and save the result in the "result" slot.',
                            ru: "<b>sum</b>(list, result) - просуммировать числовые элементы списка list, и поместить результат в слот с именем result."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_TOTAL: [ {
                        type: "text",
                        text: {
                            en: "",
                            ru: "<b>total</b>(list, result, expr) - просуммировать значения выражения expr, применяемого к каждому элементу списка list. Результат помещается в слот с именем result."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>push</b> (LIST, { n: 1, p: 10 } )     \n<b>push</b> (LIST, { n: 2, p: 20 } )     \n<b>total</b>(LIST, TOT, \\$n * \\$p)     ",
                            ru: "<b>push</b> (LIST, { n: 1, p: 10 } )     \n<b>push</b> (LIST, { n: 2, p: 20 } )     \n<b>total</b>(LIST, TOT, \\$n * \\$p)     "
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_FOREACH: [ {
                        type: "text",
                        text: {
                            en: "",
                            ru: "<b>foreach</b>(list [, cond], script) - выполнить скрипт script для каждого элемента списка list, если выполняется условие cond (оно может остутствовать). "
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "",
                            ru: "В действиях скрипта можно использовать ключи элементов списка (ставя перед ними, как обычно, $). Если такого ключа в элементе нет, то ищется слот с таким именем."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b>push</b>(LIST, \n{ amount: 1, price: 10, name: "Wine"  } \n)     \n<b>push</b>(LIST, \n{ amount: 2, price: 20, name: "Cheese"} \n)     \n\n<b>foreach</b>(LIST, \n    <b>text</b>("\\$name:  \\{\\{\\$amount * \\$price\\}\\}")\n)     ',
                            ru: '<b>push</b>(LIST, \n{ amount: 1, price: 10, name: "Wine"  } \n)     \n<b>push</b>(LIST, \n{ amount: 2, price: 20, name: "Cheese"} \n)     \n\n<b>foreach</b>(LIST, \n    <b>text</b>("\\$name:  \\{\\{\\$amount * \\$price\\}\\}")\n)     '
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_GET: [ {
                        type: "text",
                        text: {
                            en: "",
                            ru: "<b>get</b>(list, item, index) - получить элемент списка list с номером index и записать его в слот item. Нумерация начинается с нуля. Если index = -1, то это последний элемент списка, -2 предпоследний и т.д."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>push</b>(LIST, { n: 1, p: 10 } )     \n<b>push</b>(LIST, { n: 2, p: 20 } )     \n<b>get</b> (LIST, ITEM, -1)     ",
                            ru: "<b>push</b>(LIST, { n: 1, p: 10 } )     \n<b>push</b>(LIST, { n: 2, p: 20 } )     \n<b>get</b> (LIST, ITEM, -1)     "
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_PUSH: [ {
                        type: "text",
                        text: {
                            en: "<b>push</b> (list, item1 [, item2, ...]) adds item1, item2, etc. elements to the end of the list. If there is no LIST slot, or it is not a list, then it is first created as an empty list, and then the elements are added to it.",
                            ru: "<b>push</b>(list, item1 [, item2,...]) - добавить в конец списка list элементы item1, item2 и т.д. Если слота list нет или он не список, то он сначала создаётся как пустой список, а потом в него помещаются элементы."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: '<b>push</b>(LIST, \n     1, 2, "Wine", \n     {a: 1, b: "cheese"}\n)',
                            ru: '<b>push</b>(LIST, \n     1, 2, "Wine", \n     {a: 1, b: "cheese"}\n)'
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    SCRIPT_LEN: [ {
                        type: "text",
                        text: {
                            en: "<b>len</b> (LIST, SIZE) writes the number of elements of the LIST slot to the SIZE slot:",
                            ru: "<b>len</b>(LIST, SIZE) - число элементов списка (слота)  с именем LIST записывает в слот c именем SIZE:"
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "<b>set</b>(LIST, [1,2,3,4])\n<b>len</b>(LIST, SIZE)\n<b>if</b>(\\$SIZE == 4,\n   <b>text</b>('Two items')\n)",
                            ru: "<b>set</b>(LIST, [1,2,3,4])\n<b>len</b>(LIST, SIZE)\n<b>if</b>(\\$SIZE == 4,\n   <b>text</b>('Two items')\n)"
                        },
                        pre: 1
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ "
                            },
                            actions: [ {
                                state: "SCRIPT_LIST"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "⚙️",
                                ru: "⚙️"
                            },
                            actions: [ {
                                state: "SCRIPT"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 ",
                                ru: "🏠"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    COPY_STEPS: [ {
                        type: "text",
                        text: {
                            en: "To copy a step, click it and then select <b>Edit Step</b> - <b>Copy</b>.",
                            ru: "Чтобы скопировать шаг, выберите его в списке карточек и нажмите <b>Copy</b> в меню <b>Edit Step</b>"
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_copy.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "PASTE_STEPS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CREATE_STEP: [ {
                        type: "text",
                        text: {
                            en: "In order to create a new step, hover a step and click the plus icon on the right.",
                            ru: "Чтобы создать новый шаг, наведите мышь на уже существующий шаг в списке карточек и нажмите плюсик справа на рамке."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_new.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "REMOVE_STEP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    REMOVE_STEP: [ {
                        type: "text",
                        text: {
                            en: "Click on the cross in the top right to delete the step.",
                            ru: "Крестик вверху удаляет шаг."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_del.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "CREATE_STEP"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "CREATE_LINE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    CREATE_LINE: [ {
                        type: "text",
                        text: {
                            en: "To start a new line of cards (<b>View</b> - <b>Grid</b>), click the plus icon on the grey menu at the start of the line.",
                            ru: "Чтобы добавить карточку в новый ряд  (меню <b>View</b> - <b>Grid</b>), нажмите плюсик на сером меню в начале ряда."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_line.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "REMOVE_STEP"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ],
                    NEXT_STEP_INFO: [ {
                        type: "text",
                        text: {
                            en: "Learn more about card editing:",
                            ru: "Может быть, Вы хотите узнать о более сложных элементах шагов, например: "
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Select the starting card",
                                ru: "Задать начальный шаг"
                            },
                            actions: [ {
                                state: "START_STEP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Copy steps",
                                ru: "Копировать шаги"
                            },
                            actions: [ {
                                state: "COPY_STEPS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Replace cards",
                                ru: "Изменить положение карточки"
                            },
                            actions: [ {
                                state: "STEP_POSITION"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Edit folders",
                                ru: "Работа с папками"
                            },
                            actions: [ {
                                state: "FOLDERS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Templates",
                                ru: "Шаблоны"
                            },
                            actions: [ {
                                state: "INSERT_TEMPLATE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    SELECT_START: [ {
                        type: "text",
                        text: {
                            en: "Any step can be assigned as the starting one. Just click the step, and then tick the checkbox in the top right of the editor panel.",
                            ru: "Начальным шагом можно сделать любой шаг. Для этого необходимо кликнуть мышью на шаг и в панели редактора  шага нажать чекбокс в правом верхнем углу."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_start_sel.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "START_STEP"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PASTE_STEPS: [ {
                        type: "text",
                        text: {
                            en: "To paste a copied step after the selected one, click it and then select <b>Edit Step</b> - <b>Paste</b>.",
                            ru: "Чтобы вставить скопированный шаг, выберите шаг, после которого необходимо вставить скопированный и нажмите <b>Paste</b> в <b>Edit Step</b>"
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_paste.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "COPY_STEPS"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    GRID_VIEW: [ {
                        type: "text",
                        text: {
                            en: 'In the <b class="green">Grid</b>  mode, cards are cards are arranged as a table.',
                            ru: 'В режиме <b class="green">Grid</b>  карточки упорядочены в виде таблицы'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_grid.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "STEP_POSITION"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "FREE_VIEW"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FREE_VIEW: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Free</b> view mode  allows to place cards randomly. ',
                            ru: 'В режиме <b class="green">Free</b>  карточки расположены произвольно.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_free.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "GRID_VIEW"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "DRAG_STEP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    DRAG_STEP: [ {
                        type: "text",
                        text: {
                            en: "Hover the top of the card to see a hand icon, then drag and drop the card to replace it. ",
                            ru: "Карточки можно таскать. Для этого нужно нажать на шапку и перетащить в необходимую позицию."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_drag.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FREE_VIEW"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    EDIT_STEPS_IN_DIR: [ {
                        type: "text",
                        text: {
                            en: "Cards in the folder can be edited just the same way as any other cards.",
                            ru: "Когда папка раскрыта, её карточки можно редактировать так же, как и любые карточки вне папки."
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FOLDERS"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FOLDER_MENU: [ {
                        type: "text",
                        text: {
                            en: "Learn more about folder editing.",
                            ru: "Может быть, Вы хотите подробнее узнать о работе с папками, например:"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Create / Delete folders",
                                ru: "Создать / удалить папку"
                            },
                            actions: [ {
                                state: "CREATE_FOLDER"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Edit folders",
                                ru: "Редактировать папку"
                            },
                            actions: [ {
                                state: "RENAME_FOLDER"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Copy folders",
                                ru: "Копировать папку"
                            },
                            actions: [ {
                                state: "COPY_FOLDER"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Add / Remove cards",
                                ru: "Добавить / удалить карточку"
                            },
                            actions: [ {
                                state: "STEP_MOVE_IN_DIR"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Menu",
                                ru: "⇧ В меню"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    CREATE_FOLDER: [ {
                        type: "text",
                        text: {
                            en: "To create a folder, click <b>Edit Step</b> -<b>New folder</b>, enter a name in the popup, and click <b>Ok</b>.",
                            ru: "Чтобы создать папку, нажмите <b>New folder</b> в меню <b>Edit Step</b>, затем введите имя в появившемся окне и нажмите <b>Ok</b>."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder_add.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "REMOVE_FOLDERS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    REMOVE_FOLDERS: [ {
                        type: "text",
                        text: {
                            en: "To delete a folder, click on the cross in the top right.",
                            ru: "Чтобы удалить папку, кликните на крестик в верхнем правом углу папки."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder_rem.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "CREATE_FOLDER"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    COPY_FOLDER: [ {
                        type: "text",
                        text: {
                            en: "To copy a folder, click to open it, and then select <b>Edit Step</b> - <b>Copy Folder</b>.",
                            ru: "Чтобы копировать папку, раскройте необходимую папку. Нажмите на <b>Copy Folder</b> в меню <b>Edit Step</b>"
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder_copy.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "PASTE_FOLDER"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PASTE_FOLDER: [ {
                        type: "text",
                        text: {
                            en: "To paste a folder after the current card, select <b>Edit Step</b> - <b>Paste Folder</b>.",
                            ru: "Чтобы вставить папку, выберите карточку, после которой будет вставлена папка, и нажмите <b>Paste Folder</b> в меню <b>Edit Step</b>"
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder_paste.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "COPY_FOLDER"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_MOVE_IN_DIR: [ {
                        type: "text",
                        text: {
                            en: "To place a card in the folder, hover over the top border to see a hand icon, then drag and drop it in the folder.",
                            ru: "Чтобы поместить карточку в папку, нужно схватить за её заголовок и перетащить, отпустив на папке. "
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_step_in_dir.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "FOLDER_LINE"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    STEP_MOVE_OUT_DIR: [ {
                        type: "text",
                        text: {
                            en: "",
                            ru: "Аналогично карточки перетаскиваются из раскрытой папки во внешний список карточек."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_step_out_dir.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FOLDER_LINE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    RENAME_FOLDER: [ {
                        type: "text",
                        text: {
                            en: "To rename a folder, click it and select <b>Edit Step</b> - <b>Rename Folder</b>. Enter the name in the popup and click Ok.",
                            ru: "Чтобы переименовать папку раскройте её, нажмите <b>Rename Folder</b> в меню <b>Edit Step</b>. В появившемся окне введите имя папки и нажмите Ok."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder_rename.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "FOLDER_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    FOLDER_LINE: [ {
                        type: "text",
                        text: {
                            en: "To start a new line of cards in the folder (<b>View</b> - <b>Grid</b>), open  the folder, click the plus icon on the grey menu at the start of the line.",
                            ru: "Чтобы добавить карточку в новый ряд внутри папки (меню <b>View</b> - <b>Grid</b>), откройте папку и нажмите плюсик на сером меню в начале ряда."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_folder_new_line.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "STEP_MOVE_IN_DIR"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "STEP_MOVE_OUT_DIR"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    INSERT_TEMPLATE: [ {
                        type: "text",
                        text: {
                            en: "The bot editor contains localized step templates (e.g. <b>Add Yes/No</b> and <b>Add Change Language</b>). It helps to easily create most common bot steps.",
                            ru: 'В конструкторе существуют шаблонные шаги, такие как <b>"Да/Нет"</b> и <b>"Выбор языка"</b>, переведённые на поддерживаемые языки. Основное их назначение - ускорение разработки ботов, ведь данные шаги встречаются во многих ботах.'
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "INSERT_YES_NO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    INSERT_YES_NO: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Add Yes/No</b> card template adds a step with a text field and the two buttons. включает в себя две кнопки и текст.\nSelect <b>Edit Step</b> - <b>Add Yes/No</b> to create it after the current step.',
                            ru: 'Шаблон шага <b class="green">"Да/Нет"</b> включает в себя две кнопки и текст.\nЧтобы добавить шаг в бота, нажмите <b>Add Yes/No</b> в меню <b>Edit Step</b>'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_yes_no.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "INSERT_TEMPLATE"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "INSERT_ADD_LANGUAGE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    INSERT_ADD_LANGUAGE: [ {
                        type: "text",
                        text: {
                            en: '<b class="green">Add Change Language</b> is a step template with 6 language options and a text suggesting to change the language.\nClick a card and select <b>Edit Step</b> - <b>Add Change Language</b> to create it.',
                            ru: 'Шаблон шага <b class="green">"Выбор языка"</b> включает в себя 6 языков и текстовое поле с предложением выбрать язык.\nЧтобы добавить шаг в бота, нажмите <b>Add Change Language</b> в меню <b>Edit Step</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/cr_ch_language.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "INSERT_YES_NO"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "NEXT_STEP_INFO"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    OPEN_PANEL: [ {
                        type: "text",
                        text: {
                            en: "To edit the selected step, click the card on on the right. The step will open in the editor window on the left, where you can modify it.",
                            ru: "Для редактирования выбранного шага, нужно кликнуть на одну из карточек справа. Карточка будет открыта в окне редактора слева, в котором её можно изменить."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_panel.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "EDIT_STEP_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    EDIT_STEP_MENU: [ {
                        type: "text",
                        text: {
                            en: "Learn more about the step editing.",
                            ru: "Может быть, Вы хотите узнать больше о редактировании шагов?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "What does a step contain?",
                                ru: "Что может быть в шаге?"
                            },
                            actions: [ {
                                state: "ITEMS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "How do I set navigation?",
                                ru: "Как задавать переходы?"
                            },
                            actions: [ {
                                state: "LINKS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "What are actions?",
                                ru: "Что такое действия?"
                            },
                            actions: [ {
                                state: "ACTIONS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        actions: []
                    } ],
                    VISUAL_ITEMS: [ {
                        type: "text",
                        text: {
                            en: "The <b>visible</b> items are typing effects, texts, images, maps and buttons.",
                            ru: "К <b>визуальным</b> элементам относятся эффекты тайпинга, тексты, изображения, карты, кнопки."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_visual.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEMS"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "LOGICAL_ITEMS"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    LOGICAL_ITEMS: [ {
                        type: "text",
                        text: {
                            en: "The <b>logical</b> items are actions, timers and the NLU.",
                            ru: "К <b>логическим</b> элементам относятся действия, таймеры, NLU."
                        }
                    }, {
                        type: "text",
                        text: {
                            en: "Logical items are not displayed in the card previews.",
                            ru: "Логические элементы не видны на предпросмотре."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_logical.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "VISUAL_ITEMS"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ITEMS_MENU: [ {
                        type: "text",
                        text: {
                            en: "Learn more about the item editing.",
                            ru: "Может быть, Вы хотите узнать подробнее о работе с элементами?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "How to create an item?",
                                ru: "Как добавить элемент?"
                            },
                            actions: [ {
                                state: "ADD_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "How to copy an item?",
                                ru: "Как копировать элемент?"
                            },
                            actions: [ {
                                state: "COPY_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "How to delete an item?",
                                ru: "Как удалить элемент?"
                            },
                            actions: [ {
                                state: "REMOVE_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "How to replace an item?",
                                ru: "Как переместить элемент?"
                            },
                            actions: [ {
                                state: "DRAG_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "What item properties are used?",
                                ru: "Подробнее о элементах"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Menu",
                                ru: "В меню "
                            },
                            actions: [ {
                                state: "EDIT_STEP_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ADD_ITEM: [ {
                        type: "text",
                        text: {
                            en: "To create a new item, click on the plus icon on the left of the previous item, and select the item kind in the popup.",
                            ru: "Чтобы добавить элемент, нажмите на плюсик на серой панели слева элемента, после которого необходимо вставить элемент, а затем выберите элемент в появившемся меню."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_add_item.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    COPY_ITEM: [ {
                        type: "text",
                        text: {
                            en: "To copy the selected item, click the double file icon on the grey popup on its left.",
                            ru: "Чтобы скопировать элемент, нажмите на значок с двумя файлами на серой панели слева элемента, который необходимо скопировать."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_copy_item.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "PASTE_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    PASTE_ITEM: [ {
                        type: "text",
                        text: {
                            en: "To paste the copied item, click on the icon featuring a file with an arrow on the left of the previous item.",
                            ru: "Чтобы вставить скопированный объект, нажмите иконку файла со стрелкой на серой панели напротив элемента, после которого нужно вставить скопированный."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_paste_item.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "COPY_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    REMOVE_ITEM: [ {
                        type: "text",
                        text: {
                            en: "To delete an item, click the trash icon on the grey popup panel on the item left.",
                            ru: "Чтобы удалить элемент, нажмите на иконку корзины на серой панели слева от элемента, который необходимо удалить."
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_rem_item.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    DRAG_ITEM: [ {
                        type: "text",
                        text: {
                            en: "Text",
                            ru: "Чтобы переместить элемент нажмите на иконку с точками на серой панели слева от элемента и переместите элемент на необходимую позицию на панели редактирования"
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_drag_item.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Button",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ITEM_PROPERTIES: [ {
                        type: "text",
                        text: {
                            en: "Item properties are differ for each item kind. Which item kind would you like to learn more about?",
                            ru: "В зависимости от типа, у элементов могут быть различные свойства. О каком элементе Вы хотите узнать подробнее?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Text",
                                ru: "Text"
                            },
                            actions: [ {
                                state: "TEXT_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Image",
                                ru: "Image"
                            },
                            actions: [ {
                                state: "IMAGE_PROP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Map",
                                ru: "Map"
                            },
                            actions: [ {
                                state: "MAP_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Block",
                                ru: "Block"
                            },
                            actions: [ {
                                state: "BLOCK_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "NLU",
                                ru: "NLU"
                            },
                            actions: [ {
                                state: "NLU_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Note",
                                ru: "Note"
                            },
                            actions: [ {
                                state: "NOTE_PROP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Actions",
                                ru: "Actions"
                            },
                            actions: [ {
                                state: "ACTIONS_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Timer",
                                ru: "Timer"
                            },
                            actions: [ {
                                state: "TIMER_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Input",
                                ru: "Input"
                            },
                            actions: [ {
                                state: "INPUT_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Buttons",
                                ru: "Buttons"
                            },
                            actions: [ {
                                state: "BUTTONS_ITEM"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEMS_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    IMAGE_PROP: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Image</b> item sets the URL of the dislayed image.\nUplode an image to the WordPress medial library or other online storage, and indicate its URL in the item field.\nThe item has two properties:  <b>width</b> and <b>center</b>.',
                            ru: 'Элемент <b class="green">Image</b> предназначен для вывода изображение клиенту.\nИзображение задаётся ссылкой. То есть Вам нужно загрузить изображение в галерею WordPress (или другой ресурс в интернете) и указать в поле элемента ссылку, по которой изображение открывается.\nУ элемента есть пара фиксированных свойств: <b>width и center</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_img.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "IMAGE_WIDTH"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NOTE_PROP: [ {
                        type: "text",
                        text: {
                            en: '<b class="green">Note</b> is the item used for comments only displayed to a developer. The commentary is very helpful in case of complex bots that require extensive support.\nThe item has no properties.',
                            ru: 'Элемент <b class="green">Note</b> предназначен для комментариев, которые видны только разработчику бота. Комментарии очень важны, если бот сложный и планируется долгосрочная поддержка.\nЭлемент не имеет свойств.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_note.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TEXT_ITEM: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Text</b> item is the message displayed to your user. It can be localized and may contain HTML-tags (e.g. <b>&lt;b&gt;</b>, <i>&lt;i&gt;</i>, or <u>&lt;u&gt;</u>).\nThe element has a few fixed properties: <b>color, center</b> and <b>pre</b>.',
                            ru: 'Элемент <b class="green">Text</b> предназначен для вывода текста клиенту.\nТекст различается для разных языков, а также может содержать внутри себя HTML-теги (например, <b>&lt;b&gt;</b>, <i>&lt;i&gt;</i>, <u>&lt;u&gt;</u>)\nУ элемента есть несколько фиксированных свойств: <b>color, center, pre</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_text.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "TEXT_COLOR"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TEXT_COLOR: [ {
                        type: "text",
                        text: {
                            en: "The <b>color</b> property sets the message text color. Click the field to open the palette popup.",
                            ru: "Свойство <b>color</b> отвечает за цвет текста, выводимого клиенту. Выбор цвета осуществляется с помощью специального окна выбора цвета."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_text_color.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "TEXT_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next ",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "TEXT_CENTER"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TEXT_CENTER: [ {
                        type: "text",
                        text: {
                            en: "The <b>center</b> checkbox is responsible for text centering, i.e. if it's ticked, the text will be centered.",
                            ru: "Свойство <b>center</b> отвечает за центрирование текста, т.е. если свойство включено, то текст будет располагаться по центру."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_text_center.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "TEXT_COLOR"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "TEXT_PRE"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Skip tutorial",
                                ru: "Уже знаю"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TEXT_PRE: [ {
                        type: "text",
                        text: {
                            en: "If the <b>pre</b> checkbox is ticked, the text will be typewritten, and all whitespaces will be taken into account. Otherwise, multiple spaces are reduced to one, and the leading spaces are removed in each line.",
                            ru: "Свойство <b>pre</b>. Если включено данное свойство, текст будет машинописным, а также будут учитываться все пробельные отступы. (В выключенном состоянии несколько пробелов превращаются в один, а также удаляются начальные пробелы в каждой строке)"
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_text_pre.JPG",
                            center: 1
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "TEXT_CENTER"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    MAP_ITEM: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Map</b> element allows you to embed a map. Insert the URL code from Google Maps to add an &lt;iframe&gt; block.\nThe item has the only property, which is <b>height</b>.',
                            ru: 'Элемент <b class="green">Map</b> предназначен для вывода на карты.\nКарта в блок вставляется из Google maps в виде  &lt;iframe&gt; блока. (При публикации Google предлагает скопировать этот блок)\nУ элемента есть только одно свойство - <b>height</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_map.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "MAP_HEIGHT"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    IMAGE_CENTER: [ {
                        type: "text",
                        text: {
                            en: "The <b>center</b> property allows you to center the image in the message in case it's size is less than 100%.",
                            ru: "Свойство <b>center</b> отвечает за центрирование изображения в сообщении (если её размер меньше 100%)."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_img_center.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "IMAGE_WIDTH"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    MAP_HEIGHT: [ {
                        type: "text",
                        text: {
                            en: "The <b>height</b> property sets the height of the displayed map iframe in the chatbot.",
                            ru: "Свойство <b>height</b> устанавливает высоту выводимой карты в чат-боте."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_map_height.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "MAP_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    IMAGE_WIDTH: [ {
                        type: "text",
                        text: {
                            en: "The image width is set by the <b>width</b> property.\nThe height is set in proportion to the original.",
                            ru: "Свойство <b>width</b> устанавливает ширину выводимого изображения.\nВысота выставляется пропорционально исходным."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_img_width.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "IMAGE_PROP"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "IMAGE_CENTER"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BLOCK_ITEM: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Block</b> item allows you to format the current message, split the step into few messages, and set the typing animation duration.\nThe item properies are <b>typing</b> and <b>background color</b>.',
                            ru: 'Элемент <b class="green">Block</b> предназначен для форматирования текущего сообщения, разбиения шага на несколько сообщений или для установки анимации тайпинга (набор текста).\nУ элемента есть пара свойств: <b>typing и background color</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_block.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "BLOCK_TYPING"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BLOCK_TYPING: [ {
                        type: "text",
                        text: {
                            en: "The <b>typing</b> is set in milliseconds. It's the time before a new message appears.",
                            ru: "Свойство <b>typing</b> измеряется в миллисекундах и устанавливает время анимации тайпинга (набор сообщения) до появления сообщения."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_block_typing.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "BLOCK_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Next ",
                                ru: "Далее"
                            },
                            actions: [ {
                                state: "BLOCK_COLOR"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BLOCK_COLOR: [ {
                        type: "text",
                        text: {
                            en: "The <b>backbround color</b> property sets the message background color.",
                            ru: "Свойство <b>backbround color</b> устанавливает цвет фона текущего сообщения."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_block_color.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "BLOCK_TYPING"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NLU_ITEM: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">NLU</b> item is used to create "smart" bots, which can maintain a dialog with a user.\nThe item properties are called <b>"intents"</b>. ',
                            ru: 'Элемент <b class="green">NLU</b> используется для создания "умных" ботов, которые способны отвечать на сообщения клиентов.\nСвойства элемента называются <b>"намерения"</b>. '
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_nlu.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Intents",
                                ru: "Намерения"
                            },
                            actions: [ {
                                state: "NLU_INTENTS"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    ACTIONS_ITEM: [ {
                        type: "text",
                        text: {
                            en: '<b class = "green">Actions</b> is a merely logical item, aimed to perform particular <b>"actions"</b> when the item is launched.\nThe item has no properties.\nIt\'s an active item, which means it uses actions.',
                            ru: 'Элемент <b class="green">Actions</b> является полностью логическим и предназначен для выполнения каких-либо <b>"действий"</b> в момент, когда элемент будет запущен.\nУ элемента нет свойств. \nЭлемент является активным, то есть он использует <b>"действия"</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_actions.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TIMER_ITEM: [ {
                        type: "text",
                        text: {
                            en: '<b class="green">Timer</b> is the item aimed to perform particular <b>"actions"</b> when the timer expires.\nThe only property is <b>time</b>. \nIt\'s an active item, which means it uses actions.',
                            ru: 'Элемент <b class="green">Timer</b> предназначен для выполнения каких либо <b>"действий"</b> после истечения заданного интервала времени.\nУ элемента только статическое свойство <b>time</b>. \nЭлемент является активным, то есть он использует действия.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_timer.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "TIMER_TIME"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    INPUT_ITEM: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Input</b> item is used for is used to receive text messages from a user, and process them.\nСhatbot may only have one <b class="green">Input</b> item.\nThe only item property is <b>kind</b>.\nIt\'s an active item, which means it uses actions.',
                            ru: 'Элемент <b class="green">Input</b> используется для получения от клиента текстовых сообщений с последующей их обработкой.\nВ чат-боте всегда есть один <b class="green">Input</b>.\nУ элемента есть только одно свойство - <b>kind</b>.\nЭлемент является активным, то есть он использует <b>"действия"</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_input.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "INPUT_KIND"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BUTTONS_ITEM: [ {
                        type: "text",
                        text: {
                            en: 'The <b class="green">Buttons</b> item contains a set of buttons, which the botton bots are based on. Clicking a button triggers its respective <b>"actions"</b>.\nThe item itself has no properties, but every button has the <b>kind</b> property.\nIt\'s an active item, which means it uses <b>"actions"</b>.',
                            ru: 'Элемент <b class="green">Buttons</b> содержит в себе набор кнопок, которые являются основой кнопочных ботов. Нажатие на кнопку выполняет заданные <b>"действия"</b>.\nСам элемент не имеет свойств, а вот у каждой кнопки есть свойство - <b>kind</b>.\nЭлемент является активным, то есть он использует <b>"действия"</b>.'
                        }
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_buttons.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "Properties",
                                ru: "Свойства"
                            },
                            actions: [ {
                                state: "BUTTONS_KIND"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    NLU_INTENTS: [ {
                        type: "text",
                        text: {
                            en: "Each NLU <b>intent</b> has a unique name and a set of examples separated by semicolons. The examples help your bot to learn to distinguish user intents.\nEach supported language requires its set of examples.\nThe intents are used in the Input item.",
                            ru: "Каждое <b>намерение</b> NLU имеет уникальное название и набор примеров, разделённых через точку с запятой. При запуске бот обучается на примерах и способен их различать.\nПримеры намерений различаются для разных языков.\nИспользуются намерения в элементе Input."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_nlu_intents.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "MAP_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    TIMER_TIME: [ {
                        type: "text",
                        text: {
                            en: 'The <b>time</b> property sets the timer countdown time in milliseconds. When the timer runs out, <b>"actions"</b> will be performed.',
                            ru: 'Свойство <b>time</b> измеряется в миллисекундах и устанавливает время отсчёта таймера. Когда таймер отработает, будут выполнены <b>"действия"</b>.'
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_timer_time.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "TIMER_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    INPUT_KIND: [ {
                        type: "text",
                        text: {
                            en: "The <b>kind</b> property sets the expected type of user data. It may be a <i>number, e-mail, phone number, intent</i> or a random text.\nIf a user enters data of inappropriate type, <i>null</i> is returned to the INPUT slot.",
                            ru: "Свойство <b>kind</b> устанавливает тип данных, который необходимо получить от клиента. Это может быть <i>число, телефон, e-mail, намерение</i> или просто любой текст.\nЕсли пользователь введёт текст неподходящего типа, то слот INPUT будет записано <i>null</i>."
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_input_kind.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "INPUT_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    BUTTONS_KIND: [ {
                        type: "text",
                        text: {
                            en: "In the <b>kind</b> property, you can set the button type (button, checkbox or radio), and the position of the text inside the button (left, center, or right).\nWhen the button is clicked, the respective <b>actions</b> will be performed.\nIf the button type is checkbox or radio, the option will be deemed selected.",
                            ru: 'В свойстве <b>kind</b> можно выбрать тип кнопки (кнопка, чекбокс или radio), а также установить расположение текста внутри кнопки (слева, по центру, справа).\nПри клике на кнопку будут выполнены <b>"действия"</b>.\nВ случае, когда тип кнопки чекбокс или radio, кнопка будет выбрана.'
                        },
                        pre: 0
                    }, {
                        type: "images",
                        items: [ {
                            type: "image",
                            image: "https://qudata.com/qubot/im/ts/help/ed_buttons_kind.JPG"
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "BUTTONS_ITEM"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "OK",
                                ru: "Понятно"
                            },
                            actions: [ {
                                state: "ITEM_PROPERTIES"
                            } ]
                        } ]
                    }, {
                        type: "input",
                        kind: "text",
                        actions: []
                    } ],
                    LINKS_MENU: [ {
                        type: "text",
                        text: {
                            en: "Learn more about step navigation.",
                            ru: "Может быть, Вы хотите узнать о переходах больше?"
                        }
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Types of navigation",
                                ru: "Виды переходов"
                            },
                            actions: [ {
                                state: "GOTO"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "Default navigation",
                                ru: "Переход по умолчанию"
                            },
                            actions: [ {
                                state: "DEF_STEP"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: [ {
                            type: "button",
                            button: {
                                en: "⇧ Back",
                                ru: "⇧ Назад"
                            },
                            actions: [ {
                                state: "EDIT_STEP_MENU"
                            } ]
                        }, {
                            type: "button",
                            button: {
                                en: "🏠 Home",
                                ru: "🏠 В начало"
                            },
                            actions: [ {
                                state: "MAIN"
                            } ]
                        } ]
                    }, {
                        type: "row",
                        items: []
                    }, {
                        type: "input",
                        actions: [],
                        kind: "text"
                    } ]
                },
                slots: {
                    LANGUAGE: "en",
                    INPUT: ""
                },
                cards: {
                    states: {
                        MAIN: {
                            c: 1,
                            r: 0,
                            x: 556,
                            y: 185
                        },
                        COLOR_STATE: {
                            c: 2,
                            r: 4,
                            x: 300,
                            y: 49,
                            folder: "FAQ"
                        },
                        LINKS: {
                            c: 1,
                            r: 9,
                            x: 300,
                            y: 211,
                            folder: "EDIT_STEPS"
                        },
                        SCRIPT: {
                            c: 3,
                            r: 3,
                            x: 417,
                            y: 602,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_CONTENT: {
                            c: 2,
                            r: 3,
                            x: 138,
                            y: 758,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_LIST: {
                            c: 4,
                            r: 3,
                            x: 658,
                            y: 761,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_CLEAR: {
                            c: 2,
                            r: 6,
                            x: 814,
                            y: 918,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_ADD: {
                            c: 4,
                            r: 6,
                            x: 3.601104940539246,
                            y: 27.381450582458953,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_TEXT: {
                            c: 1,
                            r: 4,
                            x: 58.00987545665905,
                            y: 22.72608733260142,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_IMAGE: {
                            c: 2,
                            r: 4,
                            x: 14.020614920465402,
                            y: 7.02360216654927,
                            folder: "SCRIPTS"
                        },
                        CHANGE_LANGUAGE: {
                            c: 2,
                            r: 0,
                            x: 867,
                            y: 68
                        },
                        START_STEP: {
                            c: 2,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        GREET: {
                            c: 0,
                            r: 0,
                            x: 535,
                            y: 25
                        },
                        FAQ: {
                            c: 0,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "FAQ"
                        },
                        FOLDERS: {
                            c: 5,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        STEP_POSITION: {
                            c: 1,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        ITEMS: {
                            c: 2,
                            r: 7,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        GOTO: {
                            c: 2,
                            r: 9,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        DEF_STEP: {
                            c: 3,
                            r: 9,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        ACTIONS: {
                            c: 0,
                            r: 9,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        MAP_LINK: {
                            c: 1,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "FAQ"
                        },
                        IMG_LINK: {
                            c: 0,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "FAQ"
                        },
                        START_LANGUAGE: {
                            c: 1,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "FAQ"
                        },
                        SCRIPT_CHECK: {
                            c: 3,
                            r: 4,
                            x: 700,
                            y: 447,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_RADIO: {
                            c: 4,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_GENERAL: {
                            c: 1,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_IF: {
                            c: 2,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_SET: {
                            c: 1,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_WHILE: {
                            c: 3,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_SUM: {
                            c: 5,
                            r: 6,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_TOTAL: {
                            c: 6,
                            r: 6,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_FOREACH: {
                            c: 8,
                            r: 6,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_GET: {
                            c: 7,
                            r: 6,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_PUSH: {
                            c: 3,
                            r: 6,
                            x: 0,
                            y: 0,
                            folder: "SCRIPTS"
                        },
                        SCRIPT_LEN: {
                            c: 1,
                            r: 6,
                            x: 575,
                            y: 920,
                            folder: "SCRIPTS"
                        },
                        COPY_STEPS: {
                            c: 4,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        CREATE_STEP: {
                            c: 0,
                            r: 2,
                            x: 100,
                            y: 49,
                            folder: "CREATE_STEPS"
                        },
                        REMOVE_STEP: {
                            c: 1,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        CREATE_LINE: {
                            c: 3,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        NEXT_STEP_INFO: {
                            c: 0,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        SELECT_START: {
                            c: 5,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        PASTE_STEPS: {
                            c: 6,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        GRID_VIEW: {
                            c: 2,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        FREE_VIEW: {
                            c: 4,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        DRAG_STEP: {
                            c: 3,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        EDIT_STEPS_IN_DIR: {
                            c: 4,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        FOLDER_MENU: {
                            c: 3,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        CREATE_FOLDER: {
                            c: 2,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        REMOVE_FOLDERS: {
                            c: 1,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        COPY_FOLDER: {
                            c: 1,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        PASTE_FOLDER: {
                            c: 3,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        STEP_MOVE_IN_DIR: {
                            c: 5,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        STEP_MOVE_OUT_DIR: {
                            c: 7,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        RENAME_FOLDER: {
                            c: 0,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        FOLDER_LINE: {
                            c: 6,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        INSERT_TEMPLATE: {
                            c: 0,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        INSERT_YES_NO: {
                            c: 2,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        INSERT_ADD_LANGUAGE: {
                            c: 4,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "CREATE_STEPS"
                        },
                        OPEN_PANEL: {
                            c: 0,
                            r: 7,
                            x: 100,
                            y: 211,
                            folder: "EDIT_STEPS"
                        },
                        EDIT_STEP_MENU: {
                            c: 1,
                            r: 7,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        VISUAL_ITEMS: {
                            c: 0,
                            r: 8,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        LOGICAL_ITEMS: {
                            c: 1,
                            r: 8,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        ITEMS_MENU: {
                            c: 3,
                            r: 7,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        ADD_ITEM: {
                            c: 2,
                            r: 8,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        COPY_ITEM: {
                            c: 3,
                            r: 8,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        PASTE_ITEM: {
                            c: 4,
                            r: 8,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        REMOVE_ITEM: {
                            c: 4,
                            r: 7,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        DRAG_ITEM: {
                            c: 5,
                            r: 8,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        ITEM_PROPERTIES: {
                            c: 0,
                            r: 10,
                            x: 0,
                            y: 0,
                            folder: "EDIT_STEPS"
                        },
                        IMAGE_PROP: {
                            c: 4,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        NOTE_PROP: {
                            c: 6,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        TEXT_ITEM: {
                            c: 1,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        TEXT_COLOR: {
                            c: 3,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        TEXT_CENTER: {
                            c: 2,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        TEXT_PRE: {
                            c: 5,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        MAP_ITEM: {
                            c: 1,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        IMAGE_CENTER: {
                            c: 7,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        MAP_HEIGHT: {
                            c: 2,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        IMAGE_WIDTH: {
                            c: 6,
                            r: 2,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        BLOCK_ITEM: {
                            c: 3,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        BLOCK_TYPING: {
                            c: 4,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        BLOCK_COLOR: {
                            c: 5,
                            r: 3,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        NLU_ITEM: {
                            c: 2,
                            r: 4,
                            x: 0,
                            y: 0,
                            folder: "ITEMS"
                        },
                        ACTIONS_ITEM: {
                            c: 3,
                            r: 5,
                            x: 0,
                            y: 0,
                            folder: "ITEMS",
                            TIMER_ITEM: {
                                c: 3,
                                r: 4,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            INPUT_ITEM: {
                                c: 2,
                                r: 5,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            BUTTONS_ITEM: {
                                c: 4,
                                r: 5,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            NLU_INTENTS: {
                                c: 1,
                                r: 4,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            TIMER_TIME: {
                                c: 4,
                                r: 4,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            INPUT_KIND: {
                                c: 1,
                                r: 5,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            BUTTONS_KIND: {
                                c: 5,
                                r: 5,
                                x: 0,
                                y: 0,
                                folder: "ITEMS"
                            },
                            LINKS_MENU: {
                                c: 4,
                                r: 9,
                                x: 100,
                                y: 100,
                                folder: "EDIT_STEPS"
                            }
                        },
                        folders: {
                            CREATE_STEPS: {
                                c: 0,
                                r: 1,
                                x: 100,
                                y: 49,
                                openGrid: !1
                            },
                            SCRIPTS: {
                                c: 1,
                                r: 2,
                                x: 613,
                                y: 515,
                                openGrid: !1
                            },
                            EDIT_STEPS: {
                                c: 0,
                                r: 2,
                                x: 187,
                                y: 182,
                                openGrid: !1
                            },
                            FAQ: {
                                c: 0,
                                r: 3,
                                x: 980,
                                y: 273,
                                openGrid: !1
                            },
                            ITEMS: {
                                c: 1,
                                r: 1,
                                x: 59,
                                y: 322,
                                openGrid: !1
                            }
                        }
                    }
                }
            }
        };
    }
}

class QuBotConsole {
    constructor() {
        this.pageData = new QuBotMyBotsData(), this.pageUpload = new QuBotMyBotsUpload();
    }
    setInterface() {
        qubotSetup.setInterface(), "WP" == qubotData.type ? this.menu(document.getElementById("menu-qubot-page-bots")) : this.menu(document.getElementById("menu-qubot-page-editor"));
    }
    menu(obj) {
        qubotManager.close();
        var pages = [ "qubot-page-bots", "qubot-page-editor", "qubot-page-setup", "qubot-page-upload", "qubot-page-data" ];
        for (let i = 0; i < pages.length; i++) {
            let p = document.getElementById(pages[i]);
            p.style.display = "none", document.getElementById("menu-" + pages[i]).className = "";
        }
        obj.className = "active";
        obj = obj.id.substring(5);
        let page = document.getElementById(obj);
        "qubot-page-editor" === obj || "qubot-page-setup" === obj || ("qubot-page-data" === obj ? this.pageData.create() : "qubot-page-upload" === obj ? this.pageUpload.create() : "qubot-page-users" === obj && this.pageUsers.create()), 
        page.style.display = "flex", window.scrollTo(0, 0), "qubot-page-editor" === obj && (qubotEditor.showView(qubotEditor.bot.view), 
        qubotEditor.help.isStartInfo && qubotEditor.help.alert("StartMessage", !0), qubotEditor.listEditor.createZoom(), 
        qubotResizeWindow());
    }
}

function qubotResizeWindow(event) {
    let statesList = document.getElementById("qubot-states-list");
    var offset = statesList.getBoundingClientRect();
    statesList.style.width = window.innerWidth - offset.left - 25 + "px", statesList.style.height = window.innerHeight - offset.top + "px";
}

function qubotDragSplitter(element, direction) {
    let md;
    const first = document.getElementById("qubot-state-editor"), second = document.getElementById("qubot-states-list");
    function onMouseMove(e) {
        let delta = {
            x: e.clientX - md.e.clientX - 10,
            y: e.clientY - md.e.clientY
        };
        "H" === direction && (delta.x = Math.min(Math.max(delta.x, -md.firstWidth), md.secondWidth), 
        element.style.left = md.offsetLeft + delta.x + "px", first.style.width = md.firstWidth + delta.x + "px", 
        second.style.width = md.secondWidth - delta.x + "px"), qubotResizeWindow();
    }
    element && first && second && (element.onmousedown = function(e) {
        md = {
            e: e,
            offsetLeft: element.offsetLeft,
            offsetTop: element.offsetTop,
            firstWidth: first.offsetWidth,
            secondWidth: second.offsetWidth
        }, document.onmousemove = onMouseMove, element.onmouseover = onMouseMove, document.onmouseup = () => {
            document.onmousemove = document.onmouseup = null, element.onmouseover = null;
        };
    });
}

window.addEventListener("resize", qubotResizeWindow, !0), window.onload = function() {
    qubotManager.create(), qubotEditor.create(), qubotCodeInput.registerTemplate("syntax-highlighted", qubotCodeInput.templates.prism(Prism));
}, window.onbeforeunload = function() {
    return qubotManager.bot && (qubotManager.bot.save(), qubotManager.bot.events.send()), 
    null;
}, window.addEventListener("beforeunload", function(e) {
    return qubotManager.bot && (qubotManager.bot.save(), qubotManager.bot.events.send()), 
    null;
}, !1);

class QuBotMyBotsData extends QuBotEditorBase {
    constructor() {
        super(), this.botID = null, this.dateFrom = null, this.dateTo = null;
    }
    create() {
        let dtFrom = document.getElementById("qubot-data-from"), dtTo = (dtFrom.value = qubotData.getDate(new Date()), 
        dtFrom.setAttribute("onchange", "qubotConsole.pageData.onchangeDateFrom(this)"), 
        this.dateFrom = dtFrom.value, document.getElementById("qubot-data-to")), mybots = (dtTo.value = qubotData.getDate(new Date()), 
        dtTo.setAttribute("onchange", "qubotConsole.pageData.onchangeDateTo(this)"), this.dateTo = dtTo.value, 
        this.wrapper = document.getElementById("qubot-data-table"), document.getElementById("qubot-data-mybots"));
        for (;mybots.firstChild; ) mybots.removeChild(mybots.lastChild);
        mybots.setAttribute("onchange", "qubotConsole.pageData.onchangeBot(this)");
        for (let i = 0; i < qubotEditor.myBots.bots.length; i++) {
            let op = this.add(mybots, null, "option");
            op.value = qubotEditor.myBots.bots[i].id, op.innerHTML = qubotEditor.myBots.bots[i].name;
        }
        this.botID = qubotEditor.myBots.bots[0].id, this.showTable();
    }
    showTable() {
        for (;this.wrapper.firstChild; ) this.wrapper.removeChild(this.wrapper.firstChild);
        qubotData.receive("wp", 1, this.botID, -1, function(data) {
            if (data) {
                let keys = {}, col = 0;
                for (let i = 0; i < data.length; i++) for (var k in data[i].value) k in keys || (keys[k] = col++);
                var table = this.add(this.wrapper, null, "table"), tr = this.add(table, null, "tr");
                this.add(tr, null, "th").innerHTML = "date";
                this.add(tr, null, "th").innerHTML = "time", this.add(tr, null, "th").innerHTML = "uid";
                for (let k in keys) this.add(tr, null, "th").innerHTML = k;
                for (let i = 0; i < data.length; i++) if (!(data[i].date < this.dateFrom)) {
                    if (data[i].date > this.dateTo) break;
                    let tr = this.add(table, null, "tr"), td, vals = (td = this.add(tr, null, "td"), 
                    td.innerHTML = data[i].time.split(" ")[0], td = this.add(tr, null, "td"), td.innerHTML = data[i].time.split(" ")[1], 
                    td = this.add(tr, null, "td"), td.innerHTML = data[i].uid.substring(0, 8), data[i].value), tds = [];
                    for (let k in keys) tds.push(this.add(tr, null, "td"));
                    for (let k in vals) tds[keys[k]].innerHTML = vals[k];
                }
            }
        }.bind(this));
    }
    onchangeBot(obj) {
        this.botID = obj.value, this.showTable();
    }
    onchangeKey(obj) {
        this.key = obj.value.trim(), this.showTable();
    }
    onchangeDateTo(obj) {
        this.dateTo = obj.value, this.showTable();
    }
    onchangeDateFrom(obj) {
        this.dateFrom = obj.value, this.showTable();
    }
}

class QuBotEditor extends QuBotEditorBase {
    constructor() {
        super(), this.stateEditor = new QuBotEditorState(), this.listEditors = new QuBotEditorLists(), 
        this.listEditor = this.listEditors.select("grid"), this.resizer = null, this.myBots = new QuBotEditorMyBots(), 
        this.templates = new QuBotEditorTemplates(), this.examples = new QuBotEditorExamples(), 
        this.help = new QuBotEditorHelp(), this.bot = {}, this.copy = null, this.dontSave = !1;
    }
    init() {
        this.stateEditor.init();
    }
    save() {
        this.myBots.bot && !this.dontSave && this.myBots.save();
    }
    clear() {
        this.stateEditor.clear(), this.listEditor.clear();
    }
    create() {
        this.dontSave = !0, this.stateEditor.create(), this.help.load(this.create1.bind(this));
    }
    create1() {
        this.myBots.loadBots(this.create2.bind(this));
    }
    create2() {
        let wrapper = document.getElementById("qubot-editor");
        wrapper.setAttribute("onclick", "qubotEditor.stateEditor.onclickEditor(this)");
        var header = document.getElementById("qubot-editor-header");
        this.createMenu(this.add(header, "qubot-editor-menu"));
        let run = this.add(header, "qubot-editor-run"), label = (run.setAttribute("onclick", "qubotEditor.runBot();"), 
        this.add(run, "qubot-icon-run"), this.add(run, null));
        label.innerHTML = "RUN BOT", this.botName = this.add(header, "qubot-editor-name"), 
        this.botName.onclick = function() {
            qubotEditor.exportToYAML();
        }, 0 === this.myBots.bots.length && this.myBots.newBot("My first bot", !1), null === this.myBots.botNumber && (this.myBots.botNumber = 0), 
        this.myBots.load(this.myBots.bots[this.myBots.botNumber].id, this.create3.bind(this));
    }
    create3() {
        qubotManager.upgradeBotFormat(this.myBots.bot), this.setBot(this.myBots.bot), qubotManager.set(this.myBots.bot), 
        this.bot.state = this.bot.start, this.myBots.createMyBotsMenu(), qubotSetup.dontSave = !0, 
        this.listEditors.create(), qubotConsole.setInterface(), this.stateEditor.caption.value = this.bot.state, 
        this.stateEditor.items.set(this.bot.states[this.bot.state]), this.stateEditor.set(this.bot.states[this.bot.state]), 
        this.setCurStateEditor(this.bot.start), this.dontSave = !1, this.botName.innerHTML = this.myBots.bot.name || "", 
        qubotData.load("qubotToken", function(data) {
            data && qubotData.setClientToken(data.token);
        }), qubotResizeWindow();
    }
    setBot(bot) {
        this.cleaningBot(bot), this.bot.id = bot.id, this.bot.version = bot.version, this.bot.start = bot.start, 
        this.bot.state = bot.start, this.bot.states = bot.states, this.bot.slots = bot.slots, 
        this.bot.cards = bot.cards, this.bot.view = bot.view, qubotManager.set(this.bot);
    }
    loadBot(bot) {
        this.dontSave = !0, this.clear(), this.init(), qubotManager.upgradeBotFormat(bot), 
        this.setBot(bot), this.listEditors.create(), this.listEditor.show(), this.stateEditor.updateCard = !1, 
        this.stateEditor.init(), this.setCurStateEditor(this.bot.start), this.stateEditor.updateCard = !0, 
        this.botName.innerHTML = bot.name || "", this.dontSave = !1;
    }
    runBot() {
        qubotManager.set(this.bot), qubotManager.bot.clear(), qubotManager.bot.debug.create(), 
        qubotManager.bot.debug.clear(), qubotManager.open();
        let lang = "en";
        this.help.isRunCurLang && (lang = qubotManager.bot.slots.LANGUAGE = this.stateEditor.LANGUAGE), 
        this.help.isRunFromActive ? qubotManager.bot.showState(this.bot.state) : qubotManager.start(!0, lang), 
        qubotManager.bot.debug.show(qubotManager.bot.curState, qubotManager.bot.slots), 
        this.wasRun = !0;
    }
    runHelpBot() {
        qubotManager.set(this.templates.bots.HELP), qubotManager.bot.clear(), qubotManager.open(), 
        qubotManager.start(), qubotManager.open();
    }
    setCurStateEditor(state) {
        this.listEditor.setCurState(state), this.stateEditor.set(this.bot.states[state]);
    }
    resetEditorAndCard() {
        this.stateEditor.set(this.bot.states[this.bot.state]);
        var card = document.getElementById("qubot-state-" + this.bot.state);
        this.listEditor.openCardState(this.bot.state, card), this.listEditor.showArrows(this.bot.state);
    }
    resetEditorAndAllCards() {
        this.stateEditor.set(this.bot.states[this.bot.state]);
        var state, curState = this.bot.state;
        for (state in this.bot.states) {
            this.bot.state = state;
            var card = document.getElementById("qubot-state-" + state);
            card ? this.listEditor.openCardState(state, card) : this.error("!!!> qubotEditor.resetEditorAndAllCards: card = null for state", state);
        }
        this.bot.state = curState, this.listEditor.showArrows(this.bot.state);
    }
    createMenu(menu) {
        let a, cont, sub, b;
        for (var bot in menu.classList.add("qubot-disable-select"), sub = this.add(menu, "submenu"), 
        b = this.add(sub, "submenu-btn", "button"), b.innerHTML = "My Bots", this.menuMyBots = this.add(sub, "submenu-content"), 
        this.myBots.createMyBotsMenu(), sub = this.add(menu, "submenu"), b = this.add(sub, "submenu-btn", "button"), 
        b.innerHTML = "Edit Bot</div>", cont = this.add(sub, "submenu-content"), a = this.add(cont, "item"), 
        a.innerHTML = "New", a.setAttribute("onclick", "qubotEditor.myBots.newBot()"), a = this.add(cont, "item"), 
        a.innerHTML = "Clone", a.setAttribute("onclick", "qubotEditor.myBots.cloneBot()"), 
        a = this.add(cont, "item"), a.innerHTML = "Delete", a.setAttribute("onclick", "qubotEditor.myBots.deleteBot()"), 
        a = this.add(cont, "item"), a.innerHTML = "Rename", a.setAttribute("onclick", "qubotEditor.myBots.renameBot()"), 
        a = this.add(cont, "item"), a.innerHTML = "Export", a.setAttribute("onclick", "qubotEditor.exportToJSON()"), 
        a = this.add(cont, "item"), a.innerHTML = "Import", a.setAttribute("onclick", "qubotEditor.importFromJSON()"), 
        sub = this.add(menu, "submenu"), b = this.add(sub, "submenu-btn", "button"), b.innerHTML = "Edit Step", 
        cont = this.add(sub, "submenu-content"), a = this.add(cont, "item"), a.innerHTML = "Copy", 
        a.setAttribute("onclick", "qubotEditor.listEditor.copyStep()"), a = this.add(cont, "item"), 
        a.innerHTML = "Paste", a.setAttribute("onclick", "qubotEditor.listEditor.pasteStep()"), 
        a = this.add(cont, "item"), a.innerHTML = "New folder", a.setAttribute("onclick", "qubotEditor.listEditor.createCardFolder()"), 
        a = this.add(cont, "item"), a.innerHTML = "Copy folder", a.setAttribute("onclick", "qubotEditor.listEditor.copyCardFolder()"), 
        a = this.add(cont, "item"), a.innerHTML = "Paste folder", a.setAttribute("onclick", "qubotEditor.listEditor.pasteCardFolder()"), 
        a = this.add(cont, "item"), a.innerHTML = "Rename folder", a.setAttribute("onclick", "qubotEditor.listEditor.renameCardFolder()"), 
        a = this.add(cont, "item"), a.innerHTML = "Add Yes/No", a.setAttribute("onclick", 'qubotEditor.listEditor.onAddCard(null, true, "yesno")'), 
        a = this.add(cont, "item"), a.innerHTML = "Add Change Language", a.setAttribute("onclick", 'qubotEditor.listEditor.onAddCard(null, true, "lang")'), 
        sub = this.add(menu, "submenu"), b = this.add(sub, "submenu-btn", "button"), b.innerHTML = "Templates", 
        this.menuTemplates = cont = this.add(sub, "submenu-content"), this.templates.bots) "Help" !== this.templates.bots[bot].name && (a = this.add(cont, "item"), 
        a.innerHTML = this.templates.bots[bot].name, a.setAttribute("onclick", `qubotEditor.onclickMenuTemplate("${bot}")`));
        sub = this.add(menu, "submenu"), b = this.add(sub, "submenu-btn", "button"), b.innerHTML = "Examples", 
        this.menuExamples = cont = this.add(sub, "submenu-content");
        for (let bot in this.examples.bots) a = this.add(cont, "item"), a.innerHTML = this.examples.bots[bot].name, 
        a.setAttribute("onclick", `qubotEditor.onclickMenuExamples("${bot}")`);
        sub = this.add(menu, "submenu"), b = this.add(sub, "submenu-btn", "button"), b.innerHTML = "View", 
        this.menuView = cont = this.add(sub, "submenu-content"), a = this.add(cont, "item"), 
        a.innerHTML = "Grid", a.setAttribute("onclick", 'qubotEditor.onclickMenuView("grid")'), 
        a.classList.add("checked"), a = this.add(cont, "item"), a.innerHTML = "Free", a.setAttribute("onclick", 'qubotEditor.onclickMenuView("free")'), 
        sub = this.add(menu, "submenu"), b = this.add(sub, "submenu-btn", "button"), b.innerHTML = "Help", 
        this.menuOptions = cont = this.add(sub, "submenu-content"), a = this.add(cont, "item"), 
        a.innerHTML = "Assistant", a.setAttribute("onclick", "qubotEditor.runHelpBot()"), 
        a = this.add(cont, "item"), this.help.isStartInfo ? a.innerHTML = "Show start info" : a.innerHTML = "Hide start info", 
        a.setAttribute("onclick", 'qubotEditor.onclickMenuOptions("start")'), a = this.add(cont, "item"), 
        this.help.isHint ? a.innerHTML = "Show hint window" : a.innerHTML = "Hide hint window", 
        a.setAttribute("onclick", 'qubotEditor.onclickMenuOptions("hint")'), a = this.add(cont, "item"), 
        this.help.isRunFromActive ? a.innerHTML = "Run from active step" : a.innerHTML = "Run from start step", 
        a.setAttribute("onclick", 'qubotEditor.onclickMenuOptions("active")'), a = this.add(cont, "item"), 
        this.help.isRunCurLang ? a.innerHTML = "Use current language" : a.innerHTML = "Use default language", 
        a.setAttribute("onclick", 'qubotEditor.onclickMenuOptions("lang")'), a = this.add(cont, "item"), 
        a.innerHTML = "Version: " + window.editorConfig.version;
    }
    onclickMenuView(view) {
        this.showView(view);
    }
    showView(view) {
        let lst = this.menuView.getElementsByClassName("item");
        Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        }), ("free" === view ? lst[1] : lst[0]).classList.add("checked"), this.listEditor = this.listEditors.select(view), 
        this.listEditor.init(), this.listEditor.show();
    }
    onclickMenuTemplate(botNumber) {
        this.save(), this.loadBot(this.templates.bots[botNumber]);
        let tagsA = this.menuTemplates.getElementsByClassName("item"), i = 0;
        for (var bot in this.templates.bots) {
            if (i >= tagsA.length) break;
            tagsA[i].classList.remove("checked"), botNumber === bot && tagsA[i].classList.add("checked"), 
            i++;
        }
        var lst = this.menuMyBots.getElementsByClassName("item");
        Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        }), lst = this.menuExamples.getElementsByClassName("item"), Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        }), this.myBots.botNumber = null, this.myBots.bot = null;
    }
    onclickMenuExamples(botNumber) {
        this.save(), this.loadBot(this.examples.bots[botNumber]);
        let tagsA = this.menuExamples.getElementsByClassName("item"), i = 0;
        for (var bot in this.examples.bots) tagsA[i].classList.remove("checked"), botNumber === bot && tagsA[i].classList.add("checked"), 
        i++;
        var lst = this.menuMyBots.getElementsByClassName("item");
        Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        }), lst = this.menuTemplates.getElementsByClassName("item"), Array.from(lst).forEach(a => {
            a.classList.remove("checked");
        }), this.myBots.botNumber = null, this.myBots.bot = null;
    }
    onclickMenuOptions(item) {
        let lst = this.menuOptions.getElementsByClassName("item");
        "start" === item ? "Show start info" === lst[1].innerHTML ? (lst[1].innerHTML = "Hide start info", 
        this.help.isStartInfo = !1) : (lst[1].innerHTML = "Show start info", this.help.isStartInfo = !0) : "hint" === item ? "Show hint window" === lst[2].innerHTML ? (lst[2].innerHTML = "Hide hint window", 
        this.help.isHint = !1) : (lst[2].innerHTML = "Show hint window", this.help.isHint = !0) : "active" === item ? "Run from active step" === lst[3].innerHTML ? (lst[3].innerHTML = "Run from start step", 
        this.help.isRunFromActive = !1) : (lst[3].innerHTML = "Run from active step", this.help.isRunFromActive = !0) : "lang" === item && ("Use default language" === lst[4].innerHTML ? (lst[4].innerHTML = "Use current language", 
        this.help.isRunCurLang = !0) : (lst[4].innerHTML = "Use default language", this.help.isRunCurLang = !1)), 
        this.help.save();
    }
    newStateName() {
        let cnt = Object.keys(this.bot.states).length + 1;
        for (;"STEP_" + cnt in this.bot.states; ) cnt++;
        return "STEP_" + cnt;
    }
    renameState(oldState, newState, bot = null, refresh = !0) {
        for (var s in bot = bot || this.bot, oldState === bot.start && (bot.start = newState, 
        this.myBots.bot && (this.myBots.bot.start = newState)), bot.states) {
            let states = this.stateEditor.items.getStates(bot.states[s]);
            states.forEach(ss => {
                ss.state === oldState && ("state" in ss.action ? ss.action.state = newState : "goto" in ss.action ? ss.action.goto = newState : "run" in ss.action && (ss.action.run = newState));
            });
        }
        var old_items = bot.states[oldState];
        delete bot.states[oldState], newState && (bot.states[newState] = old_items, this.listEditor.renameState(oldState, newState), 
        this.listEditor.setCurState(newState)), refresh && (this.resetEditorAndCard(), this.myBots.save());
    }
    error(...theArgs) {
        console.log(theArgs);
    }
    copyBot() {
        this.help.alert("BotCopy"), this.copy = JSON.stringify(this.bot);
    }
    pasteBot() {
        if (this.copy) if (this.myBots.bot) {
            var state, bot = JSON.parse(this.copy);
            for (state in this.bot.states) if (state in bot.states) {
                let cnt = 1;
                for (;state + cnt in this.bot.states; ) cnt++;
                this.renameState(state, state + cnt, bot, !1);
            }
            for (let state in bot.states) this.myBots.bot.states[state] = bot.states[state];
            this.save(), this.resetEditorAndAllCards();
        } else this.help.alert("BotPasteCannot2"); else this.help.alert("BotPasteCannot1");
    }
    cleaningBot(bot) {
        var state, input, slot, slots = {};
        for (state in bot.states) {
            let items = bot.states[state], inputID = -1;
            for (let i = 0; i < items.length; i++) {
                let item = items[i];
                if ("actions" in item) this.cleaningBotActions(item.actions); else if ("row" === item.type && "items" in item) for (let j = 0; j < item.items.length; j++) this.cleaningBotActions(item.items[j].actions); else "comment" === item.type && (item.type, 
                item.comment && (item.note = item.comment, delete item.comment));
                "input" === item.type && i + 1 < items.length && (inputID = i);
            }
            0 <= inputID && (input = items[inputID], items.splice(inputID, 1), items.push(input)), 
            this.stateEditor.items.getSlots(bot.states[state], slots);
        }
        for (slot in bot.slots) slot in slots || "LANGUAGE" === slot || "INPUT" === slot || delete bot.slots[slot];
    }
    cleaningBotActions(actions) {
        if (actions) for (let i = 0; i < actions.length; i++) {
            let action = actions[i];
            action ? "true" in action ? delete action.true : "false" in action ? delete action.false : 0 === Object.keys(action).length || "state" in action && !action.state || "goto" in action && !action.goto || "run" in action && !action.run ? actions.splice(i, 1) : "comment" in action ? "note" in action || (action.note = action.comment, 
            delete action.comment) : "actions" in action && this.cleaningBotActions(action.actions) : actions.splice(i, 1);
        }
    }
    exportToYAML() {
        let res = "bid:     " + this.bot.id + "\n";
        for (var state in res += "version: " + this.bot.version + "\n", res += "view: " + this.bot.view + "\n", 
        res += "start: " + this.bot.start + "\n\n", res += "states:\n", this.bot.states) res += "    " + state + ":\n", 
        res += this.stateEditor.items.exportYAML(this.bot.states[state], "    ");
        for (var slot in res += "\nslots:\n", this.bot.slots) res += "    " + slot + ":\n", 
        res += "        value: " + this.bot.slots[slot] + "\n";
        res += "\ncards:\n", res += "    states: " + JSON.stringify(this.bot.cards.states) + "\n", 
        res += "    folders: " + JSON.stringify(this.bot.cards.folders) + "\n";
        var blob = new Blob([ res ], {
            type: "text/plain"
        }), blob = window.URL.createObjectURL(blob), anchor = document.createElement("a");
        anchor.href = blob, anchor.download = this.botName.innerHTML + ".yml", anchor.click(), 
        window.URL.revokeObjectURL(blob), anchor.remove();
    }
    itemsToYAML(items) {
        let res = "";
        for (var k in items) "type" !== k && (res += `${k}: ${items[k]}, `);
        res += "}\n";
    }
    exportToJSON() {
        var slot, res = "{", res = (res = (res = (res += '"start":"' + this.bot.start + '",') + ('"version":"' + this.bot.version + '",')) + ('"view":"' + this.bot.view + '",')) + ('"states":' + JSON.stringify(this.bot.states) + ",");
        for (slot in this.bot.slots) this.bot.slots[slot] = "";
        this.bot.slots.LANGUAGE = "en", res = (res += '"slots":' + JSON.stringify(this.bot.slots) + ",") + '"cards":' + JSON.stringify(this.bot.cards) + "}";
        var res = new Blob([ res ], {
            type: "text/plain"
        }), res = window.URL.createObjectURL(res), anchor = document.createElement("a");
        anchor.href = res, anchor.download = this.botName.innerHTML + ".json", anchor.click(), 
        window.URL.revokeObjectURL(res), anchor.remove(), this.save();
    }
    async importFromJSON() {
        if (this.myBots.bot) {
            if (await qubotEditor.help.loadFile()) try {
                var bot = JSON.parse(qubotEditor.help.inputFileContent), id = this.myBots.bot.id, name = this.myBots.bot.name;
                this.myBots.bot = bot, this.myBots.bot.id = id, this.myBots.bot.name = name, this.loadBot(bot), 
                this.save();
            } catch (e) {
                alert("I was unable to import the bot. Format error.");
            }
        } else qubotEditor.help.alert("NoActiveBot");
    }
    toMultiLang() {
        for (var state in this.bot.states) this.bot.states[state].forEach(item => {
            "text" === item.type && "string" == typeof item.text ? item.text = {
                en: item.text
            } : "row" === item.type && item.items.forEach(btn => {
                "button" === btn.type && "string" == typeof btn.button && (btn.button = {
                    en: btn.button
                });
            });
        });
    }
}

class QuBotEditorHelp extends QuBotEditorBase {
    constructor() {
        super(), this.helpLanguage = "en", this.isStartInfo = !0, this.isHint = !0, this.messages = {
            StartMessage: {
                en: `
                    There are different ways to create a bot:
                    <ol>
                    <li> Feel free to edit the current bot.<br> In <b>My bots</b> menu, you can find it as  "<i>My first bot</i>".
                    <li> Download a bot from <b>Templates</b>, then click <b>Clone</b> in <b>Edit Bot</b> menu.<br> The bot will be added to <b>My bots</b>.
                    </ol>           
                    Useful tips:    
                    <ul>
                        <li> new bots can be created by clicking <b>Edit Bot</b> - <b>New</b>;<br> you can then rename and delete the bots;
                        <li> dialog steps are arranged in a grid or as an arbitrary diagram (see <b>View</b> menu);
                        <li> click <b>Help</b> - <b>Assistant</b> to receve more information from the bot;
                        <li> the window and hints are enabled/disabled in <b>Help</b> menu.
                    </ul>
                    `,
                ru: `
                    Для создания бота, выберете одну из следующих возможностей:
                    <ol>
                    <li> Смело редактируйте текущего бота.<br> В меню <b>My bots</b> он называется "<i>My first bot</i>".
                    <li> Из меню <b>Templates</b> загрузите одного из готовых ботов.<br> Затем в меню <b>Edit Bot</b> выберете пункт <b>Clone</b>, сделав его своим (он добавится в меню <b>My bots</b>).
                    </ol>           
                    Обратите внимание, что:    
                    <ul>
                        <li> новых ботов можно создавать в меню <b>Edit Bot</b> - <b>New</b>; там же их можно переименовать или удалить;
                        <li> шаги диалога упорядочиваются в виде сетки или как произвольная диаграмма (меню <b>View</b>);
                        <li> более подробную информацию можно получить у бота, выбрав пункт меню <b>Help</b> -  <b>Assistant</b>;
                        <li> это окно и хинты отключаются или включаются в меню <b>Help</b>.
                    </ul>
                `
            },
            storageNeedQudata: {
                en: `
                    To use this repository,
                    read the following gidelines: <a href="https://qudata.com/en/qubot/wordpress/" target="_blank">qudata.com/wordpress/</a>                
                    `,
                ru: `
                    Для использования этого хранилища, 
                    прочтите инструкции по адресу <a href="https://qudata.com/ru/qubot/wordpress/" target="_blank">qudata.com/wordpress/</a>
                    `
            },
            UnknownState: {
                en: `
                    Unknown step name. Don't enter it here. 
                    Click on that entry and then on the step card where you want to go.
                    `,
                ru: `
                    Неизвестное название шага. Лучше не вводите его здесь. 
                    Щелкните по этому вводу, а затем по карточке шага, куда вы хотите перейти.
                    `
            },
            YouCannotEdit: {
                en: `
                    You cannot edit the template. Make it your own bot first. 
                    In order to do it, click <b>Edit</b> and select <b>Clone</b>.
                    `,
                ru: `Вам не следует редактировать темплейт. Сначала  сделайте его своим собственным ботом. 
                    Для этого зайдите в меню <b>Edit</b> и выберете <b>Clone</b>.
                    `
            },
            SetStartStateInOther: {
                en: `
                    This is the starting step. 
                    To change the starting step, select it and click this checkbox.
                    `,
                ru: `
                    Это стартовый шаг (сообщение) бота. 
                    Чтобы сделать стартовым  другой шаг, зайдите в него и нажмите этот чекбокс.
                    `
            },
            StateNameExists: {
                en: `
                    A step with the same name already exists. Choose a different name.
                    `,
                ru: `
                    Шаг диалога с таким именем уже существует. Выберете другое имя.
                    `
            },
            DoYouWantChabgeStartState: {
                en: `
                    Do you want to change the bot starting message to the current one?
                    `,
                ru: `
                    Вы хотите изменить стартовое сообщение бота на текущее?
                    `
            },
            RemovedPermanently: {
                en: `
                    Are you sure? It will be deleted permanently.
                    `,
                ru: `
                    Вы уверены? Он будет удалён навсегда.
                    `
            },
            NoActiveBot: {
                en: `
                    There is no active bot. First select it from the menu <b>My Bots</b>.
                    `,
                ru: `
                    У Вас нет активного бота. Сначала выберете его в меню <b>My Bots</b>.
                    `
            },
            DeleteBot: {
                en: `
                    Are you sure you want to delete the current bot?    
                    `,
                ru: `
                    Вы уверены, что хотите удалить текущего бота?
                    `
            },
            NoBotsForRename: {
                en: `
                    You haven't chosen your active bot in <b>My Bots</b>.
                    `,
                ru: `
                    Не выбран Ваш активный бот в меню <b>My Bots</b>
                    `
            },
            BotCopy: {
                en: `
                    You copied the current bot, and can now paste it to other bot. 
                    Just select this bot (<b>My bots</b> menu) and then click <b>Edit</b> -  <b>Paste</b>.
                    `,
                ru: `
                    Текущий бот скопирован. Теперь Вы можете вставить его в другой бот. 
                    Для этого выберете этот бот (меню <b>My bots</b>) и затем выполните <b>Edit</b> -  <b>Paste</b>
                    `
            },
            BotPasteCannot1: {
                en: `
                    You haven't copied the bot. Click <b>Edit</b> and select <b>Copy</b>.
                    `,
                ru: `
                    Вы ещё не скопировали бота. Войдите в меню <b>Edit</b> и выберете <b>Copy</b>.
                    `
            },
            BotPasteCannot2: {
                en: `
                    You cannot paste the copied bot to a template. 
                    First select a bot in <b>My Bots</b>.
                    `,
                ru: `
                    Нельзя вставить скопированного бота в темплейт. 
                    Сначала выберете бота в <b>My Bots</b>.
                    `
            },
            import: {
                en: `
                    It will be imported into the current bot.
                    Are you sure you want to continue?
                    `,
                ru: `
                    Импорт  произойдёт в текущего бота. 
                    Вы уверены в этом?
                    `
            },
            SaveToClipboard: {
                en: "Saved to clipboard",
                ru: "Сохранён в клипборд"
            },
            OneState: {
                en: "There has to be at least one step",
                ru: "Должен быть хотя бы один шаг"
            },
            DelState: {
                en: "Are you sure you want to delete this step?",
                ru: "Вы уверены, что хотите удалить этот шаг?"
            },
            DelFolder: {
                en: "Are you sure you want to delete this folder?",
                ru: "Вы уверены, что хотите удалить эту папку?"
            },
            NoActiveFolder: {
                en: "There is no active folder. First select it from the list.",
                ru: "У Вас нет активной папки. Сначала выберете ее из списка."
            },
            UploadBot: {
                en: "Your bot has been added to the site",
                ru: "Ваш бот успешно помещён на сайт"
            },
            UnUploadBot: {
                en: "Your bot is not displayed on the website",
                ru: "Ваш бот на сайте не показывается"
            },
            YouDontCopyActionYet: {
                en: "You haven't copied the action yet",
                ru: "Вы ещё не скопировали действие"
            },
            YouDontCopyItemYet: {
                en: "You have not copied the message element yet.",
                ru: "Вы ещё не скопировали элемент сообщения"
            },
            ClickOnFolder: {
                en: "You can edit the contents of the folder only in the <b>View</b> - <b>Grid</b> mode.",
                ru: "Редактировать содержимое папки можно только в режиме <b>View</b> - <b>Grid</b>"
            },
            NotEmptyState: {
                en: "Step name cannot be empty.",
                ru: "Имя шага не может быть пустым."
            },
            OneButton: {
                en: "There must be at least one button. <p>To remove a row of buttons, use the menu on the left.</p>",
                ru: "Должна быть хотя бы одна кнопка. <p>Для удаления ряда кнопок, используйте меню слева.</p>"
            },
            hintTextItem: {
                en: `
                    Here is the message text. It can be displayed in different languages.
                    Language can be changed in the top right of the step editor (<b>en</b> stands for English).
                    <p> You can use slot names in the texts, preceded by $: </p>
                    <p> Hello $NAME</p>
                    <p> You can also use calculations in double curly brackets: </p>
                    <p> Total: {{$AMOUNT * $PRICE}} </p>
                `,
                ru: `
                    Текст сообщения. Он может быть на различных языках. 
                    Язык меняется в правом верхнем углу редактора шага (<b>en</b> - это английский). 
                    <p>В текстах можно использовать имена слотов, ставя перед ними $:</p>
                    <p>Привет $NAME</p>
                    <p>Можно также использовать вычисления, окружая их двойными фигурными скобками:</p>
                    <p>Стоимость равна {{$AMOUNT * $PRICE}}</p>
                    `
            },
            MustBeQuotes: {
                en: "A string constant or step name has to be enclosed in quotes.",
                ru: "Если это строковая константа или имя шага, то текст нужно окружить кавычками"
            },
            hintImageItem: {
                en: `
                    Add the image url to inserst it. 
                    It can be hosted on your website or an external resource. 
                    The image width and aligment is set in the properties.
                    `,
                ru: `
                    Для вставки в сообщение картинки, необходимо указать её адрес в интернете. 
                    Он может быть на Вашем сайте или на одном из внешних сайтов с изображениями. 
                    В свойствах можно настроить ширину картинки и её центрирование.
                    `
            },
            hintVideoItem: {
                en: `
                    Add the video mp4 url to inserst it. 
                    It can be hosted on your website or an external resource.                     
                    `,
                ru: `
                    Для вставки в сообщение видео mp4, необходимо указать его адрес в интернете. 
                    Оно может быть на Вашем сайте или на одном из внешних сайтов с изображениями.                     
                    `
            },
            hintAudioItem: {
                en: `
                    Add the audio mp3 url to inserst it. 
                    It can be hosted on your website or an external resource.                     
                    `,
                ru: `
                    Для вставки в сообщение звука mp3, необходимо указать его адрес в интернете. 
                    Оно может быть на Вашем сайте или на одном из внешних сайтов с изображениями.                     
                    `
            },
            hintButtonItem: {
                en: `
                    It's a button. You can add few buttons on a row. 
                    Buttons are usually placed at the bottom of the message.
                    <p>In the Item properties, you can set the dialog step that the bot will navigate to
                    when the button is pressed, or save certain information in the slots.</p>
                    `,
                ru: `
                    Это кнопка. В линию можно добавить несколько кнопок. 
                    Обычно в сообщении кнопки стоят последними.
                    <p> В свойствах (Item properties) можно задать шаг диалога в который бот перейдёт 
                    при нажатии на кнопку или сохранить в слотах некоторую информацию.</p>
                    `
            },
            hintActionItem: {
                en: `
                    The actions performed when the dialog step is activated is described here.
                    <p> In Item properties, you can set the next step to navigate by default.
                    It will be done when a button is pressed or when text is entered, if the step isn't defined for them.</p>
                    `,
                ru: `
                    Здесь описываются действия, выполняющиеся, когда активируется этот шаг диалога.
                    <p> В Item properties вы можете задать переход на следующий шаг 
                    по умолчанию для всех кнопок или ввода текста, 
                    а также инициализировать значения слотов.</p>
                    `
            },
            hintTimerItem: {
                en: `
                    The actions performed when the timer expires is described here.
                    <p>The countdown starts when the current step is displayed.</p>
                    `,
                ru: `
                    Здесь описываются действия, выполняющиеся, когда срабатывает таймер. 
                    <p>Таймер начинает отсчёт времени при показе текущего шага.</p>
                    `
            },
            hintTime: {
                en: `
                    Time in milliseconds after which the following actions will be performed. 
                    <p>In particular, the <b>step</b> property is meant to navigate to the next dialog step.
                    `,
                ru: `
                    Время в миллисекундах после которого выполнятся идущие далее действия. 
                    <p>В частности, свойство <b>step</b> приведёт к переходу на следующий шаг диалога.</p>
                    `
            },
            hintMapItem: {
                en: "Link to Google Maps with your coordinates",
                ru: "Ссылка на карту Google Map с Вашими координатами"
            },
            hintMapHight: {
                en: "Map height in pixels",
                ru: "Высота карты в пикселях"
            },
            hintInputItem: {
                en: `
                    Click here to configure the navigation to a new dialog step when a user enters text.
                    <p> This text can be saved in a slot and then used in messages,
                    in which case the slot name is preceded by the dollar sign: $
                    `,
                ru: `
                    Кликнув сюда, Вы можете настроить переход в новый шаг диалога при вводе пользователем текста. 
                    <p>Этот текст можно сохранить в слоте и использовать затем в сообщениях, 
                    поставим перед слотом символ доллара: $.</p>
                    `
            },
            hintStateName: {
                en: `
                    It's the name of the current dialog step.
                    <p>Use meaningful names to easily set navigations in the Item properties.</p>
                    `,
                ru: `
                    Это имя текущего шага диалога. 
                    <p>Стоит задавать осмысленные имена. Тогда будет проще ориентироваться при определении  
                    переходов в Item properties.</p>
                    `
            },
            hintLANGUAGE: {
                en: `
                    Texts and buttons can be lolalized.
                    <p>When a new language is selected, the text of the previous language is displayed in gray.</p>
                    <p>Language selection can be done on a separate step by changing the value of the built-in LANGUAGE slot to ru, en, etc.</p>

                    `,
                ru: `
                    Тексты и надписи на кнопках могут быть на нескольких языках. 
                    <p>При выборе нового языка, текст предыдущего языка  отображается серым цветом.</p> 
                    <p>Смену языка можно сделать в отдельном шаге, изменив значение встроенного слота LANGUAGE на ru, en и т.д.</p>
                    `
            },
            hintStartState: {
                en: `
                    If the checkbox is active, it means the current dialog step is the starting step, which  will be initiated when the bot is launched. 
                    <p>It is marked as "start" in the step card heading.</p>                
                    `,
                ru: `
                    Когда чекбокс активен, это означает, что текущий шаг диалога является стартовым и при запуске бота он выполнится первым. 
                    <p>В заколовке карточки шага это помечается надписью "start".</p>
                    `
            },
            hintState: {
                en: `
                    This is the name of the step to navigate when an event occurs (button pressed, text entered, or timer expired).
                    <p> You needn't edit it. Simply click the input field, and then the card with the step in the list of steps on the right. </>
                    <p> However, it can be deleted or edited. </p>
                    <p> If it's http: // ... or https: // ..., then the link will open in a new browser tab. </p>
                    `,
                ru: `
                    Это имя шага для перехода при возникновении события (нажата кнопка, введен текст или сработал таймер).
                    <p> Его не стоит редактировать. Достаточно кликнуть на поле ввода, а затем  на карточку с шагом в списке шагов справа.</> 
                    <p>Хотя, при желании, его можно удалить или отредактировать.</p>
                    <p>Если это http://... или https://..., то ссылка откроется в новой закладке браузера.</p>
                    `
            },
            hintGoto: {
                en: `
                    This is the step name for the "instant" navigation.
                    All actions below are ignored and the content of the specified step is displayed immediately.
                    `,
                ru: `
                    Это имя шага для "мгновенного" перехода. 
                    Все ниже лежащие действия игнорируются и сразу показывается контент указанного шага.
                    `
            },
            hintInputKind: {
                en: `
                    By default, entered text is regarded a text. However, it can also be an email, a phone, or a number.
                    <p>For example, if you select the email, then the bot will check if the format is correct.
                    If the entered string is not email, then the <b>INPUT</b> slot will remain empty. </p>
                    <p>This can be processed using the condition <b>if</b>: not $INPUT </p>
                    `,
                ru: `
                    По умолчанию, введенный текст считается просто текстом. Однако он также может быть email-ом, телефоном или числом. 
                    <p>Если Вы выберете, например, email, то бот проверит корректность его формата. 
                    Если введенная строка не является email, то слота <b>INPUT</b> будет пустой. </p> 
                    <p>Это можно обработать при помощи условия <b>if</b>:  not $INPUT </p>                    
                    `
            },
            hintSlot: {
                en: `
                    A slot is a variable in which values can be stored. A slot has a name and values.
                    Below, the first field is the name of the slot and the second is its value.
                    `,
                ru: `
                    Слот - это переменная, в которой можно сохранять значения. Слот имеет имя и значения. 
                    Ниже первое поле - это имя слота, а второе - его значение.
                    `
            },
            hintSlotName: {
                en: `
                    You can choose any name for a slot. If you need to create a new slot, just enter its name.
                    It will then appear in the drop-down list.
                    <p>Slots can be used in message texts.
                    If you have saved the client's name in the NAME slot, then you may have "Hello $NAME!",
                    and the client's name will be substitutedin the text. </p>
                    `,
                ru: `
                    Имя слота может быть любым. Если Вам необходимо создать новый слот, просто введите его название. 
                    В дальнейшем он будет появляться в выпадающем списке.  
                    <p>Слот можно использовать в текстах сообщений. 
                    Если Вы сохранили имя клиента в слоте NAME, то в тексте можно написать: 
                    "Привет $NAME!" и это имя будет подставлено.</p>
                    `
            },
            hintSlotValue: {
                en: `
                    The slot value may be a number, a string, values of other slots,
                    or arithmetic expressions. A string has to be enclosed in quotes: "string".
                    <p> For example, the input string properties for the NAME slot can be set to $INPUT.
                    As a result, the text entered by a user will be saved to NAME. </p>
                    <p>
                    If the expression contains slot names, then they has to be preceded by the dollar sign:
                    </p>
                    <p>
                    $AMOUNT * 2
                    </p>
                    `,
                ru: `Значением слота могут быть числа, строки, значения других слотов, 
                    а также арифметические выражения. Строки нужно окружать кавычками: "строка".
                    <p>Например, в свойствах строки ввода для слота NAME можно задать значение $INPUT. 
                    В результате в NAME будет сохранён введенный пользователем текст.</p>
                    <p>                    
                    Если в выражении присутствуют имена слотов, то перед ними должен быть символ доллара:                    
                    </p>
                    <p>
                    $AMOUNT * 2
                    </p>                    
                    `
            },
            hintIf: {
                en: `
                    Any logical expression can be written in the <b>if</b> condition string.
                    If it is true, then the actions going under it are performed.
                    <p>For example, if you request a customer's name, you can check if he has entered a single letter:
                    len($INPUT) &lt; 2
                    </p>
                    <p>
                    If so, then you may set the navigation to the dialog step to reprove the user for it.</p>
                    If the input string type is email, then before saving the value in the slot,
                    add "not $INPUT" in <b>if</b> and navigate to the step informing that the entered email is incorrect. 
                    `,
                ru: `
                    В строке условия <b>if</b> можно написать любое логическое выражение. 
                    Если оно истинно, то выполняются действия, идущие под ним. 
                    <p>Например, если Вы спросили имя клиента, можно проверить не ввёл ли он одну букву:  
                    len($INPUT) &lt; 2
                    </p>
                    <p>
                    Если это так, то можно задать переход в шаг диалога где его за это пожурить.</p>
                    <p>Если строка ввода имеет тип email, то прежде чем его его запомнить в слоте, 
                    следует в <b>if</b> написать: not $INPUT и перйти в шаг сообщающий, что это адрес неверен.</p>
                    `
            },
            hintElIf: {
                en: `
                    These actions are performed if the <b>if</b>-condition above is false and the boolean condition written in the line is true.
                    `,
                ru: `
                    Эти действия выполняются, если ложно условие <b>if</b> выше и истинно логическое условие, записанное в строке.
                    `
            },
            hintElse: {
                en: `
                    These actions are performed if the <b>if</b>-condition above is false.
                    `,
                ru: `
                    Эти действия выполняются, если ложно условие <b>if</b> выше.
                    `
            },
            hintRun: {
                en: `
                    This is the name of a step that is immediately executed, after which it returns to the call point.                    
                    `,
                ru: `
                    Это имя шага, который сразу выполняется, после чего происходит возврат в точку вызова.                                         
                    `
            },
            hintMessageItem: {
                en: `Start of a new message. By default, each step is a new message.
                    You can put this object at the beginning of the step and override, for example, the background color of the message.
                    <p> You can also put this object in the middle of a step and then it will be split into two messages.
                    In addition to color, you can also set the duration of the tapping effect. </p>
                    `,
                ru: `Начало нового сообщения. По умолчанию, каждый шаг является новым сообщением. 
                     Вы можете поставить этот объект вначале шага и переопределить, например цвет фона сообщения.
                     <p>Можно также поставить этот объект в середине шага и тогда он будет разбит на два сообщения.
                     Кроме цвета, можно также задать продолжительность эффекта тайпинга.</p>
                    `
            },
            hintMessageTyping: {
                en: "The time of the taiping effect that simulates the typing of a bot.",
                ru: "Время эффекта тайпинга, имитирующего набор текста ботом."
            },
            hintMessageBack: {
                en: "The background color of the message, if different from the default.",
                ru: "Цвет фона сообщения, если он отличен от принятого по умолчанию."
            },
            hintSwitch: {
                en: `
                    The "<b>switch</b>" statement evaluates the expression.
                    <p> If its value matches the value in the following "<b>case</b>" statements, then their actions will be executed. </p>
                    <p> If no "<b>case</b>" works, then the "<b>default</b>" block will be executed.                
                    `,
                ru: `
                    Оператор "<b>switch</b>" вычисляет выражение. 
                    <p>Если его значение совпадёт со значением в следующих далее операторах "<b>case</b>", то выполнятся их действия.</p>
                    <p>Если ни один "case" не сработает, то выполнится блок "<b>default</b>".
                    `
            },
            hintCase: {
                en: `
                    The actions of the "<b>case</b>" statement are executed if the value of its expression matches the value in the above "<b>switch</b>" statement.
                    <p> If "<b>case</b>" fires, no further "<b>case</b>" statements are checked </p>
                    <p> In the "<b>case</b>" statement, you can write several expressions, separating them with a semicolon. The block will be triggered if one of these values matches the "<b>switch</b>" statement. </p>                
                    `,
                ru: `
                    Действия оператора "<b>case</b>" выполняются, если значение его выражение совпадёт со значением, в стоящем выше операторе "<b>switch</b>".
                    <p>Если "<b>case</b>" сработал, дальнейшие операторы "<b>case</b>" не проверяются</p>
                    <p>В операторе "<b>case</b>" можно записывать несколько выражений, разделяя их точкой с запятой. Блок сработает, если одно из этих значений совпадёт с оператором "<b>switch</b>".</p>
                    `
            },
            hintDefault: {
                en: `
                    The actions of the "<b>default</b>" operator are executed if none of the "<b>case</b>" statements above have worked.
                    `,
                ru: `
                    Действия оператора "<b>default</b>" выполняются, если не сработал ни один оператор "<b>case</b>", стоящий выше.
                    `
            },
            hintIntentItem: {
                en: `
                    In this item, you can describe the classes of intentions of users when they enter text.
                    <p> Examples of intentions: <b> GREET </b> (the person said hello) or <b> YES </b> (the person agreed). </p>
                    <p> For each intent, examples are given on which the bot learns. </p>
                    `,
                ru: `
                    В этом элементе можно описывать классы намерений пользователей при вводе ими текста.
                    <p>Примеры намерений: <b>GREET</b> (человек поздоровался) или <b>YES</b> (человек согласился).</p>
                    <p>Для каждого намерения приводятся примеры, на которых бот обучается.</p>
                    `
            },
            hintIntentName: {
                en: `
                    This is the name of the intent. It must be unique and must be entered without spaces.
                    <p> This name is then used to react to the text in the input field. For the bot to start recognizing intent, <b> kind: </b> intent </p> must be selected in the input field.
                    <p> A simple example can be found in the <b> Examples </b> - <b> NLU </b> menu. </p>                    
                    `,
                ru: `
                    Это имя намерения. Оно должно быть уникальным и вводится без пробелов.
                    <p>Это имя затем используется для реакции на текст в поле ввода. Чтобы бот начал распознавание намерения в полее ввода должен быть выбран <b>kind:</b> intent</p>.
                    <p>Простой пример можно найти в меню <b>Examples</b> -  <b>NLU</b>.</p>
                    `
            },
            hintIntentWords: {
                en: `
                    Examples of texts characteristic of this intention. Separate these examples with semicolons.
                    <p> When attributing a text to this intention, both single words and pairs of words (within this example) are taken into account. </p>
                    `,
                ru: `
                    Примеры текстов характерные для данного намерения. Эти примеры должны разделяться точкой с запятой.
                    <p>При отнесении текста к данному намерению учитываются как одиночные слова, так и пары слов (внутри данного примера).</p>
                    `
            },
            hintActionSave: {
                en: "When performing this action, data from the bot will be sent to external storage.",
                ru: "При выполнении этого действия данные из бота будут посланы во внешнее хранилище."
            },
            hintActionSaveValue: {
                en: `
                    The sent data is an object, with a list of key-values in curly brackets.
                    <p> { "name": $NAME, "age": $AGE, "kind": "person" } </p>
                    `,
                ru: `
                    Посылаемые данные - объект, со списком ключей и значений, окружённых фигурными скобками.
                    <p> { "name": $NAME, "age": $AGE, "kind": "person" } </p>
                    `
            },
            hintActionSaveStorage: {
                en: "What to use as data storage. Learn more here: https://qudata.com/qubot/wordpress",
                ru: "Что использовать в качестве хранилища для данных. Детали можно найти на https://qudata.com/qubot/wordpress"
            },
            hintActionEvent: {
                en: `
                    When this action is performed, information is saved for the analytics system.
                    <p>User navigation to any dialog step automatically generates an event
                    to track the bot funnel.</p>
                    <p>Here you can additionally indicate any significant event.</p>
                    `,
                ru: `
                    При выполнении этого действия сохраняется информация для системы аналитики. 
                    <p>Появление пользователя в любом шаге диалога автоматически генерит событие, 
                    для отслеживания воронки прохождения бота.</p> 
                    <p>Здесь же Вы можете  дополнительно указать важное для Вас событие.</p>
                    `
            },
            hintActionEventKey: {
                en: `
                    It's an arbitrary string aimed to track the bot funnel in the analytics system.
                    `,
                ru: `
                    Произвольная строка, по которой в системе аналитики можно отслеживать воронку 
                    прохождения пользователя по боту.
                    `
            },
            hintActionEventTarget: {
                en: `
                    The key event is marked with a tick.
                    Filter by key events in the analytics system.
                    `,
                ru: `
                    Помечается, что данное событие является ключевым. 
                    В системе аналитики поставить фильтр по ключевым событиям
                    `
            },
            hintClear: {
                en: "Delete the recent chat messages",
                ru: "Удалить из чата последние сообщения"
            },
            hintScript: {
                en: "You can call various functions for slots operation. Chat with the bot from Help",
                ru: "Можно вызвать различные функции по работе со слотами. Пообщайтесь с ботом из Help"
            },
            hintComment: {
                en: "Here you can write an arbitrary comment in the actions",
                ru: `
                    Здесь можно написать произвольный комментарий в действиям
                `
            },
            hint: {
                en: "",
                ru: ""
            }
        }, this.modal = null;
    }
    get(nick) {
        nick = this.messages[nick];
        return nick ? nick[this.helpLanguage] : "Something strange is happening ...";
    }
    alert(nick, dontShow = !1, savedKey = "isStartInfo") {
        if (!this.modal) {
            this.modal = this.add(document.body, "qubot-modal"), this.modal.style.display = "block";
            var header = this.add(this.modal, "qubot-modal-header");
            let close = this.add(header, "qubot-modal-close", "span");
            close.innerHTML = "&times;";
            var header = qubotEditor.help.add(this.modal, "qubot-modal-content"), cont = this.add(header, "qubot-modal-block");
            let text = this.add(cont, "", "p");
            if (text.innerHTML = this.get(nick), !dontShow) {
                cont = qubotEditor.help.add(header, "qubot-modal-block");
                let btnOk = qubotEditor.help.add(cont, "qubot-modal-button");
                btnOk.innerHTML = "✔ Ok", btnOk.onclick = function() {
                    qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                    qubotEditor.help.modal = null;
                };
            }
            if (dontShow) {
                nick = this.add(header, "qubot-modal-footer");
                let check = this.add(nick, null, "input"), lbl = (check.type = "checkbox", check.onclick = function(obj) {
                    (obj = (obj.path || obj.composedPath && obj.composedPath())[0]).checked ? "isStartInfo" === savedKey ? qubotEditor.onclickMenuOptions("start") : qubotEditor.help[savedKey] = !0 : qubotEditor.help[savedKey] = !1, 
                    qubotEditor.help.save();
                }, this.add(nick, "label"));
                lbl.innerHTML = "don't show again";
            }
            close.onclick = function() {
                qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                qubotEditor.help.modal = null;
            }, window.onclick = function(event) {
                event.target == qubotEditor.help.modal && (qubotEditor.help.modal.style.display = "none", 
                qubotEditor.help.modal.remove(), qubotEditor.help.modal = null);
            };
        }
    }
    confirm(nick) {
        return this.nick = nick, new Promise(function(resolve) {
            var nick = this.nick, header = (qubotEditor.help.modal = qubotEditor.help.add(document.body, "qubot-modal"), 
            this.add(qubotEditor.help.modal, "qubot-modal-header"));
            let close = this.add(header, "qubot-modal-close", "span");
            close.innerHTML = "&times;";
            var header = qubotEditor.help.add(qubotEditor.help.modal, "qubot-modal-content"), cont = qubotEditor.help.add(header, "qubot-modal-block");
            let text = qubotEditor.help.add(cont, "", "p");
            text.innerHTML = qubotEditor.help.get(nick);
            cont = qubotEditor.help.add(header, "qubot-modal-block");
            let btnOk = qubotEditor.help.add(cont, "qubot-modal-button"), btnNot = (btnOk.innerHTML = "✔ Yes", 
            qubotEditor.help.add(cont, "qubot-modal-button"));
            btnNot.innerHTML = "❌ Cancel", qubotEditor.help.modal.style.display = "block", btnOk.onclick = function() {
                qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                qubotEditor.help.modal = null, resolve(1);
            }, btnNot.onclick = function() {
                qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                qubotEditor.help.modal = null, resolve(0);
            }, close.onclick = function() {
                qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                qubotEditor.help.modal = null, resolve(0);
            };
        }.bind(this));
    }
    showHint(obj, nick) {
        if (!this.hint && this.isHint) {
            this.hint = this.add(document.body, "qubot-hint");
            let text = this.add(this.hint, "", "p");
            text.innerHTML = this.get(nick), this.hint.style.display = "block";
            nick = obj.getBoundingClientRect(), obj = text.getBoundingClientRect();
            let top = nick.top;
            nick.top + obj.height + 50 > document.documentElement.clientHeight && (top -= nick.top + obj.height - document.documentElement.clientHeight + 50), 
            this.hint.style.left = nick.right + 50 + window.scrollX + "px", this.hint.style.top = top + 0 + window.scrollY + "px";
        }
    }
    hideHint() {
        this.hint && (this.hint.style.display = "none", this.hint.remove(), this.hint = null);
    }
    loadFile() {
        if (!this.modal) return new Promise(function(resolve) {
            qubotEditor.help.modal = qubotEditor.help.add(document.body, "qubot-modal"), qubotEditor.help.modal.ttt = 1, 
            qubotEditor.help.modal.style.display = "block";
            var header = qubotEditor.help.add(qubotEditor.help.modal, "qubot-modal-header");
            const close = qubotEditor.help.add(header, "qubot-modal-close", "span"), content = (close.innerHTML = "&times;", 
            qubotEditor.help.inputFiles = null, qubotEditor.help.add(qubotEditor.help.modal, "qubot-modal-content"));
            content.addEventListener("dragover", event => {
                event.stopPropagation(), event.preventDefault(), event.dataTransfer.dropEffect = "copy";
            }), content.addEventListener("drop", event => {
                event.stopPropagation(), event.preventDefault(), qubotEditor.help.inputFiles = event.dataTransfer.files, 
                qubotEditor.help.inputFiles.length && (qubotEditor.help.inputFileLbl.innerHTML = qubotEditor.help.inputFiles[0].name);
            });
            header = qubotEditor.help.add(content, "qubot-modal-block");
            let text = qubotEditor.help.add(header, "", "p");
            text.innerHTML = qubotEditor.help.get("import"), header = qubotEditor.help.add(content, "qubot-modal-block");
            const lbl = qubotEditor.help.add(header, "qubot-import-input", "label");
            qubotEditor.help.inputFileLbl = qubotEditor.help.add(lbl, ""), qubotEditor.help.inputFileLbl.innerHTML = "Select or drag a file", 
            lbl.setAttribute("for", "upload-file-JSON"), qubotEditor.help.add(lbl, "qubot-icon-upload"), 
            qubotEditor.help.inputFile = qubotEditor.help.add(header, "qubot-import-drop-area", "input"), 
            qubotEditor.help.inputFile.style.display = "none", qubotEditor.help.inputFile.type = "file", 
            qubotEditor.help.inputFile.setAttribute("accept", ".txt, .json"), qubotEditor.help.inputFile.id = "upload-file-JSON", 
            qubotEditor.help.inputFile.onchange = function(ev) {
                qubotEditor.help.inputFile.files.length && (qubotEditor.help.inputFileLbl.innerHTML = qubotEditor.help.inputFile.files[0].name, 
                qubotEditor.help.inputFiles = qubotEditor.help.inputFile.files);
            };
            header = qubotEditor.help.add(content, "qubot-modal-block");
            const btnOk = qubotEditor.help.add(header, "qubot-modal-button"), btnNot = (btnOk.innerHTML = "✔ Yes", 
            qubotEditor.help.add(header, "qubot-modal-button"));
            btnNot.innerHTML = "❌ Cancel", btnOk.disabled = !0, btnOk.onclick = function() {
                if (qubotEditor.help.inputFiles) {
                    qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                    qubotEditor.help.modal = null;
                    const reader = new FileReader();
                    reader.addEventListener("load", event => {
                        qubotEditor.help.inputFileContent = event.target.result, resolve(1);
                    }), reader.readAsText(qubotEditor.help.inputFiles[0]);
                }
            }, btnNot.onclick = function() {
                qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                qubotEditor.help.modal = null, resolve(0);
            }, close.onclick = function() {
                qubotEditor.help.modal.style.display = "none", qubotEditor.help.modal.remove(), 
                qubotEditor.help.modal = null, resolve(0);
            };
        });
    }
    save() {
        qubotData.save("qubotEditorHelp", JSON.stringify({
            isHint: this.isHint,
            isStartInfo: this.isStartInfo,
            isRunFromActive: this.isRunFromActive,
            isRunCurLang: this.isRunCurLang
        }));
    }
    load(callback) {
        this.callback = callback, qubotData.load("qubotEditorHelp", function(data) {
            data && ("isHint" in data && (this.isHint = data.isHint), "isStartInfo" in data && (this.isStartInfo = data.isStartInfo), 
            "langWP" in data && "ru" === data.langWP ? this.helpLanguage = "ru" : this.helpLanguage = "en"), 
            this.callback();
        }.bind(this));
    }
}

class QuBotEditorState extends QuBotEditorBase {
    constructor() {
        super(), this.caption = null, this.content = null, this.items = new QuBotEditorStateItems(), 
        this.actions = null, this.LANGUAGE = "en", this.updateCard = !0, this.startEditText = !1;
    }
    init() {
        this.actions.init();
    }
    clear() {
        for (;this.content.firstChild; ) this.content.removeChild(this.content.firstChild);
    }
    create() {
        var editor = document.getElementById("qubot-state-editor");
        let info = this.add(editor, "info");
        info.classList.add("qubot-disable-select"), info.innerHTML = "Current state:";
        var caption = this.add(editor, "caption");
        this.caption = this.add(caption, "input", "input"), this.caption.setAttribute("onchange", "qubotEditor.stateEditor.onchangeStateName(this)"), 
        this.caption.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintStateName")'), 
        this.caption.onclick = function() {
            qubotEditor.stateEditor.startEditText = !0;
        }, this.caption.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.stateEditor.startEditText = !1;
        };
        let menu = this.add(caption, "qubot-editor-menu"), actions = (menu.classList.add("qubot-disable-select"), 
        menu.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintLANGUAGE")'), 
        menu.onmouseout = function() {
            qubotEditor.help.hideHint();
        }, menu.style.minWidth = "60px", this.startStateChk = this.add(caption, "checkbox", "input"), 
        this.startStateChk.type = "checkbox", this.startStateChk.setAttribute("onchange", "qubotEditor.stateEditor.onchangeStartState(this)"), 
        this.startStateChk.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintStartState")'), 
        this.startStateChk.onmouseout = function() {
            qubotEditor.help.hideHint();
        }, this.createLanguageMenu(menu), this.content = this.add(editor, "content"), info = this.add(editor, "info2"), 
        info.classList.add("qubot-disable-select"), info.innerHTML = "Item properties:", 
        this.add(editor, "actions"));
        actions.id = "qubot-actions", this.actions = new QuBotEditorStateActions(actions);
    }
    createLanguageMenu(menu) {
        menu.style.fontFamily = "Courier";
        menu = this.add(menu, "submenu");
        let b = this.add(menu, "submenu-btn", "button"), cont = (b.innerHTML = "en", b.style.fontWeight = "700", 
        this.add(menu, "submenu-content")), a = (cont.style.top = "20px", this.add(cont, "item"));
        a.innerHTML = "en", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"en")'), 
        a.classList.add("checked"), a = this.add(cont, "item"), a.innerHTML = "ru", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"ru")'), 
        a = this.add(cont, "item"), a.innerHTML = "ua", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"ua")'), 
        a = this.add(cont, "item"), a.innerHTML = "de", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"de")'), 
        a = this.add(cont, "item"), a.innerHTML = "fr", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"fr")'), 
        a = this.add(cont, "item"), a.innerHTML = "es", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"es")'), 
        a = this.add(cont, "item"), a.innerHTML = "jp", a.setAttribute("onclick", 'qubotEditor.stateEditor.setLangMenu(this,"jp")');
    }
    setLangMenu(obj, lang) {
        var all = obj.parentNode.getElementsByClassName("item");
        Array.from(all).forEach(ob => {
            ob.classList.remove("checked");
        }), obj.classList.add("checked"), this.LANGUAGE = lang, qubotEditor.stateEditor.actions.LANGUAGE = lang, 
        this.activeItem && "intents" === this.activeItem.type ? this.actions.show() : qubotEditor.resetEditorAndAllCards(), 
        obj.parentNode.parentNode.getElementsByClassName("submenu-btn")[0].innerHTML = lang;
    }
    set(items) {
        qubotEditor.dontSave = !0, this.items.set(items);
        let messages = this.content.getElementsByClassName("message");
        if (void 0 !== messages) for (;0 < messages.length; ) messages[0].remove();
        let inputs = this.content.getElementsByClassName("item-input");
        if (void 0 !== inputs) for (;0 < inputs.length; ) inputs[0].remove();
        this.caption.value = qubotEditor.bot.state, this.startStateChk.checked = qubotEditor.bot.state === qubotEditor.bot.start;
        var message = this.add(this.content, "message");
        let input = null;
        if (items) {
            this.activeItem = null, 1 === items.length && items.push({
                type: "actions",
                actions: []
            });
            for (let i = 0; i < items.length; i++) {
                var item = items[i];
                if ("text" === item.type) this.addText(message, item.text, i); else if ("row" === item.type) {
                    let row = this.addRow(message);
                    row.id = "qubot-editor-" + i;
                    for (let j = 0; j < item.items.length; j++) this.addButton(item.items[j].button, row, i, j);
                } else if ("images" === item.type) {
                    let images = this.addImages(message, i);
                    images.id = "qubot-editor-" + i;
                    for (let j = 0; j < item.items.length; j++) this.addImage(item.items[j].image, images, i, j);
                } else if ("gallery" === item.type) {
                    let gallery = this.addGallery(message, i);
                    gallery.id = "qubot-editor-" + i;
                    for (let j = 0; j < item.items.length; j++) this.addSlide(item.items[j], gallery, i, j);
                } else "actions" === item.type ? this.addActions(message, i, "actions") : "timer" === item.type ? this.addActions(message, i, "timer") : "message" === item.type ? this.addActions(message, i, "message") : "intents" === item.type ? this.addActions(message, i, "intents") : "map" === item.type ? this.addMap(message, item, i) : "video" === item.type ? this.addMedia(message, item, i, "video", "hintVideoItem") : "audio" === item.type ? this.addMedia(message, item, i, "audio", "hintAudioItem") : "note" !== item.type && "comment" !== item.type || this.addComment(message, item, i);
            }
            for (let i = 0; i < items.length; i++) {
                let item = items[i];
                "input" === item.type && (input = item);
            }
            input || (items.splice(items.length - 1, 0, {
                type: "input",
                actions: []
            }), input = items[items.length - 1]);
            let inp = this.add(this.content, "item-input");
            inp.id = "qubot-editor-" + (items.length - 1), inp.setAttribute("onclick", "qubotEditor.stateEditor.onclickInput(this)"), 
            inp.innerHTML = "Click to show actions when entering text", inp.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintInputItem")'), 
            inp.onmouseout = function() {
                qubotEditor.help.hideHint(), qubotEditor.dontSave = !1;
            }, this.actions.clear(), qubotEditor.dontSave = !1;
        } else qubotEditor.dontSave = !1;
    }
    addAddMenu(btn_add) {
        return this.createAddMenu(btn_add, [ [ "text", "qubotEditor.stateEditor.items.addTextToState  (this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "buttons", "qubotEditor.stateEditor.items.addRowToState(this.parentNode.parentNode.parentNode.parentNode);     qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "images", "qubotEditor.stateEditor.items.addImagesToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "actions", "qubotEditor.stateEditor.items.addActionsToState(this.parentNode.parentNode.parentNode.parentNode); qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "timer", "qubotEditor.stateEditor.items.addTimerToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "block", "qubotEditor.stateEditor.items.addMessageToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "map", "qubotEditor.stateEditor.items.addMapToState(this.parentNode.parentNode.parentNode.parentNode);     qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "video", "qubotEditor.stateEditor.items.addVideoToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "audio", "qubotEditor.stateEditor.items.addAudioToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "nlu", "qubotEditor.stateEditor.items.addIntentsToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ], [ "note", "qubotEditor.stateEditor.items.addCommentToState(this.parentNode.parentNode.parentNode.parentNode);  qubotEditor.save(); qubotEditor.resetEditorAndCard()" ] ]);
    }
    expandTextarea(elm) {
        elm.style.cssText = "height:auto; padding:0", elm.style.cssText = "height:" + (elm.scrollHeight + 3) + "px";
    }
    addActions(message, id, kind) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            let itemActions = this.add(message, "item-actions"), actions = (itemActions.id = "qubot-editor-" + id, 
            this.addButtonsMenu(itemActions), this.add(itemActions, "actions-text"));
            "actions" === kind ? (actions.innerHTML = "🚀", actions.setAttribute("onclick", "qubotEditor.stateEditor.onclickActions(this)"), 
            actions.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintActionItem")')) : "timer" === kind ? (actions.innerHTML = "⏳", 
            actions.setAttribute("onclick", "qubotEditor.stateEditor.onclickActions(this)"), 
            actions.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintTimerItem")')) : "message" === kind ? (actions.innerHTML = "🧾", 
            actions.setAttribute("onclick", "qubotEditor.stateEditor.onclickActions(this)"), 
            actions.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintMessageItem")')) : "intents" === kind && (actions.innerHTML = "🙊", 
            actions.setAttribute("onclick", "qubotEditor.stateEditor.onclickActions(this)"), 
            actions.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintIntentItem")')), 
            actions.onmouseout = function() {
                qubotEditor.help.hideHint(), qubotEditor.dontSave = !1;
            };
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    onclickActions(obj) {
        var ids = obj.parentNode.id.substring(13).split("_");
        this.activeItem && "intents" === this.activeItem.type && qubotEditor.resetEditorAndAllCards(), 
        this.activeItem = qubotEditor.bot.states[qubotEditor.bot.state][ids[0]], this.actions.set(this.activeItem), 
        this.removeAddClass(obj, "qubot-state-editor-active-item"), this.actions.show();
    }
    addText(message, content, id) {
        let itemText = this.add(message, "item-text"), text = (itemText.id = "qubot-editor-" + id, 
        this.addButtonsMenu(itemText), this.add(itemText, "text", "textarea"));
        text.setAttribute("spellcheck", "true");
        message = this.getText(content);
        if (message.length) text.value = message; else {
            let t = this.getNonEmplyText(content);
            0 === t.length && (t = "Text."), text.setAttribute("placeholder", t);
        }
        text.setAttribute("onclick", "qubotEditor.stateEditor.onclickText(this); qubotEditor.stateEditor.startEditText=true;"), 
        text.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintTextItem")'), 
        text.setAttribute("oninput", `qubotEditor.stateEditor.oninputText(this, ${id})`), 
        text.onchange = function() {
            qubotEditor.save();
        }, text.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.stateEditor.startEditText = !1, qubotEditor.dontSave = !1;
        }, text.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), this.expandTextarea(text), 
        qubotEditor.save();
    }
    oninputText(obj, id) {
        var card, txt;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (card = qubotEditor.listEditor.findCardByState(qubotEditor.bot.state), 
        obj && card && (txt = obj.value, this.setText(this.getItem(id).text, txt), qubotEditor.listEditor.onResizeCardEvent(card), 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, card.elem)), 
        this.expandTextarea(obj)) : qubotEditor.help.alert("YouCannotEdit");
    }
    getItem(id) {
        if (qubotEditor.bot.state in qubotEditor.bot.states) return qubotEditor.bot.states[qubotEditor.bot.state][id];
    }
    onclickText(obj) {
        this.expandTextarea(obj), this.onclickActions(obj);
    }
    addRow(message) {
        message = this.add(message, "item-row");
        return this.addButtonsMenu(message), this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state)), 
        message;
    }
    addButton(caption, row, i, j) {
        let button = this.add(row, "button"), text = (button.id = "qubot-editor-" + i + "_" + j, 
        button.setAttribute("onmouseout", "qubotEditor.stateEditor.mouseOutItem(this,  'button')"), 
        this.addBtnDel(button, "qubotEditor.stateEditor.items.deleteItem(this.parentNode);       qubotEditor.resetEditorAndCard(); qubotEditor.save()"), 
        this.addBtnAdd(button, "qubotEditor.stateEditor.items.addButtonToState(this.parentNode); qubotEditor.resetEditorAndCard(); qubotEditor.save()"), 
        this.addBtnLf(button, "qubotEditor.stateEditor.items.moveItem(this.parentNode, -1);     qubotEditor.resetEditorAndCard(); qubotEditor.save()"), 
        this.addBtnRt(button, "qubotEditor.stateEditor.items.moveItem(this.parentNode,  1);      qubotEditor.resetEditorAndCard(); qubotEditor.save()"), 
        this.add(button, "text", "input"));
        text.setAttribute("oninput", `qubotEditor.stateEditor.oninputButton(this, ${i}, ${j})`), 
        text.setAttribute("onchange", "qubotEditor.save()"), text.setAttribute("onclick", "qubotEditor.stateEditor.onclickButton(this)"), 
        text.setAttribute("onfocusout", "qubotEditor.stateEditor.buttonOnOut(this)"), text.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintButtonItem")'), 
        text.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.dontSave = !1;
        };
        row = this.getText(caption);
        return row.length ? text.value = row : (i = this.getNonEmplyText(caption), text.setAttribute("placeholder", i)), 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state)), 
        button;
    }
    onclickButton(obj) {
        var ids;
        0 !== qubotEditor.bot.state.length && qubotEditor.bot.state in qubotEditor.bot.states && (ids = obj.parentNode.id.substring(13).split("_"), 
        this.activeItem = qubotEditor.bot.states[qubotEditor.bot.state][ids[0]].items[ids[1]], 
        this.actions.set(this.activeItem), this.removeAddClass(obj, "qubot-state-editor-active-item"), 
        this.actions.show());
    }
    oninputButton(obj, i, j) {
        var card;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (card = document.getElementById("qubot-state-" + qubotEditor.bot.state), 
        obj = obj.value, this.setText(qubotEditor.bot.states[qubotEditor.bot.state][i].items[j].button, obj), 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, card)) : qubotEditor.help.alert("YouCannotEdit");
    }
    addImages(message, id) {
        message = this.add(message, "item-images");
        return this.addButtonsMenu(message), this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state)), 
        message;
    }
    addImage(caption, image, i, j) {
        let img = this.add(image, "image"), text = (img.id = "qubot-editor-" + i + "_" + j, 
        this.add(img, "text", "textarea"));
        return text.setAttribute("onchange", `qubotEditor.stateEditor.onchangeImage(this, ${i}, ${j})`), 
        text.setAttribute("onclick", "qubotEditor.stateEditor.onclickImage(this)"), text.setAttribute("oninput", "qubotEditor.stateEditor.expandTextarea(this)"), 
        text.value = caption, text.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintImageItem")'), 
        text.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.dontSave = !1;
        }, this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state)), 
        qubotEditor.save(), img;
    }
    onclickImage(obj) {
        this.expandTextarea(obj);
        var ids = obj.parentNode.id.substring(13).split("_");
        this.activeItem = qubotEditor.bot.states[qubotEditor.bot.state][ids[0]].items[ids[1]], 
        this.actions.set(this.activeItem), this.removeAddClass(obj, "qubot-state-editor-active-item"), 
        this.actions.show();
    }
    onchangeImage(obj, i, j) {
        var card;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (card = document.getElementById("qubot-state-" + qubotEditor.bot.state), 
        obj.value = obj.value.replace(/\s+/g, ""), qubotEditor.bot.states[qubotEditor.bot.state][i].items[j].image = obj.value, 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, card), 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addGallery(message, id) {
        message = this.add(message, "gallery");
        return this.addButtonsMenu(message), this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state)), 
        message;
    }
    addSlide(item, image, i, j) {
        let img = this.add(image, "slide"), text = (img.id = "qubot-editor-" + i + "_" + j, 
        this.addBtnDel(img, "qubotEditor.stateEditor.items.deleteItem(this.parentNode);       qubotEditor.resetEditorAndCard(); qubotEditor.save()", -2, -2), 
        this.addBtnAdd(img, "qubotEditor.stateEditor.items.addButtonToState(this.parentNode); qubotEditor.resetEditorAndCard(); qubotEditor.save()", !1, !1, -2, -2), 
        this.addBtnUp(img, "qubotEditor.stateEditor.items.moveItem(this.parentNode, -1);     qubotEditor.resetEditorAndCard(); qubotEditor.save()", 5, -2), 
        this.addBtnDn(img, "qubotEditor.stateEditor.items.moveItem(this.parentNode,  1);      qubotEditor.resetEditorAndCard(); qubotEditor.save()", 5, -2), 
        this.add(img, "text", "textarea"));
        return text.setAttribute("onchange", `qubotEditor.stateEditor.onchangeSlide(this, ${i}, ${j}); qubotEditor.stateEditor.expandTextarea(this)`), 
        text.setAttribute("onclick", "qubotEditor.stateEditor.onclickSlide(this)"), text.setAttribute("oninput", "qubotEditor.stateEditor.expandTextarea(this)"), 
        text.value = item.image, this.expandTextarea(text), text.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintImageItem")'), 
        text.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.dontSave = !1;
        }, this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state)), 
        qubotEditor.save(), img;
    }
    onclickSlide(obj) {
        this.expandTextarea(obj);
        var ids = obj.parentNode.id.substring(13).split("_");
        this.activeItem = qubotEditor.bot.states[qubotEditor.bot.state][ids[0]].items[ids[1]], 
        this.actions.set(this.activeItem), this.removeAddClass(obj, "qubot-state-editor-active-item"), 
        this.actions.show();
    }
    onchangeSlide(obj, i, j) {
        var card;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (card = document.getElementById("qubot-state-" + qubotEditor.bot.state), 
        obj.value = obj.value.replace(/\s+/g, ""), qubotEditor.bot.states[qubotEditor.bot.state][i].items[j].image = obj.value, 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, card), 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addMap(message, item, id) {
        let itemMap = this.add(message, "item-text"), text = (itemMap.id = "qubot-editor-" + id, 
        this.addButtonsMenu(itemMap), this.add(itemMap, "map", "textarea"));
        message = item.url;
        message.length ? text.value = message : text.setAttribute("placeholder", "Url from Google Map"), 
        text.setAttribute("onclick", "qubotEditor.stateEditor.onclickMap(this)"), text.setAttribute("oninput", "qubotEditor.stateEditor.expandTextarea(this)"), 
        text.setAttribute("onchange", `qubotEditor.stateEditor.onchangeMap(this, ${id})`), 
        text.setAttribute("onmouseover", 'qubotEditor.help.showHint(this, "hintMapItem")'), 
        text.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.stateEditor.startEditText = !1, qubotEditor.dontSave = !1;
        }, qubotEditor.save();
    }
    onclickMap(obj) {
        this.expandTextarea(obj), this.onclickActions(obj);
    }
    onchangeMap(obj, id) {
        var card;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (card = document.getElementById("qubot-state-" + qubotEditor.bot.state), 
        obj.value = obj.value.trim(), qubotEditor.bot.states[qubotEditor.bot.state][id].url = obj.value, 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, card), 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addMedia(message, item, id, className, hint) {
        let itemMap = this.add(message, "item-text"), text = (itemMap.id = "qubot-editor-" + id, 
        this.addButtonsMenu(itemMap), this.add(itemMap, className, "textarea"));
        message = item.url;
        message.length ? text.value = message : text.setAttribute("placeholder", "Url of media"), 
        this.expandTextarea(text), text.setAttribute("onchange", `qubotEditor.stateEditor.onchangeMedia(this, ${id})`), 
        text.setAttribute("onclick", "qubotEditor.stateEditor.onclickMedia(this)"), text.setAttribute("oninput", "qubotEditor.stateEditor.expandTextarea(this)"), 
        text.setAttribute("onmouseover", `qubotEditor.help.showHint(this, "${hint}")`), 
        text.onmouseout = function() {
            qubotEditor.help.hideHint(), qubotEditor.stateEditor.startEditText = !1, qubotEditor.dontSave = !1;
        }, qubotEditor.save();
    }
    onclickMedia(obj) {
        this.expandTextarea(obj), this.onclickActions(obj);
    }
    onchangeMedia(obj, id) {
        var card;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (card = document.getElementById("qubot-state-" + qubotEditor.bot.state), 
        obj.value = obj.value.replace(/\s+/g, ""), qubotEditor.bot.states[qubotEditor.bot.state][id].url = obj.value, 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, card), 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addComment(message, item, id) {
        let itemComment = this.add(message, "item-text"), text = (itemComment.id = "qubot-editor-" + id, 
        this.addButtonsMenu(itemComment), this.add(itemComment, "comments", "textarea"));
        message = item.note || item.comment;
        message && message.length ? text.value = message : text.setAttribute("placeholder", "Any comment"), 
        text.setAttribute("oninput", `qubotEditor.stateEditor.oninputComment(this, ${id})`), 
        text.setAttribute("onclick", "qubotEditor.stateEditor.onclickComment(this)"), text.onchange = function() {
            qubotEditor.save();
        }, this.expandTextarea(text), qubotEditor.save();
    }
    onclickComment(obj) {
        this.expandTextarea(obj), this.onclickActions(obj);
    }
    oninputComment(obj, id) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (qubotEditor.bot.states[qubotEditor.bot.state][id].note = obj.value, 
        this.expandTextarea(obj)) : qubotEditor.help.alert("YouCannotEdit");
    }
    onclickInput(obj) {
        let items = qubotEditor.bot.states[qubotEditor.bot.state];
        for (let i = items.length - 1; 0 <= i; i--) if ("input" === items[i].type) {
            "kind" in items[i] || (items[i].kind = "text", qubotEditor.save()), this.activeItem = items[i], 
            this.actions.set(items[i]), this.actions.show();
            break;
        }
        this.removeAddClass(obj, "qubot-state-editor-active-item");
    }
    addButtonsMenu(item) {
        this.addDragMenu(item);
    }
    moveItemInState(obj, dir) {
        obj = obj.id.substring(13);
        this.moveItem(obj, dir), this.set(qubotEditor.bot.states[qubotEditor.bot.state]), 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state));
    }
    moveButtonInState(obj, dir) {
        obj = obj.id.substring(13);
        this.moveItem(obj, dir), thisr.set(qubotEditor.bot.states[qubotEditor.bot.state]), 
        this.updateCard && qubotEditor.listEditor.openCardState(qubotEditor.bot.state, document.getElementById("qubot-state-" + qubotEditor.bot.state));
    }
    mouseOutItem(obj) {
        let btns = obj.getElementsByClassName("qubot-btn");
        btns = obj.parentNode.getElementsByClassName("qubot-btn");
        for (let i = 0; i < 4; i++) btns[i].style.visibility = "visible";
    }
    buttonOnOut(obj) {
        if (0 !== qubotEditor.bot.state.length && qubotEditor.bot.state in qubotEditor.bot.states) {
            obj = obj.parentNode.id.substring(13).split("_"), obj = this.items.getState(obj[0], obj[1]);
            let card = document.getElementById("qubot-state-" + obj);
            card && card.classList.remove("qubot-state-link-main");
        }
    }
    removeAddClass(obj, className) {
        Array.from(this.content.getElementsByClassName(className)).forEach(it => {
            it.classList.remove(className);
        }), obj.classList.add(className);
    }
    onclickEditor(obj) {
        this.removeSlotsMenu(), qubotEditor.wasRun || "Help" === qubotManager.bot.name ? qubotEditor.wasRun = !1 : qubotManager.close(), 
        qubotEditor.dontSave = !1, this.actions.wasOnClickState ? this.actions.wasOnClickState = !1 : this.actions.changingState = null;
    }
    onchangeStateName(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var oldState, newState = obj.value.trim();
            if (qubotEditor.bot.state !== newState) {
                if (0 === newState.length) return obj.value = qubotEditor.bot.state, void qubotEditor.help.alert("NotEmptyState");
                newState in qubotEditor.bot.states ? qubotEditor.help.alert("StateNameExists") : (oldState = qubotEditor.bot.state, 
                qubotEditor.renameState(oldState, newState), obj.value = newState, qubotEditor.init());
            }
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    async onchangeStartState(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            if (qubotEditor.bot.start === qubotEditor.bot.state) return obj.checked = !0, void qubotEditor.help.alert("SetStartStateInOther");
            await qubotEditor.help.confirm("DoYouWantChabgeStartState") ? qubotEditor.listEditor.setStartState(qubotEditor.bot.state) : obj.checked = !1;
        } else qubotEditor.help.alert("YouCannotEdit");
    }
}

class QuBotEditorStateItems {
    constructor() {
        this.items = null;
    }
    set(items) {
        this.items = items;
    }
    getState(row, col) {
        var item = this.items[row];
        if (item && "actions" in item) for (let i = 0; i < item.actions.length; i++) if ("state" in item.actions[i]) return item.actions[i].state;
        return null;
    }
    getStates(items) {
        items = items || this.items;
        let list = [], btnID = 0;
        return items.forEach(item => {
            "row" === item.type ? item.items.forEach(bt => {
                this.getStatesTree(list, bt.actions, btnID, bt), btnID++;
            }) : "input" === item.type ? this.getStatesTree(list, item.actions, -1, item) : "actions" !== item.type && "timer" !== item.type || this.getStatesTree(list, item.actions, -2, item);
        }), list;
    }
    getStatesTree(list, actions, itemID, item) {
        if (actions) for (let i = 0; i < actions.length; i++) {
            var action = actions[i];
            action && ("state" in action && action.state.length ? list.push({
                state: action.state,
                id: itemID,
                item: item,
                action: action
            }) : "goto" in action && action.goto.length ? list.push({
                state: action.goto,
                id: itemID,
                item: item,
                action: action
            }) : "run" in action && action.run.length ? list.push({
                state: action.run,
                id: itemID,
                item: item,
                action: action
            }) : ("if" in action || "else" in action || "switch" in action || "case" in action || "default" in action) && this.getStatesTree(list, action.actions, itemID, item));
        }
    }
    getSlots(items, slots) {
        items ? items.forEach(item => {
            "actions" in item && this.getSlotsTree(item.actions, slots);
        }) : qubotEditor.error("!!!> items.getSlots: No items");
    }
    getSlotsTree(actions, slots) {
        actions.forEach(action => {
            "slot" in action ? slots[action.slot.trim()] = "" : "actions" in action && this.getSlotsTree(action.actions, slots);
        });
    }
    insertItem(id, data) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? 1 === (id = id.split("_")).length ? this.items.splice(parseInt(id[0]) + 1, 0, data) : 2 === id.length && this.items[parseInt(id[0])].items.splice(parseInt(id[1]) + 1, 0, data) : qubotEditor.help.alert("YouCannotEdit");
    }
    async deleteItem(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            if (await qubotEditor.help.confirm("RemovedPermanently")) {
                let id = obj.id.substring(13);
                obj = id.split("_");
                if (1 === obj.length) this.items.splice(parseInt(obj[0]), 1); else if (2 === obj.length) {
                    if (!(1 < this.items[parseInt(obj[0])].items.length)) return void qubotEditor.help.alert("OneButton");
                    this.items[parseInt(obj[0])].items.splice(parseInt(obj[1]), 1);
                }
                qubotEditor.save(), qubotEditor.resetEditorAndCard();
            }
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    moveItem(obj, dir) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            obj = obj.id.substring(13).split("_");
            if (1 === obj.length) {
                var item, i = parseInt(obj[0]);
                0 === i && -1 == dir || i === this.items.length - 1 && 1 == dir || (item = this.items[i], 
                this.items.splice(i, 1), this.items.splice(i + dir, 0, item));
            } else if (2 === obj.length) {
                let i = parseInt(obj[0]), j = parseInt(obj[1]);
                if (!(0 === j && -1 == dir || j === this.items[i].items.length - 1 && 1 == dir)) {
                    let item = this.items[i].items[j];
                    this.items[i].items.splice(j, 1), this.items[i].items.splice(j + dir, 0, item);
                }
            }
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    copyItem(obj) {
        obj = obj.id.substring(13);
        this.copiedItem = JSON.parse(JSON.stringify(this.items[obj]));
    }
    pasteItem(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? this.copiedItem ? (obj = parseInt(obj.id.substring(13)), 
        this.items.splice(obj + 1, 0, JSON.parse(JSON.stringify(this.copiedItem)))) : qubotEditor.help.alert("YouDontCopyItemYet") : qubotEditor.help.alert("YouCannotEdit");
    }
    changeItemPosition(curPos, newPos) {
        if (null === qubotEditor.myBots.botNumber && !qubotEditor.dontSave) return qubotEditor.help.alert("YouCannotEdit"), 
        void qubotEditor.resetEditorAndCard();
        var item;
        0 <= curPos && curPos < this.items.length && 0 <= newPos && newPos < this.items.length && (item = this.items[curPos], 
        this.items.splice(curPos, 1), this.items.splice(newPos, 0, item)), qubotEditor.resetEditorAndCard();
    }
    addMessageToState(obj) {
        obj = obj.id.substring(13);
        this.insertItem(obj, {
            type: "message"
        });
    }
    addTextToState(obj) {
        obj = obj.id.substring(13);
        this.insertItem(obj, {
            type: "text",
            text: {
                en: "Text."
            }
        });
    }
    addRowToState(prev_row) {
        prev_row = prev_row.id.substring(13);
        this.insertItem(prev_row, {
            type: "row",
            items: [ {
                type: "button",
                button: {
                    en: "Button"
                },
                actions: [ {
                    state: ""
                } ]
            } ]
        });
    }
    addActionsToState(prev_row) {
        prev_row = prev_row.id.substring(13);
        this.insertItem(prev_row, {
            type: "actions",
            actions: []
        });
    }
    addIntentsToState(prev_row) {
        prev_row = prev_row.id.substring(13);
        this.insertItem(prev_row, {
            type: "intents",
            actions: [ {
                intent: "",
                words: {}
            } ]
        });
    }
    addTimerToState(prev_row) {
        prev_row = prev_row.id.substring(13);
        this.insertItem(prev_row, {
            type: "timer",
            time: "",
            actions: [ {
                state: ""
            } ]
        });
    }
    addCommentToState(prev_row) {
        prev_row = prev_row.id.substring(13);
        this.insertItem(prev_row, {
            type: "note",
            note: ""
        });
    }
    addButtonToState(prev_btn) {
        prev_btn = prev_btn.id.substring(13);
        this.insertItem(prev_btn, {
            type: "button",
            button: {
                en: "Button"
            },
            actions: [ {
                state: ""
            } ]
        });
    }
    addImagesToState(prev) {
        prev = prev.id.substring(13);
        this.insertItem(prev, {
            type: "images",
            items: [ {
                type: "image",
                image: "https://qudata.com/qubot/im/owl_a.png"
            } ]
        });
    }
    addImageToState(prev) {
        prev = prev.id.substring(13);
        this.insertItem(prev, {
            type: "image",
            image: ""
        });
    }
    addMapToState(obj) {
        obj = obj.id.substring(13);
        this.insertItem(obj, {
            type: "map",
            url: ""
        });
    }
    addVideoToState(obj) {
        obj = obj.id.substring(13);
        this.insertItem(obj, {
            type: "video",
            url: "https://qudata.com/qubot/qudata.mp4",
            stop: 1,
            play: 1
        });
    }
    addAudioToState(obj) {
        obj = obj.id.substring(13);
        this.insertItem(obj, {
            type: "audio",
            url: "https://qudata.com/qubot/QuOwl.mp3",
            stop: 1,
            play: 1
        });
    }
    addGalleryToState(obj) {
        obj = obj.id.substring(13);
        this.insertItem(obj, {
            type: "gallery",
            items: [ {
                type: "slide",
                image: "",
                actions: [ {
                    state: ""
                } ]
            }, {
                type: "slide",
                image: "",
                actions: [ {
                    state: ""
                } ]
            } ]
        });
    }
    exportYAML(items, skip) {
        if (items) {
            let res = "";
            return items.forEach(item => {
                "items" in item ? (res += skip + `  - ${item.type}:
`, item.items.forEach(it => {
                    "button" === it.type ? (res += skip + "      - button: " + JSON.stringify(it.button) + "\n", 
                    it.kind && (res += skip + "        kind: " + it.kind + "\n"), it.align && (res += skip + "        align: " + it.align + "\n"), 
                    res += skip + "        actions:\n", res += this.actionsYAML(it.actions, skip + skip + skip)) : "image" === it.type ? (res += skip + "      - image:\n", 
                    res += skip + "            src: " + JSON.stringify(it.image) + "\n", "center" in it && (res += skip + "            center: " + it.center + "\n"), 
                    "width" in it && (res += skip + '            width: "' + it.width + '"\n')) : res += skip + "       - " + JSON.stringify(it) + "\n";
                })) : "text" === item.type ? (res += skip + "  - text: " + JSON.stringify(item.text) + "\n", 
                "center" in item && (res += skip + "    center: " + item.center + "\n"), "color" in item && (res += skip + '    color: "' + item.color + '"\n')) : "actions" === item.type ? (res += skip + "  - actions:\n", 
                res += this.actionsYAML(item.actions, skip + skip)) : "timer" === item.type ? (res += skip + "  - timer:\n", 
                res += skip + "        time: " + item.time + "\n", res += skip + "        actions:\n", 
                res += this.actionsYAML(item.actions, skip + skip + skip)) : "intents" === item.type ? (res += skip + "  - intents:\n", 
                res += this.actionsYAML(item.actions, skip + skip)) : "input" === item.type ? (res += skip + "  - input:\n", 
                res += skip + "        kind: " + item.kind + "\n", res += skip + "        actions:\n", 
                res += this.actionsYAML(item.actions, skip + skip + skip)) : "note" === item.type ? res += skip + `  - note: "${item.note}"
` : res += skip + "  - " + JSON.stringify(item) + "\n";
            }), res += skip + "#--------------------------------------------------------------------------\n", 
            res + "\n";
        }
        qubotEditor.error("exportYAML: undefined items");
    }
    actionsYAML(actions, skip) {
        if (actions) {
            let res = "";
            for (let i = 0; i < actions.length; i++) {
                var keys, action = actions[i];
                action ? "if" in action ? (res += skip + "  - if: " + JSON.stringify(action.if) + "\n", 
                res += skip + "    actions:\n", res += this.actionsYAML(action.actions, skip + "    ")) : "else" in action ? (res += skip + "  - else: " + JSON.stringify(action.else) + "\n", 
                res += skip + "    actions:\n", res += this.actionsYAML(action.actions, skip + "    ")) : "switch" in action ? (res += skip + "  - switch: " + JSON.stringify(action.switch) + "\n", 
                res += skip + "    actions:\n", res += this.actionsYAML(action.actions, skip + "    ")) : "case" in action ? (res += skip + "  - case: " + JSON.stringify(action.case) + "\n", 
                res += skip + "    actions:\n", res += this.actionsYAML(action.actions, skip + "    ")) : "default" in action ? (res += skip + "  - default: \n", 
                res += skip + "    actions:\n", res += this.actionsYAML(action.actions, skip + "    ")) : "intent" in action ? (res += skip + `  - intent: ${action.intent}
`, res += skip + `    words:  ${JSON.stringify(action.words)}
`) : "slot" in action ? (res += skip + `  - slot: ${action.slot}
`, res += skip + `    value: ${JSON.stringify(action.value)}
`) : 1 === (keys = Object.keys(action)).length ? res += skip + "  - " + keys[0] + ": " + JSON.stringify(action[keys[0]]) + "\n" : res += skip + "  - " + JSON.stringify(action) + "\n" : res += skip + "  - action: null !!! \n";
            }
            return res;
        }
        qubotEditor.error("actionsYAML: undefined actions");
    }
}

class QuBotEditorStateActions extends QuBotEditorBase {
    constructor(container) {
        super(), this.container = container, this.LANGUAGE = "en", this.init();
    }
    init() {
        this.wasOnClickState = !1, this.changingState = null, this.item = null, this.actionID = null, 
        this.actions = [];
    }
    set(item) {
        this.item = item;
    }
    clear(container) {
        for (void 0 === container && (container = this.container); container.childElementCount; ) container.lastChild.remove();
    }
    show(item, container, actionID) {
        item = item || this.item, container = container || this.container, null == actionID && (actionID = 0), 
        "text" === item.type ? this.addTextInterface(container, item) : "image" === item.type ? this.addImageInterface(container, item) : "button" === item.type ? (this.clear(container), 
        this.showButtonInterface(container, item, actionID), this.showActions(container, item.actions, actionID)) : "input" === item.type ? (this.clear(container), 
        this.addInputInterface(container, item), this.showActions(container, item.actions, actionID)) : "actions" === item.type ? (this.clear(container), 
        this.showActions(container, item.actions, actionID)) : "timer" === item.type ? (this.clear(container), 
        this.addTimerInterface(container, item, actionID)) : "intents" === item.type ? (this.clear(container), 
        this.addIntentsInterface(container, item, actionID)) : "message" === item.type ? (this.clear(container), 
        this.addMessageInterface(container, item, actionID)) : "map" === item.type ? (this.clear(container), 
        this.addMapInterface(container, item, actionID)) : "video" === item.type || "audio" === item.type ? (this.clear(container), 
        this.addMediaInterface(container, item, actionID)) : "if" in item || "else" in item || "switch" in item || "case" in item || "default" in item ? (this.clear(container), 
        this.showActions(container, item.actions, actionID)) : ("note" in item || "comment" in item) && this.clear(container);
    }
    showActions(container, actions, actionID) {
        void 0 === actionID && (actionID = 0);
        let stateID = 0;
        if (actions) {
            for (let i = 0; i < actions.length; i++) {
                var action = actions[i];
                action && ("state" in action ? (this.addStateInterface(container, action.state, stateID, actionID, "state"), 
                stateID++, actionID++) : "goto" in action ? (this.addStateInterface(container, action.goto, stateID, actionID, "goto"), 
                stateID++, actionID++) : "run" in action ? (this.addStateInterface(container, action.run, stateID, actionID, "run"), 
                stateID++, actionID++) : "slot" in action ? (this.addSlotInterface(container, action, actionID), 
                actionID++) : "script" in action ? (this.addScriptInterface(container, action, actionID), 
                actionID++) : "note" in action || "comment" in action ? (this.addCommentInterface(container, action, actionID), 
                actionID++) : "clear" in action ? (this.addActionClearInterface(container, action, actionID), 
                actionID++) : "action" in action ? "save" === action.action ? (this.addActionSaveInterface(container, action, actionID), 
                actionID++) : "event" === action.action && (this.addActionEventInterface(container, action, actionID), 
                actionID++) : "if" in action ? actionID = this.addIfInterface(container, action, actionID, actions, i) : "else" in action ? actionID = action.else.length ? this.addElifInterface(container, action, actionID, actions, i) : this.addElseInterface(container, action, actionID, actions, i) : "switch" in action ? actionID = this.addSwitchInterface(container, action, actionID) : "case" in action ? actionID = this.addCaseInterface(container, action, actionID) : "default" in action && (actionID = this.addDefaultInterface(container, action, actionID)));
            }
            0 === actions.length && this.addAddAction(container, actionID);
        } else this.item.actions = [], this.addAddAction(container, actionID);
        return actionID;
    }
    addAddAction(container, actionID) {
        let a = this.add(container, "action");
        a = this.add(a, "action-add"), a.id = "qubot-action-add-" + (actionID - 1), a.setAttribute("onclick", "qubotEditor.stateEditor.actions.runAddAddMenu(this)"), 
        a = this.add(a, "qubot-icon-plus-row-add"), a.style.filter = "none";
    }
    runAddAddMenu(obj) {
        let menu = qubotEditor.stateEditor.actions.addAddMenu(obj);
        menu.style.display = "flex", menu.style.left = (obj.clientWidth - menu.clientWidth) / 2 + "px";
    }
    getAction(actions, actionID, totID = {
        id: 0
    }) {
        for (let i = 0; i < actions.length; i++) {
            if (actionID === totID.id) return actions[i];
            if ("if" in actions[i]) {
                totID.id++;
                var action = this.getAction(actions[i].actions, actionID, totID);
                if (action) return action;
            } else if ("else" in actions[i]) {
                totID.id++;
                let action = this.getAction(actions[i].actions, actionID, totID);
                if (action) return action;
            } else if ("switch" in actions[i]) {
                totID.id++;
                let action = this.getAction(actions[i].actions, actionID, totID);
                if (action) return action;
            } else if ("case" in actions[i]) {
                totID.id++;
                let action = this.getAction(actions[i].actions, actionID, totID);
                if (action) return action;
            } else if ("default" in actions[i]) {
                totID.id++;
                let action = this.getAction(actions[i].actions, actionID, totID);
                if (action) return action;
            } else totID.id++;
        }
        return null;
    }
    addAddMenu(btn_add, disabled = [ "elif", "else", "case", "default" ]) {
        return this.createAddMenu(btn_add, [ [ "step", "qubotEditor.stateEditor.actions.addAction(this, 'state')" ], [ "goto", "qubotEditor.stateEditor.actions.addAction(this, 'goto')" ], [ "run", "qubotEditor.stateEditor.actions.addAction(this, 'run')" ], [ "slot", "qubotEditor.stateEditor.actions.addAction(this, 'slot')" ], [ "script", "qubotEditor.stateEditor.actions.addAction(this, 'script')" ], [ "clear", "qubotEditor.stateEditor.actions.addAction(this, 'clear')" ], [ "if", "qubotEditor.stateEditor.actions.addAction(this, 'if')" ], [ "else", "qubotEditor.stateEditor.actions.addAction(this, 'else')" ], [ "elif", "qubotEditor.stateEditor.actions.addAction(this, 'elif')" ], [ "switch", "qubotEditor.stateEditor.actions.addAction(this, 'switch')" ], [ "case", "qubotEditor.stateEditor.actions.addAction(this, 'case')" ], [ "default", "qubotEditor.stateEditor.actions.addAction(this, 'default')" ], [ "save", "qubotEditor.stateEditor.actions.addAction(this, 'save')" ], [ "event", "qubotEditor.stateEditor.actions.addAction(this, 'event')" ], [ "note", "qubotEditor.stateEditor.actions.addAction(this, 'note')" ] ], disabled);
    }
    addInterfaceFrame(container, actionName, actionID, menu = !0, disabled = [ "elif", "else", "case", "default" ]) {
        let action = this.add(container, "action");
        if (null != actionID && (action.id = "qubot-action-" + actionID), menu && this.addDragMenu(action, !1, disabled), 
        actionName.length) {
            let name = this.add(action, "name");
            "state" === actionName && (actionName = "step"), name.innerHTML = actionName + ":", 
            name.setAttribute("onclick", "qubotEditor.stateEditor.actions.onclickAction(this)"), 
            "default" !== actionName && "else" !== actionName && 0 !== actionName.indexOf("typing") && 0 !== actionName.indexOf("background") && 0 !== actionName.indexOf("height") && 0 !== actionName.indexOf("time") || (action.style.flexDirection = "column", 
            name.style.width = "200px");
        }
        return action;
    }
    addInterfaceFrameCheck(container, caption) {
        container = this.add(container, "action");
        let chk = this.add(container, "checkbox", "input"), name = (chk.type = "checkbox", 
        this.add(container, "name"));
        return name.innerHTML = caption, chk;
    }
    onclickAction(obj) {
        return;
    }
    addHint(obj, nick) {
        obj.setAttribute("onmouseover", `qubotEditor.help.showHint(this, "${nick}")`), obj.setAttribute("onmouseout", "qubotEditor.help.hideHint()");
    }
    expandTextarea(elm) {
        elm.style.cssText = "height:auto; padding:0", elm.style.cssText = "height:" + (elm.scrollHeight + 2) + "px";
    }
    expandCode(elm) {
        elm.style.cssText = "height:auto; padding:0", elm.style.cssText = "height:" + elm.codeHeight() + "px";
    }
    addInputInterface(container, item) {
        this.clear(container);
        var container = this.addInterfaceFrame(container, "kind", null, !1), div = (this.addHint(container, "hintInputKind"), 
        this.add(container, "group"));
        let knd = this.add(div, "select", "select"), sel;
        knd.style.width = "100%", sel = this.add(knd, "", "option"), sel.value = "text", 
        sel.innerHTML = "✍🏽 text", sel = this.add(knd, "", "option"), sel.value = "int", 
        sel.innerHTML = "🔢 integer: 137 ", sel = this.add(knd, "", "option"), sel.value = "float", 
        sel.innerHTML = "🔢 number:  3.141592", sel = this.add(knd, "", "option"), sel.value = "email", 
        sel.innerHTML = "📧 email:   info@qudata.com", sel = this.add(knd, "", "option"), 
        sel.value = "phone", sel.innerHTML = "📞 phone:   +1 234 567 8900", sel = this.add(knd, "", "option"), 
        sel.value = "MMDDYYYY", sel.innerHTML = "📆 date:    mm/dd/yyyy", sel = this.add(knd, "", "option"), 
        sel.value = "DDMMYYYY", sel.innerHTML = "📆 date:    dd/mm/yyyy", sel = this.add(knd, "", "option"), 
        sel.value = "time", sel.innerHTML = "⌚ time:    hh:mm", sel = this.add(knd, "", "option"), 
        sel.value = "intent", sel.innerHTML = '🙊 intent: "I would like.."', item.kind && (knd.value = item.kind), 
        knd.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeInputKind(this);"), 
        this.add(container, "hr", "hr");
    }
    onchangeInputKind(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.item.kind = obj.value, 
        qubotEditor.listEditor.openCardCurState(), qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    showButtonInterface(container, item, actionID) {
        this.clear(container);
        var container = this.addInterfaceFrame(container, "kind", null, !1), div = this.add(container, "group");
        let knd = this.add(div, "select", "select"), sel;
        knd.style.width = "50%", sel = this.add(knd, "", "option"), sel.value = "button", 
        sel.innerHTML = "button", sel = this.add(knd, "", "option"), sel.value = "check", 
        sel.innerHTML = "check", sel = this.add(knd, "", "option"), sel.value = "radio", 
        sel.innerHTML = "radio", "check" === item.kind || "radio" === item.kind ? knd.value = item.kind : knd.value = "button", 
        knd.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeButtonKind(this);"), 
        knd = this.add(div, "select", "select"), knd.style.width = "50%", sel = this.add(knd, "", "option"), 
        sel.value = "center", sel.innerHTML = "center", sel = this.add(knd, "", "option"), 
        sel.value = "left", sel.innerHTML = "left", sel = this.add(knd, "", "option"), sel.value = "right", 
        sel.innerHTML = "right", knd.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeButtonAlign(this);"), 
        item.align && (knd.value = item.align), this.add(container, "hr", "hr"), 0 === item.actions.length && (item.actions = [ {
            state: ""
        } ]);
    }
    onchangeButtonKind(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? "button" === obj.value && !this.item.kind || (this.item.kind = obj.value, 
        qubotEditor.listEditor.openCardCurState(), qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    onchangeButtonAlign(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? "center" === obj.value && !this.item.align || (this.item.align = obj.value, 
        qubotEditor.listEditor.openCardCurState(), qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addMapInterface(container, item, actionID) {
        container = this.addInterfaceFrame(container, "height (px)", null, !1);
        this.addHint(container, "hintMapHight");
        let inp = this.add(container, "input", "input");
        inp.type = "number", inp.min = "0", inp.max = "1000", inp.id = "qubot-map-time-" + actionID, 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.onchangeHeightMap(this); qubotEditor.listEditor.openCardCurState ()"), 
        "height" in item && item.height.length ? inp.value = item.height : inp.value = 200;
    }
    onchangeHeightMap(obj) {
        var val = obj.value;
        val < 1 && (obj.value = 1), 1e3 < val && (obj.value = 1e3), this.item.height = obj.value.trim(), 
        qubotEditor.save();
    }
    addMediaInterface(container, item, actionID) {
        this.clear(container);
        let chk = this.addInterfaceFrameCheck(container, "stop on event");
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeMediaStop(this); qubotEditor.listEditor.openCardCurState ()"), 
        "stop" in item && item.stop && (chk.checked = !0), chk = this.addInterfaceFrameCheck(container, "autoplay"), 
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeMediaPlay(this); qubotEditor.listEditor.openCardCurState ()"), 
        "play" in item && item.play && (chk.checked = !0), chk = this.addInterfaceFrameCheck(container, "controls"), 
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeMediaControl(this); qubotEditor.listEditor.openCardCurState ()"), 
        "control" in item && item.control && (chk.checked = !0), chk = this.addInterfaceFrameCheck(container, "loop"), 
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeMediaLoop(this); qubotEditor.listEditor.openCardCurState ()"), 
        "loop" in item && item.loop && (chk.checked = !0);
    }
    onchangeMediaStop(obj) {
        this.item.stop = obj.checked ? 1 : 0, qubotEditor.save();
    }
    onchangeMediaPlay(obj) {
        this.item.play = obj.checked ? 1 : 0, qubotEditor.save();
    }
    onchangeMediaControl(obj) {
        this.item.control = obj.checked ? 1 : 0, qubotEditor.save();
    }
    onchangeMediaLoop(obj) {
        this.item.loop = obj.checked ? 1 : 0, qubotEditor.save();
    }
    addMessageInterface(container, item, actionID) {
        var action = this.addInterfaceFrame(container, "typing (ms)", null, !1);
        this.addHint(action, "hintMessageTyping");
        let inp = this.add(action, "input", "input"), col = (inp.type = "number", inp.min = "0", 
        inp.id = "qubot-message-typing", inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeMessageTyping(this);"), 
        inp.value = "0", "typing" in item && item.typing && (inp.value = "" + item.typing), 
        action = this.addInterfaceFrame(container, "background color", null, !1), this.addHint(action, "hintMessageBack"), 
        this.add(action, "color", "input"));
        col.type = "color", col.setAttribute("title", "Text color"), col.setAttribute("oninput", "qubotEditor.stateEditor.actions.onchangeMessageBack(this); qubotEditor.listEditor.openCardCurState ()"), 
        "back" in item && item.back.length ? col.value = item.back : (container = getComputedStyle(document.documentElement).getPropertyValue("--qubot-message-background").trim(), 
        col.value = container || "#f7f7f7");
    }
    onchangeMessageTyping(obj) {
        var val = obj.value;
        isNaN(val) && (obj.value = "0"), this.item.typing = obj.value, qubotEditor.save();
    }
    onchangeMessageBack(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.item.back = obj.value, 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addTimerInterface(container, item, actionID) {
        var action = this.addInterfaceFrame(container, "time (ms)", null, !1);
        this.addHint(action, "hintTime");
        let inp = this.add(action, "input", "input");
        inp.type = "number", inp.min = "100", inp.id = "qubot-timer-time", inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeTimerTime(this);"), 
        inp.value = "1000", "time" in item && item.time ? inp.value = "" + item.time : item.time = 1e3, 
        this.add(action, "hr", "hr"), this.showActions(container, item.actions, actionID);
    }
    onchangeTimerTime(obj) {
        var val = obj.value;
        (isNaN(val) || val < 100) && (obj.value = "100"), this.item.time = obj.value, qubotEditor.save();
    }
    addIntentsInterface(container, item, actionID) {
        let actions = item.actions;
        0 === actions.length && actions.push({
            intent: "",
            words: {
                en: ""
            }
        });
        for (let i = 0; i < actions.length; i++) {
            var intent = actions[i];
            let action = this.addInterfaceFrame(container, "", actionID), add = (action.style.flexDirection = "column", 
            this.addHint(action, "hintIntent"), action.children[0].children[1]), inp = (add.setAttribute("onclick", "qubotEditor.stateEditor.actions.addNewIntent(this)"), 
            this.add(action, "input", "input"));
            inp.setAttribute("placeholder", "Intent name"), inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeIntentName(this)"), 
            this.addHint(inp, "hintIntentName"), inp.value = intent.intent, inp.id = "qubot-intent-name" + actionID, 
            inp = this.add(action, "intents", "textarea"), inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeIntentWords(this)"), 
            inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandTextarea(this)"), 
            inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandTextarea(this)"), 
            this.addHint(inp, "hintIntentWords");
            var tx = this.getText(intent.words);
            if (tx.length) inp.value = tx; else {
                let t = this.getNonEmplyText(intent.words);
                0 === t.length && (t = "Intent examples"), inp.setAttribute("placeholder", t);
            }
            inp.id = "qubot-intent-words" + actionID, this.expandTextarea(inp), actionID++;
        }
        return actionID;
    }
    onchangeIntentName(obj) {
        var id = parseInt(obj.id.substring(17));
        let action = this.getAction(this.item.actions, id);
        id = obj.value.trim();
        action.intent = id, qubotEditor.save();
    }
    onchangeIntentWords(obj) {
        var id = parseInt(obj.id.substring(18));
        let action = this.getAction(this.item.actions, id);
        id = obj.value.trim();
        action.words[this.LANGUAGE] = id, qubotEditor.save();
    }
    addNewIntent(obj) {
        this.addAction(obj, "intent");
    }
    addCommentInterface(container, item, actionID) {
        container = this.addInterfaceFrame(container, "", actionID);
        this.addHint(container, "hintComment");
        let inp = this.add(container, "comments", "textarea");
        inp.id = "qubot-script-" + actionID, inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeComment(this); qubotEditor.listEditor.openCardCurState ()"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandTextarea(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandTextarea(this)");
        container = item.note || item.comment;
        container ? inp.value = container : inp.setAttribute("placeholder", "Any comment"), 
        this.expandTextarea(inp);
    }
    onchangeComment(obj) {
        var id = parseInt(obj.id.substring(13));
        let action = this.getAction(this.item.actions, id);
        action.note = obj.value.trim(), qubotEditor.save();
    }
    addTextInterface(container, item) {
        this.clear(container);
        var action = this.addInterfaceFrame(container, "color", null, !1);
        let col = this.add(action, "color", "input"), chk = (col.type = "color", col.setAttribute("title", "Text color"), 
        col.setAttribute("oninput", "qubotEditor.stateEditor.actions.onchangeTextColor(this); qubotEditor.listEditor.openCardCurState ()"), 
        "color" in item && item.color.length ? col.value = item.color : (action = getComputedStyle(document.documentElement).getPropertyValue("--qubot-message-color").trim(), 
        col.value = action || "#212529"), this.addInterfaceFrameCheck(container, "center"));
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeTextCenter(this); qubotEditor.listEditor.openCardCurState ()"), 
        "center" in item && item.center && (chk.checked = !0), chk = this.addInterfaceFrameCheck(container, "pre"), 
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeTextPre(this); qubotEditor.listEditor.openCardCurState ()"), 
        "pre" in item && item.pre && (chk.checked = !0);
    }
    onchangeTextCenter(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.item.center = obj.checked ? 1 : 0, 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    onchangeTextColor(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.item.color = obj.value, 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    onchangeTextPre(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.item.pre = obj.checked ? 1 : 0, 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addImageInterface(container, image) {
        this.clear(container);
        var action = this.addInterfaceFrame(container, "width (%) ", null, !1);
        let inp = this.add(action, "input", "input"), chk = (inp.setAttribute("placeholder", "image width in percentage (default 100%)"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeImageWidth(this);  qubotEditor.listEditor.openCardCurState ()"), 
        inp.type = "number", inp.min = "0", inp.max = "100", image.width ? inp.value = image.width : inp.value = "100", 
        this.addInterfaceFrameCheck(container, "center"));
        chk.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeImageCenter(this); qubotEditor.listEditor.openCardCurState ()"), 
        chk.type = "checkbox", void 0 === image.center ? chk.checked = !0 : chk.checked = image.center;
    }
    onchangeImageWidth(obj) {
        var val;
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? ((val = obj.value) < 1 && (obj.value = 1), 
        100 < val && (obj.value = 100), this.item.width = obj.value, qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    onchangeImageCenter(obj) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (obj.checked ? this.item.center = 1 : this.item.center = 0, 
        qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    addStateInterface(container, state, stateID, actionID, caption) {
        container = this.addInterfaceFrame(container, caption, actionID);
        "state" === caption ? this.addHint(container, "hintState") : "goto" === caption ? this.addHint(container, "hintGoto") : "run" === caption && this.addHint(container, "hintRun");
        let inp = this.add(container, "input", "input");
        inp.setAttribute("onclick", `qubotEditor.stateEditor.actions.onclickInputState(this,${actionID}, "${caption}")`), 
        inp.setAttribute("placeholder", "Click here and then on the step card"), inp.setAttribute("onchange", `qubotEditor.stateEditor.actions.onchangeInputState(this, ${actionID}, "${caption}" )`), 
        state && state.length && (inp.value = state);
    }
    onclickInputState(obj, actionID, caption) {
        this.wasOnClickState = !0, this.changingState = caption, this.actionID = parseInt(actionID);
    }
    onchangeInputState(obj, actionID, caption) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            let action = this.getAction(this.item.actions, actionID), val = obj.value.trim();
            val.length ? 0 === val.indexOf("http://") || 0 === val.indexOf("https://") ? action.state = val : val in qubotEditor.bot.states || "$" === val[0] ? (this.changeState(val, caption), 
            this.wasOnClickState = !0, this.changingState = !0) : (qubotEditor.help.alert("UnknownState"), 
            obj.value = "", action[caption] = "") : (action[caption] = "", this.wasOnClickState = !0, 
            this.changingState = !0), 0 === val.length && (this.changingState = caption), qubotEditor.listEditor.setCurState(), 
            qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    changeState(newState, caption) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.changeStateTree(newState, this.item.actions, 0, caption), 
        qubotEditor.dontSave = !1, qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    changeStateTree(newState, actions, actionID, caption) {
        for (let i = 0; i < actions.length; i++) {
            if (actionID === this.actionID) return void (actions[i][caption] = newState);
            "if" in actions[i] || "else" in actions[i] || "switch" in actions[i] || "case" in actions[i] || "default" in actions[i] ? actionID = this.changeStateTree(newState, actions[i].actions, actionID + 1, caption) : actionID++;
        }
        return actionID;
    }
    addSlotInterface(container, slot, actionID) {
        var s, container = this.addInterfaceFrame(container, "slot", actionID);
        this.addHint(container, "hintSlot");
        let inp = this.add(container, "input", "input"), lst = (inp.setAttribute("placeholder", "Slot name"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeSlotName(this)"), 
        inp.setAttribute("list", "qubot-slot-list-" + actionID), this.addHint(inp, "hintSlotName"), 
        inp.value = slot.slot, inp.id = "qubot-action-slot-var-" + actionID, this.add(container, null, "datalist"));
        for (s in lst.id = "qubot-slot-list-" + actionID, qubotEditor.bot.slots) this.add(lst, "option", "option").value = s;
        inp = this.add(container, null, "code-input"), inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), 
        inp.setAttribute("placeholder", "Slot value"), inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeSlotValue(this)"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), this.addHint(inp, "hintSlotValue"), 
        inp.value = slot.value, inp.id = "qubot-action-slot-val-" + actionID, this.expandCode(inp);
    }
    onchangeSlotName(obj) {
        var id = parseInt(obj.id.substring(22));
        let action = this.getAction(this.item.actions, id), slot = obj.value.trim();
        slot.length && "$" === slot[0] && (slot = slot.substring(1), obj.value = slot), 
        action.slot = slot, action.slot in qubotEditor.bot.slots || (qubotEditor.bot.slots[action.slot] = ""), 
        qubotEditor.save();
    }
    onchangeSlotValue(obj) {
        var id = parseInt(obj.id.substring(22));
        let action = this.getAction(this.item.actions, id);
        id = obj.value.trim();
        (action.value = id).length && "$" !== id[0] && "[" !== id[0] && "{" !== id[0] && isNaN(id) && '"' != id[0] && "'" != id[0] && qubotEditor.help.alert("MustBeQuotes"), 
        qubotEditor.save();
    }
    addScriptInterface(container, item, actionID) {
        container = this.addInterfaceFrame(container, "", actionID);
        this.addHint(container, "hintScript");
        let inp = this.add(container, null, "code-input");
        inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), inp.id = "qubot-script-" + actionID, 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeScript(this); qubotEditor.listEditor.openCardCurState ()"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("placeholder", "Script actions"), inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), 
        "script" in item && item.script.length && (inp.value = item.script), this.expandCode(inp);
    }
    onchangeScript(obj) {
        var id = parseInt(obj.id.substring(13));
        let action = this.getAction(this.item.actions, id);
        action.script = obj.value.trim(), qubotEditor.save();
    }
    addIfInterface(container, actionIf, actionID, actions, id) {
        let disabled = [ "case", "default" ];
        id + 1 < actions.length && "else" in actions[id + 1] && (disabled = disabled.concat([ "step", "goto", "run", "slot", "script", "clear", "if", "switch", "save", "event" ]));
        actions = this.addInterfaceFrame(container, "if", actionID, !0, disabled);
        this.addHint(actions, "hintIf");
        let inp = this.add(actions, null, "code-input");
        return inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeIF(this)"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("placeholder", "Enter boolean expression"), inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), 
        inp.value = actionIf.if, inp.id = "qubot-action-if-" + actionID, this.expandCode(inp), 
        actionID++, actionIf.actions && (id = this.add(actions, "actions"), actionID = this.showActions(id, actionIf.actions, actionID)), 
        actionID;
    }
    onchangeIF(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(16));
            let action = this.getAction(this.item.actions, id);
            action.if = obj.value, qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addElifInterface(container, actionIf, actionID, actions, id) {
        let disabled = [ "case", "default" ];
        id + 1 < actions.length && "else" in actions[id + 1] && (disabled = disabled.concat([ "step", "goto", "run", "slot", "script", "clear", "if", "switch", "save", "event" ]));
        actions = this.addInterfaceFrame(container, "elif", actionID, !0, disabled);
        this.addHint(actions, "hintElIf");
        let inp = this.add(actions, null, "code-input");
        return inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeELIF(this)"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("placeholder", "Enter boolean expression"), inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), 
        inp.value = actionIf.else, inp.id = "qubot-action-if-" + actionID, this.expandCode(inp), 
        actionID++, actionIf.actions && (id = this.add(actions, "actions"), actionID = this.showActions(id, actionIf.actions, actionID)), 
        actionID;
    }
    onchangeELIF(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(16));
            let action = this.getAction(this.item.actions, id);
            action.else = obj.value.trim(), qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addElseInterface(container, actionElse, actionID, actions, id) {
        var container = this.addInterfaceFrame(container, "else", actionID, !0);
        return this.addHint(container, "hintElse"), actionID++, actionElse.actions && (container = this.add(container, "actions"), 
        actionID = this.showActions(container, actionElse.actions, actionID)), actionID;
    }
    onchangeElse(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(16));
            let action = this.getAction(this.item.actions, id);
            action.if = obj.value, qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addSwitchInterface(container, actionIf, actionID) {
        var container = this.addInterfaceFrame(container, "switch", actionID);
        this.addHint(container, "hintSwitch");
        let inp = this.add(container, null, "code-input");
        return inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeSwitch(this)"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("placeholder", "Enter some expression"), inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), 
        inp.value = actionIf.switch, inp.id = "qubot-action-switch-" + actionID, this.expandCode(inp), 
        actionID++, actionIf.actions && (container = this.add(container, "actions"), actionID = this.showActions(container, actionIf.actions, actionID)), 
        actionID;
    }
    onchangeSwitch(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(20));
            let action = this.getAction(this.item.actions, id);
            action.switch = obj.value.trim(), qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addCaseInterface(container, actionIf, actionID) {
        container = this.addInterfaceFrame(container, "case", actionID, !0, [ "step", "goto", "run", "slot", "script", "clear", "if", "else", "elif", "switch", "save", "event" ]);
        this.addHint(container, "hintCase");
        let inp = this.add(container, null, "code-input");
        inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeCase(this)"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("placeholder", "Enter some expression"), inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), 
        inp.id = "qubot-action-case-" + actionID, this.expandCode(inp);
        var val = actionIf.case;
        return val && (inp.value = val), actionID++, actionIf.actions && (val = this.add(container, "actions"), 
        actionID = this.showActions(val, actionIf.actions, actionID)), actionID;
    }
    onchangeCase(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(18));
            let action = this.getAction(this.item.actions, id);
            action.case = obj.value.trim(), qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addDefaultInterface(container, actionIf, actionID) {
        var container = this.addInterfaceFrame(container, "default", actionID);
        return this.addHint(container, "hintDefault"), actionID++, actionIf.actions && (container = this.add(container, "actions"), 
        actionID = this.showActions(container, actionIf.actions, actionID)), actionID;
    }
    addActionClearInterface(container, clear, actionID) {
        container = this.addInterfaceFrame(container, "clear", actionID);
        this.addHint(container, "hintClear");
        let inp = this.add(container, "input", "input");
        inp.setAttribute("placeholder", "The number of recent messages (-1 all)"), inp.setAttribute("onchange", `qubotEditor.stateEditor.actions.onchangeInputClear(this, ${actionID} )`), 
        clear.clear && clear.clear.length && (inp.value = clear.clear);
    }
    onchangeInputClear(obj, actionID) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            let action = this.getAction(this.item.actions, actionID);
            actionID = obj.value.trim();
            action.clear = actionID, qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addActionSaveInterface(container, action, actionID) {
        container = this.addInterfaceFrame(container, "save", actionID);
        this.addHint(container, "hintActionSave");
        let des = this.add(container, "select", "select"), op = (des.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeActionSaveStorage(this);"), 
        this.addHint(des, "hintActionSaveStorage"), des.id = "qubot-action-save-des-" + actionID, 
        this.add(des, null, "option")), inp = (op.setAttribute("disabled", !0), op.setAttribute("selected", !1), 
        op.setAttribute("value", ""), op.innerHTML = "Select storage location", op = this.add(des, null, "option"), 
        op.innerHTML = "Word Press", (op.value = "wp") === action.storage && op.setAttribute("selected", !0), 
        op = this.add(des, null, "option"), op.innerHTML = "QuData.com", (op.value = "qudata") === action.storage && op.setAttribute("selected", !0), 
        this.add(container, null, "code-input"));
        inp.setAttribute("lang", "qb"), inp.setAttribute("spellcheck", "false"), inp.setAttribute("placeholder", "Data value"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeActionSaveValue(this)"), 
        inp.setAttribute("oninput", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.setAttribute("onclick", "qubotEditor.stateEditor.actions.expandCode(this)"), 
        inp.addEventListener("keydown", qubotEditor.stateEditor.onkeydownText), this.addHint(inp, "hintActionSaveValue"), 
        inp.value = action.value, inp.id = "qubot-action-save-val-" + actionID, this.expandCode(inp);
    }
    onchangeActionSaveValue(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(22));
            let action = this.getAction(this.item.actions, id);
            action.value = obj.value.trim(), qubotEditor.dontSave = !1, qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    onchangeActionSaveStorage(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(22));
            let action = this.getAction(this.item.actions, id);
            action.storage = obj.value, "qudata" !== obj.value && "sheets" !== obj.value || qubotEditor.help.alert("storageNeedQudata"), 
            qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addActionEventInterface(container, action, actionID) {
        container = this.addInterfaceFrame(container, "event", actionID);
        this.addHint(container, "hintActionEvent");
        let inp = this.add(container, "input", "input");
        inp.value = action.key, inp.id = "qubot-action-event-key-" + actionID, inp.setAttribute("placeholder", "Event key"), 
        inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeActionEventKey(this)"), 
        this.addHint(inp, "hintActionEventKey");
        container = this.add(container, "group");
        inp = this.add(container, "checkbox", "input"), inp.id = "qubot-action-event-target-" + actionID, 
        inp.type = "checkbox", inp.checked = action.target;
        let lbl = this.add(container, "label", "label");
        lbl.innerHTML = "target", inp.setAttribute("onchange", "qubotEditor.stateEditor.actions.onchangeActionEventTarget(this)"), 
        this.addHint(inp, "hintActionEventTarget");
    }
    onchangeActionEventKey(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(23));
            let action = this.getAction(this.item.actions, id);
            action.key = obj.value.trim(), qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    onchangeActionEventTarget(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            var id = parseInt(obj.id.substring(26));
            let action = this.getAction(this.item.actions, id);
            action.target = obj.checked, qubotEditor.save();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addAction(obj, type) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            if (0 === this.item.actions.length) return this.insertAction(0, this.item.actions, type), 
            void qubotEditor.save();
            if ("action-add" === obj.parentNode.parentNode.className) {
                let idStr = obj.parentNode.parentNode.id.substring(17);
                var id = parseInt(idStr), id = this.getAction(this.item.actions, id);
                if (id && id.actions) return this.insertAction(0, id.actions, type), void qubotEditor.save();
            }
            let idStr;
            if (idStr = ("intent" === type ? obj : obj.parentNode.parentNode).parentNode.parentNode.id.substring(13), 
            idStr.length && !isNaN(idStr)) {
                let id = parseInt(idStr);
                this.addAcionTree(this.item.actions, 0, id, type), qubotEditor.save();
            }
            qubotEditor.listEditor.openCardCurState();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    addAcionTree(actions, totID, id, type) {
        if (id < totID) return totID;
        for (let i = 0; i < actions.length; i++) {
            if (totID === id) return this.insertAction(i + 1, actions, type), totID + 1;
            actions[i] && ("if" in actions[i] || "else" in actions[i] || "switch" in actions[i] || "case" in actions[i] || "default" in actions[i] ? totID = this.addAcionTree(actions[i].actions, totID + 1, id, type) : totID++);
        }
        return totID;
    }
    insertAction(i, actions, type) {
        "state" === type ? actions.splice(i, 0, {
            state: ""
        }) : "goto" === type ? actions.splice(i, 0, {
            goto: ""
        }) : "run" === type ? actions.splice(i, 0, {
            run: ""
        }) : "slot" === type ? actions.splice(i, 0, {
            slot: "",
            value: ""
        }) : "clear" === type ? actions.splice(i, 0, {
            clear: ""
        }) : "save" === type ? actions.splice(i, 0, {
            action: "save",
            storage: "",
            key: "",
            value: ""
        }) : "event" === type ? actions.splice(i, 0, {
            action: "event",
            key: "",
            target: !1
        }) : "script" === type ? actions.splice(i, 0, {
            script: ""
        }) : "note" === type ? actions.splice(i, 0, {
            note: ""
        }) : "if" === type ? actions.splice(i, 0, {
            if: "",
            actions: []
        }) : "else" === type ? actions.splice(i, 0, {
            else: "",
            actions: []
        }) : "elif" === type ? actions.splice(i, 0, {
            else: " ",
            actions: []
        }) : "switch" === type ? actions.splice(i, 0, {
            switch: "",
            actions: [ {
                case: "",
                actions: []
            } ]
        }) : "case" === type ? actions.splice(i, 0, {
            case: "",
            actions: []
        }) : "default" === type ? actions.splice(i, 0, {
            default: "",
            actions: []
        }) : "intent" === type && actions.splice(i, 0, {
            intent: "",
            words: {}
        }), this.show(), qubotEditor.listEditor.openCardCurState();
    }
    delAction(obj) {
        null !== qubotEditor.myBots.botNumber ? (obj = parseInt(obj.parentNode.parentNode.id.substring(13)), 
        this.delActionTree(this.item.actions, 0, obj), qubotEditor.listEditor.openCardCurState()) : qubotEditor.help.alert("YouCannotEdit");
    }
    delActionTree(actions, totID, id) {
        for (let i = 0; i < actions.length; i++) if (actions[i]) {
            if (totID === id) return actions.splice(i, 1), void this.show();
            "if" in actions[i] || "else" in actions[i] || "switch" in actions[i] || "case" in actions[i] || "default" in actions[i] ? totID = this.delActionTree(actions[i].actions, totID + 1, id) : totID++;
        }
        return totID;
    }
    changeActionPosition(startPos, curPos, newPos) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (this.changeActionPositionTree(this.item.actions, 0, startPos, curPos, newPos), 
        this.show()) : qubotEditor.help.alert("YouCannotEdit");
    }
    changeActionPositionTree(actions, totID, startPos, curPos, newPos) {
        for (let i = 0; i < actions.length; i++) if (actions[i]) {
            var action;
            if (totID === startPos) return void ((action = actions[curPos]) && (actions.splice(curPos, 1), 
            actions.splice(newPos, 0, action)));
            "actions" in actions[i] ? totID = this.changeActionPositionTree(actions[i].actions, totID + 1, startPos, curPos, newPos) : totID++;
        }
        return totID;
    }
    moveAction(obj, dir) {
        null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave ? (obj = parseInt(obj.parentNode.id.substring(13)), 
        this.moveActionTree(this.item.actions, 0, obj, dir), qubotEditor.save()) : qubotEditor.help.alert("YouCannotEdit");
    }
    moveActionTree(actions, totID, id, dir) {
        for (let i = 0; i < actions.length; i++) {
            if (totID === id) {
                if (0 === id && -1 == dir || id === actions.length - 1 && 1 == dir) return;
                var item = actions[i];
                return actions.splice(i, 1), actions.splice(i + dir, 0, item), void this.show();
            }
            "if" in actions[i] || "else" in actions[i] || "switch" in actions[i] || "case" in actions[i] || "default" in actions[i] ? totID = this.moveActionTree(actions[i].actions, totID + 1, id, dir) : totID++;
        }
        return totID;
    }
    copyAction(obj) {
        obj = parseInt(obj.id.substring(13));
        this.copyActionTree(this.item.actions, 0, obj);
    }
    copyActionTree(actions, totID, id) {
        for (let i = 0; i < actions.length; i++) {
            if (totID === id) return void (this.copiedAction = JSON.parse(JSON.stringify(actions[i])));
            "if" in actions[i] || "else" in actions[i] || "switch" in actions[i] || "case" in actions[i] || "default" in actions[i] ? totID = this.copyActionTree(actions[i].actions, totID + 1, id) : totID++;
        }
        return totID;
    }
    pasteAction(obj) {
        if (null !== qubotEditor.myBots.botNumber || qubotEditor.dontSave) {
            if (!this.copiedAction) return qubotEditor.help.alert("YouDontCopyActionYet"), void this.show();
            obj = parseInt(obj.id.substring(13));
            this.pasteActionTree(this.item.actions, 0, obj), this.show();
        } else qubotEditor.help.alert("YouCannotEdit");
    }
    pasteActionTree(actions, totID, id) {
        for (let i = 0; i < actions.length; i++) {
            if (totID === id) return void actions.splice(i + 1, 0, JSON.parse(JSON.stringify(this.copiedAction)));
            "if" in actions[i] || "else" in actions[i] || "switch" in actions[i] || "case" in actions[i] || "default" in actions[i] ? totID = this.pasteActionTree(actions[i].actions, totID + 1, id) : totID++;
        }
        return totID;
    }
}

var qubotCodeInput = {
    usedTemplates: {},
    defaultTemplate: void 0,
    CodeInput: class extends HTMLElement {
        constructor() {
            super();
        }
        update(text) {
            this.value != text && (this.value = text), this.querySelector("textarea").value != text && (this.querySelector("textarea").value = text);
            let result_element = this.querySelector("pre code");
            "\n" == text[text.length - 1] && (text += " "), result_element.innerHTML = this.escape_html(text), 
            this.template.includeCodeInputInHighlightFunc ? this.template.highlight(result_element, this) : this.template.highlight(result_element);
        }
        sync_scroll() {
            var input_element = this.querySelector("textarea");
            let result_element = this.template.preElementStyled ? this.querySelector("pre") : this.querySelector("pre code");
            result_element.scrollTop = input_element.scrollTop, result_element.scrollLeft = input_element.scrollLeft;
        }
        codeHeight() {
            return (this.template.preElementStyled ? this.querySelector("pre") : this.querySelector("pre code")).scrollHeight;
        }
        check_tab(event) {
            if (this.template.isCode) {
                let input_element = this.querySelector("textarea"), code = input_element.value;
                if ("Tab" == event.key) {
                    if (event.preventDefault(), input_element.selectionStart == input_element.selectionEnd) {
                        var before_selection = code.slice(0, input_element.selectionStart), after_selection = code.slice(input_element.selectionEnd, input_element.value.length), cursor_pos = input_element.selectionEnd + 1;
                        input_element.value = before_selection + "\t" + after_selection, input_element.selectionStart = cursor_pos, 
                        input_element.selectionEnd = cursor_pos;
                    } else {
                        let lines = input_element.value.split("\n"), letter_i = 0;
                        before_selection = input_element.selectionStart, after_selection = input_element.selectionEnd;
                        let number_indents = 0, first_line_indents = 0;
                        for (let i = 0; i < lines.length; i++) letter_i += lines[i].length, input_element.selectionStart < letter_i && input_element.selectionEnd > letter_i - lines[i].length && (event.shiftKey ? "\t" == lines[i][0] && (lines[i] = lines[i].slice(1), 
                        0 == number_indents && first_line_indents--, number_indents--) : (lines[i] = "\t" + lines[i], 
                        0 == number_indents && first_line_indents++, number_indents++));
                        input_element.value = lines.join("\n"), input_element.selectionStart = before_selection + first_line_indents, 
                        input_element.selectionEnd = after_selection + number_indents;
                    }
                    this.update(input_element.value);
                }
            }
        }
        escape_html(text) {
            return text.replace(new RegExp("&", "g"), "&amp;").replace(new RegExp("<", "g"), "&lt;");
        }
        connectedCallback() {
            this.template = qubotCodeInput.usedTemplates[this.getAttribute("template") || qubotCodeInput.defaultTemplate], 
            this.template.preElementStyled && this.classList.add("code-input_pre-element-styled");
            var lang = this.getAttribute("lang"), placeholder = this.getAttribute("placeholder") || this.getAttribute("lang") || "", value = this.value || this.innerHTML || "";
            this.innerHTML = "";
            let textarea = document.createElement("textarea"), code = (textarea.placeholder = placeholder, 
            textarea.value = value, textarea.setAttribute("spellcheck", "false"), this.getAttribute("name") && (textarea.setAttribute("name", this.getAttribute("name")), 
            this.removeAttribute("name")), textarea.setAttribute("oninput", "this.parentElement.update(this.value); this.parentElement.sync_scroll();"), 
            textarea.setAttribute("onscroll", "this.parentElement.sync_scroll();"), textarea.setAttribute("onkeydown", "this.parentElement.check_tab(event);"), 
            this.append(textarea), document.createElement("code")), pre = (this.template.isCode && null != lang && code.classList.add("language-" + lang), 
            code.innerText = value, document.createElement("pre"));
            pre.setAttribute("aria-hidden", "true"), pre.append(code), this.append(pre), this.update(value, this);
        }
        static get observedAttributes() {
            return [ "value", "placeholder", "lang", "template" ];
        }
        attributeChangedCallback(name, oldValue, newValue) {
            switch (name) {
              case "value":
                this.update(newValue);
                break;

              case "placeholder":
                this.querySelector("textarea").placeholder = newValue;
                break;

              case "template":
                this.template = qubotCodeInput.usedTemplates[newValue || qubotCodeInput.defaultTemplate], 
                this.template.preElementStyled ? this.classList.add("code-input_pre-element-styled") : this.classList.remove("code-input_pre-element-styled"), 
                this.update(this.value);

              case "lang":
                let code = this.querySelector("pre code");
                if (code) {
                    let textarea = this.querySelector("textarea");
                    code.className = null != newValue ? "language-" + newValue : "", textarea.placeholder == oldValue && (textarea.placeholder = newValue), 
                    this.update(this.value);
                }

              default:
                this.update(this.value);
            }
        }
        get value() {
            return this.getAttribute("value");
        }
        set value(val) {
            return this.setAttribute("value", val);
        }
        get placeholder() {
            return this.getAttribute("placeholder");
        }
        set placeholder(val) {
            return this.setAttribute("placeholder", val);
        }
    },
    registerTemplate: function(template_name, template) {
        qubotCodeInput.usedTemplates[template_name] = template, qubotCodeInput.defaultTemplate = template_name;
    },
    templates: {
        custom(highlight = function() {}, preElementStyled = !0, isCode = !0, includeCodeInputInHighlightFunc = !1) {
            return {
                highlight: highlight,
                includeCodeInputInHighlightFunc: includeCodeInputInHighlightFunc,
                preElementStyled: preElementStyled,
                isCode: isCode
            };
        },
        prism(prism) {
            return {
                includeCodeInputInHighlightFunc: !1,
                highlight: prism.highlightElement,
                preElementStyled: !0,
                isCode: !0
            };
        },
        hljs(hljs) {
            return {
                includeCodeInputInHighlightFunc: !1,
                highlight: hljs.highlightElement,
                preElementStyled: !1,
                isCode: !0
            };
        },
        characterLimit() {
            return {
                highlight: function(result_element, code_input) {
                    var character_limit = Number(code_input.getAttribute("data-character-limit")), normal_characters = code_input.escape_html(code_input.value.slice(0, character_limit)), character_limit = code_input.escape_html(code_input.value.slice(character_limit));
                    result_element.innerHTML = normal_characters + `<mark class="overflow">${character_limit}</mark>`, 
                    0 < character_limit.length && (result_element.innerHTML += ` <mark class="overflow-msg">${code_input.getAttribute("data-overflow-msg") || "(Character limit reached)"}</mark>`);
                },
                includeCodeInputInHighlightFunc: !0,
                preElementStyled: !0,
                isCode: !1
            };
        },
        rainbowText(rainbow_colors = [ "red", "orangered", "orange", "goldenrod", "gold", "green", "darkgreen", "navy", "blue", "magenta" ], delimiter = "") {
            return {
                highlight: function(result_element, code_input) {
                    let html_result = [];
                    var sections = code_input.value.split(code_input.template.delimiter);
                    for (let i = 0; i < sections.length; i++) html_result.push(`<span style="color: ${code_input.template.rainbow_colors[i % code_input.template.rainbow_colors.length]}">${code_input.escape_html(sections[i])}</span>`);
                    result_element.innerHTML = html_result.join(code_input.template.delimiter);
                },
                includeCodeInputInHighlightFunc: !0,
                preElementStyled: !0,
                isCode: !1,
                rainbow_colors: rainbow_colors,
                delimiter: delimiter
            };
        }
    }
}, _self = (customElements.define("code-input", qubotCodeInput.CodeInput), "undefined" != typeof window ? window : "undefined" != typeof WorkerGlobalScope && self instanceof WorkerGlobalScope ? self : {}), Prism = function(_self) {
    var lang = /(?:^|\s)lang(?:uage)?-([\w-]+)(?=\s|$)/i, uniqueId = 0, plainTextGrammar = {}, _ = {
        manual: _self.Prism && _self.Prism.manual,
        disableWorkerMessageHandler: _self.Prism && _self.Prism.disableWorkerMessageHandler,
        util: {
            encode: function encode(tokens) {
                return tokens instanceof Token ? new Token(tokens.type, encode(tokens.content), tokens.alias) : Array.isArray(tokens) ? tokens.map(encode) : tokens.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/\u00a0/g, " ");
            },
            type: function(o) {
                return Object.prototype.toString.call(o).slice(8, -1);
            },
            objId: function(obj) {
                return obj.__id || Object.defineProperty(obj, "__id", {
                    value: ++uniqueId
                }), obj.__id;
            },
            clone: function deepClone(o, visited) {
                var clone, id;
                switch (visited = visited || {}, _.util.type(o)) {
                  case "Object":
                    if (id = _.util.objId(o), visited[id]) return visited[id];
                    for (var key in clone = {}, visited[id] = clone, o) o.hasOwnProperty(key) && (clone[key] = deepClone(o[key], visited));
                    return clone;

                  case "Array":
                    return (id = _.util.objId(o), visited[id]) ? visited[id] : (clone = [], visited[id] = clone, 
                    o.forEach(function(v, i) {
                        clone[i] = deepClone(v, visited);
                    }), clone);

                  default:
                    return o;
                }
            },
            getLanguage: function(element) {
                for (;element; ) {
                    var m = lang.exec(element.className);
                    if (m) return m[1].toLowerCase();
                    element = element.parentElement;
                }
                return "none";
            },
            setLanguage: function(element, language) {
                element.className = element.className.replace(RegExp(lang, "gi"), ""), element.classList.add("language-" + language);
            },
            currentScript: function() {
                if ("undefined" == typeof document) return null;
                if ("currentScript" in document) return document.currentScript;
                try {
                    throw new Error();
                } catch (err) {
                    var src = (/at [^(\r\n]*\((.*):[^:]+:[^:]+\)$/i.exec(err.stack) || [])[1];
                    if (src) {
                        var i, scripts = document.getElementsByTagName("script");
                        for (i in scripts) if (scripts[i].src == src) return scripts[i];
                    }
                    return null;
                }
            },
            isActive: function(element, className, defaultActivation) {
                for (var no = "no-" + className; element; ) {
                    var classList = element.classList;
                    if (classList.contains(className)) return !0;
                    if (classList.contains(no)) return !1;
                    element = element.parentElement;
                }
                return !!defaultActivation;
            }
        },
        languages: {
            plain: plainTextGrammar,
            plaintext: plainTextGrammar,
            text: plainTextGrammar,
            txt: plainTextGrammar,
            extend: function(id, redef) {
                var key, lang = _.util.clone(_.languages[id]);
                for (key in redef) lang[key] = redef[key];
                return lang;
            },
            insertBefore: function(inside, before, insert, root) {
                var token, grammar = (root = root || _.languages)[inside], ret = {};
                for (token in grammar) if (grammar.hasOwnProperty(token)) {
                    if (token == before) for (var newToken in insert) insert.hasOwnProperty(newToken) && (ret[newToken] = insert[newToken]);
                    insert.hasOwnProperty(token) || (ret[token] = grammar[token]);
                }
                var old = root[inside];
                return root[inside] = ret, _.languages.DFS(_.languages, function(key, value) {
                    value === old && key != inside && (this[key] = ret);
                }), ret;
            },
            DFS: function DFS(o, callback, type, visited) {
                visited = visited || {};
                var i, property, propertyType, objId = _.util.objId;
                for (i in o) o.hasOwnProperty(i) && (callback.call(o, i, o[i], type || i), property = o[i], 
                "Object" !== (propertyType = _.util.type(property)) || visited[objId(property)] ? "Array" !== propertyType || visited[objId(property)] || (visited[objId(property)] = !0, 
                DFS(property, callback, i, visited)) : (visited[objId(property)] = !0, DFS(property, callback, null, visited)));
            }
        },
        plugins: {},
        highlightAll: function(async, callback) {
            _.highlightAllUnder(document, async, callback);
        },
        highlightAllUnder: function(container, async, callback) {
            var env = {
                callback: callback,
                container: container,
                selector: 'code[class*="language-"], [class*="language-"] code, code[class*="lang-"], [class*="lang-"] code'
            };
            _.hooks.run("before-highlightall", env), env.elements = Array.prototype.slice.apply(env.container.querySelectorAll(env.selector)), 
            _.hooks.run("before-all-elements-highlight", env);
            for (var element, i = 0; element = env.elements[i++]; ) _.highlightElement(element, !0 === async, env.callback);
        },
        highlightElement: function(element, async, callback) {
            var language = _.util.getLanguage(element), grammar = _.languages[language], parent = (_.util.setLanguage(element, language), 
            element.parentElement);
            parent && "pre" === parent.nodeName.toLowerCase() && _.util.setLanguage(parent, language);
            var env = {
                element: element,
                language: language,
                grammar: grammar,
                code: element.textContent
            };
            function insertHighlightedCode(highlightedCode) {
                env.highlightedCode = highlightedCode, _.hooks.run("before-insert", env), env.element.innerHTML = env.highlightedCode, 
                _.hooks.run("after-highlight", env), _.hooks.run("complete", env), callback && callback.call(env.element);
            }
            if (_.hooks.run("before-sanity-check", env), (parent = env.element.parentElement) && "pre" === parent.nodeName.toLowerCase() && !parent.hasAttribute("tabindex") && parent.setAttribute("tabindex", "0"), 
            !env.code) return _.hooks.run("complete", env), void (callback && callback.call(env.element));
            _.hooks.run("before-highlight", env), env.grammar ? async && _self.Worker ? ((language = new Worker(_.filename)).onmessage = function(evt) {
                insertHighlightedCode(evt.data);
            }, language.postMessage(JSON.stringify({
                language: env.language,
                code: env.code,
                immediateClose: !0
            }))) : insertHighlightedCode(_.highlight(env.code, env.grammar, env.language)) : insertHighlightedCode(_.util.encode(env.code));
        },
        highlight: function(text, grammar, language) {
            text = {
                code: text,
                grammar: grammar,
                language: language
            };
            return _.hooks.run("before-tokenize", text), text.tokens = _.tokenize(text.code, text.grammar), 
            _.hooks.run("after-tokenize", text), Token.stringify(_.util.encode(text.tokens), text.language);
        },
        tokenize: function(text, grammar) {
            var rest = grammar.rest;
            if (rest) {
                for (var token in rest) grammar[token] = rest[token];
                delete grammar.rest;
            }
            for (var tokenList = new LinkedList(), list = (addAfter(tokenList, tokenList.head, text), 
            function matchGrammar(text, tokenList, grammar, startNode, startPos, rematch) {
                for (var token in grammar) if (grammar.hasOwnProperty(token) && grammar[token]) {
                    var patterns = grammar[token];
                    patterns = Array.isArray(patterns) ? patterns : [ patterns ];
                    for (var j = 0; j < patterns.length; ++j) {
                        if (rematch && rematch.cause == token + "," + j) return;
                        for (var flags, patternObj = patterns[j], inside = patternObj.inside, lookbehind = !!patternObj.lookbehind, greedy = !!patternObj.greedy, alias = patternObj.alias, pattern = (greedy && !patternObj.pattern.global && (flags = patternObj.pattern.toString().match(/[imsuy]*$/)[0], 
                        patternObj.pattern = RegExp(patternObj.pattern.source, flags + "g")), patternObj.pattern || patternObj), currentNode = startNode.next, pos = startPos; currentNode !== tokenList.tail && !(rematch && pos >= rematch.reach); pos += currentNode.value.length, 
                        currentNode = currentNode.next) {
                            var str = currentNode.value;
                            if (tokenList.length > text.length) return;
                            if (!(str instanceof Token)) {
                                var match, removeCount = 1;
                                if (greedy) {
                                    if (!(match = matchPattern(pattern, pos, text, lookbehind)) || match.index >= text.length) break;
                                    var from = match.index, to = match.index + match[0].length, p = pos;
                                    for (p += currentNode.value.length; p <= from; ) currentNode = currentNode.next, 
                                    p += currentNode.value.length;
                                    if (p -= currentNode.value.length, pos = p, currentNode.value instanceof Token) continue;
                                    for (var k = currentNode; k !== tokenList.tail && (p < to || "string" == typeof k.value); k = k.next) removeCount++, 
                                    p += k.value.length;
                                    removeCount--, str = text.slice(pos, p), match.index -= pos;
                                } else if (!(match = matchPattern(pattern, 0, str, lookbehind))) continue;
                                var from = match.index, matchStr = match[0], before = str.slice(0, from), after = str.slice(from + matchStr.length), str = pos + str.length, removeFrom = (rematch && str > rematch.reach && (rematch.reach = str), 
                                currentNode.prev), before = (before && (removeFrom = addAfter(tokenList, removeFrom, before), 
                                pos += before.length), removeRange(tokenList, removeFrom, removeCount), new Token(token, inside ? _.tokenize(matchStr, inside) : matchStr, alias, matchStr));
                                currentNode = addAfter(tokenList, removeFrom, before), after && addAfter(tokenList, currentNode, after), 
                                1 < removeCount && (matchStr = {
                                    cause: token + "," + j,
                                    reach: str
                                }, matchGrammar(text, tokenList, grammar, currentNode.prev, pos, matchStr), rematch && matchStr.reach > rematch.reach && (rematch.reach = matchStr.reach));
                            }
                        }
                    }
                }
            }(text, tokenList, grammar, tokenList.head, 0), tokenList), array = [], node = list.head.next; node !== list.tail; ) array.push(node.value), 
            node = node.next;
            return array;
        },
        hooks: {
            all: {},
            add: function(name, callback) {
                var hooks = _.hooks.all;
                hooks[name] = hooks[name] || [], hooks[name].push(callback);
            },
            run: function(name, env) {
                var callbacks = _.hooks.all[name];
                if (callbacks && callbacks.length) for (var callback, i = 0; callback = callbacks[i++]; ) callback(env);
            }
        },
        Token: Token
    };
    function Token(type, content, alias, matchedStr) {
        this.type = type, this.content = content, this.alias = alias, this.length = 0 | (matchedStr || "").length;
    }
    function matchPattern(pattern, pos, text, lookbehind) {
        pattern.lastIndex = pos;
        pos = pattern.exec(text);
        return pos && lookbehind && pos[1] && (pattern = pos[1].length, pos.index += pattern, 
        pos[0] = pos[0].slice(pattern)), pos;
    }
    function LinkedList() {
        var head = {
            value: null,
            prev: null,
            next: null
        }, tail = {
            value: null,
            prev: head,
            next: null
        };
        head.next = tail, this.head = head, this.tail = tail, this.length = 0;
    }
    function addAfter(list, node, value) {
        var next = node.next, value = {
            value: value,
            prev: node,
            next: next
        };
        return node.next = value, next.prev = value, list.length++, value;
    }
    function removeRange(list, node, count) {
        for (var next = node.next, i = 0; i < count && next !== list.tail; i++) next = next.next;
        (node.next = next).prev = node, list.length -= i;
    }
    if (_self.Prism = _, Token.stringify = function stringify(o, language) {
        if ("string" == typeof o) return o;
        var s;
        if (Array.isArray(o)) return s = "", o.forEach(function(e) {
            s += stringify(e, language);
        }), s;
        var name, env = {
            type: o.type,
            content: stringify(o.content, language),
            tag: "span",
            classes: [ "token", o.type ],
            attributes: {},
            language: language
        }, o = o.alias, attributes = (o && (Array.isArray(o) ? Array.prototype.push.apply(env.classes, o) : env.classes.push(o)), 
        _.hooks.run("wrap", env), "");
        for (name in env.attributes) attributes += " " + name + '="' + (env.attributes[name] || "").replace(/"/g, "&quot;") + '"';
        return "<" + env.tag + ' class="' + env.classes.join(" ") + '"' + attributes + ">" + env.content + "</" + env.tag + ">";
    }, !_self.document) return _self.addEventListener && (_.disableWorkerMessageHandler || _self.addEventListener("message", function(evt) {
        var evt = JSON.parse(evt.data), lang = evt.language, code = evt.code, evt = evt.immediateClose;
        _self.postMessage(_.highlight(code, _.languages[lang], lang)), evt && _self.close();
    }, !1)), _;
    var readyState, plainTextGrammar = _.util.currentScript();
    function highlightAutomaticallyCallback() {
        _.manual || _.highlightAll();
    }
    return plainTextGrammar && (_.filename = plainTextGrammar.src, plainTextGrammar.hasAttribute("data-manual") && (_.manual = !0)), 
    _.manual || ("loading" === (readyState = document.readyState) || "interactive" === readyState && plainTextGrammar && plainTextGrammar.defer ? document.addEventListener("DOMContentLoaded", highlightAutomaticallyCallback) : window.requestAnimationFrame ? window.requestAnimationFrame(highlightAutomaticallyCallback) : window.setTimeout(highlightAutomaticallyCallback, 16)), 
    _;
}(_self);

"undefined" != typeof module && module.exports && (module.exports = Prism), "undefined" != typeof global && (global.Prism = Prism), 
Prism.languages.markup = {
    comment: {
        pattern: /<!--(?:(?!<!--)[\s\S])*?-->/,
        greedy: !0
    },
    prolog: {
        pattern: /<\?[\s\S]+?\?>/,
        greedy: !0
    },
    doctype: {
        pattern: /<!DOCTYPE(?:[^>"'[\]]|"[^"]*"|'[^']*')+(?:\[(?:[^<"'\]]|"[^"]*"|'[^']*'|<(?!!--)|<!--(?:[^-]|-(?!->))*-->)*\]\s*)?>/i,
        greedy: !0,
        inside: {
            "internal-subset": {
                pattern: /(^[^\[]*\[)[\s\S]+(?=\]>$)/,
                lookbehind: !0,
                greedy: !0,
                inside: null
            },
            string: {
                pattern: /"[^"]*"|'[^']*'/,
                greedy: !0
            },
            punctuation: /^<!|>$|[[\]]/,
            "doctype-tag": /^DOCTYPE/i,
            name: /[^\s<>'"]+/
        }
    },
    cdata: {
        pattern: /<!\[CDATA\[[\s\S]*?\]\]>/i,
        greedy: !0
    },
    tag: {
        pattern: /<\/?(?!\d)[^\s>\/=$<%]+(?:\s(?:\s*[^\s>\/=]+(?:\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?=[\s/>])))+)?\s*\/?>/,
        greedy: !0,
        inside: {
            tag: {
                pattern: /^<\/?[^\s>\/]+/,
                inside: {
                    punctuation: /^<\/?/,
                    namespace: /^[^\s>\/:]+:/
                }
            },
            "special-attr": [],
            "attr-value": {
                pattern: /=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+)/,
                inside: {
                    punctuation: [ {
                        pattern: /^=/,
                        alias: "attr-equals"
                    }, /"|'/ ]
                }
            },
            punctuation: /\/?>/,
            "attr-name": {
                pattern: /[^\s>\/]+/,
                inside: {
                    namespace: /^[^\s>\/:]+:/
                }
            }
        }
    },
    entity: [ {
        pattern: /&[\da-z]{1,8};/i,
        alias: "named-entity"
    }, /&#x?[\da-f]{1,8};/i ]
}, Prism.languages.markup.tag.inside["attr-value"].inside.entity = Prism.languages.markup.entity, 
Prism.languages.markup.doctype.inside["internal-subset"].inside = Prism.languages.markup, 
Prism.hooks.add("wrap", function(env) {
    "entity" === env.type && (env.attributes.title = env.content.replace(/&amp;/, "&"));
}), Object.defineProperty(Prism.languages.markup.tag, "addInlined", {
    value: function(tagName, lang) {
        var includedCdataInside = {}, includedCdataInside = (includedCdataInside["language-" + lang] = {
            pattern: /(^<!\[CDATA\[)[\s\S]+?(?=\]\]>$)/i,
            lookbehind: !0,
            inside: Prism.languages[lang]
        }, includedCdataInside.cdata = /^<!\[CDATA\[|\]\]>$/i, {
            "included-cdata": {
                pattern: /<!\[CDATA\[[\s\S]*?\]\]>/i,
                inside: includedCdataInside
            }
        }), lang = (includedCdataInside["language-" + lang] = {
            pattern: /[\s\S]+/,
            inside: Prism.languages[lang]
        }, {});
        lang[tagName] = {
            pattern: RegExp(/(<__[^>]*>)(?:<!\[CDATA\[(?:[^\]]|\](?!\]>))*\]\]>|(?!<!\[CDATA\[)[\s\S])*?(?=<\/__>)/.source.replace(/__/g, function() {
                return tagName;
            }), "i"),
            lookbehind: !0,
            greedy: !0,
            inside: includedCdataInside
        }, Prism.languages.insertBefore("markup", "cdata", lang);
    }
}), Object.defineProperty(Prism.languages.markup.tag, "addAttribute", {
    value: function(attrName, lang) {
        Prism.languages.markup.tag.inside["special-attr"].push({
            pattern: RegExp(/(^|["'\s])/.source + "(?:" + attrName + ")" + /\s*=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))/.source, "i"),
            lookbehind: !0,
            inside: {
                "attr-name": /^[^\s=]+/,
                "attr-value": {
                    pattern: /=[\s\S]+/,
                    inside: {
                        value: {
                            pattern: /(^=\s*(["']|(?!["'])))\S[\s\S]*(?=\2$)/,
                            lookbehind: !0,
                            alias: [ lang, "language-" + lang ],
                            inside: Prism.languages[lang]
                        },
                        punctuation: [ {
                            pattern: /^=/,
                            alias: "attr-equals"
                        }, /"|'/ ]
                    }
                }
            }
        });
    }
}), Prism.languages.html = Prism.languages.markup, Prism.languages.mathml = Prism.languages.markup, 
Prism.languages.svg = Prism.languages.markup, Prism.languages.xml = Prism.languages.extend("markup", {}), 
Prism.languages.ssml = Prism.languages.xml, Prism.languages.atom = Prism.languages.xml, 
Prism.languages.rss = Prism.languages.xml, Prism.languages.clike = {
    comment: [ {
        pattern: /(^|[^\\])\/\*[\s\S]*?(?:\*\/|$)/,
        lookbehind: !0,
        greedy: !0
    }, {
        pattern: /(^|[^\\:])\/\/.*/,
        lookbehind: !0,
        greedy: !0
    } ],
    string: {
        pattern: /(["'])(?:\\(?:\r\n|[\s\S])|(?!\1)[^\\\r\n])*\1/,
        greedy: !0
    },
    "class-name": {
        pattern: /(\b(?:class|extends|implements|instanceof|interface|new|trait)\s+|\bcatch\s+\()[\w.\\]+/i,
        lookbehind: !0,
        inside: {
            punctuation: /[.\\]/
        }
    },
    keyword: /\b(?:break|catch|continue|do|else|finally|for|function|if|in|instanceof|new|null|return|throw|try|while)\b/,
    boolean: /\b(?:false|true)\b/,
    function: /\b\w+(?=\()/,
    number: /\b0x[\da-f]+\b|(?:\b\d+(?:\.\d*)?|\B\.\d+)(?:e[+-]?\d+)?/i,
    operator: /[<>]=?|[!=]=?=?|--?|\+\+?|&&?|\|\|?|[?*/~^%]/,
    punctuation: /[{}[\];(),.:]/
}, Prism.languages.qubot = {
    comment: [ {
        pattern: /(^|[^\\])#.*/,
        lookbehind: !0,
        greedy: !0
    }, {
        pattern: /(^|[^\\:])\/\/.*/,
        lookbehind: !0,
        greedy: !0
    } ],
    "string-interpolation": {
        pattern: /(?:f|fr|rf)(?:("""|''')[\s\S]*?\1|("|')(?:\\.|(?!\2)[^\\\r\n])*\2)/i,
        greedy: !0,
        inside: {
            interpolation: {
                pattern: /((?:^|[^{])(?:\{\{)*)\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}]|\{(?!\{)(?:[^{}])+\})+\})+\}/,
                lookbehind: !0,
                inside: {
                    "format-spec": {
                        pattern: /(:)[^:(){}]+(?=\}$)/,
                        lookbehind: !0
                    },
                    "conversion-option": {
                        pattern: /![sra](?=[:}]$)/,
                        alias: "punctuation"
                    },
                    rest: null
                }
            },
            string: /[\s\S]+/
        }
    },
    "triple-quoted-string": {
        pattern: /(?:[rub]|br|rb)?("""|''')[\s\S]*?\1/i,
        greedy: !0,
        alias: "string"
    },
    string: {
        pattern: /(?:[rub]|br|rb)?("|')(?:\\.|(?!\1)[^\\\r\n])*\1/i,
        greedy: !0
    },
    function: {
        pattern: /((?:^|\s)def[ \t]+)[a-zA-Z_]\w*(?=\s*\()/g,
        lookbehind: !0
    },
    "class-name": {
        pattern: /(\bclass\s+)\w+/i,
        lookbehind: !0
    },
    decorator: {
        pattern: /(^[\t ]*)@\w+(?:\.\w+)*/m,
        lookbehind: !0,
        alias: [ "annotation", "punctuation" ],
        inside: {
            punctuation: /\./
        }
    },
    keyword: /\b(?:_(?=\s*:)|and|not|or)\b/,
    builtin: /\b(?:set|if|while|len|clear|push|add|sum|total|get|foreach|text|images|image|set_check|get_check|set_radio|get_radio)\b/,
    boolean: /\b(?:False|None|True|false|true|null)\b/,
    number: /\b0(?:b(?:_?[01])+|o(?:_?[0-7])+|x(?:_?[a-f0-9])+)\b|(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)(?:e[+-]?\d+(?:_\d+)*)?j?(?!\w)/i,
    operator: /[-+%=]=?|!=|:=|\*\*?=?|\/\/?=?|<[<=>]?|>[=>]?|[&|^~]/,
    punctuation: /[{};(),.:]/,
    punctuation2: /[\[\]]/,
    variable: /\${1}(?:\w+\b)/
}, Prism.languages.qubot["string-interpolation"].inside.interpolation.inside.rest = Prism.languages.qubot, 
Prism.languages.qb = Prism.languages.qubot;